importScripts(
        '../../libs/moment.min.js',
        '../../libs/moment-timezone.min.js',
        '../../libs/rrule.min.js'
        );


function EventsCalcThread(events, params) {
    this.eventsPool = events;
    this.params = params;
}

EventsCalcThread.prototype.dateToRFC = function (date) {
    var dt = moment(date);
    var rfc = dt.utc().format('YYYYMMDDTHHmmss') + 'Z';
    return rfc;
};

EventsCalcThread.prototype.grep = function (elems, callback, inv) {
    var ret = [], retVal, i;
    inv = !!inv;

    // Go through the array, only saving the items
    // that pass the validator function
    for (i = 0, length = elems.length; i < length; i++) {
        retVal = !!callback(elems[i], i);
        if (inv !== retVal) {
            ret.push(elems[i]);
        }
    }

    return ret;
};

EventsCalcThread.prototype.merge = function (array1, array2) {

    return array1.concat(array2);

};

// sort by featured
// @todo recheck solution...
EventsCalcThread.prototype.sortByFeatured = function (a, b) {

    return 0;

    if (a.featured !== 0 && b.featured !== 0) {
        return a.start_date_timestamp - b.start_date_timestamp;
    } else {
        return a.featured - b.featured;
    }

};

// sort by timestamp oldest -> newest
EventsCalcThread.prototype.sortByTimestamp = function (a, b) {
    if (a.start_date_timestamp < b.start_date_timestamp) {
        return -1;
    } else if (a.start_date_timestamp > b.start_date_timestamp) {
        return 1;
    } else {
        return 0;
    }
};

EventsCalcThread.prototype.isOverriden = function (event) {

    var recurrenceId = moment.tz(event.start_date, event.timezone).format('YMMDDTHmss');
    var overrides = this.getOverrides(event.uid, recurrenceId);
    if (overrides.length > 0) {
        return true;
    }

    return false;

};

EventsCalcThread.prototype.getOverridesInRange = function (a, b, filteredPool) {

    var pool = filteredPool ? filteredPool : this.eventsPool;

    return this.grep(pool, function (event) {

        if (!event.recurrence_id) {
            return false;
        }

        if (b > event.start_date_timestamp && event.end_date_timestamp >= a) {
            return true;
        }

        return false;
    });
};

EventsCalcThread.prototype.getOverrides = function (uid, recurrenceId) {

    return this.grep(this.eventsPool, function (poolEvent) {

        var m, offset = 0;

        if (!poolEvent.recurrence_id) {
            return false;
        }

        if (poolEvent.recurrence_id.indexOf('Z') !== -1) {

            m = new moment.tz(poolEvent.recurrence_id, 'UTC');

            if (m.isDST()) {
                offset = -1 * ((poolEvent.timezone_utc_offset - 3600) / 3600);
            } else {
                offset = -1 * (poolEvent.timezone_utc_offset / 3600);
            }

            m.utcOffset(offset);

            poolEvent.recurrence_id = m.format('YYYYMMDDTHHmmss');

        } else if (poolEvent.recurrence_id.length === 8) {
            recurrenceId = recurrenceId.substr(0, 8);
        }

        return uid === poolEvent.uid && poolEvent.recurrence_id === recurrenceId;
    });
};

EventsCalcThread.prototype.getRangeUnix = function (dateOrString, endOf) {

    var momentDate;
    var dateArray = null;

    if (!dateOrString) {
        throw 'Invalid dateOrString value';
    } else if (true === dateOrString instanceof Date) {
        dateArray = [dateOrString.getFullYear(), dateOrString.getMonth(), dateOrString.getDate()];
    } else if ('string' === typeof dateOrString) {
        dateOrString = moment.utc(dateOrString).toDate();
        dateArray = [dateOrString.getFullYear(), dateOrString.getMonth(), dateOrString.getDate()];
    } else {
        console.log('Unsupported dateOrString');
    }

    momentDate = moment.tz('UTC');
    momentDate.set({year: dateArray[0], month: dateArray[1], date: dateArray[2], hour: 0, minute: 0, second: 0, millisecond: 0});

    if (endOf) {
        momentDate.add(24, 'hours');
    }

    return momentDate.unix();

};

EventsCalcThread.prototype.repeaterGet = function (event, rangeStart, rangeEnd) {

    var parent = this;

    var eventStartDate = moment.utc(event.start_date);
    var rangeStartDate = moment.unix(rangeStart).utc();
    var rangeEndDate = moment.unix(rangeEnd).utc();
    var rfcString = event.rrule + ';DTSTART=' + parent.dateToRFC(eventStartDate);

    var rruleSet;
    var rEvents = [];
    var result;

    var exdates;
    var exdatesFixedArray;

    var i, j;
    var dt, st;

    var rEvent;

    if (eventStartDate.isBefore(rangeStartDate)) {
        rangeStartDate = eventStartDate;
    }

    if (event.exdate) {

        exdates = event.exdate.split(',');
        exdatesFixedArray = [];

        for (i = 0; i < exdates.length; i++) {

            if (exdates[i].length === 8) {

                st = moment.tz(event.start_date, event.timezone);
                dt = moment.tz(exdates[i], event.timezone);

                exdatesFixedArray.push(dt
                        .hours(st.hours())
                        .minutes(st.minutes())
                        .seconds(0)
                        .format('YYYYMMDD\THHmmss') + 'Z');

            } else {

                exdatesFixedArray.push(exdates[i]);

            }
        }

        rfcString += '\nEXDATE:' + exdatesFixedArray.join(',');

    }

    rruleSet = rrule.rrulestr(rfcString, {
        forceset: true
    });

    result = rruleSet.between(rangeStartDate.toDate(), rangeEndDate.toDate(), true);

    for (j = 0; j < result.length; j++) {

        rEvent = parent.getREvent(event, result[j]);

        if (rEvent.end_date_timestamp < rangeStart) {
            continue;
        }

        rEvents.push(rEvent);
    }


    return rEvents;

};

EventsCalcThread.prototype.repeaterGetNearest = function (event, rangeStart, rangeEnd) {

    var parent = this;

    var eventStartDate = moment.utc(event.start_date);
    var rangeStartDate = moment.unix(rangeStart).utc();
    var rangeEndDate = moment.unix(rangeEnd).utc();
    var rfcString = event.rrule + ';DTSTART=' + parent.dateToRFC(eventStartDate);

    var rruleSet;
    var rEvents = [];
    var result;

    var exdates;
    var exdatesFixedArray;

    var i, j;
    var dt, st;

    var rEvent;

    if (event.exdate) {

        exdates = event.exdate.split(',');
        exdatesFixedArray = [];

        for (i = 0; i < exdates.length; i++) {

            if (exdates[i].length === 8) {

                st = moment.tz(event.start_date, event.timezone);
                dt = moment.tz(exdates[i], event.timezone);

                exdatesFixedArray.push(dt
                        .hours(st.hours())
                        .minutes(st.minutes())
                        .seconds(0)
                        .format('YYYYMMDD\THHmmss') + 'Z');

            } else {

                exdatesFixedArray.push(exdates[i]);

            }
        }

        rfcString += '\nEXDATE:' + exdatesFixedArray.join(',');

    }

    rruleSet = rrule.rrulestr(rfcString, {
        forceset: true
    });

    result = rruleSet.after(rangeStartDate.toDate(), true);

    if (!result) {
        result = rruleSet.before(rangeStartDate.toDate(), true);
    }

    if (result) {
        rEvent = parent.getREvent(event, result);

        if (rEvent.end_date_timestamp > rangeStart) {
            rEvents.push(rEvent);
        }

    }


    return rEvents;

};

EventsCalcThread.prototype.getREvent = function (event, date) {

    var rEvent;
    var offset = 0;
    var initialStartDate = moment.tz(event.start_date, event.timezone);
    var initialEndDate = moment.tz(event.end_date, event.timezone);
    var occurenceDate = moment.tz(date, event.timezone);
    occurenceDate.subtract(occurenceDate.utcOffset(), 'minutes');

    offset = occurenceDate.diff(initialStartDate, 'seconds');

    // not by reference! need fresh copy
    rEvent = JSON.parse(JSON.stringify(event));

    rEvent.start_date_timestamp = initialStartDate.unix() + offset;
    rEvent.end_date_timestamp = initialEndDate.unix() + offset;

    rEvent.start_date = initialStartDate.add(offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');
    rEvent.end_date = initialEndDate.add(offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');
    rEvent.repeat_time_offset = offset;

    return rEvent;

};

EventsCalcThread.prototype.getEvents = function () {

    var parent = this,
            a,
            b,
            i,
            j,
            start,
            end,
            pick = [],
            overrides,
            eventLoop,
            now = moment().unix();

    parent.filterGetEvents = [];
    a = parent.getRangeUnix(parent.params.from);
    b = parent.getRangeUnix(parent.params.to ? parent.params.to : parent.params.to, true);

    // Get overrides in this range and add them to "pick" array
    overrides = parent.getOverridesInRange(a, b);
    if (overrides.length > 0) {
        parent.filterGetEvents = overrides;
        overrides = parent.filterGetEvents;
        pick = parent.merge(pick, parent.filterGetEvents);
    }

    // Exclude overriden
    parent.filterGetEvents = parent.grep(parent.eventsPool, function (poolEvent) {
        return !poolEvent.recurrence_id;
    });

    for (i = 0; i < parent.filterGetEvents.length; i++) {

        if (parent.params.incAapproval === false && parseInt(parent.filterGetEvents[i].approved, 10) === 0) {
            continue;
        }

        //  If repeating event get repeater
        if (parent.filterGetEvents[i].rrule && parent.filterGetEvents[i].rrule !== '') {
            try {
                eventLoop = parent.repeaterGet(parent.filterGetEvents[i], a, b);
            } catch (err) {
                console.log(err);
                console.error('Error processing repeater for event ' + parent.filterGetEvents[i].id, 'with RRULE String: ' + parent.filterGetEvents[i].rrule);
            }
        } else {
            eventLoop = [parent.filterGetEvents[i]];
        }

        for (j = 0; j < eventLoop.length; j++) {

            if (overrides.length > 0 && parent.isOverriden(eventLoop[j])) {
                continue;
            }

            start = moment.tz(eventLoop[j].start_date, 'UTC').unix();
            end = moment.tz(eventLoop[j].end_date, 'UTC').unix();

            // if param expired_only=1 in [stachethemes_ec]
            if (true === parent.params.expiredOnly && end > now) {
                continue;
            } else

            // if param upcoming_only=1 in [stachethemes_ec]
            if (true === parent.params.upcomingOnly && start < now) {
                continue;
            }

            if (b > start && end >= a) {
                pick.push(eventLoop[j]);
            }
        }
    }

    // reorder full events by timestamp
    parent.filterGetEvents = pick.sort(parent.sortByTimestamp);

    // reorder by featured
    if (parent.params.view !== 'agenda' && parent.params.view !== 'grid' && parent.params.view !== 'boxgrid') {
        parent.filterGetEvents.sort(parent.sortByFeatured);
    }

    return parent.filterGetEvents;

};

EventsCalcThread.prototype.getNearestEvents = function () {

    var parent = this,
            a,
            b,
            i,
            j,
            start,
            end,
            pick = [],
            overrides,
            eventLoop,
            now = moment().unix();

    parent.filterGetEvents = [];
    a = parent.getRangeUnix(parent.params.from);
    b = parent.getRangeUnix(parent.params.to ? parent.params.to : parent.params.to, true);

    // Get overrides in this range and add them to "pick" array
    overrides = parent.getOverridesInRange(a, b);
    if (overrides.length > 0) {
        parent.filterGetEvents = overrides;
        overrides = parent.filterGetEvents;
        pick = parent.merge(pick, parent.filterGetEvents);
    }

    // Exclude overriden
    parent.filterGetEvents = parent.grep(parent.eventsPool, function (poolEvent) {
        return !poolEvent.recurrence_id;
    });

    for (i = 0; i < parent.filterGetEvents.length; i++) {

        if (parent.params.incAapproval === false && parseInt(parent.filterGetEvents[i].approved, 10) === 0) {
            continue;
        }

        //  If repeating event get repeater
        if (parent.filterGetEvents[i].rrule && parent.filterGetEvents[i].rrule !== '') {
            try {
                eventLoop = parent.repeaterGetNearest(parent.filterGetEvents[i], a, b);
            } catch (err) {
                console.log(err);
                console.error('Error processing repeater for event ' + parent.filterGetEvents[i].id, 'with RRULE String: ' + parent.filterGetEvents[i].rrule);
            }
        } else {
            eventLoop = [parent.filterGetEvents[i]];
        }

        for (j = 0; j < eventLoop.length; j++) {

            if (overrides.length > 0 && parent.isOverriden(eventLoop[j])) {
                continue;
            }

            start = moment.tz(eventLoop[j].start_date, 'UTC').unix();
            end = moment.tz(eventLoop[j].end_date, 'UTC').unix();

            // if param expired_only=1 in [stachethemes_ec]
            if (true === parent.params.expiredOnly && end > now) {
                continue;
            } else

            // if param upcoming_only=1 in [stachethemes_ec]
            if (true === parent.params.upcomingOnly && start < now) {
                continue;
            }

            if (b > start && end >= a) {
                pick.push(eventLoop[j]);
            }
        }
    }

    // reorder full events by timestamp
    parent.filterGetEvents = pick.sort(parent.sortByTimestamp);

    // reorder by featured
    if (parent.params.view !== 'agenda' && parent.params.view !== 'grid' && parent.params.view !== 'boxgrid') {
        parent.filterGetEvents.sort(parent.sortByFeatured);
    }

    return parent.filterGetEvents;

};

onmessage = function (e) {

    var events, params, task;

    switch (e.data.task) {

        case 'getNearestEvents' :

            events = e.data.eventsPool;
            params = e.data.params;
            task = new EventsCalcThread(events, params);

            postMessage(task.getNearestEvents());
            break;

        case 'getEvents' :

            events = e.data.eventsPool;
            params = e.data.params;
            task = new EventsCalcThread(events, params);

            postMessage(task.getEvents());

            break;
    }
};
