(function ($) {

    'use strict';

    var proto;
    var moment = window.moment;

    if (moment) {
        moment.locale('en');
    }

    window.StecHelper = function () {
        this.animate = false;
        this.glob = {};
        this.calData = {};
        this.$instance = '';
        this.instance = '';
        this.layout = '';
        this.eventsHandler = '';
        this.canVclick = typeof $.vmouse !== 'undefined';

    };

    proto = window.StecHelper.prototype;

    proto.set = function (obj, data) {

        if (typeof this[obj] !== 'undefined') {
            this[obj] = data;
        }

        return this;
    };

    proto.getSelector = function ($element, sub) {
        
        var id = $element.attr('id');
        var klasses = $element.attr('class');
        var selector = '';

        if (id) {
            selector = '#' + id;
        } else if (klasses) {
            selector = klasses.split(' ').map(function (k) {
                return (k ? '.' + k : '');
            }).join('');
        } else {
            console.warn('No selector found', [$element, sub]);
            selector = '';
        }

        return sub ? selector + ' ' + sub : selector;
    };

    proto.isEmail = function (email) {
        var regex = /^([a-zA-Z0-9_.+-])+\@(([a-zA-Z0-9-])+\.)+([a-zA-Z0-9]{2,4})+$/;
        return regex.test(email);
    };

    proto.getElementWidth = function ($element) {
        return $element.get(0).getBoundingClientRect().width;
    };

    /**
     * Date to ICS time string
     * @param {date} date
     * @returns yyyymmddThhmmssZ
     */
    proto.dateToRFC = function (date) {
        var dt = moment(date);
        var rfc = dt.utc().format('YYYYMMDDTHHmmss') + 'Z';
        return rfc;
    };

    proto.eventToGoogleCalImportLink = function (eventId, repeatOffset) {

        var event, d1, d2, startDate, endDate, description, location;

        event = this.calData.getEventById(eventId);

        d1 = moment.tz(event.start_date, event.timezone).add(repeatOffset, 'seconds');
        d2 = moment.tz(event.end_date, event.timezone).add(repeatOffset, 'seconds');

        if (event.all_day) {
            startDate = d1.format('YYYYMMDD');
            endDate = moment(d2).add(1, 'day').format('YYYYMMDD');
        } else {
            startDate = this.dateToRFC(d1);
            endDate = this.dateToRFC(d2);
        }


        description = event.description_short;
        location = event.location.full_address ? event.location.full_address : '';

        return "//calendar.google.com/calendar/render?action=TEMPLATE&text=" + event.title + "&dates=" + startDate + "/" + endDate + "&details=" + description + "&location=" + location + "&sf=true&output=xml";
    };

    proto.beautifyScheduleTimespan = function (start) {

        var d1 = new moment(start),
                format = '',
                timeFormat = 'HH:mm',
                timeFormatSetting = this.getGeneralSettings('time_format'),
                dateFormat = '',
                delimeter = '',
                trLabel = [],
                label;

        if (parseInt(timeFormatSetting, 10) === 12) {
            timeFormat = 'hh:mma';
        }

        dateFormat = this.getGeneralSettings('date_format');
        delimeter = dateFormat === 'dd.mm.yyyy' ? '. ' : ' ';

        switch (dateFormat) {
            case 'dd.mm.yyyy' :
                format = 'DD. MMM ';
                break;
            case 'dd-mm-yy' :
                format = 'DD MMM ';
                break;
            case 'mm-dd-yy' :
                format = 'MMM DD ';
                break;
            case 'yy-mm-dd' :
                format = 'MMM DD';
                break;
        }

        format += ' ' + timeFormat;

        label = d1.format(format).split(delimeter);
        trLabel = [];

        $.each(label, function () {

            var part = typeof window.stecLang[this.toLowerCase().trim()] !== 'undefined' ? window.stecLang[this.toLowerCase().trim()] : this;
            part = part.charAt(0).toUpperCase() + part.slice(1);

            trLabel.push(part);
        });


        this.glob.temp.dateTimeString = trLabel.join(delimeter);

        this.$instance.trigger('stecBeforeReturnScheduleTimespan', {
            $instance: this.$instance,
            momentDate: d1,
            temp: this.glob.temp
        });

        return this.glob.temp.dateTimeString;

    };

    proto.beautifyDate = function (date) {

        var d1,
                format,
                timeFormat,
                timeFormatSetting,
                dateFormat,
                delimeter,
                timespanLabel,
                trTimespanLabel;

        d1 = moment(date);

        format = '';
        timeFormat = 'HH:mm';

        timeFormatSetting = this.getGeneralSettings('time_format');

        if (parseInt(timeFormatSetting, 10) === 12) {
            timeFormat = 'hh:mma';
        }

        dateFormat = this.getGeneralSettings('date_format');

        delimeter = dateFormat === 'dd.mm.yyyy' ? '. ' : ' ';

        switch (dateFormat) {
            case 'dd.mm.yyyy' :
                format = 'D. MMM. YYYY';
                break;
            case 'dd-mm-yy' :
                format = 'D MMM YYYY';
                break;
            case 'mm-dd-yy' :
                format = 'MMM D YYYY';
                break;
            case 'yy-mm-dd' :
                format = 'YYYY MMM D';
                break;
            default:
                format = dateFormat;
        }

        format += ' ' + timeFormat;

        timespanLabel = d1.format(format);
        timespanLabel = timespanLabel.split(delimeter);
        trTimespanLabel = [];

        $.each(timespanLabel, function () {
            var part = typeof window.stecLang[this.toLowerCase().trim()] !== 'undefined' ? window.stecLang[this.toLowerCase().trim()] : this;
            part = part.charAt(0).toUpperCase() + part.slice(1);
            trTimespanLabel.push(part);
        });

        trTimespanLabel = trTimespanLabel.join(delimeter);

        this.glob.temp.dateTimeString = trTimespanLabel[0].toUpperCase() + trTimespanLabel.substr(1);

        this.$instance.trigger('stecBeforeReturnBeautifyDate', {
            $instance: this.$instance,
            momentDate: d1,
            temp: this.glob.temp
        });

        return this.glob.temp.dateTimeString;
    };

    /**
     * Returns user friendly timespan
     * Accepted params for start and end are String (DbDate) or Date object
     * @param {mixed} start
     * @param {mixed} end 
     * @param {int} allDay 1/0
     * @returns {String}
     * @since 2.0.0 uses moment
     */
    proto.beautifyTimespan = function (start, end, allDay) {

        var d1,
                d2,
                format,
                timeFormat,
                timeFormatSetting,
                dateFormat,
                delimeter,
                timespanLabel,
                formatEnd,
                trTimespanLabel;

        allDay = parseInt(allDay, 10);

        d1 = new moment(start);
        d2 = end ? new moment(end) : null;

        format = '';
        timeFormat = 'HH:mm';

        timeFormatSetting = this.getGeneralSettings('time_format');

        if (parseInt(timeFormatSetting, 10) === 12) {
            timeFormat = 'hh:mma';
        }

        dateFormat = this.getGeneralSettings('date_format');

        delimeter = dateFormat === 'dd.mm.yyyy' ? '. ' : ' ';

        switch (dateFormat) {
            case 'dd.mm.yyyy' :
                format = 'D. MMM. YYYY';
                break;
            case 'dd-mm-yy' :
                format = 'D MMM YYYY';
                break;
            case 'mm-dd-yy' :
                format = 'MMM D YYYY';
                break;
            case 'yy-mm-dd' :
                format = 'YYYY MMM D';
                break;
            default:
                format = dateFormat;
        }

        // Show time only if not all_day event
        if (parseInt(allDay, 10) !== 1) {
            format += ' ' + timeFormat;
        }

        if (end) {

            // Same Day
            if (d1.isSame(d2, 'day')) {

                timespanLabel = d1.format(format);

                if (parseInt(allDay, 10) !== 1) {
                    timespanLabel += " - " + d2.format(timeFormat);
                }

            } else

            // Same Month & Year
            if (d1.isSame(d2, 'month')) {

                formatEnd = format.replace('YYYY', '');

                timespanLabel = d1.format(format);
                timespanLabel += " - " + d2.format(formatEnd);

            }

            // Default
            else {

                timespanLabel = d1.format(format);
                timespanLabel += " - " + d2.format(format);
            }

        } else {
            timespanLabel = d1.format(format);
        }

        timespanLabel = timespanLabel.split(delimeter);
        trTimespanLabel = [];

        $.each(timespanLabel, function () {

            var part = typeof window.stecLang[this.toLowerCase().trim()] !== 'undefined' ? window.stecLang[this.toLowerCase().trim()] : this;
            part = part.charAt(0).toUpperCase() + part.slice(1);

            trTimespanLabel.push(part);
        });

        trTimespanLabel = trTimespanLabel.join(delimeter);

        this.glob.temp.dateTimeString = trTimespanLabel[0].toUpperCase() + trTimespanLabel.substr(1);

        this.$instance.trigger('stecBeforeReturnBeautifyTimespan', {
            $instance: this.$instance,
            startDate: start,
            endDate: end,
            allDay: allDay,
            temp: this.glob.temp
        });

        return this.glob.temp.dateTimeString;

    };

    proto.focus = function (el) {

        var $wpadminbar = $("#wpadminbar");

        if (parseInt(this.glob.options.general_settings.event_auto_focus, 10) !== 1) {
            return;
        }

        $('html, body').animate({
            scrollTop: $(el).offset().top - ($wpadminbar.length > 0 ? $wpadminbar.height() : 0) + parseInt(this.glob.options.general_settings.event_auto_focus_offset, 10)
        }, {
            duration: 750,
            easing: "easeInOutExpo"
        });
    };

    proto.nl2br = function (txt) {
        return txt.replace(/(\r\n|\n\r|\r|\n)/g, "<br>");
    };

    proto.capitalize = function (text) {
        return text.charAt(0).toUpperCase() + text.slice(1);
    };

    /**
     * Preloads images from given array
     * @param {array} imagesArray
     * @returns {Promise}
     */
    proto.preloadContentImages = function (imagesArray) {

        return new Promise(function (resolve) {

            var waitingPreload = 0;

            waitingPreload = imagesArray.length;

            if (imagesArray.length > 0) {
                $.each(imagesArray, function () {

                    var src = this;
                    var imageObject = new Image();
                    imageObject.src = src;

                    imageObject.onload = function () {
                        waitingPreload--;

                        if (waitingPreload <= 0) {
                            resolve();
                        }
                    };
                });
            } else {
                resolve();
            }
        });
    };

    proto.getImageHeight = function (src) {
        var imageObject;
        if (!src) {
            return 0;
        }

        imageObject = new Image();
        imageObject.src = src;

        return imageObject.naturalHeight;
    };

    proto.promiseImageHeight = function (src) {

        return new Promise(function (resolve) {
            var imageObject;
            if (!src) {
                return 0;
            }

            imageObject = new Image();
            imageObject.src = src;

            imageObject.onload = function () {
                return resolve(imageObject.naturalHeight);
            };

        });

    };

    proto.capitalizeFirstLetter = function (string) {
        return string.charAt(0).toUpperCase() + string.slice(1);
    };

    /**
     * @param {string} dbDate string
     * @param {int} offset seconds
     * @returns {date} date object
     * @since 2.0.0 uses moment.js
     */
    proto.dbDateOffset = function (string, offset) {
        return moment(string).add(offset, 'seconds').toDate();
    };

    /**
     * expects months range 1-12
     * converts to 0-11
     * @param {str} string yy-mm-dd h:i:s
     * @since 2.0.0 uses moment.js
     */
    proto.dbDateTimeToDate = function (string) {
        return new moment(string).toDate();
    };

    proto.getColorBrightness = function (hex) {

        var rgb, r, g, b, luma;

        /*jslint bitwise: true */

        hex = hex.substring(1);      // strip #
        rgb = parseInt(hex, 16); // convert rrggbb to decimal
        r = (rgb >> 16) & 0xff;  // extract red
        g = (rgb >> 8) & 0xff;   // extract green
        b = (rgb >> 0) & 0xff;   // extract blue
        luma = 0.2126 * r + 0.7152 * g + 0.0722 * b; // per ITU-R BT.709
        return luma;
    };

    proto.extendBind = function (funcArr, sub, rtrn) {

        var parent = this;

        if (typeof window[funcArr] !== "undefined") {

            if (sub) {

                if (typeof window[funcArr][sub] !== "undefined") {
                    $(window[funcArr][sub]).each(function () {

                        if (typeof this === "function") {

                            this({
                                instance: parent.instance,
                                $instance: parent.$instance,
                                glob: parent.glob,
                                helper: parent,
                                calData: parent.calData,
                                layout: parent.layout,
                                eventsHandler: parent.eventsHandler
                            }, rtrn ? rtrn : null);
                        }
                    });
                }

            } else {

                $(window[funcArr]).each(function () {

                    if (typeof this === "function") {

                        this({
                            instance: parent.instance,
                            $instance: parent.$instance,
                            glob: parent.glob,
                            helper: parent,
                            calData: parent.calData,
                            layout: parent.layout,
                            eventsHandler: parent.eventsHandler
                        }, rtrn ? rtrn : null);
                    }
                });
            }
        }
    };

    proto.onResizeEnd = function (callback, time, keyslug) {
        var id, handle;
        var width = $(window).width(), height = $(window).height();
        time = time ? time : 10;
        handle = keyslug ? 'resize.' + keyslug : 'resize';

        $(window).on(handle, function () {
            clearTimeout(id);
            id = setTimeout(function () {
                if (typeof callback === "function") {
                    if ($(window).width() !== width || $(window).height() !== height) {
                        callback.call();
                    }
                }
            }, time);
        });
    };

    proto.iso8601Week = function (date) {
        var time;
        var checkDate = new Date(date.getTime());

        // Find Thursday of this week starting on Monday
        checkDate.setDate(checkDate.getDate() + 4 - (checkDate.getDay() || 7));

        time = checkDate.getTime();
        checkDate.setMonth(0); // Compare with Jan 1
        checkDate.setDate(1);
        return Math.floor(Math.round((time - checkDate) / 86400000) / 7) + 1;
    };

    proto.newDate = function (y, m, d) {
        return new Date(parseInt(y, 10), parseInt(m, 10), parseInt(d, 10), 0, 0, 0, 0);
    };

    /**
     * @param {Date} alt new Date()
     */
    proto.getWeekInfo = function (alt) {

        var year, month, day, activeDate, firstDayOfTheWeek, lastDayOfTheWeek;

        if (alt) {
            year = alt.getFullYear();
            month = alt.getMonth();
            day = alt.getDate();
        } else {
            year = this.glob.options.year;
            month = this.glob.options.month;
            day = this.glob.options.day;
        }

        activeDate = this.newDate(year, month, day);

        switch (this.glob.options.general_settings.first_day_of_the_week) {
            case 'mon' :

                if (activeDate.getDay() !== 1) {
                    while (activeDate.getDay() !== 1) {
                        activeDate.setDate(activeDate.getDate() - 1);
                    }
                }

                break;

            case 'sat' :

                if (activeDate.getDay() !== 6) {
                    while (activeDate.getDay() !== 6) {
                        activeDate.setDate(activeDate.getDate() - 1);
                    }
                }

                break;

            case 'sun' :

                if (activeDate.getDay() !== 0) {
                    while (activeDate.getDay() !== 0) {
                        activeDate.setDate(activeDate.getDate() - 1);
                    }
                }

                break;
        }

        firstDayOfTheWeek = new Date(activeDate);
        lastDayOfTheWeek = new Date(firstDayOfTheWeek);
        lastDayOfTheWeek.setDate(lastDayOfTheWeek.getDate() + 6);

        return {
            start: {
                day: firstDayOfTheWeek.getDate(),
                month: firstDayOfTheWeek.getMonth(),
                year: firstDayOfTheWeek.getFullYear()
            },
            end: {
                day: lastDayOfTheWeek.getDate(),
                month: lastDayOfTheWeek.getMonth(),
                year: lastDayOfTheWeek.getFullYear()
            },
            week: this.iso8601Week(activeDate)
        };
    };

    proto.getMonthInfo = function (month, year) {

        var daysInMonth, firstDay, startingDay, monthLength, monthInfo, dayOffset = '';

        if (isNaN(month)) {
            month = parseInt(this.glob.options.month, 10);
        }

        if (isNaN(year)) {
            year = parseInt(this.glob.options.year, 10);
        }

        daysInMonth = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

        firstDay = this.newDate(year, month, 1);
        startingDay = firstDay.getDay();

        monthLength = daysInMonth[month];

        if (month === 1) {
            if ((year % 4 === 0 && year % 100 !== 0) || year % 400 === 0) {
                monthLength = 29;
            }
        }

        switch (this.glob.options.general_settings.first_day_of_the_week) {
            case 'mon' :
                dayOffset = startingDay - 1 < 0 ? startingDay + 6 : startingDay - 1;
                break;

            case 'sat' :
                dayOffset = startingDay - 7 < 0 ? startingDay + 1 : startingDay - 7;

                break;

            case 'sun' :
                dayOffset = startingDay;
                break;
        }

        monthInfo = {
            startingDay: startingDay,
            monthLength: monthLength,
            year: year,
            month: month,
            dayOffset: dayOffset,
            monthName: this.glob.options.monthLabels[month],
            monthNameShort: this.glob.options.monthLabelsShort[month]
        };

        return monthInfo;
    };

    /**
     * data-date="yyyy-mm-dd" to Date()
     * @param {string} str yyyy-mm-dd
     * @returns {Date} returns new Date()
     */
    proto.getDateFromData = function (str) {
        var d = str.split('-');

        var nd = this.newDate(parseInt(d[0], 10), parseInt(d[1], 10), parseInt(d[2], 10));

        return nd;
    };

    /**
     * Date() to yyy-mm-dd string
     * @param {Date} date
     * @returns {String}
     */
    proto.getDataFromDate = function (date) {
        return date.getFullYear() + '-' + date.getMonth() + '-' + date.getDate();
    };

    /**
     * Translates labels
     * @param {string} label
     * @param {object} week
     * @returns string
     */
    proto.getWeekLabel = function (label, week) {

        return  label.replace(/sday/g, week.start.day)
                .replace(/smonth/g, this.glob.options.monthLabelsShort[week.start.month])
                .replace(/syear/g, week.start.year)
                .replace(/eday/g, week.end.day)
                .replace(/emonth/g, this.glob.options.monthLabelsShort[week.end.month])
                .replace(/eyear/g, week.end.year);

    };

    proto.isMobile = function () {
        return (/Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent)) ? true : false;
    };

    proto.clickHandle = function (sub) {

        var tap = sub ? 'vclick.' + sub : 'vclick';
        var click = sub ? 'click.' + sub : 'click';

        return this.isMobile() && this.canVclick ? tap : click;

    };

    proto.instaClickHandle = function () {
        return this.isMobile() ? "touchstart" : "mousedown";
    };

    proto.insertAtIndex = function (str, index, insert) {
        return [str.slice(0, index), insert, str.slice(index)].join('');
    };

    proto.getPermalink = function (permalink, offset) {
        var argPos = null;

        if (offset && parseInt(offset, 10) > 0) {
            argPos = permalink.indexOf('?');
            if (-1 !== argPos) {
                permalink = this.insertAtIndex(permalink, argPos, offset.toString() + '/');
            } else {
                permalink = this.insertAtIndex(permalink, permalink.length, offset.toString() + '/');
            }
        }
        return permalink;

    };

    proto.lang = function (textToTranslate) {
        return window.stecLang[textToTranslate] ? window.stecLang[textToTranslate] : textToTranslate;
    };

    proto.attendanceHasFreeSlots = function (event, date) {
        var freeSlots = event.rsvp_limit;

        if (event.rsvp_limit === 0) {
            return 9999;
        }

        if (event.attendance.length > 0) {
            $.each(event.attendance, function () {
                if (1 === parseInt(this.status[date], 10)) {
                    freeSlots = freeSlots - this.people;
                }
            });
        }

        return Math.max(0, freeSlots);
    };

    proto.getEventState = function (event, offset) {

        var nowUnix, startUnix, endUnix;

        if (!offset) {
            offset = 0;
        }

        nowUnix = moment().unix();
        startUnix = event.start_date_timestamp + offset;
        endUnix = event.end_date_timestamp + offset;

        if (nowUnix > endUnix) {
            return 2; // expired
        }

        if (nowUnix > startUnix) {
            return 1; // in progress
        }

        if (nowUnix < startUnix) {
            return 0; // not started
        }


    };

    proto.toast = function (msg, append, delay, css) {
        var html = '', element;

        if (!delay) {
            delay = 2000;
        }

        if (!append) {
            append = 'body';
        }

        $('.stec-toast').remove();

        html = ('<div class="stec-toast">' + msg + '</div>');

        element = $(html).appendTo(append);

        if (css) {
            element.css(css);
        }

        setTimeout(function () {
            element.addClass('stec-toast-close');

            setTimeout(function () {
                element.remove();
            }, 250);

        }, delay);
    };

    proto.getMapStyles = function (theme) {

        var aubergine = new window.google.maps.StyledMapType(
                [
                    {
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#242f3e"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#746855"
                            }
                        ]
                    },
                    {
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#242f3e"
                            }
                        ]
                    }, {
                        "featureType": "administrative.locality",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#d59563"
                            }
                        ]
                    }, {
                        "featureType": "poi",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#d59563"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#263c3f"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#6b9a76"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#38414e"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#212a37"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9ca5b3"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#746855"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#1f2835"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#f3d19c"
                            }
                        ]
                    }, {
                        "featureType": "transit",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#2f3948"
                            }
                        ]
                    }, {
                        "featureType": "transit.station",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#d59563"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#17263c"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#515c6d"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#17263c"
                            }
                        ]
                    }
                ],
                {name: 'Aubergine'}

        );

        var retro = new window.google.maps.StyledMapType(
                [
                    {
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#ebe3cd"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#523735"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#f5f1e6"
                            }
                        ]
                    }, {
                        "featureType": "administrative",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#c9b2a6"
                            }
                        ]
                    }, {
                        "featureType": "administrative.land_parcel",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#dcd2be"
                            }
                        ]
                    },
                    {
                        "featureType": "administrative.land_parcel",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#ae9e90"
                            }
                        ]
                    }, {
                        "featureType": "landscape.natural",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dfd2ae"
                            }
                        ]
                    },
                    {
                        "featureType": "poi",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dfd2ae"
                            }
                        ]
                    }, {
                        "featureType": "poi",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#93817c"
                            }
                        ]
                    },
                    {
                        "featureType": "poi.park",
                        "elementType": "geometry.fill",
                        "stylers": [
                            {
                                "color": "#a5b076"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#447530"
                            }
                        ]
                    },
                    {
                        "featureType": "road",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#f5f1e6"
                            }
                        ]
                    }, {
                        "featureType": "road.arterial",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#fdfcf8"
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#f8c967"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#e9bc62"
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway.controlled_access",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#e98d58"
                            }
                        ]
                    }, {
                        "featureType": "road.highway.controlled_access",
                        "elementType": "geometry.stroke",
                        "stylers": [
                            {
                                "color": "#db8555"
                            }
                        ]
                    },
                    {
                        "featureType": "road.local",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#806b63"
                            }
                        ]
                    }, {
                        "featureType": "transit.line",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dfd2ae"
                            }
                        ]
                    },
                    {
                        "featureType": "transit.line",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#8f7d77"
                            }
                        ]
                    }, {
                        "featureType": "transit.line",
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#ebe3cd"
                            }
                        ]
                    },
                    {
                        "featureType": "transit.station",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dfd2ae"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "geometry.fill",
                        "stylers": [
                            {
                                "color": "#b9d3c2"
                            }
                        ]
                    },
                    {
                        "featureType": "water",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#92998d"
                            }
                        ]
                    }
                ],
                {name: 'Retro'}
        );

        var night = new window.google.maps.StyledMapType(
                [
                    {elementType: 'geometry', stylers: [{color: '#242f3e'}]}, {elementType: 'labels.text.stroke', stylers: [{color: '#242f3e'}]},
                    {elementType: 'labels.text.fill', stylers: [{color: '#746855'}]}, {
                        featureType: 'administrative.locality',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#d59563'}]
                    },
                    {
                        featureType: 'poi',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#d59563'}]
                    }, {
                        featureType: 'poi.park',
                        elementType: 'geometry',
                        stylers: [{color: '#263c3f'}]
                    },
                    {
                        featureType: 'poi.park',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#6b9a76'}]
                    }, {
                        featureType: 'road',
                        elementType: 'geometry',
                        stylers: [{color: '#38414e'}]
                    },
                    {
                        featureType: 'road',
                        elementType: 'geometry.stroke',
                        stylers: [{color: '#212a37'}]
                    }, {
                        featureType: 'road',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#9ca5b3'}]
                    },
                    {
                        featureType: 'road.highway',
                        elementType: 'geometry',
                        stylers: [{color: '#746855'}]
                    }, {
                        featureType: 'road.highway',
                        elementType: 'geometry.stroke',
                        stylers: [{color: '#1f2835'}]
                    },
                    {
                        featureType: 'road.highway',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#f3d19c'}]
                    }, {
                        featureType: 'transit',
                        elementType: 'geometry',
                        stylers: [{color: '#2f3948'}]
                    },
                    {
                        featureType: 'transit.station',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#d59563'}]
                    }, {
                        featureType: 'water',
                        elementType: 'geometry',
                        stylers: [{color: '#17263c'}]
                    },
                    {
                        featureType: 'water',
                        elementType: 'labels.text.fill',
                        stylers: [{color: '#515c6d'}]
                    }, {
                        featureType: 'water',
                        elementType: 'labels.text.stroke',
                        stylers: [{color: '#17263c'}]
                    }
                ], {name: 'Night'}
        );

        var silver = new window.google.maps.StyledMapType(
                [
                    {
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#f5f5f5"
                            }
                        ]
                    }, {
                        "elementType": "labels.icon",
                        "stylers": [
                            {
                                "visibility": "off"
                            }
                        ]
                    },
                    {
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#616161"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#f5f5f5"
                            }
                        ]
                    },
                    {
                        "featureType": "administrative.land_parcel",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#bdbdbd"
                            }
                        ]
                    }, {
                        "featureType": "poi",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#eeeeee"
                            }
                        ]
                    },
                    {
                        "featureType": "poi",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#e5e5e5"
                            }
                        ]
                    },
                    {
                        "featureType": "poi.park",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9e9e9e"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#ffffff"
                            }
                        ]
                    },
                    {
                        "featureType": "road.arterial",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#dadada"
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#616161"
                            }
                        ]
                    }, {
                        "featureType": "road.local",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9e9e9e"
                            }
                        ]
                    },
                    {
                        "featureType": "transit.line",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#e5e5e5"
                            }
                        ]
                    }, {
                        "featureType": "transit.station",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#eeeeee"
                            }
                        ]
                    },
                    {
                        "featureType": "water",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#c9c9c9"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9e9e9e"
                            }
                        ]
                    }
                ], {name: 'Silver'}
        );


        var dark = new window.google.maps.StyledMapType(
                [
                    {
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#212121"
                            }
                        ]
                    }, {
                        "elementType": "labels.icon",
                        "stylers": [
                            {
                                "visibility": "off"
                            }
                        ]
                    },
                    {
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#212121"
                            }
                        ]
                    },
                    {
                        "featureType": "administrative",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "administrative.country",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#9e9e9e"
                            }
                        ]
                    },
                    {
                        "featureType": "administrative.land_parcel",
                        "stylers": [
                            {
                                "visibility": "off"
                            }
                        ]
                    }, {
                        "featureType": "administrative.locality",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#bdbdbd"
                            }
                        ]
                    },
                    {
                        "featureType": "poi",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#181818"
                            }
                        ]
                    },
                    {
                        "featureType": "poi.park",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#616161"
                            }
                        ]
                    }, {
                        "featureType": "poi.park",
                        "elementType": "labels.text.stroke",
                        "stylers": [
                            {
                                "color": "#1b1b1b"
                            }
                        ]
                    },
                    {
                        "featureType": "road",
                        "elementType": "geometry.fill",
                        "stylers": [
                            {
                                "color": "#2c2c2c"
                            }
                        ]
                    }, {
                        "featureType": "road",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#8a8a8a"
                            }
                        ]
                    },
                    {
                        "featureType": "road.arterial",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#373737"
                            }
                        ]
                    }, {
                        "featureType": "road.highway",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#3c3c3c"
                            }
                        ]
                    },
                    {
                        "featureType": "road.highway.controlled_access",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#4e4e4e"
                            }
                        ]
                    }, {
                        "featureType": "road.local",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#616161"
                            }
                        ]
                    },
                    {
                        "featureType": "transit",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#757575"
                            }
                        ]
                    }, {
                        "featureType": "water",
                        "elementType": "geometry",
                        "stylers": [
                            {
                                "color": "#000000"
                            }
                        ]
                    },
                    {
                        "featureType": "water",
                        "elementType": "labels.text.fill",
                        "stylers": [
                            {
                                "color": "#3d3d3d"
                            }
                        ]
                    }], {name: 'Dark'}
        );

        switch (theme) {

            case 'silver' :
                return silver;

            case 'aubergine' :
                return aubergine;

            case 'retro' :
                return retro;

            case 'night' :
                return night;

            case 'dark' :
                return dark;

            default:
                return false;
        }
    };

    proto.getGeneralSettings = function (option) {

        if (this.glob.options.general_settings) {
            return option ? this.glob.options.general_settings[option] : this.glob.options.general_settings;
        }

        return option ? this.glob.options[option] : this.glob.options;

    };

    /**
     * OpenStreeMap 
     * returns map instance
     */
    proto.osm = function (selector) {

        var map;

        if (!window.L) {
            return false;
        }

        map = window.L.map(selector);

        this.glob.temp.osmTileLayer = window.L.tileLayer('//{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
            maxZoom: 19,
            attribution: ['&copy; <a href="//www.openstreetmap.org/copyright">OpenStreetMap</a>', '<a href="//wiki.osmfoundation.org/wiki/Privacy_Policy">Privacy Policy</a>'].join(' | ')
        });

        this.$instance.trigger('onStecOsmGetTileLayer', {
            glob: this.glob
        });

        map.addLayer(this.glob.temp.osmTileLayer);

        return map;
    };

    proto.promiseCoordinates = function (event) {

        var parent = this;

        return new Promise(function (resolve, reject) {

            if (event.location.length <= 0 || true === event.noGeocode) {
                return reject(event);
            }
            if (event.location.coordinates) {
                return resolve(event.location.coordinates);
            }

            $.getJSON('//nominatim.openstreetmap.org/search?format=json&limit=1&q=' + event.location.full_address, function (data) {

                if (data.length > 0) {
                    let coords = [data[0].lat, data[0].lon].join(',');

                    if (parent.calData) {
                        let eventById = parent.calData.getEventById(event.id);
                        eventById.location.coordinates = coords;
                    }

                    return resolve(coords);
                } else {

                    if (parent.calData) {
                        let eventById = parent.calData.getEventById(event.id);
                        eventById.noGeocode = true;
                    }

                    console.log('Unable to find coordinates for: ' + event.location.full_address);

                    return reject(event);
                }
            });
        });

    };

    proto.promiseGetAddressCoordinates = function (address) {

        return new Promise(function (resolve, reject) {

            $.getJSON('//nominatim.openstreetmap.org/search?format=json&limit=1&q=' + address, function (data) {

                if (data.length > 0) {
                    return resolve([data[0].lat, data[0].lon].join(','));
                } else {
                    console.log('Unable to find coordinates for: ' + address);
                    return reject(address);
                }
            });


        });

    };

    /**
     * @todo Events near me
     * Measure distance from point A to point B 
     * @param {string} A coordinates string "lat, lng" (e.g. "42.698398, 23.278058")
     * @param {string} B coordinates string "lat, lng"
     * @return {Number} distance in KM
     */
    proto.measureDistance = function (A, B) {

        var coordsA = A.split(',').map(function (n) {
            return parseFloat(n, 10);
        });

        var coordsB = B.split(',').map(function (n) {
            return parseFloat(n, 10);
        });

        var R = 6371000; //  6.371km
        var dLat = (coordsA[0] - coordsB[0]) * Math.PI / 180;
        var dLon = (coordsA[1] - coordsB[1]) * Math.PI / 180;
        var lat1 = (coordsB[0]) * Math.PI / 180;
        var lat2 = (coordsA[0]) * Math.PI / 180;
        var a = Math.sin(dLat / 2) * Math.sin(dLat / 2) + Math.sin(dLon / 2) * Math.sin(dLon / 2) * Math.cos(lat1) * Math.cos(lat2);
        var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        var d = R * c;
        return Math.round(d / 1000);

    };

    proto.modalWindow = function () {

        this.isInit = false;
        this.preloaderTimeout = null;
        this.content = [];
        this.settings = {
            id: null,
            $instance: null,
            glob: null,
            helper: null
        };

    };

    proto.modalWindow.prototype.set = function (obj, data) {

        if (typeof this.settings[obj] !== 'undefined') {
            this.settings[obj] = data;
        }

        return this;
    };

    proto.newModalWindow = function (id) {
        var modal = new this.modalWindow();

        modal.set('id', id ? id : Date.now().toString(36) + Math.random().toString(36).substr(2));
        modal.set('$instance', this.$instance);
        modal.set('glob', this.glob);
        modal.set('helper', this);

        return modal;
    };

    proto.modalWindow.prototype.close = function () {

        $('#' + this.settings.id).remove();

        if ($('body > .stec-modal').length <= 0) {
            $('body').removeClass('stec-modal-opened stec-no-scroll');
        }

        $('#stec-layout-event-inner-out-wrapper-' + this.settings.glob.options.id).show();

    };

    proto.modalWindow.prototype.setContent = function (content) {
        this.content.push(content);
    };

    proto.modalWindow.prototype.controller = function ($modalWindow) {

        var parent = this;

        $modalWindow.find('.stec-modal-window-close').on(this.settings.helper.clickHandle(), function (e) {
            e.stopPropagation();
            parent.close();
        });

        $modalWindow.on(this.settings.helper.clickHandle(), function (e) {
            e.stopPropagation();
            parent.close();
        });

        $modalWindow.find('.stec-modal-window-wrapper').on(this.settings.helper.clickHandle(), function (e) {
            e.stopPropagation();
        });
    };

    proto.modalWindow.prototype.promiseOpen = function () {

        var parent = this;

        $('#stec-layout-event-inner-out-wrapper-' + this.settings.glob.options.id).hide();

        return new Promise(function (resolve) {

            var html = [], $modalWindow, imagesSrcArray = [];

            html.push('<div id="' + parent.settings.id + '" class="stec-modal-window-background stec-modal loading">');
            html.push('     <div class="stec-modal-window">');
            html.push('         <div class="stec-modal-window-wrapper">');
            html.push(parent.content);
            html.push('         </div>');
            html.push('         <i class="stec-modal-window-close fa fa-times"></i>');
            html.push('     </div>');
            html.push('</div>');

            $('body').addClass('stec-modal-opened stec-no-scroll');

            $modalWindow = $(html.join('')).data('stec-modal-object', parent).appendTo('body');

            parent.controller($modalWindow);

            $.each($modalWindow.find('img'), function () {
                imagesSrcArray.push(this.src);
            });

            parent.preloaderTimeout = setTimeout(function () {
                $(parent.settings.glob.template.preloader).appendTo($modalWindow);
            }, 500);

            parent.settings.helper.preloadContentImages(imagesSrcArray).then(function () {
                clearTimeout(parent.preloaderTimeout);
                $modalWindow.find('.stec-preloader').remove();
                $modalWindow.removeClass('loading');

                return resolve($modalWindow);
            });
        });
    };

})(window.jQuery);