(function ($) {

    'use strict';

    var DB = new window.StecDB();
    var helper = new window.StecHelper();
    
    const preloader = '<div class="stec-block-action"><div class="stec-preloader"></div></div>';
    const $container = $('.stec-order-details');
    
    function markItem(itemId, orderId, markScanned) {

        return new Promise(function (resolve, reject) {

            var $itemContainer = $container.find('.stec-order-details-item[data-id="' + itemId + '"]');
            var $preloader;
            var request = DB.newRequest();
            
            request.doAbort = false;

            request.setData({
                action: 'stec_ajax_action',
                task: 'mark_ticket_scanned',
                item_id: itemId,
                order_id: orderId,
                mark_scanned: markScanned
            });

            request.setBefore(function () {
                $preloader = $(preloader).appendTo($itemContainer);
            });

            request.setOnError(function () {
                $preloader.remove();
            });

            request.setCallback(function (rtrn) {

                $preloader.remove();

                if (rtrn.error !== 0) {
                    return reject(rtrn);
                }

                if (rtrn.data.success === 1) {
                    return resolve(rtrn);
                }

                return reject(rtrn);

            });

            DB.query(request);

        });
    }

    function markAll(orderId, markScanned) {

        return new Promise(function (resolve, reject) {
            var $preloader;
            var request = DB.newRequest();
            var $btn = $container.find('.stec-order-details-mark-all.active');
            
            request.doAbort = false;

            request.setData({
                action: 'stec_ajax_action',
                task: 'mark_tickets_order_scanned',
                order_id: orderId,
                mark_scanned: markScanned
            });

            request.setBefore(function () {
                $preloader = $('<div class="stec-preloader"></div>').insertAfter($btn);
                $btn.removeClass('active');
            });

            request.setOnError(function () {
                $preloader.remove();
                $btn.addClass('active');
            });

            request.setCallback(function (rtrn) {

                $preloader.remove();
                $btn.addClass('active');

                if (rtrn.error !== 0) {
                    return reject(rtrn);
                }

                if (rtrn.data.success === 1) {
                    return resolve(rtrn);
                }

                return reject(rtrn);

            });

            DB.query(request);
        });

    }

    $container.find('button[data-action="mark-order"]').on('click', function (e) {

        var orderId, markScanned;

        try {

            orderId = parseInt($(this).data('id'), 10);
            markScanned = parseInt($(this).data('mark-scanned'), 10);

            if (isNaN(orderId)) {
                throw 'Invalid order id: ' + $(this).data('id');
            }

            if (isNaN(markScanned)) {
                throw 'Unexpected error';
            }

            markAll(orderId, markScanned).then(function (result) {

                var $itemButtons = $container.find('.stec-order-details-item-mark');
                var markScanned = result.data.mark_scanned;

                $.each($itemButtons, function () {
                    $(this).filter('[data-mark-scanned="' + markScanned + '"]').removeClass('active');
                    $(this).not('[data-mark-scanned="' + markScanned + '"]').addClass('active');
                    $container.find('.stec-order-details-mark-all').filter('[data-mark-scanned="' + markScanned + '"]').removeClass('active');
                    $container.find('.stec-order-details-mark-all').not('[data-mark-scanned="' + markScanned + '"]').addClass('active');
                });

                helper.toast(result.data.msg);


            }).catch(function (result) {

                if (result.error_msg) {
                    throw result.error_msg;
                } else {
                    throw 'Unexpected error';
                }

            });

        } catch (err) {
            e.preventDefault();
            console.warn(err);
        }

    });

    $container.find('button[data-action="mark-item"]').on('click', function (e) {
        var idsArray = [];
        var $ticket = $(this).parents('.stec-order-details-item').first();
        var markScanned = $(this).data('mark-scanned');

        try {

            idsArray = $(this).data('id').split('-').map(function (num) {
                var integer = parseInt(num, 10);
                if (isNaN(integer)) {
                    throw 'Id is not an integer: ' + num;
                }
                return integer;
            });

            if (2 !== idsArray.length) {
                throw 'Invalid item order id length: ' + idsArray;
            }

            markItem(idsArray[0], idsArray[1], markScanned).then(function (result) {

                var markScanned = result.data.mark_scanned;
                var allMarked = true;

                $ticket.find('.stec-order-details-item-mark').filter('[data-mark-scanned="' + markScanned + '"]').removeClass('active');
                $ticket.find('.stec-order-details-item-mark').not('[data-mark-scanned="' + markScanned + '"]').addClass('active');

                $container.find('.stec-order-details-item-mark:not(.active)').each(function () {

                    if (parseInt($(this).data('mark-scanned'), 10) === 0) {
                        allMarked = false;
                        return false; // break;
                    }
                });

                if (true === allMarked) {
                    $container.find('.stec-order-details-mark-all').filter('[data-mark-scanned="1"]').removeClass('active');
                    $container.find('.stec-order-details-mark-all').not('[data-mark-scanned="1"]').addClass('active');
                } else {
                    $container.find('.stec-order-details-mark-all').filter('[data-mark-scanned="1"]').addClass('active');
                    $container.find('.stec-order-details-mark-all').not('[data-mark-scanned="1"]').removeClass('active');
                }

                helper.toast(result.data.msg, $ticket, 3000, {position: 'absolute'});


            }).catch(function (result) {

                if (result.error_msg) {
                    throw result.error_msg;
                } else {
                    throw 'Unexpected error';
                }

            });

        } catch (err) {
            e.preventDefault();
            console.warn(err);
        }
    });


})(window.jQuery);