(function ($) {

    'use strict';
    var proto;
    var moment = window.moment;
    var DB = new window.StecDB();
    const __ = new window.StecHelper().lang;

    moment.locale('en');

    window.StecShop = function () {

        this.settings = {
            offset: null,
            event: null,
            glob: null,
            helper: null,
            $instance: null,
            $container: null
        };

    };

    proto = window.StecShop.prototype;

    proto.set = function (obj, data) {

        if (typeof this.settings[obj] !== 'undefined') {
            this.settings[obj] = data;
        }

        return this;

    };

    proto.unblockActions = function ($product) {
        $product.find('.stec-layout-shop-product-wait').remove();

    };

    proto.blockActions = function ($product) {

        var html = '<div class="stec-layout-shop-product-wait">' + this.settings.glob.template.preloader + '</div>';

        $(html).appendTo($product);

    };

    proto.getHtmlProductsVariables = function (product) {

        var html = [];

        if (product.variation_options.length <= 0) {
            return '';
        }

        html.push('   <div class="stec-layout-shop-product-variables">');
        html.push('     <div class="stec-style-flexbox-separate">');

        $.each(product.variation_options, function () {
            let varName = this.name;
            let slug = this.slug;
            let options = this.options;

            html.push('         <div class="stec-style-full-width">');
            html.push('             <p class="stec-style-text">' + varName + '</p>');
            html.push('             <select name="' + slug + '" class="stec-style-select" data-type="variation" data-action="variation_changed">');
            html.push('                 <option value>' + __('Choose an option') + '</option>');

            $.each(options, function () {
                html.push('             <option value="' + this + '">' + this + '</option>');
            });

            html.push('             </select>');
            html.push('         </div>');

        });

        html.push('     </div>');
        html.push('     <div class="stec-layout-shop-product-variables-actions">');
        html.push('         <div class="stec-style-button" data-action="add_to_cart">');
        html.push('             <p>' + __('Add to Cart') + '</p>');
        html.push('             <i class="fa fa-shopping-cart"></i>');
        html.push('         </div>');
        html.push('     </div>');
        html.push('   </div>');

        return html.join('');
    };

    proto.getHtmlTop = function () {

        var html = [];

        html.push('<div class="stec-layout-shop-top">');
        html.push('     <p class="stec-style-text">' + __('Product') + '</p>');
        html.push('     <p class="stec-style-text">' + __('Name') + '</p>');
        html.push('     <p class="stec-style-text">' + __('Price') + '</p>');
        html.push('     <p class="stec-style-text">' + __('Quantity') + '</p>');
        html.push('     <p class="stec-style-text">' + __('Action') + '</p>');
        html.push('</div>');

        return html.join('');
    };

    proto.getHtmlBottomLinks = function () {

        var cartUrl = window.wc_add_to_cart_params.cart_url;
        var checkoutURL = window.stec_wc_checkout_params.checkout_url;
        var checkoutText = window.stec_wc_checkout_params.i18n_checkout;
        var cartText = window.wc_add_to_cart_params.i18n_view_cart;
        var html = [];

        html.push('<div class="stec-layout-shop-links">');
        html.push('    <a class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor" href="' + checkoutURL + '">' + checkoutText + '</a>');
        html.push('    <a class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor" href="' + cartUrl + '">' + cartText + '</a>');
        html.push('</div>');

        return html.join('');
    };

    proto.getHtmlProducts = function () {
        var parent = this;
        var html = [];

        $.each(this.settings.event.products, function () {

            var product = this;
            var price = product.html_price;
            var quantity = product.stock_quantity === null ? 9999 : product.stock_quantity;
            var inStock = quantity > 0 && product.is_in_stock && product.is_purchasable;
            var image = product.image;
            var title = product.title;
            var desc = product.short_desc;
            var extraClasses = [];

            if (false === inStock) {
                extraClasses.push('out-of-stock');
            }

            if (true === product.is_featured) {
                extraClasses.push('is-featured');
            }

            if (true === product.is_on_sale) {
                extraClasses.push('is-on-sale');
            }

            if ('variable' === product.type) {
                extraClasses.push('no-product-selected');
            }

            html.push('<div class="stec-layout-shop-product ' + extraClasses.join(' ') + '" data-id="' + product.id + '">');

            html.push('          <div class="stec-layout-shop-product-main">');

            if (0 !== image.id) {
                html.push('                <div class="stec-layout-shop-product-image" data-id="' + image.id + '"><i class="fas fa-eye"></i><img src="' + image.thumb + '" /></div>');
            } else {
                html.push('                <div class="stec-layout-shop-product-image stec-layout-shop-no-gallery" data-id="' + image.id + '"><img src="' + image.thumb + '" /></div>');
            }

            html.push('                <div class="stec-layout-shop-product-desc">');
            html.push('                    <div class="stec-layout-shop-product-status">');
            html.push('                        <span class="stec-layout-shop-product-about-featured">' + __('featured') + '</span>');
            html.push('                        <span class="stec-layout-shop-product-about-sale">' + __('Sale') + '</span>');
            html.push('                    </div>');

            html.push('                    <p class="stec-style-title2">' + title + '</p>');
            html.push('                    <p class="stec-style-text">' + desc + '</p>');
            html.push('                </div>');

            html.push('                <p class="stec-layout-shop-product-price stec-style-title2"><span class="stec-style-text">' + __('PRICE:') + '</span>' + price + '</p>');

            html.push('                <p class="stec-layout-shop-product-quantity stec-style-title2">');
            html.push('                    <span class="stec-style-text">' + __('QTY:') + '</span>');
            html.push('                    <input name="qty" type="number" class="stec-style-input" value="1" min="1" max="' + quantity + '" />');
            html.push('                    <span class="stec-layout-shop-product-outofstock">' + __('Out of Stock') + '</span>');
            html.push('                </p>');

            html.push('                <div class="stec-layout-shop-product-actions">');

            if ('variable' === product.type) {
                html.push('                    <div class="stec-style-button" data-action="toggle_options">');
                html.push('                        <p>' + __('Options') + '</p>');
                html.push('                        <i class="fa fa-bars"></i>');
                html.push('                    </div>');
            } else {
                html.push('                    <div class="stec-style-button" data-action="add_to_cart">');
                html.push('                        <p>' + __('Add to Cart') + '</p>');
                html.push('                        <i class="fa fa-shopping-cart"></i>');
                html.push('                    </div>');
            }

            html.push('                </div>');
            html.push('            </div>');

            if ('variable' === product.type) {
                html.push(parent.getHtmlProductsVariables(product));
            }

            html.push('</div>');
        });

        return html.join('');

    };

    proto.addHtml = function () {

        var html = [];

        html.push('<div class="stec-layout-shop">');
        html.push(this.getHtmlTop());
        html.push(this.getHtmlProducts());
        html.push(this.getHtmlBottomLinks());
        html.push('</div>');

        $(html.join('')).appendTo(this.settings.$container.find('.stec-layout-event-inner-shop, .stec-layout-single-shop'));

    };

    proto.getAvailableVariationOptions = function (current, attributes, variations) {

        var options = [];

        $.each(variations, function () {
            let include = true;
            let varLoop = this;

            $.each(attributes, function (attribName) {
                
                if (!varLoop.attributes[attribName] || (typeof this === 'String' && varLoop.attributes[attribName].valueOf() !== this.valueOf())) {
                    include = false;
                }
            });
            
            if (true === include) {
                $.each(varLoop.attributes, function (k) {
                    if (k === current && -1 === $.inArray(this.valueOf(), options)) {
                        options.push(this.valueOf());
                    }
                });
            }
        });

        return options;

    };

    proto.updateVariationOptions = function ($selector) {

        var parent = this;
        var $productContainer = $selector.parents('.stec-layout-shop-product').first();
        var productObject = parent.getEventProductById($productContainer.data('id'));

        $.each($productContainer.find('[data-type="variation"]'), function () {
            let attributes = {};
            let $variation = $(this);
            let current = $variation.attr('name');
            let availableOptions;

            attributes[current] = '';

            $.each($productContainer.find('[data-type="variation"]').not($variation), function () {
                let $sibling = $(this);
                attributes[$sibling.attr('name')] = $sibling.val();
            });

            availableOptions = parent.getAvailableVariationOptions(current, attributes, productObject.variations);

            $variation.find('option').each(function () {
                let $opt = $(this);
                let optVal = $opt.val();

                if (optVal) {
                    if (-1 === $.inArray(optVal, availableOptions)) {
                        $opt.hide();
                    } else {
                        $opt.show();
                    }
                }
            });
        });
    };

    proto.displayProducts = function () {
        if (this.settings.$container.find('.stec-layout-shop').children().length <= 0) {
            this.addHtml();
        }
    };

    proto.getEventProductById = function (searchId) {

        var found = false;

        $.each(this.settings.event.products, function () {

            if (this.id === searchId) {
                found = this;
                return false; // break;
            }

        });

        return found;
    };

    proto.getVariationById = function (productObjectOrId, variationId, returnPrimary) {

        var found = false;
        var product = typeof productObjectOrId === 'object' ? productObjectOrId : this.getEventProductById(productObjectOrId);

        if (product) {
            $.each(product.variations, function () {
                if (this.id === variationId) {
                    found = this;
                    return false; // break;
                }
            });
        }

        if (false === found && true === returnPrimary) {
            found = product;
        }

        return found;
    };

    proto.getVariableProductData = function (productId, variation) {

        var product = this.getEventProductById(productId);
        var found = false;

        $.each(product.variations, function () {
            if (JSON.stringify(variation) === JSON.stringify(this.attributes)) {
                found = this;
                return false;
            }
        });

        return found;

    };

    proto.updateProductFromVariation = function ($productContainer, variation, productId) {

        var quantity = 0;
        let productObject;

        try {

            if (false !== variation && variation.id) {

                quantity = variation.stock_quantity === null ? 9999 : variation.stock_quantity;

                if (quantity <= 0 || false === variation.is_in_stock || false === variation.is_purchasable) {
                    throw 'out_of_stock';
                }

                $productContainer.find('.stec-layout-shop-product-quantity input[name="qty"]').attr('max', quantity);
                $productContainer.find('.stec-layout-shop-product-price').html('<span class="stec-style-text">' + __('PRICE:') + '</span>' + variation.html_price);

                if (variation.image) {
                    $productContainer.find('.stec-layout-shop-product-image')
                            .data('id', variation.image.id)
                            .find('img').attr('src', variation.image.thumb);
                }

                $productContainer.removeClass('out-of-stock no-product-selected');
                $productContainer.data('variation_id', variation.id);

            } else {
                throw 0;
            }

        } catch (e) {

            $productContainer.addClass('no-product-selected');
            $productContainer.data('variation_id', '');

            switch (e) {
                case 0:

                    productObject = this.getEventProductById(productId);
                    $productContainer.find('.stec-layout-shop-product-price').html('<span class="stec-style-text">' + __('PRICE:') + '</span>' + productObject.html_price);
                    $productContainer.find('.stec-layout-shop-product-image')
                            .data('id', productObject.image.id)
                            .find('img').attr('src', productObject.image.thumb);
                    $productContainer.removeClass('out-of-stock');

                    break;

                case 'out_of_stock' :

                    $productContainer.addClass('out-of-stock');

                    break;

                default:

                    $productContainer.removeClass('out-of-stock');

                    console.error(e);
            }

        }

    };

    proto.addToCart = function ($productContainer, params) {

        var parent = this;
        var request = DB.newRequest();

        request.setData({
            action: 'stec_public_ajax_action',
            task: 'shop_add_to_cart',
            data: {
                stec_shop_item: params
            }
        });

        request.setBefore(function () {
            $productContainer.find('.stec-toast').remove();
            parent.blockActions($productContainer);
        });

        request.setCallback(function (result) {

            var toastCss = {position: 'absolute'};
            var toastContainer = $productContainer;

            if (parent.settings.$instance.hasClass('stec-media-small') || parent.settings.$instance.hasClass('stec-layout-single-media-small')) {
                toastCss = false;
                toastContainer = 'body';
            }

            parent.unblockActions($productContainer);

            if (result && 0 === result.error) {

                parent.settings.helper.toast(__('Item added to your cart'), toastContainer, 3000, toastCss);

                // update cart fragments 
                if (result.cart && result.cart.fragments) {
                    for (let key in result.cart.fragments) {
                        if (result.cart.fragments.hasOwnProperty(key)) {
                            $(key).replaceWith(result.cart.fragments[key]);
                        }
                    }
                }

            } else {
                parent.settings.helper.toast(result.msg, toastContainer, 3000, toastCss);
            }
        });

        DB.query(request);


    };

    proto.controller = function () {

        var parent = this;
        var $theInstances = ['.stec-layout-single', '#' + parent.settings.glob.options.id, '#stec-layout-event-inner-' + parent.settings.glob.options.id];

        /**
         * Event toggle open
         * Attaching products to shop section
         */
        parent.settings.$instance.on('onEventToggleOpen', function (e, data) {

            parent.settings.event = data.event;
            parent.settings.offset = parseInt(data.repeatOffset, 10);
            parent.settings.$container = data.$container;

            parent.displayProducts();

        });

        $.each($theInstances, function () {

            /**
             * Toggle product variations
             */

            $(document).on(parent.settings.helper.clickHandle(), this + ' .stec-layout-shop [data-action="toggle_options"]', function () {

                var $button = $(this);
                var $productContainer = $button.parents('.stec-layout-shop-product').first();
                var $variablesContainer = $productContainer.find('.stec-layout-shop-product-variables');

                $variablesContainer.toggleClass('active');
                $button.toggleClass('active');

            });

            /**
             * Add to Cart
             */
            $(document).on(parent.settings.helper.clickHandle(), this + ' .stec-layout-shop [data-action="add_to_cart"]', function () {

                var $button = $(this);
                var $productContainer = $button.parents('.stec-layout-shop-product').first();
                var productId = $productContainer.data('id');
                var variationId = $productContainer.data('variation_id');
                var quantity = $productContainer.find('input[name="qty"]').val();
                var eventStartDate = parent.settings.helper.beautifyDate(moment.tz(parent.settings.event.start_date, parent.settings.event.timezone).add(parent.settings.offset, 'seconds'), true);

                if ($productContainer.hasClass('out-of-stock') || $productContainer.hasClass('no-product-selected')) {
                    return false;
                }

                parent.addToCart($productContainer, {
                    'quantity': quantity,
                    'product_id': productId,
                    'variation_id': variationId,
                    'event_id': parent.settings.event.id,
                    'event_title': parent.settings.event.title,
                    'repeat_offset': parent.settings.offset,
                    'start_date': eventStartDate,
                    'permalink': parent.settings.helper.getPermalink(parent.settings.event.permalink, parent.settings.offset)
                });
            });

            /**
             * On variation change
             */
            $(document).on('change', this + ' .stec-layout-shop [data-action="variation_changed"]', function () {
                var $variableButton = $(this);
                var $productContainer = $variableButton.parents('.stec-layout-shop-product').first();
                var $variablesContainer = $productContainer.find('.stec-layout-shop-product-variables');
                var productId = $productContainer.data('id');
                var selectedVariation = {};
                var foundVariation;

                $.each($variablesContainer.find('select'), function () {
                    selectedVariation[this.name] = this.value;
                });

                foundVariation = parent.getVariableProductData(productId, selectedVariation);

                parent.updateProductFromVariation($productContainer, foundVariation, productId);

                parent.updateVariationOptions($variableButton);
            });

            /** Imager */
            $(document).on(parent.settings.helper.clickHandle(), this + ' .stec-layout-shop .stec-layout-shop-product-image:not(.stec-layout-shop-no-gallery)', function () {

                var $product = $(this).parents('.stec-layout-shop-product').first();
                var productId = $product.data('id');
                var productObject = parent.getEventProductById(productId);
                var activeImage = $(this).data('id');

                parent.showImageGallery(activeImage, productObject.gallery ? productObject.gallery : [productObject.image]);

            });


        });


    };

    proto.showImageGallery = function (activeImageId, gallery) {

        var parent = this;
        var modalWindow = this.settings.helper.newModalWindow();
        var html = [];
        var imagesHtml = [];
        var thumbsHtml = [];
        var extraClasses = [];
        var activeImageFound = false;

        $.each(gallery, function () {
            var thumbClass = ['stec-shop-modal-media-control-thumb'];
            var imageClass = [];

            if (activeImageId === this.id) {
                thumbClass.push('active');
                imageClass.push('active');
                activeImageFound = true;
            }

            imagesHtml.push('<img class="' + imageClass.join(' ') + '" data-id="' + this.id + '" src="' + this.full + '" />');
            thumbsHtml.push('<div class="' + thumbClass.join(' ') + '" data-id="' + this.id + '"><img src="' + this.thumb + '" /></div>');
        });

        if (gallery.length <= 1) {
            extraClasses.push('stec-shop-modal-media-simple');
        }

        html.push('<div class="stec-shop-modal-media-display ' + extraClasses.join(' ') + '">');
        html.push(imagesHtml.join(''));
        html.push('</div>');

        html.push('<div class="stec-shop-modal-media-control ' + extraClasses.join(' ') + '">');
        html.push(thumbsHtml.join(''));
        html.push('</div>');

        modalWindow.setContent(html.join(''));

        parent.settings.helper.onResizeEnd(function () {
            var $modal = $('#' + modalWindow.settings.id);
            $modal.trigger('stecShopModalUpdate');
        }, 100);

        modalWindow.promiseOpen().then(function ($modalWindow) {

            $modalWindow.find('.stec-shop-modal-media-control-thumb').on(parent.settings.helper.clickHandle(), function () {

                var $modal, $th, $sib, imageId;

                $th = $(this);

                if ($th.hasClass('active')) {
                    return;
                }

                $modal = $('#' + modalWindow.settings.id);
                $sib = $th.siblings();
                imageId = $th.data('id');

                $sib.removeClass('active');
                $th.addClass('active');

                $modal.find('.stec-shop-modal-media-display img.active').removeClass('active');
                $modal.find('.stec-shop-modal-media-display img[data-id="' + imageId + '"]').addClass('active');
                $modal.trigger('stecShopModalUpdate');
            });

            $modalWindow.on('stecShopModalUpdate', function () {
                var $modal = $(this);
                var $image = $modal.find('.stec-shop-modal-media-display img.active');
                $modal.find('.stec-shop-modal-media-control').height($image.height());
            });

            if (false === activeImageFound) {
                $modalWindow.find('.stec-shop-modal-media-control-thumb').first().trigger(parent.settings.helper.clickHandle());
            } else {
                $modalWindow.trigger('stecShopModalUpdate');
            }
        });
    };

})(window.jQuery);