(function ($) {

    "use strict";

    const __ = new window.StecHelper().lang;

    var DB = new window.StecDB();
    var moment = window.moment;
    moment.locale('en');

    // Stec Extend
    (function () {

        /**
         * Helper function allows binding to certain listeners
         * Used by adds/...
         */

        $.stecExtend = function (fn, sub) {

            if (typeof window.stachethemes_ec_extend === "undefined") {
                window.stachethemes_ec_extend = [];
            }

            if (sub) {

                if (typeof window.stachethemes_ec_extend[sub] === "undefined") {
                    window.stachethemes_ec_extend[sub] = [];
                }

                window.stachethemes_ec_extend[sub].push(fn);

            } else {
                window.stachethemes_ec_extend.push(fn);
            }
        };

    })();

    // Media.js
    (function () {

        // makes sure the media is init only once on first LayoutSet
        var mediaLoaded = {};

        function Media() {
            this.$instance = null;
            this.glob = [];

            this.init = function (glob, $instance) {
                this.glob = glob;
                this.$instance = $instance;

                this.Controller();
                this.mediaTrigger();
            };

        }

        Media.prototype.mediaTrigger = function () {

            var instanceWidth = this.$instance.width();
            var bodyWidth = $('body').width();
            var $modalInner = $('#stec-layout-event-inner-' + this.glob.options.id);

            // Top menu
            if (this.glob.options.general_settings.views.length >= 7 && instanceWidth < 1015 && instanceWidth > 600) {
                this.$instance.$top.addClass('stec-media-med');
            } else {
                this.$instance.$top.removeClass('stec-media-med');
            }

            // Main
            if (instanceWidth <= 620) {
                this.$instance.removeClass("stec-media-med");
                this.$instance.addClass("stec-media-small");
            } else if (instanceWidth <= 870) {
                this.$instance.removeClass("stec-media-small");
                this.$instance.addClass("stec-media-med");
            } else {
                this.$instance.removeClass("stec-media-med stec-media-small");
            }

            // Modal
            if ($modalInner.length > 0) {

                if (bodyWidth >= 870) {
                    $modalInner
                            .addClass("stec-media-med")
                            .removeClass('stec-media-small');
                } else {
                    $modalInner
                            .addClass('stec-media-small')
                            .removeClass("stec-media-med");
                }

            }
        };

        Media.prototype.Controller = function () {

            var parent = this;

            $(window).on("resize stec-trigger-resize", function () {
                parent.mediaTrigger();
            });

        };

        $.stecExtend(function (m) {
            if (true !== mediaLoaded[m.glob.options.id]) {
                let media = new Media();
                media.init(m.glob, m.$instance);
                mediaLoaded[m.glob.options.id] = true;
            }
        }, 'onLayoutSet');

    })();

    // Animate 
    (function () {


        /**
         *  Helper function for calendar animations 
         */
        window.StecAnimate = function () {

            var accordeonSpeed = 270;

            this.eventHolder = {

                open: function ($eventHolder, callback, instant) {

                    $eventHolder.show();

                    if (instant) {
                        $eventHolder.find('.stec-layout-event-preview').addClass('stec-layout-event-preview-animate-complete');


                        if (typeof callback === 'function') {
                            callback();
                        }

                        return;
                    }

                    $eventHolder.find('.stec-layout-event').addClass('stec-layout-event-perspective stec-layout-event-animate-queue');

                    $eventHolder.find('.stec-layout-event-preview').each(function (i) {

                        var $event = $(this);

                        setTimeout(function () {
                            $event.parent().removeClass('stec-layout-event-animate-queue');
                            $event.addClass('stec-layout-event-preview-animate');

                            setTimeout(function () {
                                $event.removeClass('stec-layout-event-preview-animate');
                                $event.addClass('stec-layout-event-preview-animate-complete');
                            }, accordeonSpeed);
                        }, i * accordeonSpeed);

                    });

                    setTimeout(function () {
                        $eventHolder.find('.stec-layout-event').removeClass('stec-layout-event-perspective');

                        if (typeof callback === 'function') {
                            callback();
                        }
                    }, $eventHolder.find('.stec-layout-event-preview').length * accordeonSpeed);


                }
            };

            this.agenda = {

                openSingle: function ($el, callback, instant) {

                    var $event = null;

                    if (instant) {
                        $event = $el.find('.stec-layout-event-preview');
                        $event.addClass('stec-layout-event-preview-animate-complete');

                        if (typeof callback === 'function') {
                            callback();
                        }

                        return;
                    }

                    $el.addClass('stec-layout-event-perspective');

                    $event = $el.find('.stec-layout-event-preview');

                    $event.addClass('stec-layout-event-preview-animate');

                    setTimeout(function () {
                        $event.removeClass('stec-layout-event-preview-animate');
                        $event.addClass('stec-layout-event-preview-animate-complete');
                    }, accordeonSpeed);

                    setTimeout(function () {
                        $el.removeClass('stec-layout-event-perspective');

                        if (typeof callback === 'function') {
                            callback();
                        }
                    }, accordeonSpeed);


                },

                fillList: function ($container, callback, instant) {

                    var $events = $container
                            .find('.stec-layout-agenda-events-all')
                            .find('.stec-layout-event-preview')
                            .not('.stec-layout-event-preview-animate-complete');


                    if (instant) {

                        $events.addClass('stec-layout-event-preview-animate-complete');

                        if (typeof callback === 'function') {
                            callback();
                        }

                        return;
                    }

                    $events.each(function (i) {

                        var $event = $(this);

                        $event.parent().addClass('stec-layout-event-perspective stec-layout-event-animate-queue');

                        setTimeout(function () {
                            $event.addClass('stec-layout-event-preview-animate');
                            $event.parent().removeClass('stec-layout-event-animate-queue');

                            setTimeout(function () {
                                $event.removeClass('stec-layout-event-preview-animate');
                                $event.addClass('stec-layout-event-preview-animate-complete');
                            }, accordeonSpeed);

                        }, i * accordeonSpeed);

                    });

                    setTimeout(function () {
                        $events.removeClass('stec-layout-event-perspective');

                        if (typeof callback === 'function') {
                            callback();
                        }

                    }, $events.length * accordeonSpeed);

                }
            };

            this.tooltip = {

                t: null,

                clear: function () {
                    clearTimeout(this.t);
                },

                show: function ($el, callback) {

                    $el.show().addClass('stec-tooltip-show');

                    this.t = setTimeout(function () {
                        if (typeof callback === 'function') {
                            callback();
                        }
                    }, 250);

                },

                hide: function ($el, callback) {

                    $el.removeClass('stec-tooltip-show');

                    this.t = setTimeout(function () {
                        if (typeof callback === 'function') {
                            callback();
                        }
                    }, 250);
                }
            };
        };

    })();

    // Attendance
    (function () {

        $(document).on('onEventToggleOpen', function (e, data) {

            var helper,
                    glob,
                    $container,
                    $inner,
                    event,
                    eventOccurrenceDate,
                    template,
                    invitedUser,
                    status = 0,
                    $avatar,
                    nowDate,
                    startDate,
                    availableSlots;

            if (data.event.attendance.length <= 0) {
                return;
            }

            helper = data.helper;
            glob = data.glob;
            $container = data.$container;
            $inner = $container.find('.stec-layout-event-inner-attendance');
            event = data.event;
            eventOccurrenceDate = moment(data.event.start_date).add(data.repeatOffset, 'seconds').format('YYYY-MM-DD');

            if ($inner.length <= 0 || true === $inner.data('loaded')) {
                return;
            }

            $inner.data('loaded', true);

            template = $('#stec-layout-event-inner-attendance-attendee-template').html();
            
            availableSlots = helper.attendanceHasFreeSlots(event, eventOccurrenceDate);

            if (event.attendance.length > 0) {

                $(event.attendance).each(function (i) {

                    var th = this, $attendee;

                    $attendee = $(template).html(function (index, html) {

                        var statusKlass = '';

                        switch (parseInt(th.status[eventOccurrenceDate], 10)) {
                            case 1:
                                statusKlass = 'fa fa-check';
                                break;

                            case 2:
                                statusKlass = 'fa fa-times';
                                break;

                            default:
                                statusKlass = 'fa fa-question';
                        }

                        return html
                                .replace(/stec_replace_people/g, th.people)
                                .replace(/stec_replace_name/g, th.name)
                                .replace(/#stec_replace_avatar/g, th.avatar)
                                .replace(/stec_replace_userid/g, th.userid ? th.userid : '')
                                .replace(/stec_replace_status/g, '<li class=""><i class="' + statusKlass + '"></i></li>');

                    })
                            .appendTo($inner.find('.stec-layout-event-inner-attendance-attendees'));

                    if (parseInt(th.status[eventOccurrenceDate], 10) === 1) {
                        $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').show();
                    }
                });

            }

            invitedUser = false;

            if (!isNaN(glob.options.userid)) {

                // check if user is invited
                $(event.attendance).each(function () {
                    if (this.userid === glob.options.userid) {
                        invitedUser = true;
                        return false; // break
                    }
                });
            }

            if (invitedUser === true) {

                $(event.attendance).each(function () {
                    if (this.userid === glob.options.userid && this.status[eventOccurrenceDate]) {
                        status = parseInt(this.status[eventOccurrenceDate], 10);
                        return false; // break
                    }
                });

                $avatar = $inner.find('.stec-layout-event-inner-attendance-attendee-avatar')
                        .filter('[data-userid="' + glob.options.userid + '"]');

                if (9999 === availableSlots) {
                    $inner.find('.stec-layout-event-inner-attendance-invited')
                            .find('.stec-layout-event-inner-attendance-invited-slots-count')
                            .parent().hide();
                }

                $inner.find('.stec-layout-event-inner-attendance-invited')
                        .find('.stec-layout-event-inner-attendance-invited-slots-count')
                        .text(availableSlots);

                switch (status) {
                    case 1:
                        // accepted
                        $inner.find('.stec-layout-event-inner-attendance-invited-buttons-accept').addClass('active');
                        $inner.find('.stec-layout-event-inner-attendance-invited-buttons-decline').removeClass('active');

                        $avatar.find('li i').attr('class', 'fa fa-check');

                        break;

                    case 2:
                        // declined
                        $inner.find('.stec-layout-event-inner-attendance-invited-buttons-accept').removeClass('active');
                        $inner.find('.stec-layout-event-inner-attendance-invited-buttons-decline').addClass('active');

                        $avatar.find('li i').attr('class', 'fa fa-times');

                        break;
                }
            } else {
                $inner.find('.stec-layout-event-inner-attendance-invited').remove();
            }

            // Attend / Decline

            function ajaxAttendance(status) {

                // status
                // 0 - no decision
                // 1 - accept 
                // 2 - decline

                var request = DB.newRequest();

                request.setData({
                    action: 'stec_public_ajax_action',
                    task: 'set_user_event_attendance',
                    event_id: event.id,
                    date: eventOccurrenceDate,
                    status: status
                });

                request.setBefore(function () {
                    $inner.find('.stec-layout-event-inner-attendance-invited-buttons').hide();
                    $(glob.template.preloader).appendTo($inner.find('.stec-layout-event-inner-attendance-invited'));
                });

                request.setCallback(function (rtrn) {

                    var $avatar;
                    var $attendee;
                    var status = parseInt(rtrn.status, 10);
                    var updatedList = null;

                    $container.find('.stec-layout-event-inner-intro-attendance li').removeClass('active');
                    $inner.find('.stec-layout-event-inner-attendance-invited-buttons li').removeClass('active');

                    $avatar = $inner.find('.stec-layout-event-inner-attendance-attendee-avatar')
                            .filter('[data-userid="' + glob.options.userid + '"]');

                    $attendee = $avatar.parent();

                    switch (status) {
                        case 1 :
                            $container.find('.stec-layout-event-inner-intro-attendance-attend').addClass('active');
                            $inner.find('.stec-layout-event-inner-attendance-invited-buttons-accept').addClass('active');
                            $avatar.find('li i').attr('class', 'fa fa-check');
                            $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').show();
                            break;
                        case 2 :
                            $container.find('.stec-layout-event-inner-intro-attendance-decline').addClass('active');
                            $inner.find('.stec-layout-event-inner-attendance-invited-buttons-decline').addClass('active');
                            $avatar.find('li i').attr('class', 'fa fa-times');
                            $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').hide();
                            break;

                        case 3 :


                            updatedList = [];
                            $(event.attendance).each(function () {
                                if (this.userid === glob.options.userid) {
                                    return true; // continue;
                                }

                                updatedList.push(this);
                            });

                            event.attendance = updatedList;
                            data.glob.rsvp.settings.update = false;

                            data.helper.layout.set('layout');
                            return;

                            break;
                        default:
                            $avatar.find('li i').attr('class', 'fa fa-question');
                            $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').hide();

                    }

                    $(event.attendance).each(function () {
                        if (this.userid === glob.options.userid) {
                            this.status[eventOccurrenceDate] = status;
                            return false; // break
                        }
                    });

                    $inner.find('.stec-layout-event-inner-attendance-invited-buttons').css('display', 'flex');
                    $inner.find('.stec-layout-event-inner-attendance-invited').find('.stec-preloader').remove();

                    $inner.find('.stec-layout-event-inner-attendance-invited')
                            .find('.stec-layout-event-inner-attendance-invited-slots-count')
                            .text(rtrn.slots_left);

                });

                DB.query(request);
            }

            $inner.find('.stec-layout-event-inner-attendance-invited-buttons-accept').on(helper.clickHandle(), function (e) {

                var rsvp = data.glob.rsvp;

                e.preventDefault();

                if ($(this).hasClass('active')) {
                    ajaxAttendance(0);
                } else {
                    rsvp.settings.id = data.event.id;
                    rsvp.settings.offset = data.repeatOffset;
                    rsvp.settings.event = data.event;
                    rsvp.settings.date = moment.tz(data.event.start_date, data.event.timezone).add(data.repeatOffset, 'seconds').format('YYYY-MM-DD');
                    rsvp.settings.update = true;
                    rsvp.popOpen();
                }
            });

            $inner.find('.stec-layout-event-inner-attendance-invited-buttons-decline').on(helper.clickHandle(), function (e) {
                var status;
                e.preventDefault();
                status = $(this).hasClass('active') ? 0 : 2;
                ajaxAttendance(status);
            });

            $inner.find('.stec-layout-event-inner-attendance-invited-buttons-unlist').on(helper.clickHandle(), function (e) {
                var status;
                e.preventDefault();
                status = 3;
                ajaxAttendance(status);
            });

            // Check if event is in progress
            nowDate = moment();
            startDate = moment.tz(event.start_date, event.timezone).add(data.repeatOffset, 'seconds');

            if (nowDate >= startDate) {
                $inner.find('.stec-layout-event-inner-intro-attendance').hide();
            }

            if (nowDate >= startDate) {
                $inner.find('.stec-layout-event-inner-attendance-invited').hide();
            }
        });


    })();

    // Comments
    (function () {


        $.stecExtend(function (master) {

            var helper = master.helper;
            var glob = master.glob;
            var container;

            var disqus = {

                loadCommentsCount: function () {

                    var disqusShortname = master.glob.options.general_settings.disqus_shortname;

                    $.ajax({
                        type: "GET",
                        url: "//" + disqusShortname + ".disqus.com/count.js",
                        dataType: "script",
                        cache: true
                    });
                },

                loadComments: function ($tabCont) {

                    var $container = $(container), eventId, offset, event, link, disqusShortname, disqusTitle;

                    $container.find("#disqus_thread").remove();

                    $('<div id="disqus_thread"></div>').appendTo($tabCont);

                    eventId = $container.data('event-id');
                    offset = parseInt($container.data('repeat-time-offset'), 10);
                    event = master.calData.getEventById(eventId);
                    link = event.permalink + (offset ? offset : '');

                    disqusShortname = master.glob.options.general_settings.disqus_shortname;
                    disqusTitle = $tabCont.parents('.stec-layout-event')
                            .find('.stec-layout-event-preview-left-text-title')
                            .text();

                    window.disqus_url = link;
                    window.disqus_identifier = "stecEventID" + eventId;
                    window.disqus_title = disqusTitle;

                    if (typeof window.DISQUS === "undefined") {
                        $.ajax({
                            type: "GET",
                            url: "//" + disqusShortname + ".disqus.com/embed.js",
                            dataType:
                                    "script",
                            cache: true
                        });
                    } else {
                        window.DISQUS.reset({
                            reload: true
                        });
                    }
                }

            };

            var facebook = {
                loadComments: function ($tabCont) {

                    var $container = $(container);
                    var eventId = $container.data('event-id');
                    var offset = parseInt($container.data('repeat-time-offset'), 10);
                    var event = master.calData.getEventById(eventId);
                    var link = event.permalink + (offset ? offset : '');

                    if ($('#fb-root').length <= 0) {
                        $('<div id="fb-root"></div>').appendTo('body');
                    }

                    $('<div class="stec-fb-comments fb-comments" data-width="100%" data-href="' + link + '" data-numposts="5"></div>').appendTo($tabCont);
                    window.FB.XFBML.parse();

                }
            };

            function loadComments($container) {

                switch (glob.options.general_settings.comments_form) {
                    case 'facebook' :

                        if ($container
                                .find('.stec-layout-event-inner-comments')
                                .find('.stec-fb-comments').length <= 0) {

                            facebook.loadComments($container.find('.stec-layout-event-inner-comments'));

                            break;
                        }

                        break;

                    case 'disqus' :

                        if ($container
                                .find('.stec-layout-event-inner-comments')
                                .find('#disqus_thread').length <= 0) {
                            disqus.loadComments($container.find('.stec-layout-event-inner-comments'));

                            break;
                        }
                }
            }

            function tabToggle($container) {
                if ($container
                        .find('.stec-layout-event-inner-top-tabs')
                        .children('.active[data-tab="stec-layout-event-inner-comments"]')
                        .length > 0) {

                    loadComments($container);
                }
            }

            if (master.glob.options.general_settings.open_event_in === 'modal') {
                container = '#stec-layout-event-inner-' + master.glob.options.id;
            } else {
                container = master.instance + ' .stec-layout-event.active .stec-layout-event-inner';
            }

            $(document).on(helper.clickHandle(), container + ' .stec-layout-event-inner-top-tabs li[data-tab="stec-layout-event-inner-comments"]', function () {
                loadComments($(container));
            });

            $(document).on(helper.clickHandle(), container + ' .stec-layout-event-preview-right-event-toggle', function (e) {
                e.preventDefault();
                e.stopPropagation();
                tabToggle($(container));

            });

            $(document).on(helper.clickHandle(), container + ' .stec-layout-event', function (e) {
                e.preventDefault();
                tabToggle($(container));
            });


        });

    })();

    // Event.create
    (function () {


        $.stecExtend(function (master) {

            var $form;

            if (parseInt(master.glob.options.general_settings.show_create_event_form, 10) === 0) {
                return;
            }

            // Remove old submission form
            master.$instance.find('.stec-event-create-form').remove();

            // Append agenda form
            if ('agenda' === master.glob.options.view) {

                $form = $(master.glob.template.eventCreateForm).prependTo(master.$instance.$agenda.find('.stec-layout-events-form'));
                new window.stecEventSubmissionForm().init({
                    id: $form.find('.stec-event-submission-form').attr('id'),
                    master: master,
                    create_event_form_allow_expired: master.glob.options.general_settings.create_event_form_allow_expired,
                    create_event_form_allow_location_type: master.glob.options.general_settings.create_event_form_allow_location_type,
                    display_privacy_agreement: master.glob.options.general_settings.display_privacy_agreement,
                    captcha: master.glob.options.captcha,
                    date_format: master.glob.options.general_settings.date_format,
                    useremail: master.glob.options.useremail
                });

                if (master.helper.animate !== false) {
                    setTimeout(function () {
                        master.helper.animate.agenda.openSingle(master.$instance.$agenda.find('.stec-event-create-form'));
                    }, 0);
                }
            }

        }, 'onLayoutSet');

        $.stecExtend(function (master) {
            var $form;

            if (parseInt(master.glob.options.general_settings.show_create_event_form, 10) === 0) {
                return;
            }

            if ($.inArray(master.glob.options.view, ['month', 'week', 'day']) !== -1) {
                if (master.$instance['$' + master.glob.options.view].find('.stec-event-create-form').length <= 0) {

                    $form = $(master.glob.template.eventCreateForm).prependTo(master.$instance['$' + master.glob.options.view].find('.stec-layout-events'));

                    new window.stecEventSubmissionForm().init({
                        id: $form.find('.stec-event-submission-form').attr('id'),
                        master: master,
                        create_event_form_allow_expired: master.glob.options.general_settings.create_event_form_allow_expired,
                        create_event_form_allow_location_type: master.glob.options.general_settings.create_event_form_allow_location_type,
                        display_privacy_agreement: master.glob.options.general_settings.display_privacy_agreement,
                        captcha: master.glob.options.captcha,
                        date_format: master.glob.options.general_settings.date_format,
                        useremail: master.glob.options.useremail
                    });
                }
            }

        }, 'onEventHolderOpen');

        $.stecExtend(function (master) {

            if (parseInt(master.glob.options.general_settings.show_create_event_form, 10) === 0) {
                return;
            }

            $(document).on(master.helper.clickHandle(), master.$instance.$agenda.path + ' .stec-layout-event-create-form-inner', function (e) {
                e.stopPropagation();
            });

            $(document).on(master.helper.clickHandle(),
                    master.instance + (" .stec-event-create-form") + ',' +
                    master.instance + (" .stec-layout-event-create-form-preview-right-event-toggle"), function (e) {

                var $form;

                e.preventDefault();
                e.stopPropagation();

                $form = $(this).hasClass('stec-event-create-form') ? $(this) : $(this).parents('.stec-event-create-form');

                $form.toggleClass('active');
                $form.find('.stec-layout-event-create-form-preview-right-event-toggle').toggleClass('active');

            });

        });

    })();

    // Forecast
    (function () {

        $(document).on('onBeforeEventWeatherDataAjax', function (e, data) {

            var glob = data.glob;
            var container;
            var $tab;

            if (data.glob.options.general_settings.open_event_in === 'modal') {
                container = '#stec-layout-event-inner-' + data.glob.options.id;
            } else {
                container = '#' + data.glob.options.id + ' .stec-layout-event.active .stec-layout-event-inner';
            }

            $tab = $(container).find('.stec-layout-event-inner-forecast');
            $tab.wrapInner('<div class="stec-layout-event-inner-preload-wrap" />');
            $(glob.template.preloader).appendTo($tab);
        });

        $(document).on('onEventToggleOpen', function (e, data) {

            var glob, $instance, helper, $container, $inner, template, isLoaded, getWeather, fillData, chart, fillError;

            if (!data.event.location.coordinates || !data.glob.options.general_settings.weather_api_key) {
                return;
            }

            glob = data.glob;
            $instance = data.$instance;
            helper = data.helper;
            $container = data.$container;
            $inner = $container.find('.stec-layout-event-inner-forecast');

            if ($inner.data('loaded') === true) {
                return;
            }

            function floorFigure(figure, decimals) {
                var d;
                if (!decimals) {
                    decimals = 4;
                }

                d = Math.pow(10, decimals);
                return (parseInt(figure * d) / d).toFixed(decimals);
            }

            function getWindDir(bearing) {

                var val, arr;

                while (bearing < 0) {
                    bearing += 360;
                }

                while (bearing >= 360) {
                    bearing -= 360;
                }

                val = Math.round((bearing - 11.25) / 22.5);
                arr = [
                    __('N'),
                    __('NNE'),
                    __('NE'),
                    __('ENE'),
                    __('E'),
                    __('ESE'),
                    __('SE'),
                    __('SSE'),
                    __('S'),
                    __('SSW'),
                    __('SW'),
                    __('WSW'),
                    __('W'),
                    __('WNW'),
                    __('NW'),
                    __('NNW')
                ];
                return arr[ Math.abs(val) ];

            }

            function iconToText(icon) {

                switch (icon) {

                    case ('01d') :
                    case ('01n') :
                        return __('clear_sky');

                    case ('02d') :
                    case ('02n') :
                        return __('partly_cloudy');

                    case ('03d') :
                    case ('03n') :
                    case ('04d') :
                    case ('04n') :
                        return __('cloudy');

                    case ('09d') :
                    case ('09n') :
                    case ('10d') :
                    case ('10n') :
                        return __('rain');


                    case ('11d') :
                    case ('11n') :
                        return __('thunderstorm');

                    case ('13d') :
                    case ('13n') :
                        return __('snow');


                    case ('50d') :
                    case ('50n') :
                        return __('mist');


                }
            }

            function iconToiconHTML(icon, forceday) {

                // openweathermap.org/weather-conditions

                switch (icon) {

                    case ('01d') :
                        return '<div class="stec-forecast-icon-clear-day"></div>';

                    case ('01n') :
                        return forceday ? '<div class="stec-forecast-icon-clear-day"></div>' : '<div class="stec-forecast-icon-clear-night"></div>';

                    case ('02d') :
                    case ('04d') :
                        return '<div class="stec-forecast-icon-cloudy-day"></div>';

                    case ('02n') :
                    case ('04n') :
                        return forceday ? '<div class="stec-forecast-icon-cloudy-day"></div>' : '<div class="stec-forecast-icon-cloudy-night"></div>';

                    case ('03d') :
                    case ('03n') :
                        return '<div class="stec-forecast-icon-cloudy"></div>';

                    case ('50d') :
                    case ('50n') :
                        return '<div class="stec-forecast-icon-mist"></div>';

                    case ('11d') :
                    case ('11n') :

                    case ('10d') :
                    case ('10n') :

                    case ('09d') :
                    case ('09n') :
                        return  '<div class="stec-forecast-icon-rain"></div>';

                    case ('sleet') :
                        return '<div class="stec-forecast-icon-sleet"></div>';

                    case ('13d') :
                    case ('13n') :
                        return '<div class="stec-forecast-icon-snow"></div>';

                }
            }

            getWeather = function () {

                var location, request;

                if (data.event.forecast) {

                    if (!data.event.forecast.error) {
                        fillData();
                    } else {
                        fillError();
                    }
                    return;
                }

                location = data.event.location.coordinates;
                request = DB.newRequest();

                request.setData({
                    action: 'stec_public_ajax_action',
                    task: 'get_weather_data',
                    location: function () {
                        location = location.split(',');
                        location[0] = floorFigure(location[0]);
                        location[1] = floorFigure(location[1]);
                        location = location.join(',');
                        return location;

                    }
                });

                request.setBefore(function () {
                    $instance.trigger('onBeforeEventWeatherDataAjax', {
                        $instance: $instance,
                        glob: glob
                    });
                });

                request.setCallback(function (rtrn) {
                    if (rtrn) {

                        if (!data.event.forecast) {
                            data.event.forecast = rtrn;
                        }

                        // error ?
                        if (data.event.forecast.error || !data.event.forecast) {
                            fillError();
                            return;
                        }

                        fillData();
                    } else {
                        fillError();
                    }

                    // Remove tabs preloaders
                    $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                    $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                    $inner.find('.stec-preloader').remove();

                });

                DB.query(request);

            };

            fillData = function () {

                var fiveDays = [];
                var forecast = data.event.forecast;
                var ch;

                $(forecast.daily).each(function (index) {

                    var th, icon, d, niceDay;

                    if (index > 5) {
                        return;
                    }

                    th = this;
                    icon = iconToiconHTML(th.weather[0].icon, true);

                    // Localtime
                    d = moment.unix(th.dt).tz(data.event.timezone);

                    niceDay = helper.beautifyTimespan(d, d, 1);

                    fiveDays[index] = $(template).html(function (index, html) {

                        var tempFmin = Math.round(th.temp.min);
                        var tempCmin = Math.round((tempFmin - 32) * 5 / 9);

                        var tempFmax = Math.round(th.temp.max);
                        var tempCmax = Math.round((tempFmax - 32) * 5 / 9);

                        return html
                                .replace(/\bstec_replace_date\b/g, niceDay)
                                .replace(/\bstec_replace_min\b/g, glob.options.general_settings.temp_units === "C" ? tempCmin : tempFmin)
                                .replace(/\bstec_replace_max\b/g, glob.options.general_settings.temp_units === "C" ? tempCmax : tempFmax)
                                .replace(/\bstec_replace_temp_units\b/g, glob.options.general_settings.temp_units === "C" ? "C" : "F")
                                .replace(/\bstec_replace_icon_div\b/g, icon);
                    })[0].outerHTML;

                });

                fiveDays = fiveDays.join('');

                $inner.html(function (index, html) {

                    var icon = iconToiconHTML(forecast.current.weather[0].icon);
                    var tempF = Math.round(forecast.current.temp);
                    var tempC = Math.round((tempF - 32) * 5 / 9);
                    var apTempF = Math.round(forecast.current.feels_like);
                    var apTempC = Math.round((tempF - 32) * 5 / 9);
                    var windMPH = Math.round(forecast.current.wind_speed);
                    var windKPH = Math.round(windMPH * 1.609344);
                    var niceDay;

                    // Local time
                    var d = moment.unix(forecast.current.dt).tz(data.event.timezone);

                    niceDay = helper.beautifyTimespan(d, d, 1);

                    return html
                            .replace(/\bstec_replace_current_summary_text\b/g, iconToText(forecast.current.weather[0].icon))
                            .replace(/\bstec_replace_today_date\b/g, niceDay)
                            .replace(/\bstec_replace_location\b/g, helper.capitalizeFirstLetter(data.event.location.full_address))
                            .replace(/\bstec_replace_current_temp\b/g, glob.options.general_settings.temp_units === "C" ? tempC : tempF)
                            .replace(/\bstec_replace_current_feels_like\b/g, glob.options.general_settings.temp_units === "C" ? apTempC : apTempF)
                            .replace(/\bstec_replace_current_humidity\b/g, (forecast.current.humidity))
                            .replace(/\bstec_replace_current_temp_units/g, glob.options.general_settings.temp_units === "C" ? "C" : "F")
                            .replace(/\bstec_replace_current_wind\b/g, glob.options.general_settings.wind_units === "MPH" ? windMPH : windKPH)
                            .replace(/\bstec_replace_current_wind_units\b/g, glob.options.general_settings.wind_units === "MPH" ? "MPH" : "KPH")
                            .replace(/\bstec_replace_current_wind_direction\b/g, getWindDir(forecast.current.wind_deg))
                            .replace(/\bstec_replace_today_icon_div\b/g, icon)
                            .replace(/\bstec_replace_5days\b/g, fiveDays);

                });

                // Chart instance
                setTimeout(function () {

                    var th,
                            i,
                            humidity = [],
                            tempf,
                            tempc,
                            tempC = [],
                            tempF = [],
                            cloudness = [],
                            j = -1,
                            timeFormat,
                            d;


                    var charTimeLabels = [];

                    for (i = 0; i < 8; i++) {

                        j++;

                        th = forecast.hourly[j];

                        tempf = Math.round(th.temp);
                        tempc = Math.round((tempf - 32) * 5 / 9);

                        tempC[i] = tempc;
                        tempF[i] = tempf;
                        humidity[i] = th.humidity;
                        cloudness[i] = th.clouds;

                        timeFormat = 'HH:mm';

                        if (parseInt(glob.options.general_settings.time_format, 10) === 12) {
                            timeFormat = 'hh:mma';
                        }

                        // Local time
                        d = moment.unix(th.dt).tz(data.event.timezone);

                        charTimeLabels.push(d.format(timeFormat));

                    }

                    ch = new chart();

                    ch.setCanvas($inner.find('.stec-layout-event-inner-forecast-details-chart canvas'));

                    ch.setChartData({
                        labels: charTimeLabels,
                        datasets: [

                            {
                                yAxisID: 'mainYAxis',
                                label: __('temperature') + ' ' + '\u00B0' + (glob.options.general_settings.temp_units === "C" ? 'C' : 'F'),
                                data: glob.options.general_settings.temp_units === "C" ? tempC : tempF,
                                backgroundColor: "rgba(255, 196, 79,0.3)",
                                borderColor: "rgba(255, 196, 79,1)",
                                pointBackgroundColor: "rgba(252,183,85,1)",
                                fill: true,
                                lineTension: 0.3,
                                pointHoverRadius: 5,
                                pointHitRadius: 10,
                                borderWidth: 1
                            },
                            {
                                yAxisID: 'mainYAxis',
                                label: __('humidity_percents'),
                                data: humidity,
                                backgroundColor: "rgba(202, 212, 224,0.1)",
                                borderColor: "rgba(202, 212, 224,1)",
                                pointBackgroundColor: "rgba(202, 212, 224,1)",
                                fill: true,
                                lineTension: 0.3,
                                pointHoverRadius: 5,
                                pointHitRadius: 10,
                                borderWidth: 1
                            },
                            {
                                yAxisID: 'mainYAxis',
                                label: __('cloudness'),
                                data: cloudness,
                                backgroundColor: "rgba(141, 152, 166,0.1)",
                                borderColor: "rgba(141, 152, 166,1)",
                                pointBackgroundColor: "rgba(141, 152, 166,1)",
                                fill: true,
                                lineTension: 0.3,
                                pointHoverRadius: 5,
                                pointHitRadius: 10,
                                borderWidth: 1
                            }
                        ]
                    });

                    ch.build();

                    helper.onResizeEnd(function () {

                        if ($instance.hasClass('stec-media-small')) {
                            ch.chart.options.legend.display = false;
                        } else {
                            ch.chart.options.legend.display = true;
                        }

                        ch.chart.update();

                    }, 50, 'stec-unbind-window-resize-on-event-close-' + glob.options.id);

                }, 0);

            };

            chart = function () {

                this.ctx = null;
                this.chartData = null;
                this.chart = null;
                this.setCanvas = function ($canvas) {

                    var canvas = $canvas.get(0);

                    var w = parseInt($inner.find('.stec-layout-event-inner-forecast-details-chart').width(), 10);
                    var h = parseInt($inner.find('.stec-layout-event-inner-forecast-details-chart').height(), 10);

                    canvas.width = w;
                    canvas.height = h;

                    this.ctx = $canvas.get(0).getContext("2d");

                };
                this.setChartData = function (chartData) {
                    this.chartData = chartData;
                };
                this.destroy = function () {
                    this.chart.destroy();
                };
                this.build = function () {

                    var parent = this;
                    var generalTextColor,
                            generalFontFamily,
                            displayLegend;

                    if (this.chart) {
                        this.destroy();
                    }

                    generalTextColor = $inner.find('.stec-layout-event-inner-forecast-details-left-forecast-top p').css('color');
                    generalFontFamily = $inner.find('.stec-layout-event-inner-forecast-details-left-forecast-top p').css('font-family');
                    displayLegend = true;

                    if ($instance.hasClass('stec-media-small')) {
                        displayLegend = false;
                    }

                    this.chart = new window.Chart(this.ctx, {
                        type: 'line',
                        data: parent.chartData,
                        options: {
                            maintainAspectRatio: false,
                            responsive: true,
                            defaultFontFamily: generalFontFamily,
                            defaultFontColor: generalTextColor,
                            legend: {
                                display: displayLegend,
                                labels: {
                                    fontFamily: generalFontFamily,
                                    fontColor: generalTextColor,
                                    fontSize: 12
                                }
                            },
                            scales: {
                                xAxes: [{
                                        ticks: {
                                            fontFamily: generalFontFamily,
                                            fontSize: 11,
                                            fontColor: generalTextColor
                                        },
                                        gridLines: {
                                            color: 'rgba(0,0,0,0.1)',
                                            zeroLineColor: 'rgba(0,0,0,0)'
                                        }
                                    }],
                                yAxes: [
                                    {
                                        id: 'mainYAxis',
                                        position: 'right',
                                        ticks: {
                                            beginAtZero: true,
                                            fontFamily: generalFontFamily,
                                            fontSize: 11,
                                            fontColor: generalTextColor
                                        },
                                        gridLines: {
                                            color: 'rgba(0, 0, 0,0.1)',
                                            zeroLineColor: 'rgba(0,0,0,0)'
                                        }
                                    }
                                ]
                            },
                            tooltips: {
                                titleFontColor: '#fff',
                                titleFontStyle: generalFontFamily,
                                bodyFontFamily: generalFontFamily,
                                bodyFontColor: '#fff'
                            }

                        }
                    });
                };

            };

            fillError = function () {
                $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                $inner.find('.stec-layout-event-inner-forecast-content').remove();
                $inner.find('.stec-preloader').remove();
                $inner.find('.errorna').show();
            };

            $(window).on('stec-tab-click-' + glob.options.id, function () {
                if ($inner.data('loaded') !== true) {
                    template = $('#stec-layout-event-inner-forecast-details-left-forecast-day-template').html();
                    getWeather();
                    $inner.data('loaded', true);
                }
            });

            if ($inner.hasClass('active')) {
                getWeather();
            }

        });

    })();

    // Guests
    (function () {


        $(document).on('onEventToggleOpen', function (e, data) {

            var $container, $inner, event, template;

            if (data.event.guests.length <= 0) {
                return;
            }

            $container = data.$container;
            $inner = $container.find('.stec-layout-event-inner-guests');
            event = data.event;

            if ($inner.length <= 0 || true === $inner.data('loaded')) {
                return;
            }
            
             $inner.data('loaded', true);

            template = $('#stec-layout-event-inner-guests-guest-template').html();

            if (event.guests.length > 0) {

                $(event.guests).each(function (i) {

                    var th = this, avatar;

                    $(template).html(function (index, html) {

                        var lis = [];

                        $(th.links).each(function (pos) {
                            var link = this.link;
                            var target = '_blank';
                            if ('fas fa-envelope' === this.ico) {
                                link = 'mailto:' + link;
                                target = '_self';
                            }
                            if ('fas fa-phone' === this.ico) {
                                link = 'tel:' + link;
                                target = '_self';
                            }
                            lis[pos] = '<li class="stec-layout-event-inner-guests-guest-left-avatar-icon-position-' + pos + '"><a target="' + target + '" href="' + link + '"><i class="' + this.ico + '"></i></a></li>';
                        });

                        if (!th.photo_full) {
                            avatar = '<div class="stec-layout-event-inner-guests-guest-left-avatar-default"></div>';
                        } else {
                            avatar = '<img src="' + th.photo_full + '" alt="stec_replace_name" >';
                        }

                        return html
                                .replace(/stec_replace_ico_position/g, i)
                                .replace(/stec_replace_avatar/g, avatar)
                                .replace(/stec_replace_social/g, lis.join(''))
                                .replace(/stec_replace_name/g, th.name)
                                .replace(/stec_replace_about/g, th.about);

                    })
                            .appendTo($inner);

                });

            }

            // Remove tab preloaders
            $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
            $inner.find('.stec-layout-event-inner-preload-wrap').remove();
            $inner.find('.stec-preloader').remove();

        });

    })();

    // Intro
    (function () {


        var moment = window.moment;
        moment.locale('en');


        $(document).on('onEventToggleOpen', function (e, data) {

            var glob = data.glob;
            var $instance = data.$instance;
            var $container = data.$container;
            var $inner = $container.find('.stec-layout-event-inner-intro');
            var helper, event, DB, eventOccurrenceDate, url, nowDate, startDate, endDate, repeatOffset, googleCalImportLink, categoryTagsHtml = [];
            var slider, organizersTemplate, clock;
            var invitedUser, status, availableSlots;
            var attachmentsTemplate;

            // Init once
            if ($inner.length <= 0 || $inner.data('loaded') === false) {
                return;
            }

            $inner.data('loaded', false);

            helper = data.helper;
            event = data.event;
            repeatOffset = parseInt(data.repeatOffset, 10);
            DB = new window.StecDB();
            eventOccurrenceDate = moment.tz(data.event.start_date, data.event.timezone).add(data.repeatOffset, 'seconds').format('YYYY-MM-DD');
            url = helper.getPermalink(event.permalink, data.repeatOffset);

            // add the repeat offset
            startDate = helper.dbDateOffset(event.start_date, data.repeatOffset);
            endDate = helper.dbDateOffset(event.end_date, data.repeatOffset);

            googleCalImportLink = helper.eventToGoogleCalImportLink(event.id, data.repeatOffset);

            if (event.category.length > 0 && parseInt(glob.options.general_settings.show_event_tags, 10) === 1) {

                // Sort categories alphabetically
                event.category.sort(function (a, b) {
                    return a.title.localeCompare(b.title, undefined, {numeric: true, sensitivity: 'base'});
                });

                $.each(event.category, function () {
                    categoryTagsHtml.push('<span class="stec-layout-event-inner-intro-tag" style="background-color: ' + this.color + '">' + this.title + '</span>');
                });
            } else {
                data.$container.find('.stec-layout-event-inner-intro-tags').remove();
            }

            $inner.html(function (index, html) {

                return html

                        .replace(/stec_replace_summary/g, event.title)
                        .replace(/stec_replace_tags/g, (categoryTagsHtml.join('')))
                        .replace(/stec_replace_description/g, (event.description))
                        .replace(/#stec_replace_link_url/g, event.link && event.link.url ? event.link.url : event.link)
                        .replace(/stec_replace_link_text/g, event.link && event.link.text ? event.link.text : __('Visit Website'))
                        .replace(/#stec_replace_googlecal_import/g, googleCalImportLink)
                        .replace(/stec_replace_event_id/g, event.id)
                        .replace(/stec_replace_calendar_id/g, event.calid)
                        .replace(/#stec_replace_event_single_url/g, url)
                        .replace(/stec_replace_event_single_url/g, url);
            });

            if (!event.edit_link) {
                $inner.find('.stec-layout-event-inner-intro-edit-link').remove();
            }

            slider = {
                cslide: 0,
                offset: 0,
                total: 0,
                blockAction: false,
                visCount: 0,
                visCountSmall: 3,
                visCountBig: 4,
                imagesArray: [],
                init: function () {

                    var parent = this;
                    this.imagesArray = [];

                    if (!event.images_meta || event.images_meta.length <= 0) {

                        $inner.find('.stec-layout-event-inner-intro-media').remove();

                        // Remove tab preloaders
                        $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                        $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                        $inner.find('.stec-preloader').remove();

                        return;
                    }

                    if (event.images_meta.length === 1) {
                        $inner.find('.stec-layout-event-inner-intro-media-controls').remove();
                    } else {
                        if (event.images_meta.length < this.visCountBig) {
                            this.visCountBig = event.images_meta.length;
                        }

                        if (event.images_meta.length < this.visCountSmall) {
                            this.visCountSmall = event.images_meta.length;
                        }
                    }

                    this.html();

                    $.each(event.images_meta, function () {
                        parent.imagesArray.push(this.src);
                    });

                    helper.preloadContentImages(parent.imagesArray).then(function () {
                        // Remove tab preloaders
                        $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                        $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                        $inner.find('.stec-preloader').remove();

                        setTimeout(function () {
                            parent.controlsDimensions();
                            parent.showImage();

                            $inner.find('.stec-layout-event-inner-intro-media').fadeTo(1000, 1);
                        }, 100);
                    });

                    this.bindControls();

                },
                bindControls: function () {

                    var parent = this;

                    // resize on tab click
                    $(document).on('stec-tab-click-' + glob.options.id, function () {
                        parent.controlsDimensions();
                    });

                    helper.onResizeEnd(function () {
                        parent.controlsDimensions();
                    }, 100, 'stec-unbind-window-resize-on-event-close-' + glob.options.id);

                    $inner.find('.stec-layout-event-inner-intro-media-controls-next').on(helper.clickHandle(), function (e) {
                        e.preventDefault();
                        parent.slideNext();
                    });

                    $inner.find('.stec-layout-event-inner-intro-media-controls-prev').on(helper.clickHandle(), function (e) {
                        e.preventDefault();
                        parent.slidePrev();
                    });

                    $inner.find('.stec-layout-event-inner-intro-media-controls li').on(helper.clickHandle(), function (e) {

                        e.preventDefault();

                        if (parent.cslide === $(this).index()) {
                            return;
                        }

                        parent.cslide = $(this).index();

                        parent.showImage();

                    });

                },
                html: function () {

                    var html = [];

                    $(event.images_meta).each(function () {

                        html.push('<div style="background-image:url(' + this.src + ');">');

                        if (this.caption !== '' || this.description !== '') {
                            html.push('   <div>');

                            if (this.caption !== '') {
                                html.push('       <p>' + this.caption + '</p>');
                            }

                            if (this.description !== '') {
                                html.push('       <span>' + this.description + '</span>');
                            }

                            html.push('   </div>');
                        }

                        html.push('</div>');

                    });

                    $(html.join('')).appendTo($inner.find('.stec-layout-event-inner-intro-media-content'));

                    html = [];

                    $(event.images_meta).each(function () {
                        html.push('<li style="background-image:url(' + this.thumb + ')">');
                        html.push('</li>');
                    });

                    $(html.join('')).appendTo($inner.find('.stec-layout-event-inner-intro-media-controls-list'));

                },
                controlsDimensions: function () {

                    var parent = this;
                    var maxWidth, $li, liWidth, listWidth, left;

                    if (!$inner.is(':visible')) {
                        return;
                    }

                    if ($instance.hasClass('stec-media-small')) {
                        parent.visCount = parent.visCountSmall;
                    } else {
                        parent.visCount = parent.visCountBig;
                    }

                    if ($inner.find('.stec-layout-event-inner-intro-media-controls-list li').length === parent.visCount) {
                        $inner.find('.stec-layout-event-inner-intro-media-controls').addClass('no-side-controls');
                    } else {
                        $inner.find('.stec-layout-event-inner-intro-media-controls').removeClass('no-side-controls');
                    }

                    maxWidth = $inner.find('.stec-layout-event-inner-intro-media-controls-list-wrap').width();
                    $li = $inner.find('.stec-layout-event-inner-intro-media-controls-list li');

                    $inner.find('.stec-layout-event-inner-intro-media-content').height(Math.min(400, helper.getImageHeight(parent.imagesArray[0])));

                    // ~'calc( (100% - 2*10px) / 3 )';
                    liWidth = (maxWidth - ((this.visCount - 1) * 10)) / this.visCount;

                    listWidth = ($li.length * liWidth) + ($li.length * 10) - 10;

                    $inner.find('.stec-layout-event-inner-intro-media-controls-list').width(listWidth);
                    $li.width(liWidth);

                    this.offset = 0;

                    left = -1 * ($li.first().width() * this.offset + this.offset * 10);

                    $inner.find('.stec-layout-event-inner-intro-media-controls-list').stop().css({
                        left: left
                    });


                },
                showImage: function () {

                    var $old, $new, $textContent, height;

                    $inner.find('.stec-layout-event-inner-intro-media-controls-list .active-thumb').removeClass('active-thumb');
                    $inner.find('.stec-layout-event-inner-intro-media-controls-list li').eq(this.cslide).addClass('active-thumb');

                    $old = $inner.find('.stec-layout-event-inner-intro-media-content .active-image');
                    $new = $inner.find('.stec-layout-event-inner-intro-media-content > div').eq(this.cslide);

                    $textContent = $new.find('div');

                    if ($textContent.length > 0) {

                        $inner.find('.stec-layout-event-inner-intro-media-content-subs div').fadeTo(250, 0, function () {

                            var caption = $textContent.find('p').text();
                            var desc = $textContent.find('span').text();

                            $inner.find('.stec-layout-event-inner-intro-media-content-subs p').text(caption);
                            $inner.find('.stec-layout-event-inner-intro-media-content-subs span').text(desc);

                            height = $inner.find('.stec-layout-event-inner-intro-media-content-subs p').height() + $inner.find('.stec-layout-event-inner-intro-media-content-subs span').height();

                            if (height > 0) {
                                height = height + 40;
                            }

                            $inner.find('.stec-layout-event-inner-intro-media-content-subs').stop().animate({
                                height: height
                            }, {
                                duration: 400,
                                easing: 'easeInOutExpo',
                                complete: function () {
                                    $inner.find('.stec-layout-event-inner-intro-media-content-subs div').fadeTo(250, 1);
                                }
                            });
                        });


                    } else {

                        $inner.find('.stec-layout-event-inner-intro-media-content-subs div').fadeTo(250, 0, function () {

                            $inner.find('.stec-layout-event-inner-intro-media-content-subs').stop().animate({
                                height: 0
                            }, {
                                duration: 400,
                                easing: 'easeInOutExpo'
                            });

                        });

                    }

                    $new.addClass('fade-in');

                    setTimeout(function () {

                        $old.removeClass('active-image');
                        $new.removeClass('fade-in').addClass('active-image');

                    }, 250);
                },
                slideNext: function () {

                    var $li = $inner.find('.stec-layout-event-inner-intro-media-controls-list li');
                    var left;

                    if (this.offset + this.visCount >= $li.length) {
                        this.offset = 0;
                    } else {
                        this.offset = this.offset + this.visCount;

                        if (this.offset > $li.length - this.visCount) {
                            this.offset = $li.length - this.visCount;
                        }
                    }

                    left = -1 * ($li.first().width() * this.offset + this.offset * 10);

                    $inner.find('.stec-layout-event-inner-intro-media-controls-list').stop().animate({
                        left: left
                    }, {
                        duration: 750,
                        easing: 'easeInOutExpo'
                    });

                },
                slidePrev: function () {

                    var $li = $inner.find('.stec-layout-event-inner-intro-media-controls-list li');
                    var left;

                    if (this.offset <= 0) {
                        this.offset = $li.length - this.visCount;
                    } else {
                        this.offset = this.offset - this.visCount;

                        if (this.offset < 0) {
                            this.offset = 0;
                        }
                    }

                    left = -1 * ($li.first().width() * this.offset + this.offset * 10);

                    $inner.find('.stec-layout-event-inner-intro-media-controls-list').stop().animate({
                        left: left
                    }, {
                        duration: 750,
                        easing: 'easeInOutExpo'
                    });

                }

            };

            slider.init();

            if (event.organizers.length > 0) {
                organizersTemplate = $('#stec-layout-event-inner-intro-organizers-organizer-template').html();

                $.each(event.organizers, function () {

                    var th = this;
                    var social = '';
                    var image = th.photo_src ? 'style="background-image: url(' + th.photo_src + ')"' : "";

                    if (th.social.length > 0) {
                        $.each(th.social, function () {
                            var link = this.link;
                            var target = '_blank';

                            if ('fas fa-envelope' === this.ico) {
                                link = 'mailto:' + link;
                                target = '_self';
                            }

                            if ('fas fa-phone' === this.ico) {
                                link = 'tel:' + link;
                                target = '_self';
                            }


                            social += '<a target="' + target + '" href="' + link + '"><i class="' + this.ico + '"></i></a>';
                        });
                    }

                    $(organizersTemplate)
                            .html(function (index, html) {

                                var organizerName = th.name;

                                if (th.permalink) {
                                    organizerName = '<a href="' + th.permalink + '">' + th.name + '</a>';
                                }

                                return html
                                        .replace(/stec_replace_title/g, organizerName)
                                        .replace(/stec_replace_event/g, event.title)
                                        .replace(/stec_replace_about/g, th.details)
                                        .replace(/stec_replace_social/g, social)
                                        .replace(/stec_replace_image/g, image);

                            }).appendTo($inner.find('.stec-layout-event-inner-intro-organizers'));


                });
            } else {
                $inner.find('.stec-layout-event-inner-intro-organizers').remove();
            }

            clock = {
                days: 0,
                hours: 0,
                minutes: 0,
                seconds: 0,
                daysLabel: '',
                hoursLabel: '',
                minutesLabel: '',
                secondsLabel: '',
                interval: '',
                init: function () {

                    var nowDate, startDate, timeLeft;

                    // Check counter disabled
                    if (parseInt(event.counter, 10) !== 1) {
                        $inner.find('.stec-layout-event-inner-intro-event-status').hide();
                        $inner.find('.stec-layout-event-inner-intro-counter').hide();
                        return;
                    }

                    nowDate = moment();
                    startDate = moment.tz(event.start_date, event.timezone).add(repeatOffset, 'seconds');
                    timeLeft = Math.floor((startDate.unix() - nowDate.unix()));

                    this.days = Math.floor(timeLeft / 86400);
                    this.hours = Math.floor(timeLeft % 86400 / 3600);
                    this.minutes = Math.floor(timeLeft % 86400 % 3600 / 60);
                    this.seconds = Math.floor(timeLeft % 86400 % 3600 % 60);

                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(0).text(this.days);
                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(1).text(this.hours);
                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(2).text(this.minutes);
                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(3).text(this.seconds);

                    this.daysLabel = $inner.find('.stec-layout-event-inner-intro-counter-label').eq(0);
                    this.hoursLabel = $inner.find('.stec-layout-event-inner-intro-counter-label').eq(1);
                    this.minutesLabel = $inner.find('.stec-layout-event-inner-intro-counter-label').eq(2);
                    this.secondsLabel = $inner.find('.stec-layout-event-inner-intro-counter-label').eq(3);

                    this.daysLabel.text(this.days === 1 ? this.daysLabel.attr('data-singular-label') : this.daysLabel.attr('data-plural-label'));
                    this.hoursLabel.text(this.hours === 1 ? this.hoursLabel.attr('data-singular-label') : this.hoursLabel.attr('data-plural-label'));
                    this.minutesLabel.text(this.minutes === 1 ? this.minutesLabel.attr('data-singular-label') : this.minutesLabel.attr('data-plural-label'));
                    this.secondsLabel.text(this.seconds === 1 ? this.secondsLabel.attr('data-singular-label') : this.secondsLabel.attr('data-plural-label'));

                    if (timeLeft < 0) {
                        this.complete();
                        return;
                    }

                    $inner.find('.stec-layout-event-inner-intro-event-status-text').removeClass('active');
                    $inner.find('.stec-layout-event-inner-intro-event-status-text.event-startsin').addClass('active');

                    this.count();
                },
                count: function () {

                    var parent = this;

                    if (parent.interval) {
                        clearInterval(parent.interval);
                    }

                    parent.interval = setInterval(function () {

                        parent.seconds--;

                        if (parent.seconds < 0) {
                            parent.seconds = 59;
                            parent.minutes--;
                            if (parent.minutes < 0) {
                                parent.minutes = 59;
                                parent.hours--;
                                if (parent.hours < 0) {
                                    parent.hours = 23;
                                    if (parent.days > 0) {
                                        parent.days--;
                                    }
                                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(0).text(parent.days);
                                    parent.daysLabel.text(parent.days === 1 ? parent.daysLabel.attr('data-singular-label') : parent.daysLabel.attr('data-plural-label'));
                                }
                                $inner.find('.stec-layout-event-inner-intro-counter-num').eq(1).text(parent.hours);
                                parent.hoursLabel.text(parent.hours === 1 ? parent.hoursLabel.attr('data-singular-label') : parent.hoursLabel.attr('data-plural-label'));
                            }
                            $inner.find('.stec-layout-event-inner-intro-counter-num').eq(2).text(parent.minutes);
                            parent.minutesLabel.text(parent.minutes === 1 ? parent.minutesLabel.attr('data-singular-label') : parent.minutesLabel.attr('data-plural-label'));
                        }
                        $inner.find('.stec-layout-event-inner-intro-counter-num').eq(3).text(parent.seconds);
                        parent.secondsLabel.text(parent.seconds === 1 ? parent.secondsLabel.attr('data-singular-label') : parent.secondsLabel.attr('data-plural-label'));


                        if (parent.days === 0 && parent.hours === 0 && parent.minutes === 0 && parent.seconds === 0) {
                            parent.complete();
                        }

                    }, 1000);
                },
                complete: function () {

                    var now, endDate;

                    clearInterval(this.interval);

                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(0).text(0);
                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(1).text(0);
                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(2).text(0);
                    $inner.find('.stec-layout-event-inner-intro-counter-num').eq(3).text(0);

                    this.daysLabel.text(this.days === 1 ? this.daysLabel.attr('data-singular-label') : this.daysLabel.attr('data-plural-label'));
                    this.hoursLabel.text(this.hours === 1 ? this.hoursLabel.attr('data-singular-label') : this.hoursLabel.attr('data-plural-label'));
                    this.minutesLabel.text(this.minutes === 1 ? this.minutesLabel.attr('data-singular-label') : this.minutesLabel.attr('data-plural-label'));
                    this.secondsLabel.text(this.seconds === 1 ? this.secondsLabel.attr('data-singular-label') : this.secondsLabel.attr('data-plural-label'));

                    now = moment();
                    endDate = moment.tz(event.end_date, event.timezone).add(repeatOffset, 'seconds');

                    if (1 === data.event.all_day) {
                        endDate = endDate.endOf('day');
                    }

                    if (now >= endDate) {
                        $inner.find('.stec-layout-event-inner-intro-event-status-text').removeClass('active');
                        $inner.find('.stec-layout-event-inner-intro-event-status-text.event-expired').addClass('active');
                    } else {

                        let timeToEventEnd = Math.floor((endDate.unix() - now.unix()));

                        this.days = Math.floor(timeToEventEnd / 86400);
                        this.hours = Math.floor(timeToEventEnd % 86400 / 3600);
                        this.minutes = Math.floor(timeToEventEnd % 86400 % 3600 / 60);
                        this.seconds = Math.floor(timeToEventEnd % 86400 % 3600 % 60);

                        $inner.find('.stec-layout-event-inner-intro-counter-num').eq(0).text(this.days);
                        $inner.find('.stec-layout-event-inner-intro-counter-num').eq(1).text(this.hours);
                        $inner.find('.stec-layout-event-inner-intro-counter-num').eq(2).text(this.minutes);
                        $inner.find('.stec-layout-event-inner-intro-counter-num').eq(3).text(this.seconds);

                        this.daysLabel.text(this.days === 1 ? this.daysLabel.attr('data-singular-label') : this.daysLabel.attr('data-plural-label'));
                        this.hoursLabel.text(this.hours === 1 ? this.hoursLabel.attr('data-singular-label') : this.hoursLabel.attr('data-plural-label'));
                        this.minutesLabel.text(this.minutes === 1 ? this.minutesLabel.attr('data-singular-label') : this.minutesLabel.attr('data-plural-label'));
                        this.secondsLabel.text(this.seconds === 1 ? this.secondsLabel.attr('data-singular-label') : this.secondsLabel.attr('data-plural-label'));

                        this.count();

                        $inner.find('.stec-layout-event-inner-intro-event-status-text').removeClass('active');
                        $inner.find('.stec-layout-event-inner-intro-event-status-text.event-inprogress').addClass('active');
                    }
                }

            };

            clock.init();


            // Attend / Decline
            function ajaxAttendance(status) {

                // status
                // 0 - no decision
                // 1 - accept 
                // 2 - decline

                var request = DB.newRequest();

                request.setData({
                    action: 'stec_public_ajax_action',
                    task: 'set_user_event_attendance',
                    event_id: event.id,
                    date: eventOccurrenceDate,
                    status: status
                });

                request.setBefore(function () {
                    $inner.find('.stec-layout-event-inner-intro-attendance').children().hide();

                    $('<li>' + glob.template.preloader + '</li>').addClass('intro-attendance')
                            .appendTo($inner.find('.stec-layout-event-inner-intro-attendance'));
                });

                request.setCallback(function (rtrn) {
                    var status = parseInt(rtrn.status, 10);
                    var $avatar, $attendee, updatedList;

                    $inner.find('.stec-layout-event-inner-intro-attendance li').removeClass('active');
                    $container.find('.stec-layout-event-inner-attendance-invited-buttons li').removeClass('active');

                    $avatar = $container.find('.stec-layout-event-inner-attendance-attendee-avatar')
                            .filter('[data-userid="' + glob.options.userid + '"]');

                    $attendee = $avatar.parent();

                    switch (status) {
                        case 1 :
                            $inner.find('.stec-layout-event-inner-intro-attendance-attend').addClass('active');
                            $container.find('.stec-layout-event-inner-attendance-invited-buttons-accept').addClass('active');
                            $avatar.find('li i').attr('class', 'fa fa-check');
                            break;
                        case 2 :
                            $inner.find('.stec-layout-event-inner-intro-attendance-decline').addClass('active');
                            $container.find('.stec-layout-event-inner-attendance-invited-buttons-decline').addClass('active');
                            $avatar.find('li i').attr('class', 'fa fa-times');
                            $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').hide();
                            break;

                        case 3:

                            updatedList = [];
                            $(event.attendance).each(function () {
                                if (this.userid === glob.options.userid) {
                                    return true; // continue;
                                }

                                updatedList.push(this);
                            });

                            event.attendance = updatedList;
                            data.glob.rsvp.settings.update = false;

                            data.helper.layout.set('layout');
                            return;

                            break;

                        default:
                            $avatar.find('li i').attr('class', 'fa fa-question');
                            $attendee.find('.stec-layout-event-inner-attendance-attendee-persons').hide();

                    }

                    $(event.attendance).each(function () {

                        if (this.userid === glob.options.userid) {
                            this.status[eventOccurrenceDate] = status;
                            return false; // break
                        }

                    });

                    $inner.find('.stec-layout-event-inner-intro-attendance').children().show();
                    $inner.find('.stec-layout-event-inner-intro-attendance').children().last().remove();

                    availableSlots = rtrn.slots_left;

                    $container.find('.stec-layout-event-inner-attendance-invited-slots-count').text(availableSlots);
                });

                DB.query(request);

            }

            $inner.find('.stec-layout-event-inner-intro-attendance-attend').on(helper.clickHandle(), function (e) {

                var rsvp = data.glob.rsvp;

                e.preventDefault();

                if ($(this).hasClass('active')) {
                    ajaxAttendance(0);
                } else {
                    rsvp.settings.id = data.event.id;
                    rsvp.settings.offset = data.repeatOffset;
                    rsvp.settings.event = data.event;
                    rsvp.settings.date = moment.tz(data.event.start_date, data.event.timezone).add(data.repeatOffset, 'seconds').format('YYYY-MM-DD');
                    rsvp.settings.update = true;
                    rsvp.popOpen();
                }

            });

            $inner.find('.stec-layout-event-inner-intro-attendance-decline').on(helper.clickHandle(), function (e) {
                var status;
                e.preventDefault();
                status = $(this).hasClass('active') ? 0 : 2;
                ajaxAttendance(status);
            });

            $inner.find('.stec-layout-event-inner-intro-attendance-unlist').on(helper.clickHandle(), function (e) {
                var status;
                e.preventDefault();
                status = 3;
                ajaxAttendance(status);
            });

            invitedUser = false;
            availableSlots = helper.attendanceHasFreeSlots(event, eventOccurrenceDate);

            if (!isNaN(glob.options.userid)) {

                // check if user is invited
                $(event.attendance).each(function () {
                    if (this.userid === glob.options.userid) {
                        invitedUser = true;
                        return false; // break
                    }
                });
            }

            if (invitedUser !== false) {

                status = 0;

                $(event.attendance).each(function () {
                    if (this.userid === glob.options.userid && this.status[eventOccurrenceDate]) {
                        status = parseInt(this.status[eventOccurrenceDate], 10);
                        return false; // break
                    }
                });

                if (availableSlots <= 0 && status !== 1) {
                    $inner.find('.stec-layout-event-inner-intro-attendance').hide();
                }

                switch (status) {
                    case 1:
                        $inner.find('.stec-layout-event-inner-intro-attendance-attend').addClass('active');
                        $inner.find('.stec-layout-event-inner-intro-attendance-decline').removeClass('active');
                        break;

                    case 2:
                        $inner.find('.stec-layout-event-inner-intro-attendance-attend').removeClass('active');
                        $inner.find('.stec-layout-event-inner-intro-attendance-decline').addClass('active');
                        break;
                }
            } else {
                $inner.find('.stec-layout-event-inner-intro-attendance').hide();
            }

            nowDate = moment();
            startDate = moment.tz(event.start_date, event.timezone).add(repeatOffset, 'seconds');

            // RSVP button
            if (true === invitedUser || parseInt(event.rsvp, 10) !== 1 || nowDate >= startDate) {
                $inner.find('.stec-layout-event-inner-intro-rsvp').hide();
            }

            // Booking button
            if (event.bookable_products.length <= 0 || nowDate >= startDate) {
                $inner.find('button[data-action="open-tickets"]').remove();
            } else {
                $inner.find('button[data-action="open-tickets"]').data({
                    event: event,
                    offset: data.repeatOffset
                });
            }

            // External link button
            if (event.link !== "") {
                $inner.find('.stec-layout-event-inner-intro-external-link').show();
            }

            // Remove RSVP and Link section
            if (!event.link && $inner.find('.stec-layout-event-inner-intro-rsvp').length <= 0) {
                $inner.find('.stec-layout-event-inner-intro-external-link').parent().remove();
            }

            // Check if event is in progress
            if (nowDate >= startDate) {
                $inner.find('.stec-layout-event-inner-intro-attendance').hide();
            }

            // Attachments
            if (event.attachments.length > 0) {
                attachmentsTemplate = $('#stec-layout-event-inner-intro-attachment-template').html();

                $(event.attachments).each(function () {

                    var th = this;

                    $(attachmentsTemplate).html(function (index, html) {
                        return html
                                .replace(/stec_replace_filename/g, th.filename)
                                .replace(/stec_replace_desc/g, th.description)
                                .replace(/\#stec_replace_url/g, th.link)
                                .replace(/stec_replace_size/g, th.size);

                    }).appendTo($inner.find('.stec-layout-event-inner-intro-attachments-list'));
                });

                $inner.find('.stec-layout-event-inner-intro-attachments-toggle').on(helper.clickHandle(), function (e) {

                    e.preventDefault();

                    $(this).toggleClass('active');

                    $inner.find('.stec-layout-event-inner-intro-attachments-list').toggleClass('active');

                });
            } else {
                $inner.find('.stec-layout-event-inner-intro-attachments').remove();
            }

            $inner.find('.stec-layout-event-inner-intro-exports-toggle').on(helper.clickHandle(), function (e) {

                e.preventDefault();

                $(this).toggleClass('active');

                $inner.find('.stec-layout-event-inner-intro-exports-options').toggleClass('active');

            });

            $inner.on(helper.clickHandle(), '.fa-link', function (e) {

                var eventId = data.$container.data('event-id');
                var eventOffset = data.$container.data('repeat-time-offset');
                var linkURL = $(this).parent().attr('href');
                var sharer = new window.stecSharer($instance, glob, helper, linkURL, eventId, eventOffset);

                e.stopPropagation();
                e.preventDefault();

                sharer.init();

            });

            $inner.find('.stec-layout-event-inner-intro-share-and-export').on(helper.clickHandle(), '.fa-envelope', function (e) {

                var eventId = $(this).parents('.stec-layout-event').attr('data-id');
                var eventOffset = $(this).parents('.stec-layout-event').attr('data-repeat-time-offset');
                var linkURL = $(this).parent().attr('href');
                var emailForm = new window.stecEmailEvent($instance, glob, helper, linkURL, eventId, eventOffset);

                e.stopPropagation();
                e.preventDefault();

                emailForm.init();

            });


        });

    })();

    // Location
    (function () {

        function physicalLocation(data) {

            var glob, helper, event, $container, $inner;

            if (!data.event.location.full_address) {
                return;
            }

            glob = data.glob;
            helper = data.helper;

            event = data.event;
            $container = data.$container;
            $inner = $container.find('.stec-layout-event-inner-location');
            $inner.find('.stec-layout-event-inner-location-virtual').remove();

            if ($inner.find('.stec-layout-event-inner-location-right-map').children().length > 0) {
                return;
            }

            $inner.html(function (index, html) {

                return html
                        .replace(/stec_replace_location/g, function () {
                            if (event.location.title) {
                                return [event.location.title, event.location.full_address].join(', ');
                            }

                            return event.location.full_address;
                        })
                        .replace(/stec_replace_details/g, event.location.details ? event.location.details : '');
            });

            if ($inner.find('.stec-layout-event-inner-location-details').text().trim() === "") {
                $inner.find('.stec-layout-event-inner-location-optional-details').hide();
            }

            function fillMyLocation($el) {

                if (glob.options.myLocation) {
                    $el.val(glob.options.myLocation);
                    return;
                }

                if (navigator.geolocation) {

                    if (navigator.geolocation) {

                        navigator.geolocation.getCurrentPosition(
                                function (position) {

                                    var pos = [position.coords.latitude, position.coords.longitude];
                                    glob.options.myLocation = pos.join(', ');
                                    $el.val(glob.options.myLocation);
                                },
                                function (a, b, c) {
                                    console.log('Navigator Geolocation Error');
                                }
                        );
                    }
                }
            }

            function gmap() {

                this.$tabCont = "";
                this.directionsService = "";
                this.directionsDisplay = "";
                this.map = "";
                this.geocoder = "";
                this.mapDiv = "";
                this.marker = "";
                this.address = "";

                this.initNoMap = function ($el) {
                    var parent = this;
                    var $start, $end;

                    parent.$tabCont = $el.parents(".stec-layout-event-inner-location");
                    parent.address = data.event.location.full_address;

                    $start = parent.$tabCont.find('input[name="start"]');
                    $end = parent.$tabCont.find('input[name="end"]');

                    if ($end.val().trim() === "") {
                        $end.val(parent.$tabCont.find(".stec-layout-event-inner-location-address").text());
                    }

                    if ($start.val().trim() === "") {
                        fillMyLocation($start);
                    }

                    parent.$tabCont.find('.stec-layout-event-inner-location-directions-buttons .stec-layout-event-inner-location-left-button').remove();

                    parent.bindControls();

                    // Remove tab preloaders
                    $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                    $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                    $inner.find('.stec-preloader').remove();
                };

                this.init = function ($el) {

                    var parent = this;
                    var $start, $end;

                    if (!window.google || !window.google.maps) {
                        this.initNoMap($el);
                        return;
                    }

                    parent.$tabCont = $el.parents(".stec-layout-event-inner-location");
                    parent.address = data.event.location.full_address;

                    parent.mapDiv = $el.get(0);
                    parent.map = new window.google.maps.Map(parent.mapDiv, {zoom: glob.options.gmap_zoom ? parseInt(glob.options.gmap_zoom, 10) : 15});
                    parent.geocoder = new window.google.maps.Geocoder();
                    parent.directionsService = new window.google.maps.DirectionsService();
                    parent.directionsDisplay = new window.google.maps.DirectionsRenderer();
                    parent.directionsDisplay.setMap(parent.map);
                    parent.setLocation(parent.address);

                    if (glob.options.gmap_theme && glob.options.gmap_theme !== 'standard') {
                        parent.map.mapTypes.set('Custom_Style', helper.getMapStyles(glob.options.gmap_theme));
                        parent.map.setMapTypeId('Custom_Style');
                    }

                    // start end directions
                    $start = parent.$tabCont.find('input[name="start"]');
                    $end = parent.$tabCont.find('input[name="end"]');

                    if ($end.val().trim() === "") {
                        $end.val(event.location.full_address);
                    }

                    if ($start.val().trim() === "") {
                        fillMyLocation($start);
                    }

                    this.bindControls();

                    // Remove tab preloaders
                    $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                    $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                    $inner.find('.stec-preloader').remove();

                };

                this.bindControls = function () {

                    var parent = this;

                    helper.onResizeEnd(function () {
                        parent.refresh();
                    }, 150, 'stec-unbind-window-resize-on-event-close-' + glob.options.id);

                    parent.$tabCont
                            .parents('.stec-layout-event-inner')
                            .find('[data-tab="stec-layout-event-inner-location"]')
                            .on(helper.clickHandle(), function (e) {

                                parent.refresh(false);
                            });


                    parent.$tabCont.find(".stec-layout-event-inner-location-open-map").on(helper.clickHandle(), function (e) {

                        var $tabCont = $(this).parents(".stec-layout-event-inner-location");
                        var $start = $tabCont.find('input[name="start"]');
                        var $end = $tabCont.find('input[name="end"]');
                        var eventLocation = $end.val();
                        var latlng, href;

                        if (event.location.coordinates) {
                            latlng = event.location.coordinates.split(',');

                            eventLocation = {
                                lat: parseFloat(latlng[0].trim()),
                                lng: parseFloat(latlng[1].trim())
                            };

                            href = 'https://www.google.com/maps/dir/?api=1&origin=' + $start.val() + '&destination=' + eventLocation.lat + ',' + eventLocation.lng + '&travelmode=driving';
                            $(this).attr('href', href);
                        } else {
                            e.preventDefault();
                        }

                    });
                    parent.$tabCont.find(".stec-layout-event-inner-location-left-button").on(helper.clickHandle(), function (e) {

                        var $tabCont = $(this).parents(".stec-layout-event-inner-location");
                        var $start = $tabCont.find('input[name="start"]');
                        var $end = $tabCont.find('input[name="end"]');
                        var eventLocation = $end.val();
                        var latlng;

                        if (event.location.coordinates) {
                            latlng = event.location.coordinates.split(',');

                            eventLocation = {
                                lat: parseFloat(latlng[0].trim()),
                                lng: parseFloat(latlng[1].trim())
                            };
                        }

                        if ($start.val() && eventLocation) {
                            e.preventDefault();
                            parent.getDirection($start.val(), eventLocation);
                        }
                    });

                };

                this.refresh = function (centerOnLocation) {

                    var parent = this;

                    setTimeout(function () {
                        window.google.maps.event.trigger(parent.mapDiv, 'resize');

                        if (centerOnLocation === true) {
                            parent.setLocation(parent.address);
                        }
                    }, 10); // timeout fixes resize bug

                };

                this.setLocation = function (address) {

                    var parent = this, latlng, pos;

                    if (event.location.coordinates) {

                        latlng = event.location.coordinates.split(',');
                        pos = {
                            lat: parseFloat(latlng[0].trim()),
                            lng: parseFloat(latlng[1].trim())
                        };

                        parent.map.setCenter(pos);
                        parent.marker = new window.google.maps.Marker({
                            map: parent.map,
                            position: pos,
                            title: address
                        });

                    } else {
                        parent.geocoder.geocode({'address': address}, function (results, status) {
                            if (status === window.google.maps.GeocoderStatus.OK) {
                                parent.map.setCenter(results[0].geometry.location);
                                parent.marker = new window.google.maps.Marker({
                                    map: parent.map,
                                    position: results[0].geometry.location,
                                    title: address
                                });

                            } else {
                                console.log("Geocoder error: " + status);
                            }
                        });
                    }


                    parent.refresh();
                };

                this.getDirection = function (a, b) {

                    var parent = this;

                    parent.directionsService.route({
                        origin: a,
                        destination: b ? b : parent.marker.position,
                        travelMode: window.google.maps.TravelMode.DRIVING
                    }, function (response, status) {

                        if (status === window.google.maps.DirectionsStatus.OK) {
                            parent.directionsDisplay.setDirections(response);
                        } else {
                            console.log("Direction Service Error");

                            $inner.find('.stec-layout-event-inner-location-direction-error').stop().fadeTo(250, 1, function () {

                                setTimeout(function () {

                                    $inner.find('.stec-layout-event-inner-location-direction-error').fadeTo(250, 0);

                                }, 3000);

                            });
                        }

                    });
                };
            }

            function osm() {

                this.$tabCont = "";
                this.map = "";
                this.mapDiv = "";
                this.address = "";

                this.initNoMap = function ($el) {
                    var parent = this;
                    var $start, $end;

                    parent.$tabCont = $el.parents(".stec-layout-event-inner-location");
                    parent.address = data.event.location.full_address;

                    $start = parent.$tabCont.find('input[name="start"]');
                    $end = parent.$tabCont.find('input[name="end"]');

                    if ($end.val().trim() === "") {
                        $end.val(parent.$tabCont.find(".stec-layout-event-inner-location-address").text());
                    }

                    if ($start.val().trim() === "") {
                        fillMyLocation($start);
                    }

                    parent.bindControls();

                    // Remove tab preloaders
                    $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                    $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                    $inner.find('.stec-preloader').remove();
                };

                this.init = function ($el) {

                    var parent = this, $start, $end;

                    parent.$tabCont = $el.parents(".stec-layout-event-inner-location");
                    parent.$tabCont.find('.stec-layout-event-inner-location-open-map').remove();

                    if (!window.L) {
                        this.initNoMap($el);
                        return;
                    }

                    parent.mapDiv = $el.get(0);

                    parent.map = helper.osm(parent.mapDiv);

                    parent.setLocation();

                    // start end directions
                    $start = parent.$tabCont.find('input[name="start"]');
                    $end = parent.$tabCont.find('input[name="end"]');

                    if ($end.val().trim() === "") {
                        $end.val(event.location.full_address);
                    }

                    if ($start.val().trim() === "") {
                        fillMyLocation($start);
                    }

                    this.bindControls();

                    // Remove tab preloaders
                    $inner.find('.stec-layout-event-inner-preload-wrap').children().first().unwrap();
                    $inner.find('.stec-layout-event-inner-preload-wrap').remove();
                    $inner.find('.stec-preloader').remove();

                };

                this.setLocation = function () {

                    var parent = this;

                    parent.address = event.location.full_address;

                    helper.promiseCoordinates(event).then(function (coordinates) {
                        let latlng = coordinates.split(',');
                        let eventLocation = {
                            lat: parseFloat(latlng[0].trim()),
                            lng: parseFloat(latlng[1].trim())
                        };
                        parent.map.setView(eventLocation, 15);
                        window.L.marker(eventLocation).addTo(parent.map);
                    }).catch(function () {
                        parent.map.setView({lat: 0, lng: 0}, 1);
                    });
                };

                this.bindControls = function () {

                    var parent = this;

                    parent.$tabCont.find(".stec-layout-event-inner-location-left-button").on(helper.clickHandle(), function (e) {

                        var $tabCont = $(this).parents(".stec-layout-event-inner-location");
                        var $start = $tabCont.find('input[name="start"]');
                        var promiseCoordinates;

                        e.preventDefault();

                        if (!event.location) {
                            return;
                        }

                        promiseCoordinates = new Promise(function (resolveCoordinates) {

                            var startLocation = $start.val();
                            var endLocation = event.location.coordinates;
                            var coords = {
                                start: startLocation,
                                end: endLocation
                            };

                            if (coords.start.match(/[aA-zZ]+/) !== null) {
                                helper.promiseGetAddressCoordinates(startLocation).then(function (startCoords) {
                                    coords.start = startCoords;
                                    return resolveCoordinates(coords);
                                });
                            } else {
                                return resolveCoordinates(coords);
                            }
                        });

                        promiseCoordinates.then(function (coords) {
                            window.open('//www.openstreetmap.org/directions?engine=fossgis_osrm_car&route=' + coords.start + ';' + coords.end, '_blank');
                        });
                    });
                };
            }

            function loadMap() {

                var $event, $mapCont;

                if ($inner.is(':visible')) {
                    if (glob.options.general_settings.open_event_in === 'modal') {
                        $event = $inner.parents('.stec-layout-event-inner');
                    } else {
                        $event = $inner.parents('.stec-layout-event');
                    }
                } else {
                    return;
                }

                $mapCont = $event.find(".stec-layout-event-inner-location-right-map");

                if ($mapCont.children().length <= 0) {

                    switch (glob.options.general_settings.map_type) {

                        case 'osm' :

                            new osm().init($mapCont);

                            break;

                        case 'gmap' :
                        default:

                            new gmap().init($mapCont);

                    }

                }
            }

            $(document).on('stec-tab-click-' + glob.options.id, function () {
                loadMap();
            });

            loadMap();
        }

        function virtualLocation(data) {

            var glob, helper, event, $container, $inner;

            if (!data.event.location.full_address) {
                return;
            }

            glob = data.glob;
            helper = data.helper;
            event = data.event;
            $container = data.$container;
            $inner = $container.find('.stec-layout-event-inner-location');

            if ($inner.data('stec-tab-loaded')) {
                return;
            }

            $inner.find('.stec-layout-event-inner-location-physical').remove();
            $inner.data('stec-tab-loaded', true);

            $inner.html(function (index, html) {

                return html
                        .replace(/\bstec_replace_location\b/g, function () {
                            if (event.location.title) {
                                return [event.location.title, event.location.full_address].join(', ');
                            }

                            return event.location.full_address;
                        })
                        .replace(/#stec_replace_virtual_location_href/g, event.location.address ? event.location.address : '')
                        .replace(/#stec_replace_virtual_event_qr_code_href/g, event.location.qrcode ? event.location.qrcode : '')
                        .replace(/stec_replace_details/g, event.location.details ? event.location.details : '');
            });

            if ($inner.find('.stec-layout-event-inner-location-details').text().trim() === "") {
                $inner.find('.stec-layout-event-inner-location-optional-details').hide();
            }

            $inner.find('[data-action="stec-copy-link"]').on(helper.clickHandle(), function (e) {
                var linkURL = $(this).attr('href');
                var $temp = $("<input>");
                e.preventDefault();
                $("body").append($temp);
                $temp.val(linkURL).select();
                document.execCommand("copy");
                $temp.remove();
                helper.toast(__('copiedToClipboard'), $inner);
            });

        }

        $(document).on('onEventToggleOpen', function (e, data) {

            switch (data.event.location.type) {
                case 'virtual' :

                    virtualLocation(data);

                    break;

                case 'physical':
                default:

                    physicalLocation(data);

            }


        });

    })();

    // Schedule
    (function () {

        $(document).on('onEventToggleOpen', function (e, data) {

            var $container, $inner, helper, template;

            if (data.event.schedule.length <= 0) {
                return;
            }

            helper = data.helper;
            $container = data.$container;
            $inner = $container.find('.stec-layout-event-inner-schedule');

            if ($inner.length <= 0 || true === $inner.data('loaded')) {
                return;
            }

            $inner.data('loaded', true);

            template = $('#stec-layout-event-inner-schedule-tab-template').html();

            $(data.event.schedule).each(function () {

                var th = this;

                var customKlass = [];

                if (th.details === '') {
                    customKlass.push('stec-layout-event-inner-schedule-tab-no-desc ');
                }

                if (th.icon === "" || th.icon === "fa") {
                    customKlass.push('stec-layout-event-inner-schedule-tab-no-icon ');
                }

                $(template)
                        .addClass(customKlass.join(' '))
                        .html(function (index, html) {

                            var start = helper.dbDateTimeToDate(helper.dbDateOffset(th.start_date, data.repeatOffset));

                            if (data.event.start_date_timestamp_tz) {
                                // event has been converted to localtime
                                // convert shedule as well
                                start = new moment(th.start_date)
                                        .add(data.repeatOffset, 'seconds')
                                        .utcOffset(data.event.timezone_utc_offset / 60, true).utcOffset(moment().utcOffset());

                            }

                            return html
                                    .replace(/stec_replace_date/g, helper.beautifyScheduleTimespan(start))
                                    .replace(/stec_replace_time/g, '')
                                    .replace(/stec_replace_icon/g, th.icon)
                                    .replace(/stec_replace_color/g, 'style="color:' + th.icon_color + '"')
                                    .replace(/stec_replace_title/g, th.title)
                                    .replace(/stec_replace_desc/g, th.details);

                        })
                        .appendTo($inner);

            });

            $inner.find('.stec-layout-event-inner-schedule-isempty').remove();
            $inner.find('.stec-layout-event-inner-schedule-tab').first().addClass('open');

            $inner.find('.stec-layout-event-inner-schedule-tab-preview')
                    .off(helper.clickHandle('open'))
                    .on(helper.clickHandle('open'), function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        $(this).parents(".stec-layout-event-inner-schedule-tab").not('.stec-layout-event-inner-schedule-tab-no-desc').toggleClass("open");
                    });
        });


    })();

    // Tooltip
    (function () {

        $.stecExtend(function (master) {

            var helper = master.helper;
            var glob = master.glob;
            var instance = master.instance;
            var $instance = master.$instance;
            var calData = master.calData;
            var event;
            var repeatTimeOffset = 0;

            var tooltip = {

                init: function () {

                    if (parseInt(glob.options.general_settings.tooltip_display, 10) !== 1) {
                        return;
                    }

                    if (helper.isMobile() === true) {
                        // don't bind on mobile
                        return;
                    }

                    this.iconImageSlider();

                    this.bindControls();

                },

                bindControls: function () {

                    var parent = this;

                    var eventClass = [];
                    eventClass.push(instance + ' .stec-layout-month-daycell-event');
                    eventClass.push(instance + ' .stec-layout-week-daycell-event');

                    if (parseInt(master.glob.options.agenda_tooltip, 10) === 1) {
                        eventClass.push(instance + ' .stec-layout-agenda-daycell-event');
                    }

                    eventClass = eventClass.join(',');

                    $(document).on('mouseenter', eventClass, function (e) {

                        var th = this;

                        $('#tooltip-' + glob.options.id).remove();

                        if (helper.animate !== false) {
                            helper.animate.tooltip.clear();
                        }

                        $(glob.template.tooltip).attr('id', 'tooltip-' + glob.options.id).appendTo('body');

                        event = calData.getEventById($(th).data('id'));
                        repeatTimeOffset = $(th).data('repeatTimeOffset');

                        parent.fillHTML(event);

                        parent.position(th);

                        if (helper.animate !== false) {

                            helper.animate.tooltip.show($('#tooltip-' + glob.options.id));

                        } else {

                            $('#tooltip-' + glob.options.id).fadeTo(0, 1);

                        }

                    });

                    $(document).on('mouseleave', instance + eventClass, function (e) {

                        if (helper.animate !== false) {
                            helper.animate.tooltip.hide($('#tooltip-' + glob.options.id), function () {

                                $('#tooltip-' + glob.options.id).remove();

                            });

                        } else {

                            $('#tooltip-' + glob.options.id).fadeTo(0, 0);

                        }

                        parent.clock.clear();
                    });

                },

                iconImageSlider: function () {

                    setInterval(function () {

                        $('#tooltip-' + glob.options.id).find('.stec-layout-event-preview-left-icon-images').filter(':visible').each(function () {

                            var currentImage, nextImage;

                            if ($(this).children().length <= 1) {
                                return;
                            }

                            currentImage = $(this).children().filter('.active');
                            nextImage = currentImage.next();

                            if (nextImage.length === 0) {
                                nextImage = $(this).children().first();
                            }

                            nextImage.addClass('transition');

                            setTimeout(function () {
                                currentImage.removeClass('active');
                                nextImage.removeClass('transition').addClass('active');

                            }, 1000);

                        });

                    }, 6000);
                },

                position: function (th) {

                    var klass = '';

                    var $tooltip = $('#tooltip-' + glob.options.id);

                    $tooltip.css({
                        left: function () {

                            var x = 0;

                            if ($(th).hasClass('stec-layout-agenda-daycell-event')) {
                                x = $(th).width() + $(th).offset().left + 15;
                            } else if ($(th).parents('td').first().index() > 3) {
                                x = $(th).offset().left - $tooltip.width() - 5;
                                klass += ' stec-tooltip-pos-left';
                            } else {
                                x = $(th).width() + $(th).offset().left + 5;
                            }

                            return x;
                        },
                        top: function () {

                            var y = 0;

                            if ($(th).hasClass('stec-layout-agenda-daycell-event')) {

                                y = $(th).offset().top - 23;

                            } else if ($(th).parents('tr').first().index() > 3) {

                                klass += ' stec-tooltip-pos-top';

                                y = $instance.hasClass('stec-media-small') ? $(th).offset().top - $tooltip.height() + 29 : $(th).offset().top - $tooltip.height() + 40;

                            } else {

                                y = $instance.hasClass('stec-media-small') ? $(th).offset().top - 27 : $(th).offset().top - 15;

                            }


                            return y;
                        }
                    });

                    $tooltip.addClass(klass);
                },

                fillHTML: function (event) {

                    var $tooltip = $('#tooltip-' + glob.options.id);
                    var icon = '';
                    var d1, iconHtml, startDate, endDate, date, imageHtml;
                    var eventState = helper.getEventState(event, repeatTimeOffset);
                    var eventStatusHtml;

                    switch (event.icon_type) {

                        case 'image' :

                            if (event.icon_images.length > 0) {

                                icon += '<div class="stec-layout-event-preview-left-icon-images">';

                                $.each(event.icon_images_meta, function (i) {
                                    icon += '<div class="stec-layout-event-preview-left-icon-image ' + (i === 0 ? 'active' : '') + '" style="background-image: url(' + this.src + ')"></div>';
                                });


                                icon += '</div>';

                            } else {
                                icon = '<i class="' + event.icon + '"></i>';
                            }

                            break;

                        case 'date' :

                            d1 = moment.tz(event.start_date, event.timezone).add(repeatTimeOffset, 'seconds');

                            icon = '<span class="stec-layout-event-preview-left-icon-date"><span class="stec-layout-event-preview-left-icon-date-number">' + d1.format('D') + '</span><span class="stec-layout-event-preview-left-icon-date-month">' + d1.format('MMM') + '</span></span>';

                            break;

                        default:
                            icon = '<i class="' + event.icon + '"></i>';

                    }

                    switch (event.event_status) {

                        case 'EventCancelled' :
                            eventStatusHtml = '<p class="stec-tooltip-event-status"><span class="cancelled">' + __('Cancelled') + '</span></p>';
                            break;

                        case 'EventMovedOnline' :
                            eventStatusHtml = '<p class="stec-tooltip-event-status"><span class="moved-online">' + __('Moved Online') + '</span></p>';
                            break;

                        case 'EventPostponed' :
                            eventStatusHtml = '<p class="stec-tooltip-event-status"><span class="postponed">' + __('Postponed') + '</span></p>';
                            break;

                        case 'EventRescheduled' :
                            eventStatusHtml = '<p class="stec-tooltip-event-status"><span class="rescheduled">' + __('Rescheduled') + '</span></p>';
                            break;

                        default:
                            eventStatusHtml = '';
                    }

                    iconHtml = '<div class="stec-tooltip-icon" style="background:' + event.color + ' ">' + icon + '</div>';

                    // original date + the repeat offset
                    startDate = moment.tz(event.start_date, event.timezone).add(repeatTimeOffset, 'seconds');

                    // original date + the repeat offset
                    endDate = moment.tz(event.end_date, event.timezone).add(repeatTimeOffset, 'seconds');

                    date = helper.beautifyTimespan(startDate, parseInt(event.hide_end, 10) === 1 ? false : endDate, event.all_day);

                    imageHtml = '';

                    if (event.images_meta && event.images_meta.length > 0) {
                        imageHtml = '<div style="background-image: url(' + event.images_meta[0].src + ');"  ></div>';
                    }

                    $tooltip.html(function (index, html) {

                        return html
                                .replace(/stec_replace_event_status/g, eventStatusHtml)
                                .replace(/stec_replace_image/g, imageHtml)
                                .replace(/stec_replace_summary/g, event.title)
                                .replace(/stec_replace_desc_short/g, event.description_short)
                                .replace(/stec_replace_icon/g, iconHtml)
                                .replace(/stec_replace_location/g, function () {

                                    if ('virtual' === event.location.type) {
                                        return __('Online Event');
                                    }

                                    if (event.location.title) {
                                        return event.location.title;
                                    }

                                    return event.location.full_address;
                                })
                                .replace(/stec_replace_timespan/g, date);

                    });

                    if (!event.location.full_address) {
                        $tooltip.find('.stec-tooltip-location').hide();
                    }

                    if (imageHtml === '') {
                        $tooltip.find('.stec-tooltip-image').hide();
                        $tooltip.find('.stec-tooltip-icon').css({
                            top: 0,
                            position: 'static',
                            marginTop: 20
                        });
                    }

                    if (-1 === $.inArray(['EventCancelled', 'EventPostponed'], event.event_status)) {

                        if (eventState === 2) {
                            $tooltip.find('.stec-tooltip-expired').css('display', 'inline-block');
                        }

                        if (eventState === 1) {
                            $tooltip.find('.stec-tooltip-progress').css('display', 'inline-block');
                        }

                        // Check counter
                        if (parseInt(event.counter, 10) !== 0 && eventState === 0) {
                            $tooltip.find('.stec-tooltip-counter').css('display', 'inline-block');
                            this.clock.init(startDate);
                        }
                    }

                    helper.extendBind("stachethemes_ec_extend", "filterTooltipHtml", {
                        event: event,
                        repeat_offset: repeatTimeOffset,
                        tooltip: $tooltip
                    });

                },

                clock: {
                    days: 0,
                    hours: 0,
                    minutes: 0,
                    seconds: 0,
                    daysLabel: __('DaysAbbr'),
                    hoursLabel: __('HoursAbbr'),
                    minutesLabel: __('MinutesAbbr'),
                    secondsLabel: __('SecondsAbbr'),
                    interval: '',
                    init: function (momentDate) {

                        var nowDate, timeLeft;

                        this.clear();

                        nowDate = moment();
                        timeLeft = Math.floor((momentDate.unix() - nowDate.unix()));
                        this.days = Math.floor(timeLeft / 86400);
                        this.hours = Math.floor(timeLeft % 86400 / 3600);
                        this.minutes = Math.floor(timeLeft % 86400 % 3600 / 60);
                        this.seconds = Math.floor(timeLeft % 86400 % 3600 % 60);

                        if (timeLeft < 0) {
                            this.complete();
                            return;
                        }

                        this.count();
                    },
                    setTimer: function () {

                        var $tooltip = $('#tooltip-' + glob.options.id);
                        var countText = this.days + this.daysLabel + ' ' + this.hours + this.hoursLabel + ' ' + this.minutes + this.minutesLabel + ' ' + this.seconds + this.secondsLabel;
                        $tooltip.find('.stec-tooltip-counter span').text(countText);

                    },
                    count: function () {

                        var parent = this;

                        parent.setTimer();

                        parent.interval = setInterval(function () {

                            parent.seconds--;

                            if (parent.seconds < 0) {
                                parent.seconds = 59;
                                parent.minutes--;
                                if (parent.minutes < 0) {
                                    parent.minutes = 59;
                                    parent.hours--;
                                    if (parent.hours < 0) {
                                        parent.hours = 23;
                                        if (parent.days > 0) {
                                            parent.days--;
                                        }
                                    }
                                }
                            }

                            parent.setTimer();

                            if (parent.days === 0 && parent.hours === 0 && parent.minutes === 0 && parent.seconds === 0) {
                                parent.clear();
                                parent.complete();
                            }

                        }, 1000);
                    },

                    complete: function () {
                        this.clear();
                    },

                    clear: function () {

                        clearInterval(this.interval);
                    }

                }
            };

            tooltip.init();

        });

    })();

    // Top calfilter
    (function () {

        function uniqueSort(array) {
            return $.grep(array, function (el, index) {
                return index === $.inArray(el, array);
            });
        }

        function locationsToHtml(locationsPool) {

            var html = [];
            var arr = [];
            var country, city, address;

            arr.other = [];
            arr.other.other = [];

            // Countries
            $.each(locationsPool, function () {
                if (!this.country) {
                    return true;// continue;
                }

                if (false === this.country in arr) {
                    arr[this.country] = [];
                    arr[this.country].other = [];
                }
            });

            // Cities
            $.each(locationsPool, function () {
                if (!this.city) {
                    return true;// continue;
                }

                if (!(arr[this.country ? this.country : 'other'][this.city])) {
                    arr[this.country ? this.country : 'other'][this.city] = [];
                }

            });

            // Address
            $.each(locationsPool, function () {

                var displayName;

                if ('virtual' === this.type) {
                    return true;
                }

                displayName = this.title ? this.title : this.address;

                if (!this.address) {
                    return true;// continue;
                }

                if (!(arr[this.country ? this.country : 'other'][this.city ? this.city : 'other'])) {
                    arr[this.country ? this.country : 'other'][this.city ? this.city : 'other'] = [];
                }

                arr[this.country ? this.country : 'other'][this.city ? this.city : 'other'][displayName] = this.full_address;

            });

            for (country in arr) {

                if ('other' === country) {

                    for (city in arr[country]) {
                        if (arr[country].hasOwnProperty(city)) {
                            if ('other' === city) {
                                for (address in arr[country][city]) {
                                    if (arr[country][city].hasOwnProperty(address)) {
                                        html.push('<li class="active" data-location="' + arr[country][city][address] + '"><span><i class="fas fa-check-square"></i>' + address + '</span></li>');
                                    }
                                }
                            } else {
                                html.push('<li class="active" data-location="' + city + '"><span><i class="fas fa-check-square"></i>' + city + '</span>');
                                html.push('<ul>');

                                for (address in arr[country][city]) {
                                    if (arr[country][city].hasOwnProperty(address)) {
                                        html.push('<li class="active" data-location="' + arr[country][city][address] + '"><span><i class="fas fa-check-square"></i>' + address + '</span></li>');
                                    }
                                }

                                html.push('</ul>');
                                html.push('</li>');
                            }
                        }
                    }
                } else {

                    html.push('<li class="active" data-location="' + country + '"><span><i class="fas fa-check-square"></i>' + country + '</span>');
                    html.push('<ul>');

                    for (city in arr[country]) {

                        if ('other' === city) {
                            for (address in arr[country][city]) {
                                if (arr[country][city].hasOwnProperty(address)) {
                                    html.push('<li class="active" data-location="' + arr[country][city][address] + '"><span><i class="fas fa-check-square"></i>' + address + '</span></li>');
                                }
                            }
                        } else {
                            html.push('<li class="active" data-location="' + city + '"><span><i class="fas fa-check-square"></i>' + city + '</span>');
                            html.push('<ul>');
                            for (address in arr[country][city]) {
                                if (arr[country][city].hasOwnProperty(address)) {
                                    html.push('<li class="active" data-location="' + arr[country][city][address] + '"><span><i class="fas fa-check-square"></i>' + address + '</span></li>');
                                }
                            }

                            html.push('</ul>');
                            html.push('</li>');
                        }
                    }

                    html.push('</ul>');
                    html.push('</li>');
                }
            }

            return html.join('');
        }

        $.stecExtend(function (master, events) {

            var filtered = [];

            $(master.calData.filterGetEvents).each(function () {

                var featured = parseInt(this.featured, 10);
                var inCategory;
                var inOrganizer;
                var locationLookFor;
                var locationFound;
                var key;

                if (parseInt(master.glob.options.featured_only, 10) === 1 && featured <= 0) {
                    return true;
                }

                switch (master.calData.featuredFilter) {
                    case 1:

                        if (featured <= 0) {
                            return true;
                        }

                        break;
                    case - 1:
                        if (featured > 0) {
                            return true;
                        }
                        break;
                }

                if ($.inArray(parseInt(this.calid, 10), master.calData.calendarFilter) === -1) {
                    return true;
                }

                if (master.calData.categoriesPool.length > 0) {

                    inCategory = false;

                    if (this.category.length > 0) {
                        $.each(this.category, function () {
                            if ($.inArray(parseInt(this.id, 10), master.calData.categoriesFilter) !== -1) {
                                inCategory = true;
                                return false; // break;
                            }
                        });
                    } else {
                        if ($.inArray(parseInt(0, 10), master.calData.categoriesFilter) !== -1) {
                            inCategory = true;
                        } else {
                            inCategory = false;
                        }
                    }

                    if (false === inCategory) {
                        return true;
                    }
                }

                if (master.calData.organizersPool.length > 0) {

                    inOrganizer = false;

                    if (this.organizers.length > 0) {
                        $.each(this.organizers, function () {
                            if ($.inArray(this.id, master.calData.organizersFilter) !== -1) {
                                inOrganizer = true;
                                return false; // break;
                            }
                        });
                    } else {
                        if ($.inArray('none', master.calData.organizersFilter) !== -1) {
                            inOrganizer = true;
                        }
                    }

                    if (false === inOrganizer) {
                        return true;
                    }

                }

                if (master.glob.options.filter_location) {

                    locationLookFor = master.glob.options.filter_location.toLowerCase();
                    locationFound = false;

                    for (key in this.location) {
                        if (this.location.hasOwnProperty(key)) {
                            if (key === 'id') {
                                continue;
                            }
                            if (this.location[key].toString().toLowerCase() === locationLookFor) {
                                locationFound = true;
                                break;
                            }
                        }
                    }

                    if (false === locationFound) {
                        return true;
                    }

                } else if ('virtual' === this.location.type && $.inArray('stec-virtual', master.calData.locationsFilter) !== -1) {
                    // Accept virtual location if checked
                } else if (this.location.full_address) {

                    if ($.inArray(this.location.full_address, master.calData.locationsFilter) === -1 &&
                            $.inArray(this.location.country, master.calData.locationsFilter) === -1 &&
                            $.inArray(this.location.city, master.calData.locationsFilter) === -1) {

                        return true;
                    }
                } else {
                    if ($.inArray('none', master.calData.locationsFilter) === -1) {
                        return true;
                    }
                }

                filtered.push(this);

            });

            master.calData.filterGetEvents = filtered;

        }, 'beforeProccessGetEvents');

        $.stecExtend(function (master, eventsPool) {

            var calendars = [];
            var categories = [];
            var locations = [];
            var organizers = [];
            var useCatColor = parseInt(master.glob.options.general_settings.use_cat_color, 10);
            var html = [];

            /**
             * @todo better implementation of sort by unique key value ?
             */

            function insertSearchInput(appElement) {
                $('<div class="stec-top-menu-quick-search-wrap"><input class="stec-top-menu-quick-search" type="text" placeholder="' + __('Search') + '" /></div>')
                        .insertBefore(appElement.find('.stec-scrollable'));
            }

            // If no events hide filter
            if (eventsPool.length <= 0) {
                master.$instance.$top.find('.stec-top-menu-filter').hide();
                return;
            }

            $(eventsPool).each(function () {
                var event = this;
                var calendarPushed = false;
                var locationPushed = false;

                $(calendars).each(function () {
                    if (this.id === event.calid) {
                        calendarPushed = true;
                    }
                });

                if (calendarPushed === false) {

                    calendars.push({
                        id: event.calid,
                        title: event.calendar.title,
                        color: event.calendar.color
                    });
                }

                if (event.location.full_address) {
                    $(locations).each(function () {
                        if (this.id === event.location.id) {
                            locationPushed = true;
                        }
                    });

                    if (locationPushed === false) {
                        locations.push(event.location);
                    }
                }

                if (event.category.length > 0) {
                    if (1 === useCatColor) {
                        event.color = event.category[0].color;
                    }

                    $.each(event.category, function () {

                        var eventCategory = this;
                        var categoryPushed = false;

                        $.each(categories, function () {
                            if (this.id === eventCategory.id) {
                                categoryPushed = true;
                            }
                        });

                        if (categoryPushed === false) {
                            categories.push({
                                id: this.id,
                                title: this.title,
                                color: this.color
                            });
                        }
                    });
                }

                if (event.organizers.length > 0) {
                    $.each(event.organizers, function () {

                        var eventOrganizer = this;
                        var organizerPushed = false;

                        $.each(organizers, function () {
                            if (this.id === eventOrganizer.id) {
                                organizerPushed = true;
                            }
                        });

                        if (organizerPushed === false) {
                            organizers.push({
                                id: this.id,
                                name: this.name
                            });
                        }
                    });
                }

            });

            // Sort locations alphabetically
            locations.sort(function (a, b) {
                return a.full_address.localeCompare(b.full_address, undefined, {numeric: true, sensitivity: 'base'});
            });

            // Sort categories alphabetically
            categories.sort(function (a, b) {
                return a.title.localeCompare(b.title, undefined, {numeric: true, sensitivity: 'base'});
            });

            master.calData.calendarsPool = calendars;
            master.calData.organizersPool = organizers;
            master.calData.categoriesPool = categories;
            master.calData.locationsPool = master.glob.options.filter_location ? [] : locations;

            // build HTML for calendar filter list
            if (master.calData.calendarsPool.length > 1) {
                html.push('<li data-exclude-qsf="1" class="stec-select-all active"><span><i class="fas fa-check-double"></i>' + __('select_all') + '</span></li>');
            }

            $(master.calData.calendarsPool).each(function () {

                // add to filter by default
                master.calData.calendarFilter.push(parseInt(this.id, 10));
                master.calData.calendarFilter = uniqueSort(master.calData.calendarFilter);

                html.push('<li class="active" data-calid="' + this.id + '"><span><i class="fas fa-check-square"></i>' + this.title + '</span></li>');
            });

            master.$instance.$top.find('.stec-top-menu-filter-by-calendar').find('ul').children('li').remove();
            $(html.join('')).appendTo(master.$instance.$top.find('.stec-top-menu-filter-by-calendar').find('ul'));

            html = [];

            if (master.calData.categoriesPool.length > 1) {
                html.push('<li data-exclude-qsf="1" class="stec-select-all active"><span><i class="fas fa-check-double"></i>' + __('select_all') + '</span></li>');
            }
            html.push('<li class="active" data-catid="0"><span><i class="fas fa-check-square"></i>' + __('uncategorized') + '</span></li>');

            $(master.calData.categoriesPool).each(function () {

                // add to filter by default
                master.calData.categoriesFilter.push(parseInt(this.id, 10));
                master.calData.categoriesFilter = uniqueSort(master.calData.categoriesFilter);

                html.push('<li class="active" data-catid="' + this.id + '"><span><i class="fas fa-check-square"></i>' + this.title + '</span></li>');
            });

            master.$instance.$top.find('.stec-top-menu-filter-by-category').find('ul').children('li').remove();
            $(html.join('')).appendTo(master.$instance.$top.find('.stec-top-menu-filter-by-category').find('ul'));

            html = [];

            if (master.calData.organizersPool.length > 1) {
                html.push('<li data-exclude-qsf="1" class="stec-select-all active"><span><i class="fas fa-check-double"></i>' + __('select_all') + '</span></li>');
            }

            html.push('<li class="active" data-organizer="none"><span><i class="fas fa-check-square"></i>' + __('noorganizer') + '</span></li>');

            $(master.calData.organizersPool).each(function () {

                // add to filter by default
                master.calData.organizersFilter.push(this.id);
                master.calData.organizersFilter = uniqueSort(master.calData.organizersFilter);

                html.push('<li class="active" data-organizer="' + this.id + '"><span><i class="fas fa-check-square"></i>' + this.name + '</span></li>');
            });

            master.$instance.$top.find('.stec-top-menu-filter-by-organizer').find('ul').children('li').remove();
            $(html.join('')).appendTo(master.$instance.$top.find('.stec-top-menu-filter-by-organizer').find('ul'));

            html = [];

            if (master.calData.locationsPool.length > 1) {
                html.push('<li data-exclude-qsf="1" class="stec-select-all active"><span><i class="fas fa-check-double"></i>' + __('select_all') + '</span></li>');
            }

            html.push('<li class="active" data-location="none"><span><i class="fas fa-check-square"></i>' + __('nolocation') + '</span></li>');
            html.push('<li class="active" data-location="stec-virtual"><span><i class="fas fa-check-square"></i>' + __('Online Events') + '</span></li>');

            $(master.calData.locationsPool).each(function () {

                if ('virtual' === this.type) {
                    return true;
                }

                // add to filter by default
                if (this.country) {
                    master.calData.locationsFilter.push(this.country);
                }
                if (this.city) {
                    master.calData.locationsFilter.push(this.city);
                }
                if (this.full_address && this.full_address !== [this.city, this.country].join(', ')) {
                    master.calData.locationsFilter.push(this.full_address);
                }
                master.calData.locationsFilter = uniqueSort(master.calData.locationsFilter);
            });

            html.push(locationsToHtml(master.calData.locationsPool));

            master.$instance.$top.find('.stec-top-menu-filter-by-location').find('ul').children('li').remove();
            $(html.join('')).appendTo(master.$instance.$top.find('.stec-top-menu-filter-by-location').find('ul'));

            // Hide category filter menu if no categories are found
            if (master.calData.categoriesPool.length === 0) {
                master.$instance.$top.find('.stec-top-menu-filter-by-category').hide();
            }

            // Hide location filter menu if no locations are found
            if (master.calData.locationsPool.length === 0) {
                master.$instance.$top.find('.stec-top-menu-filter-by-location').hide();
            }

            // Hide calendar filter menu if only 1 calendars is found
            if (master.calData.calendarsPool.length <= 1) {
                master.$instance.$top.find('.stec-top-menu-filter-by-calendar').hide();
            }

            // Hide organizers filter menu if no organizers
            if (master.calData.organizersPool.length <= 0) {
                master.$instance.$top.find('.stec-top-menu-filter-by-organizer').hide();
            }

            if (master.calData.categoriesPool.length > 8) {
                insertSearchInput(master.$instance.$top.find('.stec-top-menu-filter-by-category'));
            }

            if (master.calData.organizersPool.length > 8) {
                insertSearchInput(master.$instance.$top.find('.stec-top-menu-filter-by-organizer'));
            }

            if (master.calData.locationsPool.length > 8) {
                insertSearchInput(master.$instance.$top.find('.stec-top-menu-filter-by-location'));
            }

            if (master.calData.calendarsPool.length > 8) {
                insertSearchInput(master.$instance.$top.find('.stec-top-menu-filter-by-calendar'));
            }


        }, 'onAddToEventsPool');

        $.stecExtend(function (master) {

            var $instance = master.$instance;
            var helper = master.helper;
            var quickSearchTimeout = null;

            function resetMenus(exclude) {

                var filterBys;

                $instance.find('.stec-top-menu-filter-dropdown').children('.stec-style-flexbox').children().not(exclude).removeClass('active');

                filterBys = $instance.find('.stec-top-menu-filter-dropdown .stec-top-menu-filter-by')
                        .not(exclude);

                filterBys
                        .find(' > div > i:last-child')
                        .removeClass('fa-chevron-down')
                        .addClass('fa-chevron-right');

            }

            // Hide filter on search toggle
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-search", function (e) {

                $instance.find('.stec-top-menu-filter').removeClass('active');
                $instance.find('.stec-top-menu-filter-dropdown:not(.static)').removeClass('active');

                resetMenus();
            });

            // Calendar filter button toggle show/hide
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter", function (e) {
                e.preventDefault();
                e.stopPropagation();

                $instance.find('.stec-top-menu-filter').toggleClass('active');
                $instance.find('.stec-top-menu-filter-dropdown').toggleClass('active');

                // Reset dropdowns to closed
                if (!$instance.find('.stec-top-menu-filter').hasClass('active')) {
                    resetMenus();
                }

                // Hide search on filter toggle
                $instance.find('.stec-top-menu-search').removeClass('active');

                // Hide mobile view menu
                if (helper.isMobile()) {
                    $instance.$top.find('.stec-top-dropmenu-layouts').children('li').removeClass('mobile-hover');
                }

            });

            // Calendar filter button toggle show/hide
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-dropdown .stec-top-menu-filter-by", function (e) {

                var scr;

                e.preventDefault();
                e.stopPropagation();

                $(this).toggleClass('active');

                $(this).children('div').children('i').last().toggleClass('fa-chevron-right fa-chevron-down');

                resetMenus(this);

                if (!$(this).find('.stec-scrollable').data('stec-scroll')) {
                    scr = new window.StecScroll();
                    scr.create($(this).find('.stec-scrollable'));
                    $(this).find('.stec-scrollable').data('stec-scroll', scr);
                }

            });

            // Toggle calendars 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-by-calendar ul li:not('.stec-select-all')", function (e) {

                var index;
                var allActive = true;

                e.preventDefault();
                e.stopPropagation();

                $(this).toggleClass('active');
                $(this).find('i').toggleClass('fas fa-check-square far fa-square');

                if ($(this).hasClass('active')) {

                    // add to filter
                    master.calData.calendarFilter.push(parseInt($(this).attr('data-calid'), 10));

                } else {

                    // remove from filter

                    if (master.calData.calendarFilter !== false) {
                        index = master.calData.calendarFilter.indexOf(parseInt($(this).attr('data-calid'), 10));
                        if (index > -1) {
                            master.calData.calendarFilter.splice(index, 1);
                        }
                    }
                }

                master.calData.calendarFilter = uniqueSort(master.calData.calendarFilter);

                $instance.$top.find(".stec-top-menu-filter-by-calendar ul li:not('.stec-select-all')").each(function () {
                    if (!$(this).hasClass('active')) {
                        allActive = false;
                        return false; // break;
                    }
                });

                if (allActive) {
                    $instance.$top.find(".stec-top-menu-filter-by-calendar ul li.stec-select-all").addClass('active');
                } else {
                    $instance.$top.find(".stec-top-menu-filter-by-calendar ul li.stec-select-all").removeClass('active');
                }

                master.layout.set();

            });

            // Toggle all calendars 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-by-calendar ul li.stec-select-all", function (e) {

                var index;

                e.preventDefault();
                e.stopPropagation();

                $(this).toggleClass('active');

                if ($(this).hasClass('active')) {

                    $instance.$top.find(".stec-top-menu-filter-by-calendar ul li:not('.stec-select-all')").filter(':visible').each(function () {
                        // add to filter
                        $(this).addClass('active');
                        $(this).find('i').removeClass('far fa-square');
                        $(this).find('i').addClass('fas fa-check-square');
                        master.calData.calendarFilter.push(parseInt($(this).attr('data-calid'), 10));
                    });

                } else {

                    // remove from filter
                    $instance.$top.find(".stec-top-menu-filter-by-calendar ul li:not('.stec-select-all')").filter(':visible').each(function () {

                        $(this).removeClass('active');
                        $(this).find('i').removeClass('fas fa-check-square');
                        $(this).find('i').addClass('far fa-square');

                        if (master.calData.calendarFilter !== false) {
                            index = master.calData.calendarFilter.indexOf(parseInt($(this).attr('data-calid'), 10));
                            if (index > -1) {
                                master.calData.calendarFilter.splice(index, 1);
                            }
                        }
                    });
                }

                master.calData.calendarFilter = uniqueSort(master.calData.calendarFilter);

                master.layout.set();

            });

            // Toggle categories 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-by-category ul li:not('.stec-select-all')", function (e) {

                var index, allActive = true;

                e.preventDefault();
                e.stopPropagation();

                $(this).toggleClass('active');

                $(this).find('i').toggleClass('fas fa-check-square far fa-square');

                if ($(this).hasClass('active')) {

                    // add to filter
                    master.calData.categoriesFilter.push(parseInt($(this).attr('data-catid'), 10));

                } else {

                    // remove from filter
                    if (master.calData.categoriesFilter !== false) {
                        index = master.calData.categoriesFilter.indexOf(parseInt($(this).attr('data-catid'), 10));
                        if (index > -1) {
                            master.calData.categoriesFilter.splice(index, 1);
                        }
                    }
                }

                master.calData.categoriesFilter = uniqueSort(master.calData.categoriesFilter);

                allActive = true;

                $instance.$top.find(".stec-top-menu-filter-by-category ul li:not('.stec-select-all')").each(function () {
                    if (!$(this).hasClass('active')) {
                        allActive = false;
                        return false; // break;
                    }
                });

                if (allActive) {
                    $instance.$top.find(".stec-top-menu-filter-by-category ul li.stec-select-all").addClass('active');
                } else {
                    $instance.$top.find(".stec-top-menu-filter-by-category ul li.stec-select-all").removeClass('active');
                }

                master.layout.set();

            });

            // Toggle all categories 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-by-category ul li.stec-select-all", function (e) {

                e.preventDefault();
                e.stopPropagation();

                $(this).toggleClass('active');

                if ($(this).hasClass('active')) {

                    $instance.$top.find(".stec-top-menu-filter-by-category ul li:not(.stec-select-all)").filter(':visible').each(function () {
                        // add to filter
                        $(this).addClass('active');
                        $(this).find('i').removeClass('far fa-square');
                        $(this).find('i').addClass('fas fa-check-square');
                        master.calData.categoriesFilter.push(parseInt($(this).attr('data-catid'), 10));
                    });

                } else {

                    // remove from filter
                    $instance.$top.find(".stec-top-menu-filter-by-category ul li:not(.stec-select-all)").filter(':visible').each(function () {

                        $(this).removeClass('active');
                        $(this).find('i').removeClass('fas fa-check-square');
                        $(this).find('i').addClass('far fa-square');

                        master.calData.categoriesFilter = [];
                    });
                }

                master.calData.categoriesFilter = uniqueSort(master.calData.categoriesFilter);

                master.layout.set();

            });

            // Toggle organizers 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-by-organizer ul li:not('.stec-select-all')", function (e) {

                var index, allActive = true;

                e.preventDefault();
                e.stopPropagation();

                $(this).toggleClass('active');
                $(this).find('i').toggleClass('fas fa-check-square far fa-square');

                if ($(this).hasClass('active')) {

                    // add to filter
                    master.calData.organizersFilter.push(parseInt($(this).attr('data-organizer'), 10));

                } else {

                    // remove from filter

                    if (master.calData.organizersFilter !== false) {
                        index = master.calData.organizersFilter.indexOf(parseInt($(this).attr('data-organizer'), 10));
                        if (index > -1) {
                            master.calData.organizersFilter.splice(index, 1);
                        }
                    }
                }

                master.calData.organizersFilter = uniqueSort(master.calData.organizersFilter);

                allActive = true;

                $instance.$top.find(".stec-top-menu-filter-by-organizer ul li:not('.stec-select-all')").each(function () {
                    if (!$(this).hasClass('active')) {
                        allActive = false;
                        return false; // break;
                    }
                });

                if (allActive) {
                    $instance.$top.find(".stec-top-menu-filter-by-organizer ul li.stec-select-all").addClass('active');
                } else {
                    $instance.$top.find(".stec-top-menu-filter-by-organizer ul li.stec-select-all").removeClass('active');
                }

                master.layout.set();

            });

            // Toggle all organizers 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-by-organizer ul li.stec-select-all", function (e) {

                var index;

                e.preventDefault();
                e.stopPropagation();

                $(this).toggleClass('active');

                if ($(this).hasClass('active')) {

                    $instance.$top.find(".stec-top-menu-filter-by-organizer ul li:not('.stec-select-all')").filter(':visible').each(function () {
                        // add to filter
                        $(this).addClass('active');
                        $(this).find('i').removeClass('far fa-square');
                        $(this).find('i').addClass('fas fa-check-square');
                        master.calData.organizersFilter.push(parseInt($(this).attr('data-organizer'), 10));
                    });

                } else {

                    // remove from filter
                    $instance.$top.find(".stec-top-menu-filter-by-organizer ul li:not('.stec-select-all')").filter(':visible').each(function () {

                        $(this).removeClass('active');
                        $(this).find('i').removeClass('fas fa-check-square');
                        $(this).find('i').addClass('far fa-square');

                        if (master.calData.organizersFilter !== false) {
                            index = master.calData.organizersFilter.indexOf(parseInt($(this).attr('data-organizer'), 10));
                            if (index > -1) {
                                master.calData.organizersFilter.splice(index, 1);
                            }
                        }
                    });
                }

                master.calData.organizersFilter = uniqueSort(master.calData.organizersFilter);

                master.layout.set();

            });

            // Toggle locations 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-by-location ul li:not('.stec-select-all')", function (e) {

                var locations;
                var allActive = true;

                e.preventDefault();
                e.stopPropagation();

                $(this).toggleClass('active');
                $(this).find('> span i').toggleClass('fas fa-check-square far fa-square');

                if ($(this).hasClass('active')) {

                    // add to filter
                    master.calData.locationsFilter.push($(this).attr('data-location'));

                    // activate children
                    $.each($(this).find('li'), function () {
                        $(this).addClass('active');
                        master.calData.locationsFilter.push($(this).attr('data-location'));
                        $(this).find('> span i').removeClass('far fa-square');
                        $(this).find('> span i').addClass('fas fa-check-square');
                    });

                } else {

                    locations = [$(this).attr('data-location')];

                    // activate children
                    $.each($(this).find('li'), function () {
                        $(this).removeClass('active');
                        $(this).find('> span i').addClass('far fa-square');
                        $(this).find('> span i').removeClass('fas fa-check-square');
                        locations.push($(this).attr('data-location'));
                    });

                    // remove from filter
                    if (master.calData.locationsFilter !== false) {
                        $.each(locations, function () {

                            var index = master.calData.locationsFilter.indexOf(this.toString());
                            if (index > -1) {
                                master.calData.locationsFilter.splice(index, 1);
                            }
                        });

                    }
                }

                master.calData.locationsFilter = uniqueSort(master.calData.locationsFilter);

                allActive = true;

                $instance.$top.find(".stec-top-menu-filter-by-location ul li:not('.stec-select-all')").each(function () {
                    if (!$(this).hasClass('active')) {
                        allActive = false;
                        return false; // break;
                    }
                });

                if (allActive) {
                    $instance.$top.find(".stec-top-menu-filter-by-location ul li.stec-select-all").addClass('active');
                } else {
                    $instance.$top.find(".stec-top-menu-filter-by-location ul li.stec-select-all").removeClass('active');
                }

                master.layout.set();

            });

            // Toggle all locations 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-by-location ul li.stec-select-all", function (e) {

                var index;

                e.preventDefault();
                e.stopPropagation();

                $(this).toggleClass('active');

                if ($(this).hasClass('active')) {

                    $instance.$top.find(".stec-top-menu-filter-by-location ul li:not('.stec-select-all')").filter(':visible').each(function () {
                        // add to filter
                        $(this).addClass('active');
                        $(this).find('i').removeClass('far fa-square');
                        $(this).find('i').addClass('fas fa-check-square');
                        master.calData.locationsFilter.push($(this).attr('data-location'));

                    });

                } else {

                    // remove from filter
                    $instance.$top.find(".stec-top-menu-filter-by-location ul li:not('.stec-select-all')").filter(':visible').each(function () {

                        $(this).removeClass('active');
                        $(this).find('i').removeClass('fas fa-check-square');
                        $(this).find('i').addClass('far fa-square');

                        if (master.calData.locationsFilter !== false) {
                            index = master.calData.locationsFilter.indexOf($(this).attr('data-location'));
                            if (index > -1) {
                                master.calData.locationsFilter.splice(index, 1);
                            }
                        }
                    });
                }

                master.calData.locationsFilter = uniqueSort(master.calData.locationsFilter);

                master.layout.set();

            });

            // Toggle featured 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-filter-by-featured ul li", function (e) {

                e.preventDefault();
                e.stopPropagation();

                $(this).addClass('active');
                $(this).find('i')
                        .removeClass('far fa-square')
                        .addClass('fas fa-check-square');

                if ($(this).hasClass('active')) {
                    // set to filter
                    master.calData.featuredFilter = (parseInt($(this).attr('data-featured'), 10));
                }

                $(this).siblings().removeClass('active');
                $(this).siblings().find('i').removeClass('fas fa-check-square').addClass('far fa-square');

                master.layout.set();

            });

            // Quick Search box 
            $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-quick-search", function (e) {
                e.stopPropagation();
            });

            $(document).on('input', $instance.$top.path + " .stec-top-menu-quick-search", function (e) {

                var th = this;

                e.preventDefault();
                e.stopPropagation();

                if (quickSearchTimeout) {
                    clearTimeout(quickSearchTimeout);
                }

                quickSearchTimeout = setTimeout(function () {
                    var $th = $(th);
                    var $container = $th.parents('.stec-top-menu-filter-by').first();
                    var $contents = $container.find('.stec-top-menu-filter-contents');
                    var searchWord = $th.val().toLowerCase().trim();
                    var scroller = $container.find('.stec-scrollable').data('stec-scroll');

                    if (!searchWord) {
                        $contents.children('li').show();
                    } else {
                        $contents.children('li').each(function () {

                            let $th, contentText;

                            $th = $(this);

                            if ($th.attr('data-exclude-qsf')) {
                                return true; // continue;
                            }

                            contentText = $th.find('span').text().toLowerCase();

                            if (-1 !== contentText.indexOf(searchWord)) {
                                $(this).show();
                            } else {
                                $(this).hide();
                            }
                        });
                    }

                    if (scroller.originalStyle !== null) {
                        scroller.reset();
                    }

                }, 350);

            });

        });

    })();

    // Search
    (function () {

        $.stecExtend(function (master) {

            var $instance = master.$instance;
            var helper = master.helper;
            var glob = master.glob;
            var layout = master.layout;
            var calData = master.calData;

            /**
             * Top Search form handling
             */
            var search = {

                init: function () {
                    this.bindControls();
                },

                bindControls: function () {

                    var parent = this;

                    var suggestionTimeout;

                    $instance.$top.find(".stec-top-search-dropdown").on("keyup", function (e) {

                        var value, $lis, $selected;

                        clearTimeout(suggestionTimeout);

                        value = $instance.$top.find(".stec-top-search-form input").val();
                        $lis = $instance.$top.find(".stec-top-search-results li");

                        switch (e.which) {

                            // esc
                            case 27 :
                                parent.closeSearch();
                                break;

                                // enter 
                            case 13 :

                                if ($lis.filter(".active").length <= 0) {
                                    parent.getResults(value);
                                    return;
                                }

                                $selected = $lis.filter(".active");

                                // Jump date check
                                if (typeof $selected.attr("data-jumpdate") !== "undefined") {
                                    parent.jumpToDate($selected);
                                    parent.closeSearch();
                                }

                                break;

                                // up arrow
                            case 38 :
                                if ($lis.filter(".active").length > 0) {
                                    $lis.filter(".active")
                                            .removeClass("active")
                                            .prev()
                                            .addClass("active");
                                } else {
                                    $lis.filter(":last").addClass("active");
                                }
                                break;

                                // down arrow
                            case 40 :
                                if ($lis.filter(".active").length > 0) {
                                    $lis.filter(".active")
                                            .removeClass("active")
                                            .next()
                                            .addClass("active");
                                } else {
                                    $lis.first().addClass("active");
                                }
                                break;

                            default:
                                suggestionTimeout = setTimeout(function () {
                                    parent.getResults(value);
                                }, 250);

                        }

                    });

                    // Result list click handle
                    $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-search-results li", function (e) {

                        var $lis = $instance.$top.find(".stec-top-search-results li");
                        var $selected;

                        $lis.removeClass("active");
                        $(this).addClass("active");

                        $selected = $lis.filter(".active");

                        if (typeof $selected.attr("data-jumpsingle") === "undefined") {
                            e.preventDefault();
                        }

                        // Jump date check
                        if (typeof $selected.attr("data-jumpdate") !== "undefined") {
                            parent.jumpToDate($selected);
                            parent.closeSearch();
                        }

                    });

                    // Search button click handle
                    $instance.$top.find(".stec-top-search-form a").on(helper.clickHandle(), function (e) {

                        var value;

                        e.preventDefault();

                        value = $instance.$top.find("input").val();
                        parent.getResults(value);
                    });

                    // Search main button toggle show/hide
                    $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-menu-search", function (e) {
                        var $dropdown;

                        e.preventDefault();
                        e.stopPropagation();

                        // fix for left offset since today button is not fixed width
                        $dropdown = $(this).find('.stec-top-search-dropdown');

                        $dropdown.css({
                            left: -1 * Math.round($(this).position().left)
                        });

                        $(this).toggleClass("active");
                    });

                    // Block search toggle for inner content
                    $(document).on(helper.clickHandle(), $instance.$top.path + " .stec-top-search-dropdown", function (e) {
                        // preventDefault blocks mobile keyboard
                        e.stopPropagation();
                    });

                },

                /**
                 * jump to $selected data-jumpdate attribute
                 * data-jumpdate="yyyy-mm-dd"
                 * @param {object} $selected $(element)
                 */
                jumpToDate: function ($selected) {

                    var date = helper.getDateFromData($selected.attr("data-jumpdate"));

                    glob.options.year = date.getFullYear();
                    glob.options.month = date.getMonth();
                    glob.options.day = date.getDate();

                    layout.set();

                },

                closeSearch: function () {
                    this.resetResults();
                    $instance.$top.find(".stec-top-menu-search").removeClass("active");
                },

                resetResults: function () {
                    $instance.$top.find('.stec-top-search-dropdown-noresult').hide();
                    $instance.$top.find(".stec-top-search-results").empty();
                },

                escapeHtml: function (string) {

                    var entityMap = {
                        "&": "&amp;",
                        "<": "&lt;",
                        ">": "&gt;",
                        '"': '&quot;',
                        "'": '&#39;',
                        "/": '&#x2F;'
                    };

                    return String(string).replace(/[&<>"'\/]/g, function (s) {
                        return entityMap[s];
                    });
                },

                getResults: function (keyword) {

                    var parent = this;
                    var keywordArray, WORD, html;
                    var dateData, searchDate, dateString;
                    var uniqueResults;
                    var result;

                    parent.resetResults();

                    keyword = keyword.trim();

                    if (keyword === "") {
                        return;
                    }

                    keywordArray = keyword.split(" ");

                    WORD = {
                        generic: false,
                        date: {
                            year: false,
                            month: false,
                            day: false
                        }
                    };

                    keywordArray = $.grep(keywordArray, function (el, index) {
                        return index === $.inArray(el, keywordArray);
                    });

                    $(keywordArray).each(function () {

                        // Start as Generic word
                        var generic = true;
                        var monthName;

                        if (parent.isYear(this)) {
                            WORD.date.year = this;
                            generic = false;
                        }

                        if (parent.isMonthString(this)) {
                            monthName = this;

                            $(glob.options.monthLabels).each(function (monthNumber) {
                                if (this.toLowerCase() === monthName.toLowerCase()) {
                                    WORD.date.month = monthNumber;
                                    generic = false;
                                }
                            });

                            if (WORD.date.month === false) {
                                // check shortname
                                $(glob.options.monthLabelsShort).each(function (monthNumber) {
                                    if (this.toLowerCase() === monthName.toLowerCase()) {
                                        WORD.date.month = monthNumber;
                                        generic = false;
                                    }
                                });
                            }
                        }

                        if (parent.isDay(this)) {
                            WORD.date.day = this;
                            generic = false;
                        }

                        // NO Func Match Found
                        if (generic === true) {
                            WORD.generic = true;
                        }

                    });

                    html = '';

                    // Not a generic word
                    if (WORD.generic === false) {
                        if (WORD.date.year !== false || WORD.date.month !== false || WORD.date.day !== false) {

                            if (WORD.date.year === false) {
                                WORD.date.year = glob.options.year;
                            }
                            if (WORD.date.month === false) {
                                WORD.date.month = glob.options.month;
                            }
                            if (WORD.date.day === false) {
                                WORD.date.day = glob.options.day;
                            }

                            // Suggest navigate to Date

                            dateData = WORD.date.year + "-" + WORD.date.month + "-" + WORD.date.day;
                            searchDate = new Date(WORD.date.year, WORD.date.month, WORD.date.day);

                            dateString = helper.beautifyTimespan(searchDate, searchDate, 1);

                            html = '<li data-jumpDate="' + dateData + '"><i class="fas fa-calendar-check"></i> <p>' + dateString + '</p></li>';
                            $(html).appendTo($instance.$top.find(".stec-top-search-results"));

                        }

                    } else {
                        // Generic
                        // Search for keywords

                        result = [];

                        $(keywordArray).each(function () {

                            var word = this;
                            var eventsPool;

                            if (word.length <= 2) {
                                return true;
                            }

                            word = word.toLowerCase();

                            eventsPool = [];

                            eventsPool = calData.eventsPool;

                            $(eventsPool).each(function () {

                                var summary, location, keyWords, closest, found = [];

                                if (calData.calendarFilter.indexOf(parseInt(this.calid, 10)) === -1) {
                                    return; // continue loop
                                }

                                summary = this.title.toLowerCase();
                                location = this.location.full_address ? this.location.full_address.toLowerCase() : '';
                                keyWords = this.keywords.toLowerCase();

                                if (keyWords.indexOf(word) > -1 || summary.indexOf(word) > -1 || location.indexOf(word) > -1) {

                                    if (this.rrule) {
                                        closest = calData.repeaterGetClosest(this);
                                        found.push(closest[0]);
                                    } else {
                                        found.push(this);
                                    }

                                    if (found.length > 0) {
                                        $.each(found, function () {
                                            var state = helper.getEventState(this);

                                            if (0 !== state && parseInt(glob.options.upcoming_only, 10) === 1) {
                                                return true; // continue
                                            }

                                            if (0 === state && parseInt(glob.options.expired_only, 10) === 1) {
                                                return true; // continue
                                            }

                                            result.push(this);

                                        });
                                    }

                                }
                            });

                        });

                        if (result.length > 0) {
                            // found match

                            html = [];

                            uniqueResults = [];

                            $.each(result, function () {
                                var th = this;
                                var found = false;
                                $.each(uniqueResults, function () {
                                    if (th.id === this.id) {
                                        found = true;
                                        return false; // break;
                                    }
                                });

                                if (false === found) {
                                    uniqueResults.push(th);
                                }
                            });

                            result = uniqueResults;

                            $(result).each(function () {

                                var permalink = helper.getPermalink(this.permalink, this.repeat_time_offset);

                                html.push('<li data-jumpsingle><a href="' + permalink + '"><i class="' + this.icon + '"></i><p>' + this.title + '</p></a></li>');
                            });

                            $(html.join('')).appendTo($instance.$top.find(".stec-top-search-results"));

                        } else {
                            // no match found
                            $instance.$top.find('.stec-top-search-dropdown-noresult').show();
                        }
                    }
                },

                isMonthString: function (keyword) {
                    var found = false;
                    if (isNaN(keyword)) {

                        if ($.inArray(keyword.toLowerCase(), glob.options.monthLabels) !== -1) {
                            found = true;
                        }

                        if ($.inArray(keyword.toLowerCase(), glob.options.monthLabelsShort) !== -1) {
                            found = true;
                        }

                    }

                    return found;
                },

                isDay: function (keyword) {
                    var found = false;
                    if (!isNaN(keyword)) {
                        // numbers
                        if (keyword.length <= 2) {
                            if (keyword <= 31 && keyword > 0) {
                                // use as day
                                found = true;
                            }
                        }
                    }
                    return found;
                },

                isYear: function (keyword) {
                    var found = false;
                    if (!isNaN(keyword)) {
                        // numbers
                        if (keyword.length === 4 && keyword >= 1800 && keyword <= 2200) {
                            // use as year
                            found = true;
                        }
                    }
                    return found;
                }
            };

            search.init();

        });

    })();

    // CalData
    (function () {


        var proto;

        window.StecCalData = function () {
            this.page = 0;
            this.perPage = 500;
            this.featuredFilter = 0;
            this.calendarFilter = [];
            this.calendarsPool = [];
            this.categoriesPool = [0];
            this.categoriesFilter = [0];
            this.locationsPool = ['none', 'stec-virtual'];
            this.locationsFilter = ['none', 'stec-virtual'];
            this.organizersPool = ['none'];
            this.organizersFilter = ['none'];
            this.helper = '';
            this.glob = '';
            this.DB = '';
            this.eventsPool = [];
            this.eventsWorker = [];
        };

        proto = window.StecCalData.prototype;

        proto.set = function (obj, data) {

            if (typeof this[obj] !== 'undefined') {
                this[obj] = data;
            }

            return this;

        };

        /**
         * Returns event by id
         * 
         * @param {int} eventId
         * @param {int} offset repeat time offset
         * @returns event object
         * @todo add params for time offset
         * 
         * NOTE IF OFFSET IS PRESENT THE EVENT IS RETURNED AS A COPY 
         * OTHERWISE A REFERENCE IS RETURNED
         */
        proto.getEventById = function (eventId, offset) {

            var event = [];

            $(this.eventsPool).each(function () {
                if (parseInt(this.id, 10) === parseInt(eventId, 10)) {

                    event = this;

                    if (offset && offset > 0) {

                        event = JSON.parse(JSON.stringify(event));

                        event.start_date_timestamp = event.start_date_timestamp + offset;
                        event.end_date_timestamp = event.end_date_timestamp + offset;

                        event.start_date = moment.tz(event.start_date, event.timezone).add(offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');
                        event.end_date = moment.tz(event.end_date, event.timezone).add(offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');
                        event.repeat_time_offset = offset;
                    }

                    return false; // break;
                }
            });

            return event;
        };

        // sort by featured
        // @todo recheck solution...
        proto.sortByFeatured = function (a, b) {

            return 0;

            if (a.featured !== 0 && b.featured !== 0) {
                return a.start_date_timestamp - b.start_date_timestamp;
            } else {
                return a.featured - b.featured;
            }

        };

        // sort by timestamp oldest -> newest
        proto.sortByTimestamp = function (a, b) {
            if (a.start_date_timestamp < b.start_date_timestamp) {
                return -1;
            } else if (a.start_date_timestamp > b.start_date_timestamp) {
                return 1;
            } else {
                return 0;
            }
        };

        proto.removeFromEventsPool = function (eventId) {

            var newPool = []; // probably the safest way to do it...

            $(this.eventsPool).each(function () {
                if (this.id !== eventId) {
                    newPool.push(this);
                }
            });

            this.eventsPool = newPool;
        };

        /**
         * Add events to the events ppol
         * Note since 2.9: Processes rrules
         * @param {type} events
         */
        proto.addToEventsPool = function (events) {

            $.each(events, function () {
                this.start_date_timestamp = moment.tz(this.start_date, this.timezone).unix();
                this.end_date_timestamp = moment.tz(this.end_date, this.timezone).unix();
            });

            this.eventsPool = this.eventsPool.concat(events);

            this.helper.extendBind("stachethemes_ec_extend", "onAddToEventsPool", this.eventsPool);
        };

        proto.addDataToEvent = function (data) {

            var parent = this;

            $(parent.eventsPool).each(function () {
                if (parseInt(this.id, 10) === parseInt(data.general.id), 10) {
                    this.data = data;
                }
            });
        };

        /**
         * Pulls all events from now up to 12 months in the future, including repeated events
         */
        proto.getFutureEvents = function (incAapproval) {

            var events;
            var date = new Date(this.glob.options.year, this.glob.options.month, this.glob.options.day);
            var a = date;
            var b = new Date(a);
            var ahead = parseInt(this.glob.options.future_months_ahead, 10);

            this.glob.options.future_months_ahead = isNaN(ahead) ? 12 : ahead;

            b.setMonth(b.getMonth() + this.glob.options.future_months_ahead);

            events = this.getEvents(a, b, incAapproval);

            return events;
        };

        proto.repeaterGetNearest = function (event, rangeStart, rangeEnd) {

            var parent = this;

            var eventStartDate = moment.utc(event.start_date);
            var rangeStartDate = moment.unix(rangeStart).utc();
            var rangeEndDate = moment.unix(rangeEnd).utc();
            var rfcString = event.rrule + ';DTSTART=' + this.helper.dateToRFC(eventStartDate);

            var rruleSet;
            var rEvents = [];
            var result;

            var exdates;
            var exdatesFixedArray;

            var i, j;
            var dt, st;

            var rEvent;

            if (event.exdate) {

                exdates = event.exdate.split(',');
                exdatesFixedArray = [];

                for (i = 0; i < exdates.length; i++) {

                    if (exdates[i].length === 8) {

                        st = moment.tz(event.start_date, event.timezone);
                        dt = moment.tz(exdates[i], event.timezone);

                        exdatesFixedArray.push(dt
                                .hours(st.hours())
                                .minutes(st.minutes())
                                .seconds(0)
                                .format('YYYYMMDD\THHmmss') + 'Z');

                    } else {

                        exdatesFixedArray.push(exdates[i]);

                    }
                }

                rfcString += '\nEXDATE:' + exdatesFixedArray.join(',');

            }

            rruleSet = window.rrule.rrulestr(rfcString, {
                forceset: true
            });

            result = rruleSet.after(rangeStartDate.toDate(), true);

            if (!result) {
                result = rruleSet.before(rangeStartDate.toDate(), true);
            }

            if (result) {
                rEvent = parent.getREvent(event, result);

                if (rEvent.end_date_timestamp > rangeStart) {
                    rEvents.push(rEvent);
                }

            }

            return rEvents;

        };

        /**
         * 
         * Return all events for given timespan
         * IF EVENT IS REPEATED IT WILL SEARCH FOR NEAREST 
         * TO DATE RECURRENCE AND WILL RETURN ONLY THIS INSTANCE
         * Nearest requirement: Must be in between range and closest to now
         * 
         * @param {date} startDate
         * @param {date} endDate
         * @param {bool} incAapproval
         * @returns {array} 
         */
        proto.getNearestDateEvents = function (startDate, endDate, incAapproval) {

            var parent = this,
                    a,
                    b,
                    overrides,
                    pick = [],
                    now = moment().unix();

            if (!incAapproval) {
                incAapproval = false;
            }

            a = this.getRangeUnix(startDate);
            b = this.getRangeUnix(endDate ? endDate : startDate, true);

            // Get overrides in this range and add them to "pick" array
            overrides = this.getOverridesInRange(a, b);
            if (overrides.length > 0) {
                parent.filterGetEvents = [];
                parent.filterGetEvents = overrides;
                overrides = parent.filterGetEvents;
                $.merge(pick, parent.filterGetEvents);
            }

            // Exclude overriden
            parent.filterGetEvents = [];
            parent.filterGetEvents = $.grep(this.eventsPool, function (poolEvent) {
                return !poolEvent.recurrence_id;
            });

            this.helper.extendBind("stachethemes_ec_extend", "beforeProccessGetEvents");

            $(parent.filterGetEvents).each(function () {

                var eventLoop = this;

                if (incAapproval !== true && parseInt(this.approved, 10) === 0) {
                    return true;
                }

                //  If repeating event get repeater
                if (this.rrule && this.rrule !== '') {
                    try {
                        eventLoop = parent.repeaterGetNearest(this, a, b);
                    } catch (err) {
                        console.log(err);
                        console.error('Error processing repeater for event ' + this.id, 'with RRULE String: ' + this.rrule);
                    }
                }

                // Picker
                $(eventLoop).each(function () {

                    var start, end;

                    if (overrides.length > 0 && parent.isOverriden(this)) {
                        return true; // continue;
                    }

                    start = this.start_date_timestamp;
                    end = this.end_date_timestamp;

                    // if param expired_only=1 in [stachethemes_ec]
                    if (parseInt(parent.glob.options.expired_only, 10) === 1 && end > now) {
                        return true; // continue;
                    } else

                    // if param upcoming_only=1 in [stachethemes_ec]
                    if (parseInt(parent.glob.options.upcoming_only, 10) === 1 && start < now) {
                        return true; // continue;
                    }

                    if (b > start && end >= a) {
                        pick.push(this);
                    }
                });

            });

            // reorder full events by timestamp
            parent.filterGetEvents = pick.sort(parent.sortByTimestamp);

            // reorder by featured
            if (this.glob.options.view !== 'agenda' && this.glob.options.view !== 'grid' && this.glob.options.view !== 'boxgrid') {
                parent.filterGetEvents.sort(this.sortByFeatured);
            }

            this.helper.extendBind("stachethemes_ec_extend", "beforeReturnGetEvents");

            return parent.filterGetEvents;
        };

        /**
         * Helper function for getEvents()
         * Gets Year Month and Date from date or string
         * Dates are treated as UTC
         * returns the utc timespan for given date
         * 
         * @param {mixed} JS Date or String
         * @param {bool} endOf adds 24 hours to the date
         * @returns {string} unixstamp
         */
        proto.getRangeUnix = function (dateOrString, endOf) {

            var momentDate;
            var dateArray = null;

            if (!dateOrString) {
                dateArray = [this.glob.options.year, this.glob.options.month, this.glob.options.day];
            } else if (true === dateOrString instanceof Date) {
                dateArray = [dateOrString.getFullYear(), dateOrString.getMonth(), dateOrString.getDate()];
            } else if ('string' === typeof dateOrString) {
                dateOrString = moment.utc(dateOrString).toDate();
                dateArray = [dateOrString.getFullYear(), dateOrString.getMonth(), dateOrString.getDate()];
            } else {
                console.log('Unsupported dateOrString');
            }

            momentDate = window.moment.tz('UTC');
            momentDate.set({year: dateArray[0], month: dateArray[1], date: dateArray[2], hour: 0, minute: 0, second: 0, millisecond: 0});

            if (endOf) {
                momentDate.add(24, 'hours');
            }

            return momentDate.unix();

        };

        /**
         * Return all events for given timespan
         * Note version > 2.9: No longer processes RRULE strings
         * @param {Date} startDate
         * @param {Date} endDate
         * @param {bool} incAapproval
         * @returns {Array|Window.filterGetEvents}
         */
        proto.getEvents = function (startDate, endDate, incAapproval) {

            var parent = this,
                    a,
                    b,
                    pick = [],
                    overrides,
                    now = moment().unix();

            if (!incAapproval) {
                incAapproval = false;
            }

            parent.filterGetEvents = [];
            a = this.getRangeUnix(startDate);
            b = this.getRangeUnix(endDate ? endDate : startDate, true);

            // Get overrides in this range and add them to "pick" array
            overrides = this.getOverridesInRange(a, b);
            if (overrides.length > 0) {
                parent.filterGetEvents = overrides;
                overrides = parent.filterGetEvents;
                $.merge(pick, parent.filterGetEvents);
            }

            // Exclude overriden
            parent.filterGetEvents = $.grep(this.eventsPool, function (poolEvent) {
                return !poolEvent.recurrence_id;
            });

            this.helper.extendBind("stachethemes_ec_extend", "beforeProccessGetEvents");

            $(parent.filterGetEvents).each(function () {

                var eventLoop = this;

                if (incAapproval !== true && parseInt(eventLoop.approved, 10) === 0) {
                    return true;
                }

                //  If repeating event get repeater
                if (eventLoop.rrule && eventLoop.rrule !== '') {
                    try {
                        eventLoop = parent.repeaterGet(eventLoop, a, b);
                    } catch (err) {
                        console.log(err);
                        console.error('Error processing repeater for event ' + this.id, 'with RRULE String: ' + this.rrule);
                    }
                }

                // Picker
                $(eventLoop).each(function () {
                    var start, end;

                    if (overrides.length > 0 && parent.isOverriden(this)) {
                        return true; // continue;
                    }

                    /**
                     * Update 3.0.1
                     * Since time is relative treat dates as UTC
                     */
                    start = moment.tz(this.start_date, 'UTC').unix();
                    end = moment.tz(this.end_date, 'UTC').unix();

                    // if param expired_only=1 in [stachethemes_ec]
                    if (parseInt(parent.glob.options.expired_only, 10) === 1 && end > now) {
                        return true; // continue;
                    } else

                    // if param upcoming_only=1 in [stachethemes_ec]
                    if (parseInt(parent.glob.options.upcoming_only, 10) === 1 && start < now) {
                        return true; // continue;
                    }

                    if (b > start && end >= a) {
                        pick.push(this);
                    }
                });

            });

            // reorder full events by timestamp
            parent.filterGetEvents = pick.sort(parent.sortByTimestamp);

            // reorder by featured
            if (this.glob.options.view !== 'agenda' && this.glob.options.view !== 'grid' && this.glob.options.view !== 'boxgrid') {
                parent.filterGetEvents.sort(this.sortByFeatured);
            }

            this.helper.extendBind("stachethemes_ec_extend", "beforeReturnGetEvents");

            return parent.filterGetEvents;
        };

        proto.isOverriden = function (event) {

            var recurrenceId = moment.tz(event.start_date, event.timezone).format('YMMDDTHmss');
            var overrides = this.getOverrides(event.uid, recurrenceId);
            if (overrides.length > 0) {
                return true;
            }

            return false;

        };

        proto.getOverridesInRange = function (a, b, filteredPool) {

            var pool = filteredPool ? filteredPool : this.eventsPool;

            return  $.grep(pool, function (event) {

                if (!event.recurrence_id) {
                    return false;
                }

                if (b > event.start_date_timestamp && event.end_date_timestamp >= a) {
                    return true;
                }

                return false;
            });
        };

        /**
         * Looks for recurrence-id override
         * @param {string} uid 
         * @param {string} recurrenceId
         * 
         * The "recurrence_id" value must contain the override time in format YYYYMMDDTHHmmss
         * 
         * @return Array
         */
        proto.getOverrides = function (uid, recurrenceId) {

            return  $.grep(this.eventsPool, function (poolEvent) {

                var m, offset = 0;

                if (!poolEvent.recurrence_id) {
                    return false;
                }

                if (poolEvent.recurrence_id.indexOf('Z') !== -1) {

                    m = new moment.tz(poolEvent.recurrence_id, 'UTC');

                    if (m.isDST()) {
                        offset = -1 * ((poolEvent.timezone_utc_offset - 3600) / 3600);
                    } else {
                        offset = -1 * (poolEvent.timezone_utc_offset / 3600);
                    }

                    m.utcOffset(offset);

                    poolEvent.recurrence_id = m.format('YYYYMMDDTHHmmss');

                } else if (poolEvent.recurrence_id.length === 8) {
                    recurrenceId = recurrenceId.substr(0, 8);
                }

                return uid === poolEvent.uid && poolEvent.recurrence_id === recurrenceId;
            });
        };

        proto.repeaterGet = function (event, rangeStart, rangeEnd) {

            var parent = this;

            var eventStartDate = moment.utc(event.start_date);
            var rangeStartDate = moment.unix(rangeStart).utc();
            var rangeEndDate = moment.unix(rangeEnd).utc();
            var rfcString = event.rrule + ';DTSTART=' + this.helper.dateToRFC(eventStartDate);

            var rruleSet;
            var rEvents = [];
            var result;

            var exdates;
            var exdatesFixedArray;

            if (eventStartDate.isBefore(rangeStartDate)) {
                rangeStartDate = eventStartDate;
            }

            if (event.exdate) {

                exdates = event.exdate.split(',');
                exdatesFixedArray = [];

                $.each(exdates, function () {

                    var dt, st;

                    if (this.length === 8) {

                        st = moment.tz(event.start_date, event.timezone);
                        dt = moment.tz(this, event.timezone);

                        exdatesFixedArray.push(dt
                                .hours(st.hours())
                                .minutes(st.minutes())
                                .seconds(0)
                                .format('YYYYMMDD\THHmmss') + 'Z');

                    } else {

                        exdatesFixedArray.push(this);

                    }
                });

                rfcString += '\nEXDATE:' + exdatesFixedArray.join(',');

            }

            rruleSet = window.rrule.rrulestr(rfcString, {
                forceset: true
            });

            result = rruleSet.between(rangeStartDate.toDate(), rangeEndDate.toDate(), true);

            $.each(result, function () {

                var rEvent = parent.getREvent(event, this);

                if (rEvent.end_date_timestamp < rangeStart) {
                    return true;
                }

                rEvents.push(rEvent);
            });

            return rEvents;

        };

        proto.getREvent = function (event, date) {

            var rEvent;
            var offset = 0;
            var initialStartDate = moment.tz(event.start_date, event.timezone);
            var initialEndDate = moment.tz(event.end_date, event.timezone);
            var occurenceDate = moment.tz(date, event.timezone);
            occurenceDate.subtract(occurenceDate.utcOffset(), 'minutes');

            offset = occurenceDate.diff(initialStartDate, 'seconds');

            // not by reference! need fresh copy
            rEvent = JSON.parse(JSON.stringify(event));

            rEvent.start_date_timestamp = initialStartDate.unix() + offset;
            rEvent.end_date_timestamp = initialEndDate.unix() + offset;

            rEvent.start_date = initialStartDate.add(offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');
            rEvent.end_date = initialEndDate.add(offset, 'seconds').format('YYYY-MM-DD HH:mm:ss');
            rEvent.repeat_time_offset = offset;

            return rEvent;

        };

        proto.repeaterGetClosest = function (event, date) {

            var parent = this;
            var eventStartDate = moment.utc(event.start_date);
            var rfcString = event.rrule + ';DTSTART=' + this.helper.dateToRFC(eventStartDate);
            var exdates;
            var exdatesFixedArray;
            var rruleSet;
            var rEvents = [];
            var result;

            if (!date) {
                date = moment.utc().toDate();
            }

            if (event.exdate) {

                exdates = event.exdate.split(',');
                exdatesFixedArray = [];

                $.each(exdates, function () {

                    var dt;

                    if (this.length === 8) {

                        dt = moment(this);

                        exdatesFixedArray.push(dt
                                .hours(eventStartDate.toDate().getHours())
                                .minutes(eventStartDate.toDate().getMinutes())
                                .seconds(0)
                                .utc().format('YYYYMMDD\THHmmss') + 'Z');

                    } else {

                        exdatesFixedArray.push(this);

                    }
                });

                rfcString += '\nEXDATE:' + exdatesFixedArray.join(',');
            }

            rruleSet = window.rrule.rrulestr(rfcString, {
                forceset: true
            });

            result = rruleSet.after(date, true);

            if (!result || result.length <= 0) {
                result = rruleSet.before(date, true);
            }

            if (!result || result.length <= 0) {
                return [];
            }

            $(result).each(function () {

                var rEvent = parent.getREvent(event, this);

                rEvents.push(rEvent);

            });

            return rEvents;
        };

        proto.queryEventsStep = function (stepCallback) {

            var request;
            var DB = this.DB;
            var glob = this.glob;

            request = DB.newRequest();

            request.setData({
                action: 'stec_public_ajax_action',
                cal: glob.options.cal ? glob.options.cal : '',
                cat: glob.options.cat ? glob.options.cat : '',
                organizer_id: glob.options.organizer_id ? glob.options.organizer_id : '',
                min_date: glob.options.min_date ? glob.options.min_date : null,
                max_date: glob.options.max_date ? glob.options.max_date : null,
                page: this.page,
                per_page: this.perPage,
                task: 'get_events'
            });

            request.setCallback(function (data) {
                stepCallback(data);
            });

            request.setOnError(function (xhr, status, thrown) {

                console.error("Stachethemes Event Calendar encountered an error (" + status + ") during get_events ajax request", {
                    status: xhr.status,
                    response: xhr.responseText
                });

                stepCallback(false);

            });

            DB.query(request);
        };

        proto.queryEvents = function (callback, step) {

            var th = this;

            if (this.glob.options.general_settings.events_per_request) {
                this.perPage = parseInt(this.glob.options.general_settings.events_per_request, 10);
            }

            this.queryEventsStep(function (data) {

                if (data && data.length > 0) {

                    th.addToEventsPool(data);
                    th.page++;

                    if (typeof step === 'function') {
                        step();
                    }

                    if (data.length <= 0) {
                        callback();
                    } else {
                        th.queryEvents(callback, step);
                    }


                } else {
                    callback();
                }

            });
        };

        /**
         * Pulls all events from now up to 12 months (or custom) in the future, including repeated events
         */
        proto.getFutureEventsThread = function (incAapproval, threadIndex) {

            var parent = this;

            return new Promise(function (resolve) {

                var date, a, b, ahead;

                if (typeof Worker === 'undefined') {
                    return resolve(parent.getFutureEvents(incAapproval));
                }

                date = new Date(parent.glob.options.year, parent.glob.options.month, parent.glob.options.day);
                a = date;
                b = new Date(a);
                ahead = parseInt(parent.glob.options.future_months_ahead, 10);

                parent.glob.options.future_months_ahead = isNaN(ahead) ? 12 : ahead;

                b.setMonth(b.getMonth() + parent.glob.options.future_months_ahead);

                parent.getEventsThread(a, b, incAapproval, threadIndex).then(function (events) {
                    return resolve(events);
                });

            });

        };

        /**
         * Retrieves events for given timespan
         * Uses Worker interface https://developer.mozilla.org/en-US/docs/Web/API/Worker
         * If Workers are not supported fallsback to default getEvents method
         * @param {date} startDate
         * @param {date} endDate
         * @param {bool} incAapproval
         * @returns {Promise} events array
         * @since 3.0.5
         */
        proto.getEventsThread = function (startDate, endDate, incAapproval, threadIndex) {

            var parent = this;

            return new Promise(function (resolve) {

                if (typeof Worker === 'undefined') {
                    return resolve(parent.getEvents(startDate, endDate, incAapproval));
                }

                if (true === parent.eventsWorker[threadIndex] instanceof Worker) {
                    parent.eventsWorker[threadIndex].terminate();
                }

                parent.eventsWorker[threadIndex] = new Worker(window.stecPluginPath.url + 'assets/js/background/worker/events.min.js');

                parent.filterGetEvents = parent.eventsPool;

                parent.helper.extendBind("stachethemes_ec_extend", "beforeProccessGetEvents");

                parent.eventsWorker[threadIndex].postMessage({
                    eventsPool: parent.filterGetEvents,
                    task: 'getEvents',
                    params: {
                        from: startDate,
                        to: endDate,
                        incAapproval: incAapproval ? true : false,
                        upcomingOnly: parseInt(parent.glob.options.upcoming_only, 10) === 1 ? true : false,
                        expiredOnly: parseInt(parent.glob.options.expired_only, 10) === 1 ? true : false,
                        view: parent.glob.options.view
                    }
                });

                parent.eventsWorker[threadIndex].onmessage = function (e) {
                    parent.eventsWorker[threadIndex].terminate();

                    parent.filterGetEvents = e.data;

                    parent.helper.extendBind("stachethemes_ec_extend", "beforeReturnGetEvents");

                    resolve(parent.filterGetEvents);
                };
            });

        };

        /**
         * Return all events for given timespan
         * Uses Worker interface https://developer.mozilla.org/en-US/docs/Web/API/Worker
         * IF EVENT IS REPEATED IT WILL SEARCH FOR NEAREST 
         * TO DATE RECURRENCE AND WILL RETURN ONLY THIS INSTANCE
         * Nearest requirement: Must be in between range and closest to now
         * 
         * @param {date} startDate
         * @param {date} endDate
         * @returns {array} 
         */
        proto.getNearestDateEventsThread = function (startDate, endDate, incAapproval, threadIndex) {

            var parent = this;

            return new Promise(function (resolve) {

                if (typeof Worker === 'undefined') {
                    return resolve(parent.getNearestDateEvents(startDate, endDate, incAapproval));
                }

                if (true === parent.eventsWorker[threadIndex] instanceof Worker) {
                    parent.eventsWorker[threadIndex].terminate();
                }

                parent.eventsWorker[threadIndex] = new Worker(window.stecPluginPath.url + 'assets/js/background/worker/events.min.js');

                parent.filterGetEvents = parent.eventsPool;

                parent.helper.extendBind("stachethemes_ec_extend", "beforeProccessGetEvents");

                parent.eventsWorker[threadIndex].postMessage({
                    eventsPool: parent.filterGetEvents,
                    task: 'getNearestEvents',
                    params: {
                        from: startDate,
                        to: endDate,
                        incAapproval: incAapproval ? true : false,
                        upcomingOnly: parseInt(parent.glob.options.upcoming_only, 10) === 1 ? true : false,
                        expiredOnly: parseInt(parent.glob.options.expired_only, 10) === 1 ? true : false,
                        view: parent.glob.options.view
                    }
                });

                parent.eventsWorker[threadIndex].onmessage = function (e) {
                    parent.eventsWorker[threadIndex].terminate();

                    parent.filterGetEvents = e.data;

                    parent.helper.extendBind("stachethemes_ec_extend", "beforeReturnGetEvents");

                    resolve(parent.filterGetEvents);

                };
            });

        };

    })();

})(window.jQuery);
