(function ($) {

    "use strict";

    /**
     *  Main Calendar front javascript
     */
    function StachethemesEventCalendar() {

        const __ = new window.StecHelper().lang;

        var main = this;

        var moment = window.moment;

        var instance = '', $instance = '';

        var glob = {

            options: {
                date: {
                    day: 1,
                    month: 1,
                    year: 2021
                },

                set day(value) {
                    this.date.day = parseInt(value, 10);
                    $instance.find('.stec-event-submission-form').trigger('stec-day-change', this.date);
                },

                set month(value) {
                    this.date.month = parseInt(value, 10);
                },

                set year(value) {
                    this.date.year = parseInt(value, 10);
                },

                get day() {
                    return this.date.day;
                },

                get month() {
                    return this.date.month;
                },

                get year() {
                    return this.date.year;
                },

                monthLabelsShort: [
                    __('jan'),
                    __('feb'),
                    __('mar'),
                    __('apr'),
                    __('may'),
                    __('jun'),
                    __('jul'),
                    __('aug'),
                    __('sep'),
                    __('oct'),
                    __('nov'),
                    __('dec')
                ],
                monthLabels: [
                    __('january'),
                    __('february'),
                    __('march'),
                    __('april'),
                    __('may'),
                    __('june'),
                    __('july'),
                    __('august'),
                    __('september'),
                    __('october'),
                    __('november'),
                    __('december')
                ],
                dayLabels: [
                    __('sunday'),
                    __('monday'),
                    __('tuesday'),
                    __('wednesday'),
                    __('thursday'),
                    __('friday'),
                    __('saturday')
                ],
                dayLabelsShort: [
                    __('sun'),
                    __('mon'),
                    __('tue'),
                    __('wed'),
                    __('thu'),
                    __('fri'),
                    __('sat')
                ],
                myLocation: ''
            },

            template: {
                eventCreateForm: '',
                event: '',
                gridevent: '',
                boxgridevent: '',
                eventInner: '',
                tooltip: '',
                preloader: '',
                share: '',
                emailEvent: ''
            },

            temp: [],

            blockAction: false,

            ajax: null,

            timeout: []
        };

        var helper, calData, DB, reminder, rsvp, booking, shop;

        /**
         * Calendar pre-init loader 
         */
        var preloader = {

            add: function () {
                $('#stec-init-preloader-' + glob.options.id).show();
            },

            count: function (number) {
                $('#stec-init-preloader-' + glob.options.id).find('.stec-init-preloader-count').text(number);
            },

            destroy: function () {
                $('#stec-init-preloader-' + glob.options.id).remove();
            }

        };

        /**
         * handles events
         */
        var eventsHandler = {

            init: function () {
                this.bindControls();
            },

            bindControls: function () {

                var parent = this;

                // activate anchors for tabs content
                $(document).on(helper.clickHandle(), $instance.$events.path + (" a"), function (e) {
                    e.stopPropagation();
                    return true;
                });

                // excludes .create-form toggle
                // .stec-event-create-form is handled by adds/event.create.js
                $(document).on(helper.clickHandle(), $instance.$events.path + (".stec-layout-event-preview-right-event-toggle:not('.stec-layout-event-create-form-preview-right-event-toggle')"), function (e) {
                    var $event;
                    e.preventDefault();
                    e.stopPropagation();
                    $event = $(this).parents(".stec-layout-event");
                    parent.eventToggle($event);

                });

                // excludes .create-form toggle, .stec-layout-event-awaiting-approval
                // .stec-event-create-form is handled by adds/event.create.js
                $(document).on(helper.clickHandle(), $instance.$events.path + (".stec-layout-event:not('.stec-event-create-form, .stec-layout-event-awaiting-approval')"), function (e) {
                    var $event;
                    e.preventDefault();
                    $event = $(this);
                    parent.eventToggle($event);
                });

                // Prevent toggling from inner content
                $(document).on(helper.clickHandle(), $instance.$events.path + '.stec-layout-event-inner', function (e) {
                    e.stopPropagation();
                });

                $(document).on(helper.clickHandle(), $instance.$events.path + (".stec-layout-event-inner-top-tabs li"), function (e) {
                    e.preventDefault();
                    parent.activateTab($(this));
                });

                $(document).on(helper.clickHandle(), '#stec-layout-event-inner-' + glob.options.id + (" .stec-layout-event-inner-top-tabs li"), function (e) {
                    e.preventDefault();
                    parent.activateTab($(this));
                });

                $(document).on(helper.clickHandle(), '#stec-layout-event-inner-' + glob.options.id + (" .stec-layout-event-inner-close"), function (e) {
                    e.preventDefault();
                    $(this).parents('.stec-layout-event-inner-out-wrapper').remove();
                    $('body').removeClass('stec-modal-opened stec-no-scroll');
                });

                $(document).on(helper.clickHandle(), '#stec-layout-event-inner-' + glob.options.id, function (e) {
                    e.stopPropagation();
                });

                $(document).on(helper.clickHandle(), '#stec-layout-event-inner-out-wrapper-' + glob.options.id, function (e) {
                    e.preventDefault();
                    $(this).remove();
                    $('body').removeClass('stec-modal-opened stec-no-scroll');

                    $instance.trigger('onEventToggleClose');

                });

            },

            /**
             * Hides tab if event has no data for this tab
             * @param {type} $event the event jquery html object
             * @todo automate
             * @todo hasCustomTabs check not reliable
             */
            setEventTabs: function ($event) {

                // Remove unused tabs
                var event = calData.getEventById($event.attr('data-id'));
                var tabs = 0;
                var hasCustomTabs = $event.find('.stec-layout-event-inner-top-tabs-content > div').length > 8 ? true : false;

                // comments
                if (parseInt(event.comments, 10) === 0) {
                    $event.find('.stec-layout-event-inner-top-tabs [data-tab="stec-layout-event-inner-comments"]').remove();
                } else {
                    tabs++;
                }

                // location
                if (!event.location.full_address) {
                    $event.find('.stec-layout-event-inner-top-tabs [data-tab="stec-layout-event-inner-location"]').remove();
                } else {
                    tabs++;
                }

                // forecast
                if (!event.location.coordinates || !glob.options.general_settings.weather_api_key) {
                    $event.find('.stec-layout-event-inner-top-tabs [data-tab="stec-layout-event-inner-forecast"]').remove();
                } else {
                    tabs++;
                }

                // schedule
                if (event.schedule.length <= 0) {
                    $event.find('.stec-layout-event-inner-top-tabs [data-tab="stec-layout-event-inner-schedule"]').remove();
                } else {
                    tabs++;
                }

                // guests
                if (event.guests.length <= 0) {
                    $event.find('.stec-layout-event-inner-top-tabs [data-tab="stec-layout-event-inner-guests"]').remove();
                } else {
                    tabs++;
                }

                // attendance
                if (event.attendance.length <= 0) {
                    $event.find('.stec-layout-event-inner-top-tabs [data-tab="stec-layout-event-inner-attendance"]').remove();
                } else {
                    tabs++;
                }

                // woocommerce
                if (event.products.length <= 0) {
                    $event.find('.stec-layout-event-inner-top-tabs [data-tab="stec-layout-event-inner-shop"]').remove();
                } else {
                    tabs++;
                }

                if (tabs < 1 && false === hasCustomTabs) {
                    $event.find('.stec-layout-event-inner-top-tabs').hide();
                }
            },

            eventToggle: function ($event) {

                var event, id, external, $siblings, $modal, $container, target = '_blank';

                if (glob.options.general_settings.event_open_target) {
                    target = glob.options.general_settings.event_open_target;
                }

                switch (glob.options.general_settings.open_event_in) {
                    case 'single' :
                        window.open($event.data('permalink'), target);
                        return false;

                    case 'external' :
                        id = $event.attr('data-id');
                        event = calData.getEventById(id);
                        external = event.link.url;
                        if (external) {
                            window.open(external, target);
                            return false;
                        }
                        break;

                    case 'modal' :

                        // Add inner content
                        if ($event.find(".stec-layout-event-inner").length <= 0) {
                            $(glob.template.eventInner).addClass('stec-modal').appendTo($event);
                        }

                        this.setEventTabs($event);

                        event = calData.getEventById($event.attr('data-id'));

                        $modal = $event.find('.stec-modal').appendTo('body');
                        $modal.attr('id', 'stec-layout-event-inner-' + glob.options.id);
                        $modal.data('event-id', $event.attr('data-id'));
                        $modal.data('repeat-time-offset', $event.attr('data-repeat-time-offset'));
                        $modal.wrap('<div id="stec-layout-event-inner-out-wrapper-' + glob.options.id + '" class="stec-layout-event-inner-out-wrapper stec-modal"></div>');

                        $('<i class="stec-layout-event-inner-close fa fa-times"></i>').appendTo($modal);

                        // Load first tab if no tab is active
                        if ($modal.find('.stec-layout-event-inner-top-tabs').children('li.active').length <= 0) {
                            $modal.find('.stec-layout-event-inner-top-tabs').children('li').first().trigger(helper.clickHandle());
                        }

                        $(window).trigger("stec-trigger-resize");

                        $('body')
                                .addClass('stec-modal-opened stec-no-scroll');

                        $modal.show();
                        helper.focus($modal);

                        $instance.trigger('onEventToggleOpen', {
                            $instance: $instance,
                            $container: $modal,
                            event: event,
                            glob: glob,
                            helper: helper,
                            repeatOffset: $event.attr('data-repeat-time-offset')
                        });

                        break;

                    default:

                        if (!$event.hasClass("active")) {

                            // Add inner content
                            if ($event.find(".stec-layout-event-inner").length <= 0) {
                                $(glob.template.eventInner).appendTo($event);
                            }

                            // Set active 
                            $event
                                    .addClass("active")
                                    .find(".stec-layout-event-preview-right-event-toggle").addClass("active");

                            $siblings = $instance.$events.find(".stec-layout-event").not($event);

                            $siblings
                                    .removeClass("active")
                                    .find(".stec-layout-event-preview-right-event-toggle")
                                    .removeClass("active");


                            this.setEventTabs($event);

                            // Load first tab if no tab is active
                            if ($event.find('.stec-layout-event-inner-top-tabs').children('li.active').length <= 0) {
                                $event.find('.stec-layout-event-inner-top-tabs').children('li').first().trigger(helper.clickHandle());
                            }

                            event = calData.getEventById($event.attr('data-id'));

                            $container = $event.find('.stec-layout-event-inner');
                            $container.data('event-id', $event.attr('data-id'));
                            $container.data('repeat-time-offset', $event.attr('data-repeat-time-offset'));

                            $instance.trigger('onEventToggleOpen', {
                                $instance: $instance,
                                $container: $container,
                                event: event,
                                glob: glob,
                                helper: helper,
                                repeatOffset: $event.attr('data-repeat-time-offset')
                            });

                            // Focus on event
                            helper.focus($event);

                        } else {

                            $event
                                    .removeClass("active")
                                    .find(".stec-layout-event-preview-right-event-toggle").removeClass("active");

                            $instance.trigger('onEventToggleClose');
                        }
                }

            },

            activateTab: function ($tab) {

                var tabClass;

                if ($tab.hasClass("active")) {
                    return;
                }

                $tab.addClass("active")
                        .siblings()
                        .removeClass("active");

                // remove only children active
                $tab
                        .parents(".stec-layout-event-inner")
                        .find(".stec-layout-event-inner-top-tabs-content")
                        .children('.active')
                        .removeClass("active");

                tabClass = "." + $tab.attr("data-tab");

                $tab
                        .parents(".stec-layout-event-inner")
                        .find(tabClass)
                        .addClass("active");

                $(document).trigger('stec-tab-click-' + glob.options.id);

            },

            eventHolder: {

                getEventHolder: function () {

                    var $eventHolder = "";

                    switch (glob.options.view) {

                        case "agenda":
                            $eventHolder = $instance.$agenda.find(".stec-event-holder");
                            break;

                        case "month":
                            $eventHolder = $instance.$month.find(".stec-event-holder");
                            break;

                        case "week":
                            $eventHolder = $instance.$week.find(".stec-event-holder");
                            break;

                        case "day":
                            $eventHolder = $instance.$day.find(".stec-event-holder");
                            break;

                        case "map":
                            $eventHolder = $instance.$map.find(".stec-event-holder");
                            break;
                    }

                    return $eventHolder;
                },

                removeEvents: function () {

                    // prevents duplicate triggers  
                    $(document).unbind('stec-tab-click-' + glob.options.id);

                    // resize.stec-unbind-window-resize-on-event-close
                    // prevents duplicate triggers 
                    $(window).unbind('resize.stec-unbind-window-resize-on-event-close-' + glob.options.id);

                    $instance.$events.not('.stec-layout-agenda-events-all').children().remove();
                },

                /**
                 * Displays events for active date
                 * @param {array} eventsArray array of events
                 * @returns bool true or false if no events
                 */
                displayEvents: function (eventsArray) {

                    var d = helper.newDate(glob.options.year, glob.options.month, glob.options.day);
                    var events = eventsArray ? eventsArray : calData.getEvents(d, false);

                    if (events.length <= 0) {
                        // no events
                        return false;
                    }

                    $(events).each(function () {

                        var event = this;
                        var extraClass = [];
                        var date;
                        var gmtUtcOffset;
                        var timezoneOffsetLabel;
                        var icon = [];
                        var d1;
                        var eventState = helper.getEventState(this);

                        if (eventState > 0 || 'EventCancelled' === event.event_status || '0' === glob.options.general_settings.reminder) {
                            extraClass.push('stec-layout-event-no-reminder');
                        }

                        date = helper.beautifyTimespan(event.start_date, parseInt(event.hide_end, 10) === 1 ? false : event.end_date, event.all_day);

                        gmtUtcOffset = parseInt(event.timezone_utc_offset, 10) / 3600;
                        gmtUtcOffset = gmtUtcOffset > 0 ? '+' + gmtUtcOffset : gmtUtcOffset;

                        if (gmtUtcOffset === 0) {
                            gmtUtcOffset = '';
                        }

                        timezoneOffsetLabel = parseInt(glob.options.general_settings.date_label_gmtutc, 10) === 0 ? '' : 'UTC/GMT ' + gmtUtcOffset;

                        switch (parseInt(this.featured, 10)) {
                            case 1:
                                extraClass.push('stec-event-featured');
                                break;

                            case 2:
                                extraClass.push('stec-event-featured stec-event-featured-bg');
                                break;
                        }

                        if ('EventCancelled' === event.event_status) {
                            extraClass.push('stec-event-cancelled');
                        }

                        if (event.icon_type === 'icon' && (!event.icon || event.icon === 'fa')) {
                            extraClass.push('stec-no-icon');
                        }

                        if (!event.edit_link) {
                            extraClass.push('stec-no-edit');
                        }

                        if (event.description_short && '1' === glob.options.general_settings.preview_desc ||
                                event.location.full_address && '1' === glob.options.general_settings.preview_location) {
                            extraClass.push('stec-layout-event-preview-size-big');
                        }

                        switch (event.icon_type) {

                            case 'image' :

                                if (event.icon_images.length > 0) {
                                    icon.push('<div class="stec-layout-event-preview-left-icon-images">');

                                    $.each(event.icon_images_meta, function (i) {
                                        icon.push('<div class="stec-layout-event-preview-left-icon-image ' + (i === 0 ? 'active' : '') + '" style="background-image: url(' + this.src + ')"></div>');
                                    });
                                    icon.push('</div>');

                                } else {
                                    icon.push('<i class="' + event.icon + '"></i>');
                                }

                                break;

                            case 'date' :
                                d1 = moment(event.start_date);

                                icon.push('<span class="stec-layout-event-preview-left-icon-date"><span class="stec-layout-event-preview-left-icon-date-number">' + d1.format('D') + '</span><span class="stec-layout-event-preview-left-icon-date-month">' + __(d1.format('MMM').toLowerCase()) + '</span></span>');

                                break;

                            default:
                                icon.push('<i class="' + event.icon + '"></i>');

                        }

                        $(glob.template.event)
                                .addClass(extraClass.join(' '))
                                .attr('data-id', event.id)
                                .attr('data-repeat-time-offset', event.repeat_time_offset ? event.repeat_time_offset : 0)
                                .data('permalink', helper.getPermalink(event.permalink, event.repeat_time_offset))
                                .html(function (index, html) {

                                    var $html = $(html);
                                    var eventStatusHtml = [];

                                    if (!event.location.full_address ||
                                            '1' !== glob.options.general_settings.preview_location) {
                                        $html.find('.stec-layout-event-preview-left-text-location').remove();
                                    }

                                    if (!event.description_short || '1' !== glob.options.general_settings.preview_desc) {
                                        $html.find('.stec-layout-event-preview-left-text-desc').remove();
                                    }

                                    glob.temp.EventPreviewHtml = $html.get(0).outerHTML;

                                    $instance.trigger('stecBeforeReturnEventPreviewHtml', {
                                        $instance: $instance,
                                        event: event,
                                        temp: glob.temp
                                    });

                                    switch (event.event_status) {

                                        case 'EventCancelled' :
                                            eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="cancelled">' + __('Cancelled') + '</span></p>');
                                            break;

                                        case 'EventMovedOnline' :
                                            eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="moved-online">' + __('Moved Online') + '</span></p>');
                                            break;

                                        case 'EventPostponed' :
                                            eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="postponed">' + __('Postponed') + '</span></p>');
                                            break;

                                        case 'EventRescheduled' :
                                            eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="rescheduled">' + __('Rescheduled') + '</span></p>');
                                            break;

                                        default:
                                            eventStatusHtml = [];
                                    }

                                    if (event.approved === 0) {
                                        eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="unapproved">' + __('Awaiting Approval') + '</span></p>');
                                    }

                                    return glob.temp.EventPreviewHtml
                                            .replace('stec_replace_event_status', eventStatusHtml.join(''))
                                            .replace('stec_replace_summary', event.title)
                                            .replace('stec_replace_location', function () {

                                                if ('virtual' === event.location.type) {
                                                    return __('Online Event');
                                                }

                                                if (event.location.title) {
                                                    return [event.location.title, event.location.full_address].join(', ');
                                                }

                                                return event.location.full_address;
                                            })
                                            .replace('stec_replace_desc_short', event.description_short)
                                            .replace('stec_replace_date', date + ' ' + timezoneOffsetLabel)
                                            .replace('stec_replace_event_background=""', ' style="background:' + event.color + '" ') // edge is retarded
                                            .replace('stec_replace_icon', icon.join(''))
                                            .replace(/#stec_replace_edit_link/g, event.edit_link);

                                }).appendTo($instance.$events.not('.stec-layout-agenda-events-all'));

                    });

                    switch (glob.options.general_settings.open_event_in) {
                        case 'single':
                        case 'external' :
                        case 'modal' :
                            // Remove + when single pages
                            $instance.$events
                                    .not('.stec-layout-agenda-events-all')
                                    .find('.stec-layout-event-preview-right-event-toggle')
                                    .remove();
                            break;

                    }

                    return true;
                },

                close: function () {

                    $instance.$events
                            .not('.stec-layout-agenda-events-all')
                            .find(".active")
                            .removeClass("active");

                    this.getEventHolder().hide();

                    // last visible children fix for border-radius
                    if (glob.options.view === "month") {
                        $instance.$month.find(".stec-layout-month-weekrow").last().addClass("stec-layout-month-weekrow-last");
                    }

                    if (glob.options.view === "week") {
                        $instance.$week.find(".stec-layout-week-weekrow").addClass("stec-layout-week-weekrow-last");
                    }

                    helper.extendBind("stachethemes_ec_extend", "onEventHolderClose");

                    // Remove inner content
                    this.removeEvents();
                },

                open: function () {

                    var result;

                    this.close();

                    result = this.displayEvents();

                    // Day layout specifics
                    if (glob.options.view === 'day') {

                        if (result === false) {
                            $instance.$day.find('.stec-layout-day-noevents').show();
                        } else {
                            $instance.$day.find('.stec-layout-day-noevents').hide();
                        }

                    }

                    // If create form is disabled and there are no events do not open the event holder
                    if (result === false && parseInt(glob.options.general_settings.show_create_event_form, 10) === 0) {
                        return;
                    }

                    // If there is event holder...

                    // Month layout specifics
                    if (glob.options.view === "month") {

                        // last visible children fix...

                        if ($instance.$month.find(".stec-layout-month-eventholder").is(":last-child")) {
                            $instance.$month.find(".stec-layout-month-weekrow-last").removeClass("stec-layout-month-weekrow-last");
                        } else {
                            $instance.$month.find("tr").last().addClass("stec-layout-month-weekrow-last");
                        }

                        // focus on event
                        helper.focus($instance.$month.find('.stec-layout-month-daycell.active'));
                    }

                    // Week layout specifics
                    if (glob.options.view === "week") {
                        $instance.$week.find(".stec-layout-week-weekrow-last").removeClass("stec-layout-week-weekrow-last");
                    }

                    helper.extendBind("stachethemes_ec_extend", "onEventHolderOpen");

                    helper.animate.eventHolder.open(this.getEventHolder());

                },

                openEvents: function (eventsArray, callback, instant) {
                    var result;

                    this.close();

                    result = this.displayEvents(eventsArray);

                    // Day layout specifics
                    if (glob.options.view === 'day') {

                        if (result === false) {
                            $instance.$day.find('.stec-layout-day-noevents').show();
                        } else {
                            $instance.$day.find('.stec-layout-day-noevents').hide();
                        }

                    }

                    // If create form is disabled and there are no events do not open the event holder
                    if (result === false && parseInt(glob.options.general_settings.show_create_event_form, 10) === 0) {
                        return;
                    }


                    // If there is event holder...

                    // Month layout specifics
                    if (glob.options.view === "month") {

                        // last visible children fix...

                        if ($instance.$month.find(".stec-layout-month-eventholder").is(":last-child")) {
                            $instance.$month.find(".stec-layout-month-weekrow-last").removeClass("stec-layout-month-weekrow-last");
                        } else {
                            $instance.$month.find("tr").last().addClass("stec-layout-month-weekrow-last");
                        }

                        // focus on event
                        helper.focus($instance.$month.find('.stec-layout-month-daycell.active'));
                    }

                    // Week layout specifics
                    if (glob.options.view === "week") {
                        $instance.$week.find(".stec-layout-week-weekrow-last").removeClass("stec-layout-week-weekrow-last");
                    }

                    helper.extendBind("stachethemes_ec_extend", "onEventHolderOpen");

                    helper.animate.eventHolder.open(this.getEventHolder(), callback, instant);

                }
            }
        };

        /**
         * handles top bar
         */
        var top = {

            dropDownScrollSpeed: 1000,

            init: function () {
                this.bindControls();
                this.bindMobile();

                if (parseInt(glob.options.general_settings.show_calfilter, 10) === 2) {
                    this.setStaticFilterMenu();
                }

                if ($instance.$top.find('.stec-top-menu').children().length <= 0) {
                    $instance.$top.find('.stec-top-menu').remove();
                }
            },

            setStaticFilterMenu: function () {

                var $filterIcon = $instance.$top.find('.stec-top-menu-filter');
                var $filterMenu = $instance.$top.find('.stec-top-menu-filter-dropdown');

                $filterIcon.remove();
                $filterMenu.addClass('active static');
            },

            bindMobile: function () {

                if (!helper.isMobile()) {
                    return;
                }

                $(document).on(helper.clickHandle(), function () {

                    $instance.$top.find('.mobile-hover').removeClass('mobile-hover');

                });

                $instance.$top.find('.stec-top-dropmenu-layouts').children('li').on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                    $(this).toggleClass('mobile-hover');

                    // Hide filter menu
                    $instance.find('.stec-top-menu-filter').removeClass('active');
                    $instance.find('.stec-top-menu-filter-dropdown:not(.static)').removeClass('active');
                    $instance.find('.stec-top-menu-filter-dropdown')
                            .children('.stec-style-flexbox')
                            .children()
                            .removeClass('active');
                    $instance.find('.stec-top-menu-filter-dropdown > div > div')
                            .find('> div > i:last-child')
                            .removeClass('fa-chevron-down')
                            .addClass('fa-chevron-right');
                });

                $instance.$top.find('.stec-top-menu-date').children('li').on(helper.clickHandle(), function (e) {

                    var html = [], theMonth, i, d, maxMonthDays, j, weekNumber, theWeek, format, label, y;

                    e.preventDefault();

                    $(this).toggleClass('mobile-hover');

                    // on active; fill dropdowns
                    if ($(this).hasClass('mobile-hover')) {
                        e.stopPropagation();
                        // dropdown day display
                        if ($(this).hasClass('stec-top-menu-date-day')) {

                            $(this).prop("hovering", true);

                            $(this).find(".stec-top-menu-date-dropdown ul").remove();

                            d = glob.options.day;

                            html.push('<ul>');

                            theMonth = helper.getMonthInfo();
                            maxMonthDays = theMonth.monthLength;

                            // center current year
                            for (j = 0; j < 3; j++) {
                                d = d - 1 < 1 ? maxMonthDays : d - 1;
                            }

                            for (i = 0; i < 5; i++) {

                                d = d + 1 > maxMonthDays ? 1 : d + 1;

                                html.push('<li><p data-day="' + d + '">' + d + '</p></li>');
                            }
                            html.push('</ul>');

                            $(this).find(".stec-top-menu-date-control-up").after(html.join(''));

                            top.dateDropdownSetActive();
                        }

                        // Dropdown week display
                        if ($(this).hasClass('stec-top-menu-date-week')) {

                            $(this).find(".stec-top-menu-date-dropdown ul").remove();

                            weekNumber = helper.newDate(glob.options.year, glob.options.month, glob.options.day);
                            weekNumber.setDate(weekNumber.getDate() - 3 * 7);

                            format = '';

                            switch (glob.options.general_settings.date_format) {
                                case 'dd.mm.yyyy' :
                                    format = 'sday. smonth - eday. emonth. eyear';
                                    break;
                                case 'dd-mm-yy' :
                                    format = 'sday smonth - eday emonth eyear';
                                    break;
                                case 'mm-dd-yy' :
                                    format = 'smonth sday - emonth eday eyear';
                                    break;
                                case 'yy-mm-dd' :
                                default:
                                    format = 'smonth sday - eyear emonth eday';
                                    break;
                            }

                            html.push('<ul>');
                            for (i = 0; i < 5; i++) {
                                weekNumber.setDate(weekNumber.getDate() + 7);
                                theWeek = helper.getWeekInfo(weekNumber);
                                label = helper.getWeekLabel(format, theWeek);
                                html.push('<li><p data-week="' + theWeek.week + '" data-date="' + theWeek.start.year + "-" + theWeek.start.month + "-" + theWeek.start.day + '">' + label + '</p></li>');
                            }
                            html.push('</ul>');

                            $(this).find(".stec-top-menu-date-control-up").after(html.join(''));

                            top.dateDropdownSetActive();
                        }

                        // Dropdown month display
                        if ($(this).hasClass('stec-top-menu-date-month')) {

                            $(this).find(".stec-top-menu-date-dropdown ul").remove();

                            theMonth = glob.options.month;

                            // center current month
                            for (i = 0; i < 3; i++) {
                                theMonth = theMonth - 1 < 0 ? 11 : theMonth - 1;
                            }

                            html.push('<ul>');
                            for (i = 0; i < 5; i++) {
                                theMonth = theMonth + 1 > 11 ? 0 : theMonth + 1;
                                html.push('<li><p data-month="' + theMonth + '">' + glob.options.monthLabels[theMonth] + '</p></li>');
                            }
                            html.push('</ul>');

                            $(this).find(".stec-top-menu-date-control-up").after(html.join(''));

                            top.dateDropdownSetActive();
                        }


                        // Dropdown year display
                        if ($(this).hasClass('stec-top-menu-date-year')) {

                            $(this).find(".stec-top-menu-date-dropdown ul").remove();

                            y = glob.options.year;

                            html.push('<ul>');

                            // center current year
                            y = y - 3;

                            for (i = 0; i < 5; i++) {
                                y++;
                                html.push('<li><p data-year="' + y + '">' + y + '</p></li>');
                            }
                            html.push('</ul>');

                            $(this).find(".stec-top-menu-date-control-up").after(html.join(''));

                            top.dateDropdownSetActive();
                        }

                    }

                });

                $instance.$top.find('.stec-top-menu-date-control-up, .stec-top-menu-date-control-down').on(helper.clickHandle(), function (e) {
                    e.stopPropagation();
                });

            },

            bindControls: function () {

                var parent = this;

                // Close all menus when clicking outside them
                $(document).on(helper.clickHandle(), function () {
                    $instance.find('.stec-top-menu-filter').removeClass('active');
                    $instance.find('.stec-top-menu-filter-dropdown:not(.static)').removeClass('active');
                    $instance.find('.stec-top-menu-search').removeClass('active');
                });

                // Click handle for top view buttons
                $instance.$top.find('[data-view]').on(helper.clickHandle(), function () {
                    glob.options.view = $(this).attr("data-view");
                    main.refreshState('layout');
                });

                // Mousewheel for year dropdown
                if ($instance.$top.find(".stec-top-menu-date-year").length > 0) {
                    $instance.$top.find(".stec-top-menu-date-year").get(0).addEventListener('wheel', function (e) {
                        e.preventDefault();

                        if (e.wheelDelta > 0 || e.detail < 0) {
                            parent.dateYearScrollUp();
                        } else {
                            parent.dateYearScrollDown();
                        }

                    }, {passive: false});
                }

                // Mousewheel for month dropdown
                if ($instance.$top.find(".stec-top-menu-date-month").length > 0) {
                    $instance.$top.find(".stec-top-menu-date-month").get(0).addEventListener('wheel', function (e) {

                        e.preventDefault();

                        if (e.wheelDelta > 0 || e.detail < 0) {
                            parent.dateMonthScrollUp();
                        } else {
                            parent.dateMonthScrollDown();
                        }
                    }, {passive: false});
                }

                // Mousewheel for week dropdown
                if ($instance.$top.find(".stec-top-menu-date-week").length > 0) {
                    $instance.$top.find(".stec-top-menu-date-week").get(0).addEventListener('wheel', function (e) {

                        e.preventDefault();

                        if (e.wheelDelta > 0 || e.detail < 0) {
                            parent.dateWeekScrollUp();
                        } else {
                            parent.dateWeekScrollDown();
                        }
                    }, {passive: false});
                }

                // Mousewheel for day dropdown
                if ($instance.$top.find(".stec-top-menu-date-day").length > 0) {
                    $instance.$top.find(".stec-top-menu-date-day").get(0).addEventListener('wheel', function (e) {

                        e.preventDefault();

                        if (e.wheelDelta > 0 || e.detail < 0) {
                            parent.dateDayScrollUp();
                        } else {
                            parent.dateDayScrollDown();
                        }
                    }, {passive: false});
                }

                // Remove hovering from dropdown menu on mouseout
                $instance.$top.find(".stec-top-menu-date-year, .stec-top-menu-date-month, .stec-top-menu-date-week, .stec-top-menu-date-day").on("mouseleave", function () {
                    $(this).prop("hovering", null);
                });

                // Dropdown day mouseenter display
                $instance.$top.find(".stec-top-menu-date-day").on("mouseenter", function () {

                    var html = [], dayNumber, theMonth, maxD, i, j;

                    if ($(this).prop("hovering")) {
                        return;
                    }

                    $(this).prop("hovering", true);

                    $(this).find(".stec-top-menu-date-dropdown ul").remove();

                    dayNumber = glob.options.day;

                    html.push('<ul>');

                    theMonth = helper.getMonthInfo();
                    maxD = theMonth.monthLength;

                    // center current year
                    for (j = 0; j < 3; j++) {
                        dayNumber = dayNumber - 1 < 1 ? maxD : dayNumber - 1;
                    }

                    for (i = 0; i < 5; i++) {

                        dayNumber = dayNumber + 1 > maxD ? 1 : dayNumber + 1;

                        html.push('<li><p data-day="' + dayNumber + '">' + dayNumber + '</p></li>');
                    }
                    html.push('</ul>');

                    $(this).find(".stec-top-menu-date-control-up").after(html.join(''));

                    top.dateDropdownSetActive();
                });

                // Dropdown week mouseenter display
                $instance.$top.find(".stec-top-menu-date-week").on("mouseenter", function () {

                    var format, weekNumber, i, html = [], theWeek, label;

                    if ($(this).prop("hovering")) {
                        return;
                    }

                    $(this).prop("hovering", true);

                    $(this).find(".stec-top-menu-date-dropdown ul").remove();


                    weekNumber = helper.newDate(glob.options.year, glob.options.month, glob.options.day);

                    for (let x = 0; x < 3; x++) {

                        if (weekNumber.getFullYear() <= 0) {
                            break;
                        }

                        weekNumber.setDate(weekNumber.getDate() - 7);
                    }

                    format = '';

                    switch (glob.options.general_settings.date_format) {
                        case 'dd.mm.yyyy' :
                            format = 'sday. smonth - eday. emonth. eyear';
                            break;
                        case 'dd-mm-yy' :
                            format = 'sday smonth - eday emonth eyear';
                            break;
                        case 'mm-dd-yy' :
                            format = 'smonth sday - emonth eday eyear';
                            break;
                        case 'yy-mm-dd' :
                        default:
                            format = 'smonth sday - eyear emonth eday';
                            break;
                    }

                    html.push('<ul>');
                    for (i = 0; i < 5; i++) {

                        weekNumber.setDate(weekNumber.getDate() + 7);
                        theWeek = helper.getWeekInfo(weekNumber);
                        label = helper.getWeekLabel(format, theWeek);

                        html.push('<li><p data-week="' + theWeek.week + '" data-date="' + theWeek.start.year + "-" + theWeek.start.month + "-" + theWeek.start.day + '">' + label + '</p></li>');
                    }
                    html.push('</ul>');

                    $(this).find(".stec-top-menu-date-control-up").after(html.join(''));

                    top.dateDropdownSetActive();
                });

                // Dropdown month mouseenter display
                $instance.$top.find(".stec-top-menu-date-month").on("mouseenter", function () {

                    var html = [], monthNumber, i;

                    if ($(this).prop("hovering")) {
                        return;
                    }

                    $(this).prop("hovering", true);

                    $(this).find(".stec-top-menu-date-dropdown ul").remove();

                    monthNumber = glob.options.month;

                    // center current month
                    for (i = 0; i < 3; i++) {
                        monthNumber = monthNumber - 1 < 0 ? 11 : monthNumber - 1;
                    }

                    html.push('<ul>');
                    for (i = 0; i < 5; i++) {
                        monthNumber = monthNumber + 1 > 11 ? 0 : monthNumber + 1;
                        html.push('<li><p data-month="' + monthNumber + '">' + glob.options.monthLabels[monthNumber] + '</p></li>');
                    }
                    html.push('</ul>');

                    $(this).find(".stec-top-menu-date-control-up").after(html.join(''));

                    top.dateDropdownSetActive();
                });

                // Dropdown year mouseenter display
                $instance.$top.find(".stec-top-menu-date-year").on("mouseenter", function () {

                    var html = [], y, i;

                    if ($(this).prop("hovering")) {
                        return;
                    }

                    $(this).prop("hovering", true);

                    $(this).find(".stec-top-menu-date-dropdown ul").remove();

                    y = glob.options.year;

                    html.push('<ul>');

                    // center current year
                    for (let x = 0; x < 3; x++) {
                        if (y <= 0) {
                            break;
                        }
                        y--;
                    }

                    for (i = 0; i < 5; i++) {
                        y++;
                        html.push('<li><p data-year="' + y + '">' + y + '</p></li>');
                    }
                    html.push('</ul>');

                    $(this).find(".stec-top-menu-date-control-up").after(html.join(''));

                    top.dateDropdownSetActive();
                });

                // Dropdown week control up
                $instance.$top.find(".stec-top-menu-date-week .stec-top-menu-date-control-up").on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.dateWeekScrollUp();
                });

                // Dropdown week control down
                $instance.$top.find(".stec-top-menu-date-week .stec-top-menu-date-control-down").on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.dateWeekScrollDown();
                });

                // Dropdown year control up
                $instance.$top.find(".stec-top-menu-date-year .stec-top-menu-date-control-up").on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.dateYearScrollUp();
                });

                // Dropdown year control down
                $instance.$top.find(".stec-top-menu-date-year .stec-top-menu-date-control-down").on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.dateYearScrollDown();
                });

                // Dropdown month control up
                $instance.$top.find(".stec-top-menu-date-month .stec-top-menu-date-control-up").on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.dateMonthScrollUp();
                });

                // Dropdown month control down
                $instance.$top.find(".stec-top-menu-date-month .stec-top-menu-date-control-down").on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.dateMonthScrollDown();
                });

                // Dropdown day control up
                $instance.$top.find(".stec-top-menu-date-day .stec-top-menu-date-control-up").on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.dateDayScrollUp();
                });

                // Dropdown day control down
                $instance.$top.find(".stec-top-menu-date-day .stec-top-menu-date-control-down").on(helper.clickHandle(), function (e) {
                    e.preventDefault();
                    parent.dateDayScrollDown();
                });

                // Dropdown month li pick
                $(document).on(helper.clickHandle(), $instance.$top.path + ".stec-top-menu-date-month ul li", function (e) {

                    var month, theMonth;

                    e.preventDefault();

                    month = $(this).find("[data-month]").attr("data-month");

                    glob.options.month = parseInt(month, 10);

                    theMonth = helper.getMonthInfo();

                    if (glob.options.day > theMonth.monthLength) {
                        glob.options.day = parseInt(theMonth.monthLength, 10);
                    }

                    main.refreshState('layout');

                });

                // Dropdown year li pick
                $(document).on(helper.clickHandle(), $instance.$top.path + ".stec-top-menu-date-year ul li", function (e) {

                    var year;

                    e.preventDefault();

                    year = $(this).find("[data-year]").attr("data-year");

                    glob.options.year = parseInt(year, 10);

                    main.refreshState('layout');

                });

                // Dropdown week li pick
                $(document).on(helper.clickHandle(), $instance.$top.path + ".stec-top-menu-date-week ul li", function (e) {

                    var d;

                    e.preventDefault();

                    d = helper.getDateFromData($(this).find("[data-date]").attr("data-date"));

                    glob.options.year = d.getFullYear();
                    glob.options.month = d.getMonth();
                    glob.options.day = d.getDate();

                    main.refreshState('layout');

                });

                // Dropdown day li pick
                $(document).on(helper.clickHandle(), $instance.$top.path + ".stec-top-menu-date-day ul li", function (e) {

                    e.preventDefault();

                    glob.options.day = parseInt($(this).find("[data-day]").attr("data-day"), 10);

                    main.refreshState('layout');

                });

                // Previous date depending on view layout
                $instance.$top.find("[data-action='previous']").on(helper.clickHandle(), function (e) {

                    var theMonth, theWeek, next;

                    e.preventDefault();

                    switch (glob.options.view) {

                        case "boxgrid":
                        case "grid":
                        case "agenda" :

                            glob.options.month = glob.options.month - 1;

                            if (glob.options.month < 0) {
                                glob.options.month = 11;
                                glob.options.year = glob.options.year - 1;
                            }

                            glob.options.day = 1;

                            if (glob.options.year <= 0) {
                                glob.options.day = 1;
                                glob.options.month = 0;
                                glob.options.year = 1;
                            }

                            main.refreshState('layout');

                            break;

                        case "map" :
                        case "month" :

                            glob.options.month = glob.options.month - 1;

                            if (glob.options.month < 0) {
                                glob.options.month = 11;
                                glob.options.year = glob.options.year - 1;
                            }

                            if (glob.options.year <= 0) {
                                glob.options.day = 1;
                                glob.options.month = 0;
                                glob.options.year = 1;
                            }

                            main.refreshState('layout');

                            break;

                        case "week" :

                            theWeek = helper.getWeekInfo();
                            next = helper.newDate(theWeek.start.year, theWeek.start.month, theWeek.start.day - 7);

                            glob.options.year = next.getFullYear();
                            glob.options.month = next.getMonth();
                            glob.options.day = next.getDate();

                            if (glob.options.year <= 0) {
                                glob.options.day = 1;
                                glob.options.month = 0;
                                glob.options.year = 1;
                            }

                            main.refreshState('layout');

                            break;

                        case "day" :

                            if (glob.options.day - 1 < 1) {

                                if (glob.options.month - 1 < 0) {

                                    glob.options.year = glob.options.year - 1;
                                    glob.options.month = 11;

                                    theMonth = helper.getMonthInfo();
                                    glob.options.day = theMonth.monthLength;

                                } else {

                                    glob.options.month = glob.options.month - 1;

                                    theMonth = helper.getMonthInfo();
                                    glob.options.day = theMonth.monthLength;
                                }

                            } else {
                                glob.options.day = glob.options.day - 1;
                            }

                            if (glob.options.year <= 0) {
                                glob.options.day = 1;
                                glob.options.month = 0;
                                glob.options.year = 1;
                            }

                            main.refreshState('layout');
                            break;
                    }
                });

                // Next date depending on view layout
                $instance.$top.find("[data-action='next']").on(helper.clickHandle(), function (e) {

                    var theWeek, next, theMonth;

                    e.preventDefault();

                    switch (glob.options.view) {

                        case "boxgrid":
                        case "grid":
                        case "agenda" :

                            glob.options.month = glob.options.month + 1;

                            if (glob.options.month > 11) {
                                glob.options.month = 0;
                                glob.options.year = glob.options.year + 1;
                            }

                            glob.options.day = 1;

                            main.refreshState('layout');

                            break;

                        case "map" :
                        case "month" :

                            glob.options.month = glob.options.month + 1;

                            if (glob.options.month > 11) {
                                glob.options.month = 0;
                                glob.options.year = glob.options.year + 1;
                            }

                            main.refreshState('layout');

                            break;

                        case "week" :

                            theWeek = helper.getWeekInfo();
                            next = helper.newDate(theWeek.start.year, theWeek.start.month, theWeek.start.day + 7);

                            glob.options.year = next.getFullYear();
                            glob.options.month = next.getMonth();
                            glob.options.day = next.getDate();

                            main.refreshState('layout');

                            break;

                        case "day" :

                            theMonth = helper.getMonthInfo();

                            if (glob.options.day + 1 > theMonth.monthLength) {

                                if (glob.options.month + 1 > 11) {

                                    glob.options.year = glob.options.year + 1;
                                    glob.options.month = 0;
                                    glob.options.day = 1;

                                } else {

                                    glob.options.month = glob.options.month + 1;
                                    glob.options.day = 1;
                                }

                            } else {
                                glob.options.day = glob.options.day + 1;
                            }

                            main.refreshState('layout');
                            break;
                    }
                });

                // Today date depending on view layout
                $instance.$top.find("[data-action='today']").on(helper.clickHandle(), function (e) {

                    var today;
                    e.preventDefault();

                    today = new Date();

                    glob.options.year = today.getFullYear();
                    glob.options.month = today.getMonth();
                    glob.options.day = today.getDate();

                    main.refreshState('layout');

                });

            },

            /**
             * Set all top data
             */
            set: function () {
                // today events count
                var events = []; //calData.getEvents(now);
                var rad;
                var theWeek;
                var label;
                var format;

                if (events.length > 0) {
                    $instance.$top.find('.stec-top-menu-count').text(events.length);
                    $instance.$top.find('.stec-top-menu-count').show();
                } else {
                    $instance.$top.find('.stec-top-menu-count').hide();
                }

                // Set .active current view 
                $instance.$top.find("[data-view]").removeClass("active");
                $instance.$top.find("[data-view='" + glob.options.view + "']").addClass("active");

                // Hide date menu
                $instance.$top.find(".stec-top-menu-date-month").hide();
                $instance.$top.find(".stec-top-menu-date-year").hide();
                $instance.$top.find(".stec-top-menu-date-week").hide();
                $instance.$top.find(".stec-top-menu-date-day").hide();

                // Show date menu for the active view
                // Sets date label visible on small devices only
                switch (glob.options.view) {

                    case "boxgrid":
                    case "grid":
                    case "agenda" :

                        rad = $instance.$top.find(".stec-top-menu-date-year").css("border-top-right-radius");

                        $instance.$top.find(".stec-top-menu-date-month, .stec-top-menu-date-month .stec-top-menu-date-dropdown").css({
                            borderTopLeftRadius: rad,
                            borderBottomLeftRadius: rad
                        });

                        $instance.$top.find(".stec-top-menu-date-month").show();
                        $instance.$top.find(".stec-top-menu-date-year").show();

                        $instance.$top.find(".stec-top-menu-date-month > [data-month]")
                                .attr("data-month", glob.options.month)
                                .text(glob.options.monthLabels[glob.options.month]);

                        $instance.$top.find(".stec-top-menu-date-day").show();
                        $instance.$top.find(".stec-top-menu-date-day > [data-day]")
                                .attr("data-month", glob.options.day)
                                .text(glob.options.day);

                        $instance.find(".stec-top-menu-date-year > [data-year]")
                                .attr("data-year", glob.options.year)
                                .text(glob.options.year);


                        switch (glob.options.general_settings.date_format) {
                            case 'dd.mm.yyyy' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.day + ". " + glob.options.monthLabels[glob.options.month] + " " + glob.options.year);
                                break;
                            case 'dd-mm-yy' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.day + " " + glob.options.monthLabels[glob.options.month] + " " + glob.options.year);
                                break;
                            case 'mm-dd-yy' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.monthLabels[glob.options.month] + " " + glob.options.day + " " + glob.options.year);
                                break;
                            case 'yy-mm-dd' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.year + " " + glob.options.monthLabels[glob.options.month] + " " + glob.options.day);
                                break;
                        }

                        break;

                    case "map" :
                    case "month" :

                        rad = $instance.$top.find(".stec-top-menu-date-year").css("border-top-right-radius");

                        $instance.$top.find(".stec-top-menu-date-month, .stec-top-menu-date-month .stec-top-menu-date-dropdown").css({
                            borderTopLeftRadius: rad,
                            borderBottomLeftRadius: rad
                        });

                        $instance.$top.find(".stec-top-menu-date-month").show();
                        $instance.$top.find(".stec-top-menu-date-year").show();

                        $instance.$top.find(".stec-top-menu-date-month > [data-month]")
                                .attr("data-month", glob.options.month)
                                .text(glob.options.monthLabels[glob.options.month]);

                        $instance.find(".stec-top-menu-date-year > [data-year]")
                                .attr("data-year", glob.options.year)
                                .text(glob.options.year);


                        switch (glob.options.general_settings.date_format) {
                            case 'dd.mm.yyyy' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.monthLabels[glob.options.month] + " " + glob.options.year);
                                break;
                            case 'dd-mm-yy' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.monthLabels[glob.options.month] + " " + glob.options.year);
                                break;
                            case 'mm-dd-yy' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.monthLabels[glob.options.month] + " " + glob.options.year);
                                break;
                            case 'yy-mm-dd' :
                            default:
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.year + " " + glob.options.monthLabels[glob.options.month]);
                                break;
                        }


                        break;

                    case "week" :

                        theWeek = helper.getWeekInfo();

                        if (theWeek.start.year === theWeek.end.year) {
                            if (theWeek.start.month === theWeek.end.month) {

                                switch (glob.options.general_settings.date_format) {
                                    case 'dd.mm.yyyy' :
                                        format = 'sday. - eday. emonth. eyear';
                                        break;
                                    case 'dd-mm-yy' :
                                        format = 'sday - eday emonth eyear';
                                        break;
                                    case 'mm-dd-yy' :
                                        format = 'sday - emonth eday eyear';
                                        break;
                                    case 'yy-mm-dd' :
                                    default:
                                        format = 'sday - eyear emonth eday';
                                        break;
                                }

                            } else {

                                switch (glob.options.general_settings.date_format) {
                                    case 'dd.mm.yyyy' :
                                        format = 'sday. smonth. - eday. emonth. eyear';
                                        break;
                                    case 'dd-mm-yy' :
                                        format = 'sday smonth - eday emonth eyear';
                                        break;
                                    case 'mm-dd-yy' :
                                        format = 'smonth sday - emonth eday eyear';
                                        break;
                                    case 'yy-mm-dd' :
                                    default:
                                        format = 'smonth sday - eyear emonth eday';
                                        break;
                                }

                            }

                        } else {

                            switch (glob.options.general_settings.date_format) {
                                case 'dd.mm.yyyy' :
                                    format = 'sday. smonth. syear - eday. emonth. eyear';
                                    break;
                                case 'dd-mm-yy' :
                                    format = 'sday smonth syear - eday emonth eyear';
                                    break;
                                case 'mm-dd-yy' :
                                    format = 'smonth sday syear - emonth eday eyear';
                                    break;
                                case 'yy-mm-dd' :
                                default:
                                    format = 'syear smonth sday - eyear emonth eday';
                                    break;
                            }

                        }

                        label = format;

                        label = helper.getWeekLabel(label, theWeek);

                        $instance.$top.find(".stec-top-menu-date-week").show();

                        $instance.$top.find(".stec-top-menu-date-week > [data-week]")
                                .attr("data-week", theWeek.week)
                                .text(label);

                        $instance.$top.find(".stec-top-menu-date-small").text(label);

                        break;

                    case "day" :

                        $instance.$top.find(".stec-top-menu-date-month").css({
                            borderRadius: 0
                        }).find(".stec-top-menu-date-dropdown").css({
                            borderTopLeftRadius: 0
                        });

                        $instance.$top.find(".stec-top-menu-date-day").show();
                        $instance.$top.find(".stec-top-menu-date-month").show();
                        $instance.$top.find(".stec-top-menu-date-year").show();

                        $instance.$top.find(".stec-top-menu-date-day > [data-day]")
                                .attr("data-day", glob.options.day)
                                .text(glob.options.day);

                        $instance.$top.find(".stec-top-menu-date-month > [data-month]")
                                .attr("data-month", glob.options.month)
                                .text(glob.options.monthLabels[glob.options.month]);

                        $instance.$top.find(".stec-top-menu-date-year > [data-year]")
                                .attr("data-year", glob.options.year)
                                .text(glob.options.year);


                        switch (glob.options.general_settings.date_format) {
                            case 'dd.mm.yyyy' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.day + ". " + glob.options.monthLabels[glob.options.month] + " " + glob.options.year);
                                break;
                            case 'dd-mm-yy' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.day + " " + glob.options.monthLabels[glob.options.month] + " " + glob.options.year);
                                break;
                            case 'mm-dd-yy' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.monthLabels[glob.options.month] + " " + glob.options.day + " " + glob.options.year);
                                break;
                            case 'yy-mm-dd' :
                                $instance.$top.find(".stec-top-menu-date-small")
                                        .text(glob.options.year + " " + glob.options.monthLabels[glob.options.month] + " " + glob.options.day);
                                break;
                        }

                        break;
                }

                // Add active class for the given year,month,week,day for menu dropdown
                this.dateDropdownSetActive();
            },

            /**
             * Add active class for the given year,month,week,day for menu dropdown
             */
            dateDropdownSetActive: function () {

                var theWeek, weekDate;

                $instance.$top
                        .find(".stec-top-menu-date-dropdown .active").removeClass("active");

                $instance.$top
                        .find(".stec-top-menu-date-dropdown")
                        .find('[data-day="' + glob.options.day + '"]')
                        .parent()
                        .addClass('active');

                $instance.$top
                        .find(".stec-top-menu-date-dropdown")
                        .find('[data-month="' + glob.options.month + '"]')
                        .parent()
                        .addClass('active');

                $instance.$top
                        .find(".stec-top-menu-date-dropdown")
                        .find('[data-year="' + glob.options.year + '"]')
                        .parent()
                        .addClass('active');

                theWeek = helper.getWeekInfo();
                weekDate = theWeek.start.year + "-" + theWeek.start.month + "-" + theWeek.start.day;

                $instance.$top
                        .find(".stec-top-menu-date-dropdown")
                        .find('[data-date="' + weekDate + '"]')
                        .parent()
                        .addClass('active');

            },

            /**
             * Month scrolldown action
             */
            dateMonthScrollDown: function () {

                var parent = this, monthNumber, html = [], i;

                $instance.$top.find(".stec-top-menu-date-month").find("ul").stop(true, true);

                monthNumber = $instance.$top.find(".stec-top-menu-date-month").find("ul").first().find("li").last().find("[data-month]").attr("data-month");
                monthNumber = parseInt(monthNumber, 10);

                for (i = 0; i <= 5; i++) {
                    monthNumber = monthNumber + 1 > 11 ? 0 : monthNumber + 1;
                    html.push('<li><p data-month="' + monthNumber + '">' + glob.options.monthLabels[monthNumber] + '</p></li>');
                }

                $instance.$top.find(".stec-top-menu-date-month ul li").last().after(html.join(''));

                $instance.$top.find(".stec-top-menu-date-month").find("ul").css({
                    top: 45
                }).stop(true, true).animate({
                    top: -1 * 4 * 45
                }, {
                    duration: parent.dropDownScrollSpeed,
                    easing: "easeOutExpo",
                    complete: function () {
                        $(this).css("top", 45).find("li").slice(0, 5).remove();
                    }
                });

                this.dateDropdownSetActive();
            },

            /**
             * Month scrollup action
             */
            dateMonthScrollUp: function () {

                var parent = this;

                var monthNumber = $instance.$top.find(".stec-top-menu-date-month").find("ul").first().find("li").first().find("[data-month]").attr("data-month");
                var html;
                var mArr = [];
                var i;

                for (i = 0; i <= 4; i++) {
                    monthNumber = monthNumber - 1 < 0 ? 11 : monthNumber - 1;
                    mArr[i] = '<li><p data-month="' + monthNumber + '">' + glob.options.monthLabels[monthNumber] + '</p></li>';
                }

                html = (mArr.reverse().join(''));

                $instance.$top.find(".stec-top-menu-date-month ul li").first().before(html);

                $instance.$top.find(".stec-top-menu-date-month").find("ul").css({
                    top: -1 * 4 * 45
                }).stop().animate({
                    top: 45
                }, {
                    duration: parent.dropDownScrollSpeed,
                    easing: "easeOutExpo",
                    complete: function () {
                        $(this).css("top", 45).find("li").slice(6).remove();
                    }
                });

                this.dateDropdownSetActive();

            },

            /**
             * Year scrolldown action
             */
            dateYearScrollDown: function () {

                var parent = this;
                var yearNumber, i, html = [];

                $instance.$top.find(".stec-top-menu-date-year").find("ul").stop(true, true);

                yearNumber = $instance.$top.find(".stec-top-menu-date-year").find("ul").first().find("li").last().find("[data-year]").attr("data-year");
                yearNumber = parseInt(yearNumber, 10);

                for (i = 0; i <= 5; i++) {
                    yearNumber = yearNumber + 1;
                    html.push('<li><p data-year="' + yearNumber + '">' + yearNumber + '</p></li>');
                }

                $instance.$top.find(".stec-top-menu-date-year ul li").last().after(html.join(''));
                $instance.$top.find(".stec-top-menu-date-year").find("ul").css({
                    top: 45
                }).stop(true, true).animate({
                    top: -1 * 4 * 45
                }, {
                    duration: parent.dropDownScrollSpeed,
                    easing: "easeOutExpo",
                    complete: function () {
                        $(this).css("top", 45).find("li").slice(0, 5).remove();
                    }
                });

                this.dateDropdownSetActive();

            },

            /**
             * Year scrollup action
             */
            dateYearScrollUp: function () {

                var parent = this;
                var yearNumber = parseInt($instance.$top.find(".stec-top-menu-date-year").find("ul").find("li").find("[data-year]").attr("data-year"), 10);
                var html;
                var mArr = [];
                var i;

                for (i = 0; i <= 4; i++) {
                    yearNumber = yearNumber - 1;

                    if (yearNumber <= 0) {
                        break;
                    }

                    mArr[i] = '<li><p data-year="' + yearNumber + '">' + yearNumber + '</p></li>';

                }

                html = (mArr.reverse().join(''));

                $instance.$top.find(".stec-top-menu-date-year ul li").first().before(html);

                $instance.$top.find(".stec-top-menu-date-year").find("ul").css({
                    top: -1 * 4 * 45
                }).stop().animate({
                    top: 45
                }, {
                    duration: parent.dropDownScrollSpeed,
                    easing: "easeOutExpo",
                    complete: function () {
                        $(this).css("top", 45).find("li").slice(6).remove();
                    }
                });

                this.dateDropdownSetActive();
            },

            /**
             * Week scrolldown action
             */
            dateWeekScrollDown: function () {

                var parent = this;
                var lastWeek, html = [], format, theWeek, label, weekNumber, i, date;

                $instance.$top.find(".stec-top-menu-date-week").find("ul").stop(true, true);

                lastWeek = $instance.$top.find(".stec-top-menu-date-week").find("ul").last().find("li").last().find("[data-date]").attr("data-date");

                switch (glob.options.general_settings.date_format) {
                    case 'dd.mm.yyyy' :
                        format = 'sday. smonth - eday. emonth. eyear';
                        break;
                    case 'dd-mm-yy' :
                        format = 'sday smonth - eday emonth eyear';
                        break;
                    case 'mm-dd-yy' :
                        format = 'smonth sday - emonth eday eyear';
                        break;
                    case 'yy-mm-dd' :
                    default:
                        format = 'smonth sday - eyear emonth eday';
                        break;
                }

                weekNumber = helper.getDateFromData(lastWeek);

                for (i = 1; i <= 6; i++) {

                    weekNumber.setDate(weekNumber.getDate() + 7);

                    theWeek = helper.getWeekInfo(weekNumber);

                    label = helper.getWeekLabel(format, theWeek);

                    date = theWeek.start.year + "-" + theWeek.start.month + "-" + theWeek.start.day;

                    html.push('<li><p data-week="' + theWeek.week + '" data-date="' + date + '">' + label + '</p></li>');

                }

                $instance.$top.find(".stec-top-menu-date-week ul li").last().after(html.join(''));

                $instance.$top.find(".stec-top-menu-date-week").find("ul").css({
                    top: 45
                }).stop(true, true).animate({
                    top: -1 * 4 * 45
                }, {
                    duration: parent.dropDownScrollSpeed,
                    easing: "easeOutExpo",
                    complete: function () {
                        $(this).css("top", 45).find("li").slice(0, 5).remove();
                    }
                });

                this.dateDropdownSetActive();

            },

            dateWeekScrollUp: function () {

                var parent = this;
                var firstWeek = $instance.$top.find(".stec-top-menu-date-week").find("ul").first().find("li").first().find("[data-date]").attr("data-date");
                var html, theWeek, label, weekNumber, mArr = [];
                var format;
                var i = 0, date;

                switch (glob.options.general_settings.date_format) {
                    case 'dd.mm.yyyy' :
                        format = 'sday. smonth - eday. emonth. eyear';
                        break;
                    case 'dd-mm-yy' :
                        format = 'sday smonth - eday emonth eyear';
                        break;
                    case 'mm-dd-yy' :
                        format = 'smonth sday - emonth eday eyear';
                        break;
                    case 'yy-mm-dd' :
                    default:
                        format = 'smonth sday - eyear emonth eday';
                        break;
                }

                weekNumber = helper.getDateFromData(firstWeek);

                for (i = 1; i <= 5; i++) {

                    weekNumber.setDate(weekNumber.getDate() - 7);

                    theWeek = helper.getWeekInfo(weekNumber);

                    if (theWeek.start.year <= 0) {
                        break;
                    }

                    label = helper.getWeekLabel(format, theWeek);

                    date = theWeek.start.year + "-" + theWeek.start.month + "-" + theWeek.start.day;

                    mArr[i] = '<li><p data-week="' + theWeek.week + '" data-date="' + date + '">' + label + '</p></li>';

                }

                html = (mArr.reverse().join(''));

                $instance.$top.find(".stec-top-menu-date-week ul li").first().before(html);

                $instance.$top.find(".stec-top-menu-date-week").find("ul").css({
                    top: -1 * 4 * 45
                }).stop().animate({
                    top: 45
                }, {
                    duration: parent.dropDownScrollSpeed,
                    easing: "easeOutExpo",
                    complete: function () {
                        $(this).css("top", 45).find("li").slice(6).remove();
                    }
                });

                this.dateDropdownSetActive();

            },

            dateDayScrollDown: function () {

                var parent = this;
                var dayNumber, theMonth, maxD, html = [], i;

                $instance.$top.find(".stec-top-menu-date-day").find("ul").stop(true, true);

                dayNumber = $instance.$top.find(".stec-top-menu-date-day").find("ul").first().find("li").last().find("[data-day]").attr("data-day");
                dayNumber = parseInt(dayNumber, 10);

                theMonth = helper.getMonthInfo();
                maxD = theMonth.monthLength;


                for (i = 0; i <= 5; i++) {
                    dayNumber = dayNumber + 1 > maxD ? 1 : dayNumber + 1;
                    html.push('<li><p data-day="' + dayNumber + '">' + dayNumber + '</p></li>');
                }

                $instance.$top.find(".stec-top-menu-date-day ul li").last().after(html.join(''));

                $instance.$top.find(".stec-top-menu-date-day").find("ul").css({
                    top: 45
                }).stop(true, true).animate({
                    top: -1 * 4 * 45
                }, {
                    duration: parent.dropDownScrollSpeed,
                    easing: "easeOutExpo",
                    complete: function () {
                        $(this).css("top", 45).find("li").slice(0, 5).remove();
                    }
                });

                this.dateDropdownSetActive();

            },

            dateDayScrollUp: function () {

                var parent = this;
                var dayNumber = $instance.$top.find(".stec-top-menu-date-day").find("ul").first().find("li").first().find("[data-day]").attr("data-day");
                var theMonth = helper.getMonthInfo();
                var maxMonthDays = theMonth.monthLength;
                var html;
                var mArr = [];
                var i;

                for (i = 0; i <= 4; i++) {
                    dayNumber = dayNumber - 1 < 1 ? maxMonthDays : dayNumber - 1;
                    mArr[i] = '<li><p data-day="' + dayNumber + '">' + dayNumber + '</p></li>';
                }

                html = (mArr.reverse().join(''));

                $instance.$top.find(".stec-top-menu-date-day ul li").first().before(html);

                $instance.$top.find(".stec-top-menu-date-day").find("ul").css({
                    top: -1 * 4 * 45
                }).stop().animate({
                    top: 45
                }, {
                    duration: parent.dropDownScrollSpeed,
                    easing: "easeOutExpo",
                    complete: function () {
                        $(this).css("top", 45).find("li").slice(6).remove();
                    }
                });

                this.dateDropdownSetActive();
            }
        };

        var eventsLoader = {
            show: function () {
                clearTimeout(glob.timeout.eventsLoading);
                glob.timeout.eventsLoading = setTimeout(function () {
                    $instance.addClass('events-loading');
                    $instance.trigger('onEventsLoaderShow');
                }, 500);

            },

            hide: function () {
                clearTimeout(glob.timeout.eventsLoading);
                $instance.removeClass('events-loading');
                $instance.trigger('onEventsLoaderHide');
            }
        };

        var layout = {
            init: function () {

                this.initViews();
                this.bindControls();
            },

            initViews: function () {

                if (typeof glob.options.general_settings.views === 'string') {
                    glob.options.general_settings.views = glob.options.general_settings.views.split(',');
                }

                if ($.inArray('agenda', glob.options.general_settings.views) !== -1) {
                    this.agenda.bindControls();
                } else {
                    $instance.$agenda.remove();
                    $instance.$top.find('[data-view="agenda"]').remove();
                }

                if ($.inArray('month', glob.options.general_settings.views) !== -1) {
                    this.month.bindControls();
                } else {
                    $instance.$month.remove();
                    $instance.$top.find('[data-view="month"]').remove();
                }

                if ($.inArray('week', glob.options.general_settings.views) !== -1) {
                    this.week.bindControls();
                } else {
                    $instance.$week.remove();
                    $instance.$top.find('[data-view="week"]').remove();
                }

                if ($.inArray('day', glob.options.general_settings.views) !== -1) {
                    this.day.bindControls();
                } else {
                    $instance.$day.remove();
                    $instance.$top.find('[data-view="day"]').remove();
                }

                if ($.inArray('grid', glob.options.general_settings.views) !== -1) {
                    this.grid.bindControls();
                } else {
                    $instance.$grid.remove();
                    $instance.$top.find('[data-view="grid"]').remove();
                }

                if ($.inArray('boxgrid', glob.options.general_settings.views) !== -1) {
                    this.boxgrid.bindControls();
                } else {
                    $instance.$boxgrid.remove();
                    $instance.$top.find('[data-view="boxgrid"]').remove();
                }

                if ($.inArray('map', glob.options.general_settings.views) !== -1) {
                    this.map.bindControls();
                    this.map.sidebar.bindControls();
                } else {
                    $instance.$map.remove();
                    $instance.$top.find('[data-view="map"]').remove();
                }

                // If active view is not in views array set first from list as active
                if ($.inArray(glob.options.view, glob.options.general_settings.views) === -1) {
                    glob.options.view = glob.options.general_settings.views[0];
                }
            },

            set: function () {

                $instance.$agenda.hide();
                $instance.$month.hide();
                $instance.$week.hide();
                $instance.$day.hide();
                $instance.$grid.hide();
                $instance.$boxgrid.hide();
                $instance.$map.hide();

                main.refreshState('top');

                switch (glob.options.view) {

                    case "agenda" :
                        this.agenda.set();
                        break;

                    case "month" :
                        this.month.set();
                        break;

                    case "week" :
                        this.week.set();
                        break;

                    case "day" :
                        this.day.set();
                        break;

                    case "grid" :
                        this.grid.set();
                        break;

                    case "boxgrid" :
                        this.boxgrid.set();
                        break;

                    case "map" :
                        this.map.set();
                        break;
                }

                // after layout set
                helper.extendBind("stachethemes_ec_extend", "onLayoutSet");

            },

            /**
             * AGENDA
             */
            agenda: {

                cache: {
                    getNfutureEvents: false
                },

                bindControls: function () {

                    var parent = this;

                    // Agenda layout needs readjusting on window resize

                    helper.onResizeEnd(function () {
                        if ($instance.$agenda.is(":visible")) {
                            parent.set(true);
                        }

                    }, 50);

                    $instance.$agenda.find('.stec-layout-agenda-events-all-load-more').on(helper.clickHandle(), function (e) {
                        e.preventDefault();
                        parent.fillAgendaAllList();
                    });

                    // Click handle for cell click
                    $(document).on(helper.clickHandle(), $instance.$agenda.path + ' .stec-layout-agenda-daycell', function (e) {

                        var date;

                        e.preventDefault();

                        if ($(this).hasClass("active")) {
                            // Close active cell
                            eventsHandler.eventHolder.close();
                            $(this).removeClass("active");
                            return;
                        }

                        date = helper.getDateFromData($(this).attr('data-date'));

                        glob.options.day = date.getDate();
                        glob.options.month = date.getMonth();
                        glob.options.year = date.getFullYear();
                        main.refreshState('top');

                        parent.setActiveCell();

                    });

                    // Instant click handle for cell click. Used for drag
                    $(document).on(helper.instaClickHandle(), $instance.$agenda.path + ' .stec-layout-agenda-daycell', function (e) {

                        var date, curr;

                        e.preventDefault();

                        date = helper.getDateFromData($(this).attr('data-date'));
                        curr = helper.newDate(glob.options.year, glob.options.month, glob.options.day);

                        if (date.getTime() !== curr.getTime()) {

                            glob.options.day = date.getDate();
                            glob.options.month = date.getMonth();
                            glob.options.year = date.getFullYear();
                            main.refreshState('top');
                        }

                    });

                    // Draggable slider
                    $instance.$agenda.find(".stec-layout-agenda-list").draggable({

                        axis: "x",

                        start: function (event, ui) {

                            $instance.$agenda.find(".stec-layout-agenda-list").stop();
                            this.previousPosition = ui.position;
                            this.time = new Date().getTime();

                        },

                        stop: function (event, ui) {

                            var time = new Date().getTime() - this.time;
                            var moved = Math.abs(ui.position.left - this.previousPosition.left);

                            parent.dragFill($(this), ui.position.left);
                            parent.inertia($(this), this.previousPosition.left > ui.position.left ? 1 : -1, time, moved);

                        },

                        drag: function (event, ui) {
                            parent.dragFill($(this), ui.position.left);
                        }

                    });
                },

                // Set agenda layout
                set: function (resizeOnly) {

                    var DIM, cells;

                    // Check if Agenda Slider is set to show; else remove it
                    if (parseInt(glob.options.general_settings.agenda_cal_display, 10) !== 0) {

                        DIM = this.getSlideDimensions(true);

                        $instance.$agenda.find(".stec-layout-agenda-list")
                                .stop()
                                .css({
                                    left: 0,
                                    width: DIM.width
                                });

                        cells = this.getCells(false, true, 1, true);

                        $instance.$agenda.find(".stec-layout-agenda-list-b").empty().css('left',
                                -1 * $instance.$agenda.find(".stec-layout-agenda-list-a").width()
                                );

                        this.fillHTML($instance.$agenda.find(".stec-layout-agenda-list-a"), cells);

                    } else {

                        $instance.$agenda.find('.stec-layout-agenda-list-wrap').remove();

                    }

                    if (resizeOnly !== true) {
                        eventsHandler.eventHolder.close();
                        this.clearAgendaAllList();
                        this.fillAgendaAllList();
                    }

                    $instance.$agenda.show();

                },

                setActiveCell: function () {
                    var eventsData;
                    var events = [];
                    var activeCell = glob.options.year + "-" + glob.options.month + "-" + glob.options.day;
                    var $activeCell = $instance.$agenda.find(".stec-layout-agenda-daycell[data-date='" + activeCell + "']");

                    $instance.$agenda
                            .find(".stec-layout-agenda-daycell")
                            .removeClass("active");

                    $activeCell.addClass("active");

                    // Retrieves events from events cell data
                    eventsData = $activeCell.find('.stec-layout-agenda-daycell-events').data('events');

                    if (eventsData) {
                        $(eventsData).each(function () {
                            var event;

                            event = calData.getEventById(this.id, this.offset);
                            if (event) {
                                events.push(event);
                            }
                        });
                    }

                    eventsHandler.eventHolder.openEvents(events);
                },

                getSlideDimensions: function (refresh) {
                    var windowWidth, innerMaxCells, cellWidth, maxCells, width, DIM;

                    if (refresh !== true && this.getSlideDimensions.cache) {
                        return this.getSlideDimensions.cache;
                    }

                    windowWidth = $(window).width() < 2000 ? 2000 : $(window).width();
                    innerMaxCells = Math.floor($instance.width() / 80);
                    cellWidth = Math.floor($instance.width() / innerMaxCells);
                    maxCells = Math.round(windowWidth / cellWidth);
                    width = maxCells * cellWidth;

                    DIM = {
                        width: width,
                        maxCells: maxCells,
                        cellWidth: cellWidth
                    };

                    this.getSlideDimensions.cache = DIM;

                    return DIM;
                },

                /**
                 * Return cells data for active date
                 * @param {Date} date Alternative date
                 * @param {bool} centerOnDate Center on active date
                 * @param {int} direction -1 backwards 1 forwards
                 * @param {bool} borderMonthCell prevents el2 slide monthcell overlap
                 * @returns {array} returns cells data 
                 */
                getCells: function (date, centerOnDate, direction, borderMonthCell) {

                    var DIM, dayNumber, theMonth, yearNumber, j, cellArray, count, i;

                    if (!date) {
                        date = helper.newDate(glob.options.year, glob.options.month, glob.options.day);
                    }

                    DIM = this.getSlideDimensions();

                    dayNumber = date.getDate();
                    theMonth = helper.getMonthInfo(date.getMonth());
                    yearNumber = date.getFullYear();

                    if (centerOnDate === true) {

                        for (j = 0; j < Math.round(DIM.maxCells / ((DIM.width / $instance.width()) * 2)); j++) {

                            dayNumber = dayNumber - 1;

                            if (dayNumber <= 0) {

                                theMonth = theMonth.month - 1;

                                if (theMonth < 0) {
                                    theMonth = 11;
                                    yearNumber = yearNumber - 1;
                                }

                                theMonth = helper.getMonthInfo(theMonth, yearNumber);
                                dayNumber = theMonth.monthLength;
                            }
                        }
                    }

                    cellArray = [];
                    count = 0;

                    // Fill cells
                    if (direction === 1) {

                        for (i = 0; i < DIM.maxCells; i++) {

                            dayNumber = dayNumber + 1;

                            if (dayNumber > theMonth.monthLength) {

                                theMonth = theMonth.month + 1;

                                if (theMonth > 11) {
                                    yearNumber = yearNumber + 1;
                                    theMonth = 0;
                                }

                                theMonth = helper.getMonthInfo(theMonth, yearNumber);
                                dayNumber = 1;

                                if (count !== 0 || borderMonthCell === true) {
                                    cellArray[count] = {
                                        dataDate: false,
                                        dayNum: false,
                                        day: false,
                                        year: yearNumber,
                                        month: theMonth.month,
                                        monthStartCell: true,
                                        hasEvents: false
                                    };

                                    count++;
                                }


                            }

                            date = helper.newDate(yearNumber, theMonth.month, dayNumber);

                            cellArray[count] = {
                                dataDate: yearNumber + "-" + theMonth.month + "-" + dayNumber,
                                dayNum: date.getDay(),
                                day: dayNumber,
                                year: yearNumber,
                                month: theMonth.month,
                                monthStartCell: false,
                                hasEvents: false
                            };

                            count++;
                        }

                        cellArray = cellArray.slice(0, DIM.maxCells);

                    } else {

                        for (i = 0; i < DIM.maxCells; i++) {

                            dayNumber = dayNumber - 1;

                            if (dayNumber <= 0) {

                                if (count !== 0 || borderMonthCell === true) {
                                    cellArray[count] = {
                                        dataDate: false,
                                        dayNum: false,
                                        day: false,
                                        year: yearNumber,
                                        month: theMonth.month,
                                        monthStartCell: true,
                                        hasEvents: false
                                    };

                                    count++;
                                }

                                theMonth = theMonth.month - 1;

                                if (theMonth < 0) {
                                    yearNumber = yearNumber - 1;
                                    theMonth = 11;
                                }

                                theMonth = helper.getMonthInfo(theMonth, yearNumber);
                                dayNumber = theMonth.monthLength;

                            }

                            date = helper.newDate(yearNumber, theMonth.month, dayNumber);

                            cellArray[count] = {
                                dataDate: yearNumber + "-" + theMonth.month + "-" + dayNumber,
                                dayNum: date.getDay(),
                                day: dayNumber,
                                year: yearNumber,
                                month: theMonth.month,
                                monthStartCell: false,
                                hasEvents: false
                            };

                            count++;

                        }

                        cellArray = cellArray.slice(0, DIM.maxCells);

                        cellArray.reverse();
                    }

                    return cellArray;

                },

                /**
                 * Create and append html for given cells
                 * @param {object} $el Element to append to
                 * @param {array} cells array
                 */
                fillHTML: function ($el, cells) {

                    var parent = this;
                    var DIM = this.getSlideDimensions();
                    var today, date, html = [];

                    $el.empty();

                    $(cells).each(function () {

                        var cell = this;

                        if (cell.monthStartCell === true) {

                            html.push('<li style="width: ' + DIM.cellWidth + 'px" class="stec-layout-agenda-monthstart" data-year="' + cell.year + '" data-month="' + cell.month + '" >');
                            html.push('<div class="stec-layout-agenda-monthstart-wrap">');
                            html.push('<p class="stec-layout-agenda-monthstart-year">' + cell.year + '</p>');
                            html.push('<p class="stec-layout-agenda-monthstart-month">' + glob.options.monthLabelsShort[cell.month] + '</p>');
                            html.push('</div>');
                            html.push('</li>');

                        } else {

                            html.push('<li style="width: ' + DIM.cellWidth + 'px" class="stec-layout-agenda-daycell" data-date="' + cell.dataDate + '">');
                            html.push('<div class="stec-layout-agenda-daycell-wrap">');
                            html.push('<p class="stec-layout-agenda-daycell-label">' + glob.options.dayLabelsShort[cell.dayNum] + '</p>');
                            html.push('<p class="stec-layout-agenda-daycell-num">' + cell.day + '</p>');
                            html.push('<div class="stec-layout-agenda-daycell-events">');
//                                    html.push('<div class="stec-layout-agenda-daycell-event"></div>');
                            html.push('</div>');
                            html.push('</div>');
                            html.push('</li>');
                        }

                    });

                    // === $(html.join('')).appendTo($el); somewhat faster
                    $el[0].innerHTML += html.join('');

                    // Set Today 
                    today = new Date();
                    date = today.getFullYear() + "-" + today.getMonth() + "-" + today.getDate();

                    $instance.$agenda
                            .find(".stec-layout-agenda-daycell[data-date='" + date + "']")
                            .addClass("stec-layout-agenda-daycell-today");

                    parent.fillEvents();

                },

                fillEvents: function () {

                    var $a, $b, start, end, startDate, endDate;

                    $a = $instance.$agenda.find(".stec-layout-agenda-list-a");
                    $b = $instance.$agenda.find(".stec-layout-agenda-list-b");

                    if ($b.children().length <= 0) {
                        // b is empty (init start)

                        start = $a.children('.stec-layout-agenda-daycell').first().attr('data-date');
                        end = $a.children('.stec-layout-agenda-daycell').last().attr('data-date');

                    } else {

                        if ($a[0].offsetLeft > $b[0].offsetLeft) {
                            // b is behind

                            start = $b.children('.stec-layout-agenda-daycell').first().attr('data-date');
                            end = $a.children('.stec-layout-agenda-daycell').last().attr('data-date');
                        } else {

                            // b is ahead
                            end = $b.children('.stec-layout-agenda-daycell').last().attr('data-date');
                            start = $a.children('.stec-layout-agenda-daycell').first().attr('data-date');
                        }
                    }

                    $instance.$agenda
                            .children('.stec-layout-agenda-list-wrap')
                            .find('.stec-layout-agenda-daycell-events').empty();

                    // Populate cells with events

                    startDate = helper.getDateFromData(start);
                    endDate = helper.getDateFromData(end);

                    eventsLoader.show();

                    calData.getEventsThread(startDate, endDate, true, 'agenda').then(function (events) {

                        eventsLoader.hide();

                        if (events.length <= 0) {
                            // no events
                            return;
                        }

                        $instance.$agenda
                                .find('.stec-layout-agenda-daycell')
                                .find('.stec-layout-agenda-daycell-events').data('events', null);

                        $(events).each(function () {

                            var startDate = moment.tz(this.start_date, this.timezone);
                            var endDate = moment.tz(this.end_date, this.timezone);
                            var days = 0;
                            var day, stamp, dataDate;
                            var extraClass = [], html;
                            var eventState = helper.getEventState(this);
                            var $eventCont;
                            var dataEvents;

                            // we care for number of difference dates not per 24 hours
                            days = Math.abs(startDate.startOf('day').diff(endDate.startOf('day'), 'days'));

                            for (day = 0; day <= days; day++) {

                                stamp = moment(this.start_date).add(day, 'days').unix();
                                dataDate = helper.getDataFromDate(new Date(stamp * 1000));

                                $eventCont = $instance.$agenda
                                        .find('.stec-layout-agenda-daycell[data-date="' + dataDate + '"]')
                                        .find('.stec-layout-agenda-daycell-events');

                                if ($eventCont.children('.stec-layout-agenda-daycell-event').length > 2) {

                                    // cells are full

                                } else {

                                    if (eventState > 0) {
                                        extraClass.push('stec-layout-agenda-daycell-event-expired');
                                    }

                                    extraClass.push('stec-layout-agenda-daycell-event');

                                    html = '<div></div>';

                                    $(html)
                                            .attr({
                                                'data-id': this.id,
                                                'data-repeat-time-offset': this.repeat_time_offset,
                                                'style': 'background-color: ' + this.color,
                                                'class': extraClass.join(' ')
                                            })
                                            .appendTo(
                                                    $instance.$agenda.children('.stec-layout-agenda-list-wrap')
                                                    .find('.stec-layout-agenda-daycell[data-date="' + dataDate + '"]')
                                                    .find('.stec-layout-agenda-daycell-events')
                                                    );
                                }


                                // Attaches events cell data
                                dataEvents = $eventCont.data('events');

                                if (!dataEvents) {
                                    dataEvents = [];
                                }

                                dataEvents.push({
                                    id: this.id,
                                    offset: this.repeat_time_offset
                                });

                                $eventCont.data('events', dataEvents);

                            }


                        });

                        helper.extendBind("stachethemes_ec_extend", "afterFillEvents");
                    });
                },

                /**
                 * Build cells on the run while dragging if required
                 * @param {object} $el Dragged element
                 * @param {Number} pos current position
                 */
                dragFill: function ($el, pos) {

                    var rebuild = false, d1, d2, borderMonthCell, cells;
                    var $el2 = $el.parent().children(".stec-layout-agenda-list").not($el);

                    $el2.css('left', pos + (pos < 0 ? 1 : -1) * $el.width());

                    if ($el2[0].offsetLeft > $el[0].offsetLeft) {
                        // increment el2

                        rebuild = false;

                        d1 = helper.getDateFromData($el.children('.stec-layout-agenda-daycell').last().attr('data-date'));
                        d2 = $el2.children('.stec-layout-agenda-daycell').first().attr('data-date');

                        if (!d2) {
                            rebuild = true;
                        } else {
                            d2 = helper.getDateFromData(d2);
                            d2.setDate(d2.getDate() - 1);

                            if (d1.getTime() !== d2.getTime()) {
                                rebuild = true;
                            }
                        }

                        if (rebuild === true) {

                            borderMonthCell = true;

                            if ($el.children("li").last().hasClass("stec-layout-agenda-monthstart")) {
                                borderMonthCell = false;
                            }

                            d1.setDate(d1.getDate());
                            cells = this.getCells(d1, false, 1, borderMonthCell);
                            this.fillHTML($el2, cells);

                        }

                    } else {
                        // decrement el2

                        rebuild = false;

                        d1 = helper.getDateFromData($el.children('.stec-layout-agenda-daycell').first().attr('data-date'));
                        d2 = $el2.children('.stec-layout-agenda-daycell').last().attr('data-date');

                        if (!d2) {
                            rebuild = true;
                        } else {
                            d2 = helper.getDateFromData(d2);
                            d2.setDate(d2.getDate() + 1);
                            if (d1.getTime() !== d2.getTime()) {
                                rebuild = true;
                            }
                        }

                        if (rebuild === true) {

                            borderMonthCell = true;

                            if ($el.children("li").first().hasClass("stec-layout-agenda-monthstart")) {
                                borderMonthCell = false;
                            }

                            cells = this.getCells(d1, false, -1, borderMonthCell);
                            this.fillHTML($el2, cells);
                        }
                    }
                },

                inertia: function ($el, dir, time, moved) {

                    var parent = this;
                    var x = 0;

                    x = moved / time * 100;

                    x = x > $el.width() ? $el.width() / 4 : x;

                    $instance.$agenda.find(".stec-layout-agenda-list").stop();

                    switch (dir) {

                        case 1:
                            $el.animate({
                                left: $el.position().left - x
                            }, {
                                easing: "easeOutExpo",
                                duration: 1000,
                                step: function (a, b) {
                                    parent.dragFill($el, b.now);
                                }
                            });
                            break;

                        case - 1:
                            $el.animate({
                                left: $el.position().left + x
                            }, {
                                easing: "easeOutExpo",
                                duration: 1000,
                                step: function (a, b) {
                                    parent.dragFill($el, b.now);
                                }
                            });
                            break;

                    }

                },

                /**
                 * Reset agenda all-list
                 */
                clearAgendaAllList: function () {
                    this.cache.getNfutureEvents = false;
                    $instance.$agenda.find('.stec-layout-agenda-events-all ul').remove();
                    $instance.$agenda.find('.stec-layout-agenda-events-all-datetext').remove();
                },

                /**
                 * Caches all future eventis initially
                 * Each call pulls N events from the cache array
                 * @return (object) events list
                 */
                getNfutureEvents: function () {

                    var parent = this;

                    return new Promise(function (resolve) {

                        var x, y, returnData = {events: [], loadMore: false};

                        if (!parent.cache.getNfutureEvents) {

                            // Load cache

                            parent.getNfutureEvents.n = parseInt(glob.options.general_settings.agenda_get_n, 10);

                            calData.getFutureEventsThread(true, 'getNfutureEvents').then(function (events) {


                                if (parseInt(glob.options.general_settings.reverse_agenda_list, 10) === 1) {
                                    parent.cache.getNfutureEvents = $(events).get().reverse();
                                } else {
                                    parent.cache.getNfutureEvents = events;
                                }

                                parent.getNfutureEvents.i = 0;

                                x = parent.getNfutureEvents.i++ * parent.getNfutureEvents.n;
                                y = x + parent.getNfutureEvents.n;

                                returnData.events = parent.cache.getNfutureEvents.slice(x, y);
                                returnData.loadMore = parent.cache.getNfutureEvents.slice(y, y + 1).length > 0 ? true : false;

                                return resolve(returnData);

                            });

                        } else {
                            x = parent.getNfutureEvents.i++ * parent.getNfutureEvents.n;
                            y = x + parent.getNfutureEvents.n;

                            returnData.events = parent.cache.getNfutureEvents.slice(x, y);
                            returnData.loadMore = parent.cache.getNfutureEvents.slice(y, y + 1).length > 0 ? true : false;

                            return resolve(returnData);
                        }

                    });

                },

                /**
                 * Builds agenda all list events html
                 */
                fillAgendaAllList: function () {

                    var lastLabel, d1;

                    if (parseInt(glob.options.general_settings.agenda_list_display, 10) === 0) {
                        return;
                    }

                    $instance.$agenda.find('.stec-layout-agenda-events-all-control').hide();

                    this.getNfutureEvents().then(function (data) {

                        var events = data.events;
                        var loadMore = data.loadMore;

                        // Hide controls and display "No events found" text
                        if (!events || events.length <= 0) {

                            if ($instance.$agenda.find('.stec-layout-agenda-events-all-list').children().length <= 0) {
                                $instance.$agenda.find('.stec-layout-agenda-noevents').show();
                            }
                            return;
                        }

                        // If there are no more events - hide control
                        if (true === loadMore) {
                            $instance.$agenda.find('.stec-layout-agenda-events-all-control').show();
                        }

                        lastLabel = $instance.$agenda.find('.stec-layout-agenda-events-all-datetext').last();
                        lastLabel.month = lastLabel.attr('data-month');
                        lastLabel.year = lastLabel.attr('data-year');

                        $(events).each(function (i) {

                            var event = this;
                            var extraClass = [];
                            var icon = [];
                            var eventState = helper.getEventState(this);
                            var d = helper.dbDateTimeToDate(this.start_date);

                            if (eventState > 0 || 'EventCancelled' === event.event_status || '0' === glob.options.general_settings.reminder) {
                                extraClass.push('stec-layout-event-no-reminder');
                            }

                            if (parseInt(lastLabel.month, 10) !== d.getMonth() || parseInt(lastLabel.year, 10) !== d.getFullYear()) {
                                // Add new label
                                lastLabel.month = d.getMonth();
                                lastLabel.year = d.getFullYear();

                                $('<p data-year="' + d.getFullYear() + '" data-month="' + d.getMonth() + '" class="stec-layout-agenda-events-all-datetext">' + glob.options.monthLabels[d.getMonth()] + ' ' + d.getFullYear() + '</p>')
                                        .insertBefore($instance.$agenda.find('.stec-layout-agenda-events-all-control'));

                                $('<ul class="stec-layout-agenda-events-all-list"></ul>')
                                        .insertAfter($instance.$agenda.find('.stec-layout-agenda-events-all-datetext').last());
                            }

                            switch (parseInt(this.featured, 10)) {
                                case 1:
                                    extraClass.push('stec-event-featured');
                                    break;

                                case 2:
                                    extraClass.push('stec-event-featured stec-event-featured-bg');
                                    break;
                            }

                            if ('EventCancelled' === event.event_status) {
                                extraClass.push('stec-event-cancelled');
                            }

                            if (event.icon_type === 'icon' && (!event.icon || event.icon === 'fa')) {
                                extraClass.push('stec-no-icon');
                            }

                            if (!event.edit_link) {
                                extraClass.push('stec-no-edit');
                            }

                            if (event.description_short && '1' === glob.options.general_settings.preview_desc ||
                                    event.location.full_address && '1' === glob.options.general_settings.preview_location) {
                                extraClass.push('stec-layout-event-preview-size-big');
                            }

                            switch (event.icon_type) {

                                case 'image' :

                                    if (event.icon_images.length > 0) {

                                        icon.push('<div class="stec-layout-event-preview-left-icon-images">');

                                        $.each(event.icon_images_meta, function (i) {
                                            icon.push('<div class="stec-layout-event-preview-left-icon-image ' + (i === 0 ? 'active' : '') + '" style="background-image: url(' + this.src + ')"></div>');
                                        });

                                        icon.push('</div>');

                                    } else {
                                        icon.push('<i class="' + event.icon + '"></i>');
                                    }

                                    break;

                                case 'date' :
                                    d1 = moment(event.start_date);

                                    icon.push('<span class="stec-layout-event-preview-left-icon-date"><span class="stec-layout-event-preview-left-icon-date-number">' + d1.format('D') + '</span><span class="stec-layout-event-preview-left-icon-date-month">' + __(d1.format('MMM').toLowerCase()) + '</span></span>');

                                    break;

                                default:
                                    icon.push('<i class="' + event.icon + '"></i>');

                            }

                            $(glob.template.event)
                                    .addClass(extraClass.join(' '))
                                    .attr('data-id', event.id)
                                    .attr('data-repeat-time-offset', event.repeat_time_offset ? event.repeat_time_offset : 0)
                                    .data('permalink', helper.getPermalink(event.permalink, event.repeat_time_offset))
                                    .html(function (index, html) {

                                        var $html = $(html);
                                        var date = helper.beautifyTimespan(event.start_date, parseInt(event.hide_end, 10) === 1 ? false : event.end_date, event.all_day);
                                        var gmtUtcOffset = parseInt(event.timezone_utc_offset, 10) / 3600;
                                        var timezoneOffsetLabel;
                                        var eventStatusHtml = [];

                                        gmtUtcOffset = gmtUtcOffset > 0 ? '+' + gmtUtcOffset : gmtUtcOffset;

                                        if (gmtUtcOffset === 0) {
                                            gmtUtcOffset = '';
                                        }

                                        timezoneOffsetLabel = (parseInt(glob.options.general_settings.date_label_gmtutc, 10) === 0 ? '' : 'UTC/GMT ' + gmtUtcOffset).trim();

                                        if (!event.location.full_address ||
                                                '1' !== glob.options.general_settings.preview_location) {
                                            $html.find('.stec-layout-event-preview-left-text-location').remove();
                                        }

                                        if (!event.description_short || '1' !== glob.options.general_settings.preview_desc) {
                                            $html.find('.stec-layout-event-preview-left-text-desc').remove();
                                        }

                                        glob.temp.EventPreviewHtml = $html.get(0).outerHTML;

                                        $instance.trigger('stecBeforeReturnEventPreviewHtml', {
                                            $instance: $instance,
                                            event: event,
                                            temp: glob.temp
                                        });

                                        switch (event.event_status) {

                                            case 'EventCancelled' :
                                                eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="cancelled">' + __('Cancelled') + '</span></p>');
                                                break;

                                            case 'EventMovedOnline' :
                                                eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="moved-online">' + __('Moved Online') + '</span></p>');
                                                break;

                                            case 'EventPostponed' :
                                                eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="postponed">' + __('Postponed') + '</span></p>');
                                                break;

                                            case 'EventRescheduled' :
                                                eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="rescheduled">' + __('Rescheduled') + '</span></p>');
                                                break;

                                            default:
                                                eventStatusHtml = [];
                                        }

                                        if (event.approved === 0) {
                                            eventStatusHtml.push('<p class="stec-layout-event-preview-left-text-event-status"><span class="unapproved">' + __('Awaiting Approval') + '</span></p>');
                                        }


                                        // Agenda Preview
                                        return glob.temp.EventPreviewHtml
                                                .replace('stec_replace_event_status', eventStatusHtml.join(''))
                                                .replace('stec_replace_summary', event.title)
                                                .replace('stec_replace_date', date + ' ' + timezoneOffsetLabel)
                                                .replace('stec_replace_location', function () {

                                                    if ('virtual' === event.location.type) {
                                                        return __('Online Event');
                                                    }

                                                    if (event.location.title) {
                                                        return [event.location.title, event.location.full_address].join(', ');
                                                    }

                                                    return event.location.full_address;
                                                })
                                                .replace('stec_replace_desc_short', event.description_short)
                                                .replace('stec_replace_event_background=""', 'style="background:' + event.color + '"') // edge is retarded
                                                .replace('stec_replace_icon', icon.join(''))
                                                .replace(/#stec_replace_edit_link/g, event.edit_link);

                                    }).appendTo($instance.find('.stec-layout-agenda-events-all-list').last());

                        });

                        switch (glob.options.general_settings.open_event_in) {
                            case 'single':
                            case 'external' :
                            case 'modal' :
                                // Remove + when single pages or modal
                                $instance
                                        .find('.stec-layout-agenda-events-all')
                                        .find('.stec-layout-event-preview-right-event-toggle')
                                        .remove();
                                break;
                        }

                        $instance.$agenda.find('.stec-layout-agenda-noevents').hide();

                        helper.animate.agenda.fillList($instance.$agenda);

                    });
                }
            },

            /**
             * MONTH LAYOUT
             */
            month: {
                bindControls: function () {

                    var parent = this;

                    // Day cell click handle
                    $instance.$month.find(".stec-layout-month-daycell").on(helper.clickHandle(), function (e) {

                        var date;

                        e.preventDefault();

                        if ($(this).hasClass("active")) {
                            // Close active cell
                            eventsHandler.eventHolder.close();
                            $(this).removeClass("active");
                            return;
                        }

                        date = helper.getDateFromData($(this).attr("data-date"));

                        glob.options.year = date.getFullYear();
                        glob.options.month = date.getMonth();
                        glob.options.day = date.getDate();

                        parent.setActiveCell();

                    });

                },

                // Set to month layout
                set: function () {
                    $instance.$month.show();
                    this.setDayLabels();
                    this.fillGridDays();
                    this.fillEvents();
                },

                setDayLabels: function () {

                    var offset = 0;
                    var a;

                    switch (glob.options.general_settings.first_day_of_the_week) {
                        case 'mon' :
                            offset = 1;
                            break;

                        case 'sat' :
                            offset = 6;
                            break;

                        case 'sun' :
                            offset = 0;
                            break;
                    }

                    a = offset;

                    $instance.$month.find('.stec-layout-month-daylabel td').each(function (i) {

                        var label = helper.capitalizeFirstLetter(glob.options.dayLabels[a]);
                        var labelShort = helper.capitalizeFirstLetter(glob.options.dayLabelsShort[a]);

                        $(this).find('p').eq(0).text(label);
                        $(this).find('p').eq(1).text(labelShort);
                        $(this).find('p').eq(2).text(labelShort.charAt(0));

                        a = a + 1 >= glob.options.dayLabels.length ? 0 : a + 1;

                    });
                },

                setActiveCell: function () {

                    var events = [];
                    var eventsData;

                    var activeCell = glob.options.year + "-" + glob.options.month + "-" + glob.options.day;

                    var $activeCell = $instance.$month.find(".stec-layout-month-daycell[data-date='" + activeCell + "']");

                    $instance.$month
                            .find(".stec-layout-month-daycell")
                            .removeClass("active");

                    $activeCell.addClass("active");

                    $instance.$month.find(".stec-layout-month-eventholder")
                            .insertAfter(
                                    $instance.$month
                                    .find(".stec-layout-month-daycell.active")
                                    .parents("tr")
                                    );

                    // Retrieves events from events cell data
                    eventsData = $activeCell.find('.stec-layout-month-daycell-events').data('events');

                    if (eventsData) {
                        $(eventsData).each(function () {
                            var event;

                            event = calData.getEventById(this.id, this.offset);
                            if (event) {
                                events.push(event);
                            }
                        });
                    }

                    eventsHandler.eventHolder.openEvents(events);
                },

                resetGridCells: function () {

                    $instance.$month
                            .find(".stec-layout-month-daylabel td")
                            .removeClass("stec-layout-month-daylabel-today");

                    $instance.$month
                            .find(".stec-layout-month-daycell")
                            .removeAttr("data-date")
                            .removeClass("stec-layout-month-daycell-today stec-layout-month-daycell-inactive active");

                    $instance.$month
                            .find(".stec-layout-month-daycell")
                            .removeClass("active");

                    $instance.$month
                            .find('.stec-layout-month-daycell-events').empty();

                    eventsHandler.eventHolder.close();

                },

                fillGridDays: function () {

                    var parent = this;
                    var i, realDayNumber, offset;
                    var activeMonthInfo;
                    var prevMonthInfo;
                    var nextMonthInfo;
                    var today;
                    var date;

                    parent.resetGridCells();

                    // Active Month
                    activeMonthInfo = helper.getMonthInfo();

                    for (i = 0; i < activeMonthInfo.monthLength; i++) {
                        realDayNumber = i + 1;

                        $instance.$month
                                .find(".stec-layout-month-daycell")
                                .eq(activeMonthInfo.dayOffset + i)
                                .attr("data-date", activeMonthInfo.year + "-" + activeMonthInfo.month + "-" + realDayNumber)
                                .find(".stec-layout-month-daycell-num").text(realDayNumber);
                    }

                    // Prev Month 
                    prevMonthInfo = helper.getMonthInfo(activeMonthInfo.month - 1 < 0 ? 11 : activeMonthInfo.month - 1, activeMonthInfo.month - 1 < 0 ? activeMonthInfo.year - 1 : activeMonthInfo.year);
                    for (i = activeMonthInfo.dayOffset; i > 0; i--) {
                        realDayNumber = prevMonthInfo.monthLength - activeMonthInfo.dayOffset + i;
                        $instance.$month
                                .find(".stec-layout-month-daycell").eq(i - 1)
                                .addClass("stec-layout-month-daycell-inactive")
                                .attr("data-date", prevMonthInfo.year + "-" + prevMonthInfo.month + "-" + realDayNumber)
                                .find(".stec-layout-month-daycell-num").text(realDayNumber);
                    }

                    // Next Month 
                    nextMonthInfo = helper.getMonthInfo(activeMonthInfo.month + 1 > 11 ? 0 : activeMonthInfo.month + 1, activeMonthInfo.month + 1 > 11 ? activeMonthInfo.year + 1 : activeMonthInfo.year);
                    for (i = 0; i < 6 * 7 - (activeMonthInfo.monthLength + activeMonthInfo.dayOffset); i++) {
                        offset = activeMonthInfo.monthLength + activeMonthInfo.dayOffset + i;
                        realDayNumber = i + 1;
                        $instance.$month
                                .find(".stec-layout-month-daycell").eq(offset)
                                .addClass("stec-layout-month-daycell-inactive")
                                .attr("data-date", nextMonthInfo.year + "-" + nextMonthInfo.month + "-" + realDayNumber)
                                .find(".stec-layout-month-daycell-num").text(realDayNumber);
                    }

                    // Set Today 
                    today = new Date();
                    date = today.getFullYear() + "-" + today.getMonth() + "-" + today.getDate();

                    $instance.$month
                            .find(".stec-layout-month-daycell[data-date='" + date + "']")
                            .addClass("stec-layout-month-daycell-today");

                    if ($instance.$month.find(".stec-layout-month-daycell-today").length > 0) {

                        offset = $instance.$month
                                .find(".stec-layout-month-weekrow")
                                .find(".stec-layout-month-daycell[data-date='" + date + "']").index();

                        $instance.$month
                                .find(".stec-layout-month-daylabel td")
                                .eq(offset)
                                .addClass("stec-layout-month-daylabel-today");
                    }
                },

                fillEvents: function () {

                    var parent = this;

                    // Get layout date span

                    var from = helper.getDateFromData($instance.$month.find('.stec-layout-month-daycell').first().attr('data-date'));
                    var to = helper.getDateFromData($instance.$month.find('.stec-layout-month-daycell').last().attr('data-date'));

                    // Clean old events data from cells
                    $instance.$month
                            .find('.stec-layout-month-daycell')
                            .find('.stec-layout-month-daycell-events').data('events', null);

                    eventsLoader.show();

                    // Get all events for this timespan
                    calData.getEventsThread(from, to, true, 'base').then(function (events) {

                        eventsLoader.hide();

                        // Loop each event
                        $(events).each(function () {

                            var hiddenEvents = [];
                            var startDate = '';
                            var endDate = '';
                            var days = 0;
                            var day, dataDate, $eventCont, html = [];
                            var text, num, newText;
                            var extraClass;
                            var backgroundColorStyle;
                            var brightness;
                            var pos;
                            var repeatTimeOffset;
                            var posId;
                            var eventState;
                            var dataEvents;

                            startDate = moment.tz(this.start_date, this.timezone);
                            endDate = moment.tz(this.end_date, this.timezone);
                            eventState = helper.getEventState(this);

                            // we care for number of difference dates not per 24 hours
                            days = Math.abs(startDate.startOf('day').diff(endDate.startOf('day'), 'days'));

                            for (day = 0; day <= days; day++) {

                                extraClass = [];

                                dataDate = helper.getDataFromDate(moment(this.start_date).add(day, 'days').toDate());

                                $eventCont = $instance.$month
                                        .find('.stec-layout-month-daycell[data-date="' + dataDate + '"]')
                                        .find('.stec-layout-month-daycell-events');

                                if ($eventCont.find('.stec-layout-month-daycell-event').length > 2 || (hiddenEvents.indexOf(this.id) > -1)) {

                                    // if container is full || event started from full container

                                    if ($eventCont.find('.stec-layout-month-daycell-eventmore').length > 0) {

                                        text = $eventCont.find('.stec-layout-month-daycell-eventmore-count').text();
                                        num = parseInt(text.match(/[0-9]+/), 10) + 1;
                                        newText = '+' + num + ' ' + __('morePlural');

                                        $eventCont.find('.stec-layout-month-daycell-eventmore-count').text(newText);

                                    } else {

                                        html = [];
                                        html.push('<li class="stec-layout-month-daycell-eventmore">');
                                        html.push('   <p class="stec-layout-month-daycell-eventmore-count">+1 ' + __('moreSingular') + '</p>');
                                        html.push('   <p class="stec-layout-month-daycell-eventmore-count-dot"></p>');
                                        html.push('   <p class="stec-layout-month-daycell-eventmore-count-dot"></p>');
                                        html.push('   <p class="stec-layout-month-daycell-eventmore-count-dot"></p>');
                                        html.push('</li>');

                                        $(html.join('')).appendTo($eventCont);
                                    }

                                    repeatTimeOffset = this.repeat_time_offset ? this.repeat_time_offset : 0;
                                    hiddenEvents.push(this.id);

                                } else {

                                    // Add event cell
                                    backgroundColorStyle = 'background-color:' + this.color;

                                    if (day === 0) {
                                        extraClass.push('stec-layout-month-daycell-event-start');
                                    }

                                    if (day === days) {
                                        extraClass.push('stec-layout-month-daycell-event-end');
                                    }

                                    switch (parseInt(this.featured, 10)) {
                                        case 1:
                                            extraClass.push('stec-event-featured');
                                            break;

                                        case 2:
                                            extraClass.push('stec-event-featured stec-event-featured-bg');
                                            break;
                                    }

                                    pos = parent.getEventPosition(this, $eventCont);

                                    brightness = helper.getColorBrightness(this.color);

                                    if (brightness > 170) {
                                        extraClass.push("stec-layout-month-daycell-event-bright");
                                    }

                                    if (eventState > 0) {
                                        extraClass.push("stec-layout-month-daycell-event-expired");
                                    }

                                    repeatTimeOffset = this.repeat_time_offset ? this.repeat_time_offset : 0;

                                    posId = this.id + '-' + repeatTimeOffset;

                                    html = [];
                                    html.push('<li>');

                                    if (day === 0 || parseInt(glob.options.general_settings.show_event_title_all_cells, 10) !== 0) {
                                        html.push('<p class="stec-layout-month-daycell-event-name">' + this.title + '</p>');
                                    }

                                    html.push('</li>');

                                    extraClass.push('stec-layout-month-daycell-event');

                                    if ('EventCancelled' === this.event_status) {
                                        extraClass.push('stec-event-cancelled');
                                    }

                                    $(html.join('')).attr({
                                        'class': extraClass.join(' '),
                                        'style': backgroundColorStyle,
                                        'data-pos': pos,
                                        'data-pos-id': posId,
                                        'data-repeat-time-offset': repeatTimeOffset,
                                        'data-id': this.id
                                    }).appendTo($eventCont);
                                }

                                // Attaches events cell data
                                dataEvents = $eventCont.data('events');

                                if (!dataEvents) {
                                    dataEvents = [];
                                }

                                dataEvents.push({
                                    id: this.id,
                                    offset: repeatTimeOffset
                                });

                                $eventCont.data('events', dataEvents);

                            }
                        });

                        helper.extendBind("stachethemes_ec_extend", "afterFillEvents");
                    });

                },

                getEventPosition: function (event, $eventCont) {

                    var position = 0;
                    var positions = [1, 2, 3];

                    // repeat instance acts like subid of the event
                    var repeatTimeOffset = event.repeat_time_offset ? event.repeat_time_offset : 0;
                    var posId = event.id + '-' + repeatTimeOffset;
                    var $first = $instance.$month.find('.stec-layout-month-daycell-event[data-pos-id="' + posId + '"]').first();

                    if ($first.length > 0) {

                        position = $first.attr('data-pos');

                    } else {

                        $eventCont.find('.stec-layout-month-daycell-event').each(function () {
                            var i = positions.indexOf(parseInt($(this).attr('data-pos'), 10));
                            if (i > -1) {
                                positions.splice(i, 1);
                            }
                        });

                        position = positions[0];
                    }

                    return position;
                }
            },

            /**
             * WEEK LAYOUT
             */
            week: {
                bindControls: function () {

                    var parent = this;
                    var date;

                    // Week daycell click handle
                    $instance.$week.find(".stec-layout-week-daycell").on(helper.clickHandle(), function (e) {

                        e.preventDefault();

                        if ($(this).hasClass("active")) {
                            // Close active cell
                            eventsHandler.eventHolder.close();
                            $(this).removeClass("active");
                            return;
                        }

                        date = helper.getDateFromData($(this).attr("data-date"));

                        glob.options.year = date.getFullYear();
                        glob.options.month = date.getMonth();
                        glob.options.day = date.getDate();

                        parent.setActiveCell();


                    });

                    helper.onResizeEnd(function () {
                        parent.calcHeight();
                    });
                },

                set: function () {
                    $instance.$week.show();
                    this.setDayLabels();
                    this.fillGridDays();
                    this.fillEvents();
                },

                setDayLabels: function () {

                    var offset = 0;
                    var a;

                    switch (glob.options.general_settings.first_day_of_the_week) {
                        case 'mon' :
                            offset = 1;
                            break;

                        case 'sat' :
                            offset = 6;
                            break;

                        case 'sun' :
                            offset = 0;
                            break;
                    }

                    a = offset;

                    $instance.$week.find('.stec-layout-week-daylabel td').each(function (i) {

                        var label = helper.capitalizeFirstLetter(glob.options.dayLabels[a]);
                        var labelShort = helper.capitalizeFirstLetter(glob.options.dayLabelsShort[a]);

                        $(this).find('p').eq(0).text(label);
                        $(this).find('p').eq(1).text(labelShort);
                        $(this).find('p').eq(2).text(labelShort.charAt(0));

                        a = a + 1 >= glob.options.dayLabels.length ? 0 : a + 1;

                    });
                },

                setActiveCell: function () {

                    // Set Active Cell

                    var activeCell = glob.options.year + "-" + glob.options.month + "-" + glob.options.day;
                    var $activeCell = $instance.$week.find(".stec-layout-week-daycell[data-date='" + activeCell + "']");
                    var eventsData = [];
                    var events = [];

                    $instance.$week
                            .find(".stec-layout-week-daycell")
                            .removeClass("active");

                    $activeCell.addClass("active");

                    // Retrieves events from events cell data
                    eventsData = $activeCell.find('.stec-layout-week-daycell-events').data('events');

                    if (eventsData) {
                        $(eventsData).each(function () {
                            var event;

                            event = calData.getEventById(this.id, this.offset);
                            if (event) {
                                events.push(event);
                            }
                        });
                    }

                    eventsHandler.eventHolder.openEvents(events);

                },

                resetGridCells: function () {

                    // Reset data 

                    $instance.$week
                            .find(".stec-layout-week-daylabel td")
                            .removeClass("stec-layout-week-daylabel-today");

                    $instance.$week
                            .find(".stec-layout-week-daycell")
                            .removeAttr("data-date")
                            .removeClass("stec-layout-week-daycell-today stec-layout-week-daycell-inactive active");

                    $instance.$week
                            .find(".stec-layout-week-daycell")
                            .removeClass("active");

                    $instance.$week
                            .find('.stec-layout-week-daycell-events').empty();

                    eventsHandler.eventHolder.close();

                },

                fillGridDays: function () {

                    var parent = this;
                    var theWeek = helper.getWeekInfo();
                    var cellDate = helper.newDate(theWeek.start.year, theWeek.start.month, theWeek.start.day);
                    var today;
                    var date;
                    var offset;

                    parent.resetGridCells();

                    // Active Week
                    $instance.$week
                            .find(".stec-layout-week-daycell").each(function (i) {

                        let next = moment(cellDate).add(i, 'days').toDate();

                        $(this)
                                .attr("data-date", next.getFullYear() + "-" + next.getMonth() + "-" + next.getDate())
                                .find(".stec-layout-week-daycell-num").text(next.getDate());
                    });

                    // Set Today 
                    today = new Date();
                    date = today.getFullYear() + "-" + today.getMonth() + "-" + today.getDate();

                    $instance.$week
                            .find(".stec-layout-week-daycell[data-date='" + date + "']")
                            .addClass("stec-layout-week-daycell-today");

                    if ($instance.$week.find(".stec-layout-week-daycell-today").length > 0) {

                        offset = $instance.$week
                                .find(".stec-layout-week-weekrow")
                                .find(".stec-layout-week-daycell[data-date='" + date + "']").index();

                        $instance.$week
                                .find(".stec-layout-week-daylabel td")
                                .eq(offset)
                                .addClass("stec-layout-week-daylabel-today");
                    }
                },

                calcHeight: function () {

                    var minHeight = $instance.hasClass('stec-media-small') ? 50 : 160;
                    var minULheight = $instance.hasClass('stec-media-small') ? 20 : 113;
                    var margin = $instance.hasClass('stec-media-small') ? 6 : 29;
                    var a = $instance.hasClass('stec-media-small') ? 40 : 70;
                    $instance.$week.find('.stec-layout-week-daycell').css('height', minHeight);
                    $instance.$week.find('.stec-layout-week-daycell-events').css('height', minULheight);
                    $instance.$week.find('.stec-layout-week-daycell-events').children().each(function () {
                        var pos = parseInt($(this).attr('data-pos'), 10);
                        $(this).css({
                            top: margin * pos
                        });
                        // Calc min cell height
                        minHeight = Math.max(minHeight, a + margin * pos);
                        minULheight = minHeight - 30;
                    });
                    $instance.$week.find('.stec-layout-week-daycell').css('height', minHeight);
                    $instance.$week.find('.stec-layout-week-daycell-events').css('height', minULheight);

                },

                fillEvents: function () {

                    var parent = this;
                    var hiddenEvents;

                    // Get layout date span

                    var from = helper.getDateFromData($instance.$week.find('.stec-layout-week-daycell').first().attr('data-date'));
                    var to = helper.getDateFromData($instance.$week.find('.stec-layout-week-daycell').last().attr('data-date'));

                    // Clean old events data from cells
                    $instance.$week
                            .find('.stec-layout-week-daycell')
                            .find('.stec-layout-week-daycell-events').data('events', null);

                    eventsLoader.show();

                    // Get all events for this timespan
                    calData.getEventsThread(from, to, true, 'base').then(function (events) {

                        eventsLoader.hide();

                        if (!events) {
                            return;
                        }

                        $(events).each(function () {

                            var startDate = moment.tz(this.start_date, this.timezone);
                            var endDate = moment.tz(this.end_date, this.timezone);
                            var days = 0;
                            var day, dataDate;
                            var $eventCont, html;
                            var extraClass, backgroundColorStyle;
                            var posId;
                            var repeatTimeOffset;
                            var pos;
                            var brightness;
                            var eventState = helper.getEventState(this);
                            var dataEvents = [];

                            hiddenEvents = [];

                            // we care for number of difference dates not per 24 hours
                            days = Math.abs(startDate.startOf('day').diff(endDate.startOf('day'), 'days'));

                            for (day = 0; day <= days; day++) {

                                extraClass = [];

                                dataDate = helper.getDataFromDate(moment(this.start_date).add(day, 'days').toDate());

                                $eventCont = $instance.$week
                                        .find('.stec-layout-week-daycell[data-date="' + dataDate + '"]')
                                        .find('.stec-layout-week-daycell-events');

                                // Add event cell
                                backgroundColorStyle = 'background-color:' + this.color;

                                if (day === 0) {
                                    extraClass.push('stec-layout-week-daycell-event-start');
                                }

                                if (day === days) {
                                    extraClass.push('stec-layout-week-daycell-event-end');
                                }

                                switch (parseInt(this.featured, 10)) {
                                    case 1:
                                        extraClass.push('stec-event-featured');
                                        break;

                                    case 2:
                                        extraClass.push('stec-event-featured stec-event-featured-bg');
                                        break;

                                }

                                repeatTimeOffset = this.repeat_time_offset ? this.repeat_time_offset : 0;
                                posId = this.id + '-' + repeatTimeOffset;
                                pos = parent.getEventPosition(this, $eventCont);

                                brightness = helper.getColorBrightness(this.color);

                                if (brightness > 170) {
                                    extraClass.push("stec-layout-week-daycell-event-bright");
                                }

                                if (eventState > 0) {
                                    extraClass.push("stec-layout-week-daycell-event-expired");
                                }

                                extraClass.push('stec-layout-week-daycell-event');

                                html = [];
                                html.push('<li>');

                                if (day === 0 || parseInt(glob.options.general_settings.show_event_title_all_cells, 10) !== 0) {
                                    html.push('<p class="stec-layout-week-daycell-event-name">' + this.title + '</p>');
                                }

                                html.push('</li>');

                                if ('EventCancelled' === this.event_status) {
                                    extraClass.push('stec-event-cancelled');
                                }


                                $(html.join('')).attr({
                                    'class': extraClass.join(' '),
                                    'style': backgroundColorStyle,
                                    'data-pos': pos,
                                    'data-pos-id': posId,
                                    'data-repeat-time-offset': repeatTimeOffset,
                                    'data-id': this.id
                                }).appendTo($eventCont);


                                // Attaches events cell data
                                dataEvents = $eventCont.data('events');

                                if (!dataEvents) {
                                    dataEvents = [];
                                }

                                dataEvents.push({
                                    id: this.id,
                                    offset: repeatTimeOffset
                                });

                                $eventCont.data('events', dataEvents);


                            }
                        });

                        parent.calcHeight();

                        helper.extendBind("stachethemes_ec_extend", "afterFillEvents");
                    });

                },

                getEventPosition: function (event, $eventCont) {

                    // determine event position
                    var positions = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24];
                    var position = 0;

                    // repeat instance acts like subid of the event
                    var repeatTimeOffset = event.repeat_time_offset ? event.repeat_time_offset : 0;
                    var posId = event.id + '-' + repeatTimeOffset;

                    var $first = $instance.$week.find('.stec-layout-week-daycell-event[data-pos-id="' + posId + '"]').first();

                    if ($first.length > 0) {

                        position = $first.attr('data-pos');

                    } else {

                        $eventCont.find('.stec-layout-week-daycell-event').each(function () {

                            var i = positions.indexOf(parseInt($(this).attr('data-pos'), 10));

                            if (i > -1) {
                                positions.splice(i, 1);
                            }
                        });

                        position = positions[0];

                    }

                    return position;
                }
            },

            /**
             * DAY LAYOUT
             */
            day: {

                bindControls: function () {

                },
                set: function () {

                    var from = helper.newDate(glob.options.year, glob.options.month, glob.options.day);
                    var to = from;

                    eventsHandler.eventHolder.close();

                    $instance.$day.show();

                    // Get all events for this timespan
                    calData.getEventsThread(from, to, true, 'base').then(function (events) {
                        eventsHandler.eventHolder.openEvents(events);
                        helper.extendBind("stachethemes_ec_extend", "afterFillEvents");
                    });
                }

            },

            /**
             * GRID
             */
            grid: {
                cache: {
                    getNfutureEvents: false
                },

                columns: 0,
                options: {
                    offset: 0,
                    perClick: 4,
                    blockMinWidth: 270,
                    defaultColumns: 4,
                    gutter: 10
                },

                bindControls: function () {

                    var parent = this;

                    this.options.defaultColumns = parseInt(glob.options.general_settings.grid_columns, 10);
                    this.options.gutter = parseInt(glob.options.general_settings.grid_gutter, 10);
                    this.options.perClick = parseInt(glob.options.general_settings.grid_per_click, 10);

                    if (!this.options.gutter) {
                        $instance.$grid.find('.stec-layout-grid-events-all-load-more').css({
                            marginTop: 10
                        });
                    }

                    $instance.$grid.find('.stec-layout-grid-events-all-load-more').on(helper.clickHandle(), function (e) {
                        e.preventDefault();
                        parent.fill();
                    });

                    if (glob.options.general_settings.open_event_in === 'modal') {
                        $instance.$grid.on(helper.clickHandle(), '.stec-layout-grid-event .stec-layout-grid-event-title, .stec-layout-grid-event .stec-layout-grid-event-image', function (e) {
                            e.preventDefault();
                            eventsHandler.eventToggle($(this).parents('.stec-layout-grid-event').first());
                        });
                    }

                    helper.onResizeEnd(function () {
                        parent.resizeEvents();
                    }, 50);

                    $instance.on('onEventsLoaderShow', function () {
                        if ('grid' === glob.options.view) {
                            layout.grid.placeholders.remove();
                            layout.grid.placeholders.add();
                        }
                    });

                    $instance.on('onEventsLoaderHide', function () {
                        if ('grid' === glob.options.view) {
                            layout.grid.placeholders.remove();
                        }
                    });

                },

                set: function () {

                    $instance.$grid.show();
                    this.clear();
                    this.fill();

                },

                clear: function () {

                    this.cache.getNfutureEvents = false;
                    $instance.$grid.find('.stec-layout-grid-events').children('.stec-layout-grid-event').remove();
                    $instance.$grid.find('.stec-layout-grid-noevents').hide();

                },

                getNfutureEvents: function () {

                    var parent = this;

                    return new Promise(function (resolve) {

                        var x, y, returnData = {events: [], loadMore: false};

                        if (!parent.cache.getNfutureEvents) {

                            // Load cache

                            parent.getNfutureEvents.n = parent.options.perClick;

                            calData.getFutureEventsThread(true, 'getFutureEventsThread').then(function (events) {

                                if (parseInt(glob.options.general_settings.reverse_agenda_list, 10) === 1) {
                                    parent.cache.getNfutureEvents = $(events).get().reverse();
                                } else {
                                    parent.cache.getNfutureEvents = events;
                                }

                                parent.getNfutureEvents.i = 0;

                                x = parent.getNfutureEvents.i++ * parent.getNfutureEvents.n;
                                y = x + parent.getNfutureEvents.n;

                                returnData.events = parent.cache.getNfutureEvents.slice(x, y);
                                returnData.loadMore = parent.cache.getNfutureEvents.slice(y, y + 1).length > 0 ? true : false;

                                return resolve(returnData);

                            });

                        } else {
                            x = parent.getNfutureEvents.i++ * parent.getNfutureEvents.n;
                            y = x + parent.getNfutureEvents.n;

                            returnData.events = parent.cache.getNfutureEvents.slice(x, y);
                            returnData.loadMore = parent.cache.getNfutureEvents.slice(y, y + 1).length > 0 ? true : false;

                            return resolve(returnData);
                        }

                    });

                },

                placeholders: {

                    html: function () {
                        var html = [];

                        html.push('<div class="stec-layout-grid-event-placeholder">');
                        html.push('<span class="noanim"></span>');
                        html.push('<span></span>');
                        html.push('<div><span></span><span></span></div>');
                        html.push('<div><span></span><span></span></div>');
                        html.push('</div>');

                        return html.join('');
                    },

                    add: function () {

                        var i;
                        var placeholders = [];

                        placeholders.push('<div class="stec-layout-grid-event-placeholders-wrap">');

                        layout.grid.calcColumns();

                        for (i = 0; i < layout.grid.columns; i++) {

                            placeholders.push(this.html());
                        }

                        placeholders.push('</div>');

                        $(placeholders.join('')).appendTo($instance.$grid.find('.stec-layout-grid-events'));


                    },

                    remove: function () {
                        $instance.$grid.find('.stec-layout-grid-events').children('.stec-layout-grid-event-placeholders-wrap').remove();
                    }
                },

                fill: function () {

                    var parent = this;

                    eventsLoader.show();

                    this.getNfutureEvents().then(function (data) {

                        var events = data.events;
                        var loadMore = data.loadMore;

                        eventsLoader.hide();

                        if (!events || events.length <= 0) {

                            // no events
                            $instance.$grid.find('.stec-layout-grid-events-all-control').hide();

                            if ($instance.$grid.find('.stec-layout-grid-events').children().length <= 0) {
                                $instance.$grid.find('.stec-layout-grid-noevents').show();
                                $instance.$grid.find('.stec-layout-grid-events').height(0);
                            }

                            return;

                        }

                        if (false === loadMore) {
                            $instance.$grid.find('.stec-layout-grid-events-all-control').hide();
                        } else {
                            $instance.$grid.find('.stec-layout-grid-events-all-control').show();
                        }

                        $(events).each(function () {

                            var event = this;
                            var invitedUser = false;
                            var tagsHtml = [];
                            var extraClass = [];
                            var eventState = helper.getEventState(this);

                            if (eventState > 0 || 'EventCancelled' === event.event_status || '0' === glob.options.general_settings.reminder) {
                                extraClass.push('stec-layout-event-no-reminder');
                            }

                            switch (parseInt(this.featured, 10)) {
                                case 1:
                                    extraClass.push('stec-event-featured');
                                    break;

                                case 2:
                                    extraClass.push('stec-event-featured stec-event-featured-bg');
                                    break;
                            }

                            if (event.icon_type === 'icon' && (!event.icon || event.icon === 'fa')) {
                                extraClass.push('stec-no-icon');
                            }

                            if (!event.edit_link) {
                                extraClass.push('stec-no-edit');
                            }

                            if (event.images_meta.length > 0) {
                                extraClass.push('stec-has-image');
                            }

                            if (event.products.length > 0) {
                                extraClass.push('stec-has-products');
                            }

                            if (event.location.full_address) {
                                extraClass.push('stec-has-location');
                            }

                            if (event.organizers.length > 0) {
                                extraClass.push('stec-has-organizers');
                            }

                            switch (event.event_status) {
                                case 'EventCancelled' :
                                    tagsHtml.push('<span class="cancelled">' + __('Cancelled') + '</span>');
                                    break;

                                case 'EventMovedOnline' :
                                    tagsHtml.push('<span class="moved-online">' + __('Moved Online') + '</span>');
                                    break;

                                case 'EventPostponed' :
                                    tagsHtml.push('<span class="postponed">' + __('Postponed') + '</span>');
                                    break;

                                case 'EventRescheduled' :
                                    tagsHtml.push('<span class="rescheduled">' + __('Rescheduled') + '</span>');
                                    break;
                            }

                            if (event.approved === 0) {
                                tagsHtml.push('<span class="unapproved">' + __('Awaiting Approval') + '</span>');
                            }

                            if (event.featured > 0) {
                                tagsHtml.push('<span class="stec-featured-tag">' + __('featured') + '</span>');
                            }

                            $.each(event.category, function () {
                                tagsHtml.push('<span style="background-color:' + this.color + '">' + this.title + '</span>');
                            });

                            if (tagsHtml.length > 0) {
                                extraClass.push('stec-has-tags');
                            }

                            if (!isNaN(glob.options.userid)) {

                                // check if user is invited
                                $(event.attendance).each(function () {
                                    if (this.userid === glob.options.userid) {
                                        invitedUser = true;
                                        return false; // break
                                    }
                                });
                            }

                            if (invitedUser) {
                                extraClass.push('stec-user-invited');
                            } else {
                                if (parseInt(event.rsvp, 10) === 1) {
                                    extraClass.push('stec-can-rsvp');
                                }
                            }

                            if (-1 !== $.inArray(['EventCancelled', 'EventPostponed'], event.event_status)) {

                                if (eventState === 2) {
                                    extraClass.push('stec-expired');
                                } else if (eventState === 1) {
                                    extraClass.push('stec-in-progress');
                                }
                            }

                            if ('EventCancelled' === event.event_status) {
                                extraClass.push('stec-event-cancelled');
                            }

                            $(glob.template.gridevent)
                                    .addClass(extraClass.join(' '))
                                    .attr({
                                        'data-id': event.id,
                                        'data-repeat-time-offset': event.repeat_time_offset ? event.repeat_time_offset : 0
                                    })
                                    .html(function (index, html) {
                                        var date = helper.beautifyTimespan(event.start_date, parseInt(event.hide_end, 10) === 1 ? false : event.end_date, event.all_day);
                                        var gmtUtcOffset = parseInt(event.timezone_utc_offset, 10) / 3600;
                                        var timezoneOffsetLabel;
                                        var organizerString = '';
                                        var productsString = '';
                                        var icon = [];
                                        var d1;
                                        var permalink;

                                        gmtUtcOffset = gmtUtcOffset > 0 ? '+' + gmtUtcOffset : gmtUtcOffset;

                                        if (gmtUtcOffset === 0) {
                                            gmtUtcOffset = '';
                                        }

                                        timezoneOffsetLabel = parseInt(glob.options.general_settings.date_label_gmtutc, 10) === 0 ? '' : 'UTC/GMT ' + gmtUtcOffset;

                                        if (event.organizers.length > 0) {
                                            organizerString = event.organizers[0].name;

                                            if (event.organizers.length > 1) {
                                                organizerString += ' +' + (event.organizers.length - 1) + ' ' + (event.organizers.length - 1 > 1 ? __('morePlural') : __('moreSingular'));
                                            }
                                        }

                                        if (event.products.length > 0) {
                                            productsString = event.products[0].title;
                                            if (event.products.length > 1) {
                                                productsString += ' +' + (event.products.length - 1) + ' ' + (event.products.length - 1 > 1 ? __('morePlural') : __('moreSingular'));
                                            }
                                        }

                                        switch (event.icon_type) {

                                            case 'image' :

                                                if (event.icon_images.length > 0) {

                                                    icon.push('<div class="stec-layout-event-preview-left-icon-images">');

                                                    $.each(event.icon_images_meta, function (i) {
                                                        icon.push('<div class="stec-layout-event-preview-left-icon-image ' + (i === 0 ? 'active' : '') + '" style="background-image: url(' + this.src + ')"></div>');
                                                    });


                                                    icon.push('</div>');

                                                } else {
                                                    icon.push('<i class="' + event.icon + '"></i>');
                                                }

                                                break;

                                            case 'date' :
                                                d1 = moment(event.start_date);

                                                icon.push('<span class="stec-layout-event-preview-left-icon-date"><span class="stec-layout-event-preview-left-icon-date-number">' + d1.format('D') + '</span><span class="stec-layout-event-preview-left-icon-date-month">' + __(d1.format('MMM').toLowerCase()) + '</span></span>');

                                                break;

                                            default:
                                                icon.push('<i class="' + event.icon + '"></i>');

                                        }

                                        permalink = helper.getPermalink(event.permalink, event.repeat_time_offset);

                                        if (glob.options.general_settings.open_event_in === 'external') {
                                            if (event.link) {
                                                permalink = event.link.url;
                                            }
                                        }

                                        return html
                                                .replace(/#stec_replace_permalink/g, permalink)
                                                .replace(/stec_replace_atarget/g, glob.options.general_settings.event_open_target)
                                                .replace('stec_replace_tags', tagsHtml.join(''))
                                                .replace('stec_replace_summary', event.title)
                                                .replace('stec_replace_date', date + ' ' + timezoneOffsetLabel)
                                                .replace('stec_replace_image', 'style="background-image:url(' + (event.images_meta.length > 0 ? event.images_meta[0].src : '') + ')"')
                                                .replace('stec_replace_event_background=""', 'style="background:' + event.color + '"') // edge is retarded
                                                .replace('stec_replace_organizer_image', 'style="background-image: url(' + (event.organizers.length > 0 && event.organizers[0].photo_src ? event.organizers[0].photo_src : '') + ')"')
                                                .replace('stec_replace_organizer_name', organizerString)
                                                .replace('stec_replace_product_name', productsString)
                                                .replace('stec_replace_icon', icon.join(''))
                                                .replace('stec_replace_location', function () {

                                                    if ('virtual' === event.location.type) {
                                                        return __('Online Event');
                                                    }

                                                    if (event.location.title) {
                                                        return event.location.title;
                                                    }

                                                    return event.location.full_address;
                                                })
                                                .replace('stec_replace_short_desc', event.description_short)
                                                .replace(/#stec_replace_edit_link/g, event.edit_link);

                                    }).appendTo($instance.$grid.find('.stec-layout-grid-events'));

                        });

                        parent.resizeEvents();

                        helper.extendBind("stachethemes_ec_extend", "afterFillEvents");
                    });

                },

                calcColumns: function () {
                    var parent = this;

                    parent.columns = parent.options.defaultColumns;

                    while (($instance.$grid.find('.stec-layout-grid-events').width() - ((parent.columns - 1) * parent.options.gutter)) / parent.columns < parent.options.blockMinWidth) {
                        parent.columns--;

                        if (parent.columns === 1) {
                            break;
                        }
                    }

                },

                resizeEvents: function () {

                    var parent;

                    if (!$instance.$grid.is(':visible')) {
                        return;
                    }

                    parent = this;

                    parent.calcColumns();

                    $instance.$grid.find('.stec-layout-grid-event').css({
                        width: function () {
                            var val = ($instance.$grid.find('.stec-layout-grid-events').outerWidth() - ((parent.columns - 1) * parent.options.gutter)) / parent.columns;
                            return Math.floor(val);
                        }
                    });

                    this.positionEvents();
                },

                positionEvents: function () {

                    var parent = this;
                    var maxHeight = 0;
                    var currentRow = 0;
                    var currentColumn = 0;
                    var $prev;

                    $instance.$grid.find('.stec-layout-grid-event').each(function (i) {

                        var x = 0, y = 0, currentHeight;

                        if (i > 0 && i % parent.columns === 0) {
                            currentRow++;
                            currentColumn = 0;
                        }

                        if (currentColumn > 0) {
                            $prev = $(this).prev();
                            x = $prev.outerWidth() + $prev.position().left + parent.options.gutter;
                        }

                        if (currentRow > 0) {
                            $prev = $instance.$grid.find('.stec-layout-grid-event').eq(i - parent.columns);
                            y = $prev.outerHeight() + $prev.position().top + parent.options.gutter;
                        }

                        $(this).css({
                            top: y,
                            left: x
                        });

                        currentHeight = y + $(this).outerHeight() + parent.options.gutter;

                        maxHeight = Math.max(currentHeight, maxHeight);

                        currentColumn++;
                    });

                    $instance.$grid.find('.stec-layout-grid-events').height(maxHeight);
                }
            },

            /**
             * BOXGRID
             */
            boxgrid: {
                cache: {
                    getNfutureEvents: false
                },

                columns: 0,
                options: {
                    offset: 0,
                    perClick: 4,
                    blockMinWidth: 270,
                    defaultColumns: 4,
                    gutter: 0
                },

                placeholders: {

                    html: function () {
                        var html = [];

                        html.push('<div class="stec-layout-boxgrid-event-placeholder">');
                        html.push('<span></span>');
                        html.push('<span></span>');
                        html.push('<span></span>');
                        html.push('<span></span>');
                        html.push('<div><span></span><span></span></div>');
                        html.push('<div><span></span><span></span></div>');
                        html.push('</div>');

                        return html.join('');
                    },

                    add: function () {

                        var i;
                        var placeholders = [];

                        placeholders.push('<div class="stec-layout-boxgrid-event-placeholders-wrap">');

                        layout.boxgrid.calcColumns();

                        for (i = 0; i < layout.boxgrid.columns; i++) {

                            placeholders.push(this.html());
                        }

                        placeholders.push('</div>');

                        $(placeholders.join('')).appendTo($instance.$boxgrid.find('.stec-layout-boxgrid-events'));

                    },

                    remove: function () {
                        $instance.$boxgrid.find('.stec-layout-boxgrid-events').children('.stec-layout-boxgrid-event-placeholders-wrap').remove();
                    }
                },

                bindControls: function () {

                    var parent = this;
                    this.options.defaultColumns = parseInt(glob.options.general_settings.boxgrid_columns, 10);
                    this.options.perClick = parseInt(glob.options.general_settings.boxgrid_per_click, 10);
                    this.options.gutter = parseInt(glob.options.general_settings.boxgrid_gutter, 10);

                    if (!this.options.gutter) {
                        $instance.$boxgrid.find('.stec-layout-boxgrid-events-all-load-more').css({
                            marginTop: 10
                        });
                    }

                    $instance.$boxgrid.find('.stec-layout-boxgrid-events-all-load-more').on(helper.clickHandle(), function (e) {
                        e.preventDefault();
                        parent.fill();
                    });

                    if (glob.options.general_settings.open_event_in === 'modal') {
                        $instance.$boxgrid.on(helper.clickHandle(), '.stec-layout-boxgrid-event', function (e) {
                            e.preventDefault();
                            eventsHandler.eventToggle($(this));
                        });
                    }

                    helper.onResizeEnd(function () {
                        parent.resizeEvents();
                    }, 50);

                    $instance.on('onEventsLoaderShow', function () {
                        if ('boxgrid' === glob.options.view) {
                            layout.boxgrid.placeholders.remove();
                            layout.boxgrid.placeholders.add();
                        }
                    });

                    $instance.on('onEventsLoaderHide', function () {
                        if ('boxgrid' === glob.options.view) {
                            layout.boxgrid.placeholders.remove();
                        }
                    });
                },

                set: function () {

                    $instance.$boxgrid.show();
                    this.clear();
                    this.fill();

                },

                clear: function () {
                    this.cache.getNfutureEvents = false;
                    $instance.$boxgrid.find('.stec-layout-boxgrid-events').empty();
                    $instance.$boxgrid.find('.stec-layout-boxgrid-noevents').hide();

                },

                getNfutureEvents: function () {

                    var parent = this;

                    return new Promise(function (resolve) {

                        var x, y, returnData = {events: [], loadMore: false};

                        if (!parent.cache.getNfutureEvents) {

                            // Load cache

                            parent.getNfutureEvents.n = parent.options.perClick;

                            calData.getFutureEventsThread(true, 'getFutureEventsThread').then(function (events) {

                                if (parseInt(glob.options.general_settings.reverse_agenda_list, 10) === 1) {
                                    parent.cache.getNfutureEvents = $(events).get().reverse();
                                } else {
                                    parent.cache.getNfutureEvents = events;
                                }

                                parent.getNfutureEvents.i = 0;

                                x = parent.getNfutureEvents.i++ * parent.getNfutureEvents.n;
                                y = x + parent.getNfutureEvents.n;

                                returnData.events = parent.cache.getNfutureEvents.slice(x, y);
                                returnData.loadMore = parent.cache.getNfutureEvents.slice(y, y + 1).length > 0 ? true : false;

                                return resolve(returnData);

                            });

                        } else {
                            x = parent.getNfutureEvents.i++ * parent.getNfutureEvents.n;
                            y = x + parent.getNfutureEvents.n;

                            returnData.events = parent.cache.getNfutureEvents.slice(x, y);
                            returnData.loadMore = parent.cache.getNfutureEvents.slice(y, y + 1).length > 0 ? true : false;

                            return resolve(returnData);
                        }

                    });

                },

                fill: function () {

                    var parent = this;

                    eventsLoader.show();

                    this.getNfutureEvents().then(function (data) {

                        var events = data.events;
                        var loadMore = data.loadMore;

                        eventsLoader.hide();

                        if (!events || events.length <= 0) {

                            // no events
                            $instance.$boxgrid.find('.stec-layout-boxgrid-events-all-control').hide();

                            if ($instance.$boxgrid.find('.stec-layout-boxgrid-events').children().length <= 0) {
                                $instance.$boxgrid.find('.stec-layout-boxgrid-noevents').show();
                            }

                            return;
                        }

                        if (false === loadMore) {
                            $instance.$boxgrid.find('.stec-layout-boxgrid-events-all-control').hide();
                        } else {
                            $instance.$boxgrid.find('.stec-layout-boxgrid-events-all-control').show();
                        }

                        $(events).each(function () {
                            var event = this;
                            var extraClass = [];
                            var tagsHtml = [];
                            var invitedUser = false;
                            var startDate;
                            var endDate;
                            var eventState = helper.getEventState(this);

                            if (eventState > 0 || 'EventCancelled' === event.event_status || '0' === glob.options.general_settings.reminder) {
                                extraClass.push('stec-layout-event-no-reminder');
                            }

                            switch (parseInt(this.featured, 10)) {
                                case 1:
                                    extraClass.push('stec-event-featured');
                                    break;

                                case 2:
                                    extraClass.push('stec-event-featured stec-event-featured-bg');
                                    break;
                            }

                            if ((event.icon_type === 'icon' && (!event.icon || event.icon === 'fa')) || event.icon_type === 'date') {
                                extraClass.push('stec-no-icon');
                            }

                            if (event.images_meta.length > 0) {
                                extraClass.push('stec-has-image');
                            }

                            switch (event.event_status) {
                                case 'EventCancelled' :
                                    tagsHtml.push('<span class="cancelled">' + __('Cancelled') + '</span>');
                                    break;

                                case 'EventMovedOnline' :
                                    tagsHtml.push('<span class="moved-online">' + __('Moved Online') + '</span>');
                                    break;

                                case 'EventPostponed' :
                                    tagsHtml.push('<span class="postponed">' + __('Postponed') + '</span>');
                                    break;

                                case 'EventRescheduled' :
                                    tagsHtml.push('<span class="rescheduled">' + __('Rescheduled') + '</span>');
                                    break;
                            }

                            if (event.approved === 0) {
                                tagsHtml.push('<span class="unapproved">' + __('Awaiting Approval') + '</span>');
                            }

                            if (event.featured > 0) {
                                tagsHtml.push('<span class="stec-featured-tag">' + __('featured') + '</span>');
                            }

                            $.each(event.category, function () {
                                tagsHtml.push('<span style="background-color:' + this.color + '">' + this.title + '</span>');
                            });

                            if (tagsHtml.length > 0) {
                                extraClass.push('stec-has-tags');
                            }

                            if (!isNaN(glob.options.userid)) {

                                // check if user is invited
                                $(event.attendance).each(function () {
                                    if (this.userid === glob.options.userid) {
                                        invitedUser = true;
                                        return false; // break
                                    }
                                });
                            }

                            if (invitedUser) {
                                extraClass.push('stec-user-invited');
                            }


                            if (eventState === 2) {
                                extraClass.push('stec-expired');
                            } else if (eventState === 1) {
                                extraClass.push('stec-in-progress');
                            }

                            if ('EventCancelled' === event.event_status) {
                                extraClass.push('stec-event-cancelled');
                            }

                            $(glob.template.boxgridevent)
                                    .addClass(extraClass.join(' '))
                                    .attr({
                                        'data-id': event.id,
                                        'data-repeat-time-offset': event.repeat_time_offset ? event.repeat_time_offset : 0
                                    })
                                    .html(function (index, html) {

                                        var timezoneOffsetLabel;
                                        var dateBigStart;
                                        var dateBigEnd;
                                        var dateBig;
                                        var productsString = '';
                                        var icon = [];
                                        var images = '';
                                        var permalink;
                                        var d1;

                                        var date = helper.beautifyTimespan(event.start_date, parseInt(event.hide_end, 10) === 1 ? false : event.end_date, event.all_day);
                                        var gmtUtcOffset = parseInt(event.timezone_utc_offset, 10) / 3600;
                                        gmtUtcOffset = gmtUtcOffset > 0 ? '+' + gmtUtcOffset : gmtUtcOffset;

                                        if (gmtUtcOffset === 0) {
                                            gmtUtcOffset = '';
                                        }

                                        timezoneOffsetLabel = parseInt(glob.options.general_settings.date_label_gmtutc, 10) === 0 ? '' : 'UTC/GMT ' + gmtUtcOffset;
                                        dateBigStart = '<span>' + moment(event.start_date).format('D') + '</span> ' + __(moment(event.start_date).format('MMM').toLowerCase());
                                        dateBigEnd = '<span>' + moment(event.end_date).format('D') + '</span> ' + __(moment(event.end_date).format('MMM').toLowerCase());
                                        dateBig = dateBigStart + ' ' + (dateBigStart !== dateBigEnd ? ' - ' + dateBigEnd : '');

                                        if (event.products.length > 0) {
                                            productsString = event.products[0].title;
                                            if (event.products.length > 1) {
                                                productsString += ' +' + (event.products.length - 1) + ' ' + (event.products.length - 1 > 1 ? __('morePlural') : __('moreSingular'));
                                            }
                                        }

                                        switch (event.icon_type) {

                                            case 'image' :

                                                if (event.icon_images.length > 0) {

                                                    icon.push('<div class="stec-layout-event-preview-left-icon-images">');

                                                    $.each(event.icon_images_meta, function (i) {
                                                        icon.push('<div class="stec-layout-event-preview-left-icon-image ' + (i === 0 ? 'active' : '') + '" style="background-image: url(' + this.src + ')"></div>');
                                                    });

                                                    icon.push('</div>');

                                                } else {
                                                    icon.push('<i class="' + event.icon + '"></i>');
                                                }

                                                break;

                                            case 'date' :
                                                d1 = moment(event.start_date);

                                                icon.push('<span class="stec-layout-event-preview-left-icon-date"><span class="stec-layout-event-preview-left-icon-date-number">' + d1.format('D') + '</span><span class="stec-layout-event-preview-left-icon-date-month">' + __(d1.format('MMM').toLowerCase()) + '</span></span>');

                                                break;

                                            default:
                                                icon.push('<i class="' + event.icon + '"></i>');

                                        }

                                        if (event.images.length > 0) {
                                            $.each(event.images_meta, function (i) {
                                                /**
                                                 * @todo change to optimal image size
                                                 */
                                                images += '<div class="stec-layout-boxgrid-image ' + (i === 0 ? 'active' : '') + '" style="background-image: url(' + this.src + ')"></div>';
                                            });
                                        }

                                        permalink = helper.getPermalink(event.permalink, event.repeat_time_offset);

                                        if (glob.options.general_settings.open_event_in === 'external') {
                                            if (event.link) {
                                                permalink = event.link.url;
                                            }
                                        }

                                        return html
                                                .replace(/#stec_replace_permalink/g, permalink)
                                                .replace(/stec_replace_atarget/g, glob.options.general_settings.event_open_target)
                                                .replace('stec_replace_tags', tagsHtml.join(''))
                                                .replace('stec_replace_summary', event.title)
                                                .replace('stec_replace_date_big', dateBig)
                                                .replace('stec_replace_date', date + ' ' + timezoneOffsetLabel)
                                                .replace('stec_replace_image', images)
                                                .replace('stec_replace_event_background=""', 'style="background:' + event.color + '"') // edge is retarded
                                                .replace('stec_replace_icon', icon.join(''))
                                                .replace('stec_replace_short_desc', event.description_short);

                                    }).appendTo($instance.$boxgrid.find('.stec-layout-boxgrid-events'));

                        });

                        helper.extendBind("stachethemes_ec_extend", "afterFillEvents");

                        parent.resizeEvents();

                    });

                },

                resizeEvents: function () {

                    var parent, $boxGridEvents, hasGutter;

                    if (!$instance.$boxgrid.is(':visible')) {
                        return;
                    }

                    parent = this;
                    $boxGridEvents = $instance.$boxgrid.find('.stec-layout-boxgrid-event');
                    hasGutter = parent.options.gutter ? true : false;

                    parent.calcColumns();

                    if (hasGutter) {
                        $boxGridEvents.css({
                            marginRight: 0,
                            marginBottom: parent.options.gutter
                        });
                    }

                    $boxGridEvents.css({
                        width: function () {
                            var boxGridWidth = Math.floor(helper.getElementWidth($instance.$boxgrid.find('.stec-layout-boxgrid-events')));
                            var val = (boxGridWidth - ((parent.columns - 1) * parent.options.gutter)) / parent.columns;
                            return Math.floor(val);
                        }
                    });

                    if (hasGutter) {
                        $boxGridEvents.filter(':not(:nth-child(' + parent.columns + 'n))').css({
                            marginRight: parent.options.gutter
                        });
                    }
                },

                calcColumns: function () {
                    var parent = this;
                    var boxGridWidth = Math.floor(helper.getElementWidth($instance.$boxgrid.find('.stec-layout-boxgrid-events')));

                    parent.columns = parent.options.defaultColumns;

                    while ((boxGridWidth - ((parent.columns - 1) * parent.options.gutter)) / parent.columns < parent.options.blockMinWidth) {
                        parent.columns--;

                        if (parent.columns === 1) {
                            break;
                        }
                    }
                }
            },

            /**
             * MAP
             */
            map: {
                bindControls: function () {

                },

                set: function () {

                    $instance.$map.show();

                    if (false === layout.map.sidebar.hasSidebar()) {
                        $instance.$map.find('.stec-layout-map-container').width('100%');
                    }

                    switch (glob.options.general_settings.map_type) {
                        case 'gmap' :
                            this.gmap.set();
                            break;
                        case 'osm' :
                            this.osm.set();
                            break;
                    }

                    this.sidebar.set();

                },

                getRanges: function () {

                    var ranges = [], from, to;

                    function calendarRanges() {
                        let d, m, y, sd, from, to;

                        d = glob.options.day;
                        m = glob.options.month;
                        y = glob.options.year;
                        sd = moment();

                        sd.set('year', y);
                        sd.set('month', m);
                        sd.set('date', d);
                        sd.set('hour', 0);
                        sd.set('minute', 0);
                        sd.set('second', 0);

                        from = sd.startOf('month').toDate();
                        to = sd.endOf('month').toDate();

                        return [from, to];
                    }

                    if (glob.options.map_date_start && glob.options.map_date_end) {

                        from = moment(glob.options.map_date_start);
                        to = moment(glob.options.map_date_end);

                        if (false === from.isValid() || false === to.isValid()) {
                            ranges = calendarRanges();
                        } else {
                            ranges = [from.toDate(), to.toDate()];

                            $instance.$top.find(".stec-top-menu-date-month").hide();
                            $instance.$top.find(".stec-top-menu-date-year").hide();
                            $instance.$top.find("[data-action='previous']").hide();
                            $instance.$top.find("[data-action='next']").hide();

                        }
                    } else {
                        ranges = calendarRanges();
                    }

                    return ranges;

                },

                getEventsWithPhysicalLocations: function (events) {
                    var filtered = [];

                    $.each(events, function () {

                        if (this.location.id && 'virtual' !== this.location.type) {
                            filtered.push(this);
                        }

                    });

                    return filtered;
                },

                getMarkerHtml: function (event) {

                    var html = [];
                    var date = helper.beautifyTimespan(event.start_date, event.end_date, event.all_day);

                    html.push('<div style="box-sizing: border-box; max-width:270px; padding:10px;">');
                    html.push('<h3>' + event.title + '</h3>');
                    html.push('<p style="margin-top:5px;">' + date + '</p>');

                    if (event.location.full_address) {
                        html.push('<p style="margin-top:5px;">' + event.location.full_address + '</p>');
                    }

                    html.push('<p style="margin-top:5px;">' + event.description_short + '</p>');

                    if (false === layout.map.sidebar.hasSidebar()) {
                        html.push('<a class="stec-style-button" target="' + (glob.options.general_settings.event_open_target ? glob.options.general_settings.event_open_target : '_blank') + '" href="' + helper.getPermalink(event.permalink, event.repeat_time_offset) + '" style="margin-top:10px; display: block;">' + __('View Event') + '</a>');
                    }

                    html.push('</div>');

                    return html.join('');

                },

                gmap: {
                    zoom: 8,
                    gmap: null,
                    geocoder: null,
                    container: null,
                    location: [0, 0],
                    markers: [],
                    activeInfoWindow: null,

                    canvasImage: function (width, height, color, bg, lineWidth) {
                        var context, x, y, w, h, r;
                        var canvas = document.createElement("canvas");
                        canvas.width = width;
                        canvas.height = height;
                        context = canvas.getContext("2d");

                        context.beginPath();
                        context.beginPath();

                        x = 0;
                        y = 0;
                        w = width;
                        h = height;
                        r = 4;

                        context.moveTo(x + r, y);
                        context.lineTo(x + w - r, y);
                        context.quadraticCurveTo(x + w, y, x + w, y + r);
                        context.lineTo(x + w, y + h - r);
                        context.quadraticCurveTo(x + w, y + h, x + w - r, y + h);
                        context.lineTo(x + r, y + h);
                        context.quadraticCurveTo(x, y + h, x, y + h - r);
                        context.lineTo(x, y + r);
                        context.quadraticCurveTo(x, y, x + r, y);

                        context.fill();

                        context.fillStyle = color;
                        context.fill();
                        context.lineWidth = lineWidth;
                        context.strokeStyle = bg;
                        context.stroke();

                        return canvas.toDataURL();
                    },

                    set: function () {

                        var parent = this;

                        if (parent.gmap === null) {

                            parent.zoom = glob.options.map_zoom ? parseInt(glob.options.map_zoom, 10) : parent.zoom;

                            parent.container = $instance.$map.find('.stec-layout-map-container').get(0);

                            parent.gmap = new window.google.maps.Map(parent.container, {
                                zoom: parent.zoom,
                                mapTypeControlOptions: {
                                    mapTypeIds: []
                                }
                            });

                            if (glob.options.gmap_theme && glob.options.gmap_theme !== 'standard') {
                                parent.gmap.mapTypes.set('Custom_Style', helper.getMapStyles(glob.options.gmap_theme));
                                parent.gmap.setMapTypeId('Custom_Style');
                            }

                            parent.geocoder = new window.google.maps.Geocoder();

                            if (glob.options.map_focus) {
                                parent.focusLocation(glob.options.map_focus);
                            } else {
                                parent.focusLocation(null, parent.location);
                            }
                        } else {
                            parent.refresh(true);
                        }

                        parent.drawActiveMarkers();

                    },

                    clearMarkers: function () {

                        var i;

                        for (i = 0; i < this.markers.length; i++) {
                            this.markers[i].setMap(null);
                        }
                    },

                    drawActiveMarkers: function () {

                        var parent = this;
                        var ranges = layout.map.getRanges();
                        this.clearMarkers();

                        calData.getNearestDateEventsThread(ranges[0], ranges[1], true, 'drawActiveMarkers').then(function (events) {

                            var physicalEvents = layout.map.getEventsWithPhysicalLocations(events);
                            var focused = false;

                            if (physicalEvents.length <= 0) {
                                layout.map.sidebar.noEventsDisplay();
                                helper.extendBind("stachethemes_ec_extend", "afterFillEvents");
                                return;
                            }

                            $.each(physicalEvents, function () {

                                let event = this;
                                let locationString;

                                if (false === focused) {

                                    if (!glob.options.map_focus) {

                                        if (event.location.coordinates) {
                                            parent.location = event.location.coordinates.split(',');
                                            parent.focusLocation(null, parent.location);
                                        } else {
                                            parent.focusLocation(event.location.full_address);
                                        }
                                    }

                                    focused = true;

                                }

                                locationString = event.location.full_address;

                                if (!event.location.coordinates) {

                                    parent.getCoordinates(locationString, function (coords) {

                                        if (coords !== false) {
                                            event.location.coordinates = coords.toString().replace(/\(|\)/gi, '');
                                            parent.addMarker(event);
                                        }
                                    });

                                } else {
                                    parent.addMarker(event);
                                }

                            });

                            layout.map.sidebar.listEvents(physicalEvents);
                            helper.extendBind("stachethemes_ec_extend", "afterFillEvents");
                        });

                    },

                    addMarker: function (event) {

                        var parent = this, marker, infoWindow;
                        var coords = event.location.coordinates.split(',').map(function (x) {
                            return Number(x);
                        });
                        var latlng = {lat: coords[0], lng: coords[1]};

                        glob.temp.stecMapMarkerIcon = parent.canvasImage(50, 50, event.color, 'white', 4);


                        glob.temp.stecMapPopupHtml = layout.map.getMarkerHtml(event);

                        $instance.trigger('stecMapMarkerIcon', {
                            $instance: $instance,
                            event: event,
                            temp: glob.temp
                        });

                        marker = new window.google.maps.Marker({
                            position: latlng,
                            map: parent.gmap,
                            icon: glob.temp.stecMapMarkerIcon,
                            title: event.title,
                            zIndex: 1000,
                            stec: {
                                event: event
                            }
                        });

                        infoWindow = new window.google.maps.InfoWindow({
                            content: glob.temp.stecMapPopupHtml
                        });

                        marker.addListener('click', function () {

                            if (parent.activeInfoWindow) {
                                parent.activeInfoWindow.close();
                            }

                            infoWindow.open(parent.gmap.gmap, marker);

                            if (layout.map.sidebar.hasSidebar()) {
                                layout.map.sidebar.focusEvent(this.stec.event);
                            }


                            parent.activeInfoWindow = infoWindow;
                        });


                        window.google.maps.event.addListener(infoWindow, 'closeclick', function () {
                            layout.map.sidebar.clearFocus();

                            if (layout.map.sidebar.isSmallScreen()) {
                                // restore sidebar scroller for small screen
                                layout.map.sidebar.scroller.doScroll();
                            }
                        });


                        parent.markers.push(marker);
                    },

                    getCoordinates: function (location, callback) {

                        var parent = this;

                        parent.geocoder.geocode({'address': location}, function (results, status) {

                            var coords = false;

                            if (status === window.google.maps.GeocoderStatus.OK) {

                                coords = results[0].geometry.location;

                                if (typeof callback === 'function') {
                                    callback(coords);
                                }

                            } else {

                                if ("OVER_QUERY_LIMIT" === status) {
                                    setTimeout(function () {
                                        parent.getCoordinates(location, callback);
                                    }, 3000);

                                    return;
                                } else {
                                    console.log("Geocoder error: " + status);
                                }

                                if (typeof callback === 'function') {
                                    callback(false);
                                }
                            }
                        });
                    },

                    focusLocation: function (location, coordinates) {

                        var parent = this;

                        if (!location && coordinates) {
                            parent.gmap.setCenter(new window.google.maps.LatLng(coordinates[0], coordinates[1]));
                            parent.refresh();
                            return;
                        }

                        if (!location) {
                            location = parent.location;
                        }

                        parent.geocoder.geocode({'address': location}, function (results, status) {
                            if (status === window.google.maps.GeocoderStatus.OK) {
                                parent.gmap.setCenter(results[0].geometry.location);
                            } else {
                                console.log("Geocoder error: " + status);
                            }
                        });


                        parent.refresh();
                    },

                    refresh: function (focusLocation) {

                        var parent = this;

                        setTimeout(function () {
                            window.google.maps.event.trigger(parent.container, 'resize');

                            if (!glob.options.map_focus && focusLocation === true) {
                                parent.focusLocation(null, parent.location);
                            }

                        }, 10); // timeout fixes resize bug

                    }
                },

                osm: {
                    zoom: 8,
                    omap: null,
                    container: null,
                    location: [0, 0],
                    markers: [],
                    markersLayer: null,
                    infoWindow: null,

                    set: function () {

                        var parent = this;

                        if (parent.omap === null) {

                            parent.zoom = glob.options.map_zoom ? parseInt(glob.options.map_zoom, 10) : parent.zoom;

                            parent.container = $instance.$map.find('.stec-layout-map-container').get(0);

                            parent.omap = helper.osm(parent.container);

                            if (glob.options.map_focus) {
                                if (glob.options.map_focus.match(/[aA-zZ]+/) === null) {
                                    parent.location = glob.options.map_focus.split(',');
                                    parent.omap.setView(parent.location, parent.zoom);
                                    glob.options.map_focus = null;
                                } else {
                                    helper.promiseGetAddressCoordinates(glob.options.map_focus).then(function (coords) {
                                        parent.location = coords.split(',');
                                        parent.omap.setView(parent.location, parent.zoom);
                                        glob.options.map_focus = null;
                                    }).catch(function () {
                                        parent.omap.setView(parent.location, parent.zoom);
                                    });
                                }
                            } else {
                                parent.omap.setView(parent.location, parent.zoom);
                            }
                        }

                        parent.drawActiveMarkers();

                    },
                    clearMarkers: function () {
                        if (this.markersLayer) {
                            this.omap.removeLayer(this.markersLayer);
                            this.markersLayer = null;
                            this.markers = [];
                        }
                    },
                    drawActiveMarkers: function () {

                        var parent = this;
                        var ranges = layout.map.getRanges();
                        var loopIndex = 0;

                        parent.clearMarkers();

                        // Get all events for this timespan
                        calData.getNearestDateEventsThread(ranges[0], ranges[1], true, 'drawActiveMarkers').then(function (events) {

                            var eventsToDraw = [];
                            var physicalEvents = layout.map.getEventsWithPhysicalLocations(events);
                            var phEventsLength = physicalEvents.length;

                            if (phEventsLength <= 0) {
                                layout.map.sidebar.noEventsDisplay();
                                helper.extendBind("stachethemes_ec_extend", "afterFillEvents");
                                return;
                            }

                            $.each(physicalEvents, function () {

                                let event = this;

                                helper.promiseCoordinates(event).then(function (coords) {
                                    event.location.coordinates = coords;
                                    parent.addMarker(event);
                                    eventsToDraw.push(event);

                                }).finally(function () {

                                    loopIndex++;

                                    if (loopIndex === phEventsLength) {
                                        if (!glob.options.map_focus) {
                                            parent.location = eventsToDraw[0].location.coordinates.split(',');
                                            parent.omap.setView(parent.location, parent.zoom);
                                        }

                                        parent.markersLayer = window.L.layerGroup(parent.markers);
                                        parent.omap.addLayer(parent.markersLayer);

                                        helper.extendBind("stachethemes_ec_extend", "afterFillEvents");
                                    }
                                });
                            });

                            layout.map.sidebar.listEvents(physicalEvents);

                        });

                    },

                    addMarker: function (event) {
                        var parent = this, d1, iconHtml = [];

                        var coords = event.location.coordinates.split(',').map(function (x) {
                            return Number(x);
                        });

                        var marker;

                        switch (event.icon_type) {

                            case 'image' :

                                if (event.icon_images.length > 0) {

                                    iconHtml.push("<div style='background-image: url(" + event.icon_images_meta[0].thumb + ")'></div>");

                                } else {
                                    iconHtml.push("<div style='background-color:" + event.color + "'><i class='" + event.icon + "'></i></div>");
                                }

                                break;

                            case 'date' :
                                d1 = moment(event.start_date);

                                iconHtml.push("<div style='background-color:" + event.color + "'><span class='icon-date'>" + d1.format('D') + "</span> <span class='icon-month'>" + __(d1.format('MMM').toLowerCase()) + "</span></div>");
                                break;

                            default:
                                iconHtml.push("<div style='background-color:" + event.color + "'><i class='" + event.icon + "'></i></div>");

                        }

                        glob.temp.stecMapMarkerIcon = new window.L.divIcon({
                            className: 'stec-omap-marker-pin',
                            html: iconHtml.join(''),
                            iconSize: [54, 54],
                            iconAnchor: [22, 22]
                        });

                        glob.temp.stecMapPopupHtml = layout.map.getMarkerHtml(event);

                        $instance.trigger('stecMapMarkerIcon', {
                            $instance: $instance,
                            event: event,
                            temp: glob.temp
                        });

                        $instance.trigger('stecMapPopup', {
                            $instance: $instance,
                            event: event,
                            temp: glob.temp
                        });

                        marker = window.L.marker(coords, {
                            icon: glob.temp.stecMapMarkerIcon,
                            riseOnHover: true
                        });

                        marker.bindPopup(glob.temp.stecMapPopupHtml);

                        marker.on('click', function () {

                            if (layout.map.sidebar.hasSidebar()) {
                                layout.map.sidebar.focusEvent(event);
                            }
                        });

                        marker.getPopup().on('remove', function () {

                            if (layout.map.sidebar.hasSidebar()) {
                                layout.map.sidebar.clearFocus();

                                if (layout.map.sidebar.isSmallScreen()) {
                                    // restore sidebar scroller for small screen
                                    layout.map.sidebar.scroller.doScroll();
                                }
                            }
                        });

                        parent.markers.push(marker);

                    }
                },

                sidebar: {

                    isSidebarEnabled: null,
                    maxHeight: 600,

                    hasSidebar: function () {

                        if (null === this.isSidebarEnabled) {
                            this.isSidebarEnabled = !(glob.options.map_list && parseInt(glob.options.map_list, 10) === 0);
                        }

                        return this.isSidebarEnabled;
                    },

                    set: function () {

                        if (false === this.hasSidebar()) {
                            $instance.$map.find('.stec-layout-map-sidebar').hide();
                            return;
                        }

                        $instance.$map.removeClass('sidebar-fullwidth');
                        eventsHandler.eventHolder.close();
                        this.clearFocus();
                        $instance.$map.find('.stec-layout-map-sidebar .stec-scrollable').css('max-height', this.maxHeight);
                    },

                    isSmallScreen: function () {
                        return (helper.isMobile() || $instance.hasClass('stec-media-small') || $instance.hasClass('stec-media-med'));
                    },

                    scroller: {

                        doScroll: function () {

                            var scr;

                            if ($instance.$map.find('.stec-scrollable').data('stec-scroll')) {
                                scr = $instance.$map.find('.stec-scrollable').data('stec-scroll');
                                scr.reset();
                            } else {
                                scr = new window.StecScroll();
                                scr.create($instance.$map.find('.stec-scrollable'));
                                $instance.$map.find('.stec-scrollable').data('stec-scroll', scr);
                            }

                            return scr;
                        },

                        removeScroller: function () {
                            var scr = $instance.$map.find('.stec-scrollable').data('stec-scroll');

                            if (scr) {
                                scr.destroy();
                                $instance.$map.find('.stec-scrollable').data('stec-scroll', null);
                            }

                        },

                        scrollTo: function (y) {
                            var scr = $instance.$map.find('.stec-scrollable').data('stec-scroll');

                            if (scr && scr.outer) {
                                scr.scrollTo(y);
                            }
                        }
                    },

                    clearFocus: function () {
                        var $eventsContainer = $instance.$map.find('.stec-layout-events');
                        var $event = $eventsContainer.find('.stec-layout-event');

                        $eventsContainer.removeClass('container-focused');
                        $event.removeClass('event-focused');
                    },

                    focusLocation: function (coordinatesArray) {

                        if ('1' === glob.options.map_list_nofocus) {
                            return;
                        }

                        switch (glob.options.general_settings.map_type) {
                            case 'gmap' :
                                layout.map.gmap.gmap.panTo(new window.google.maps.LatLng(coordinatesArray[0], coordinatesArray[1]));
                                break;
                            case 'osm' :
                                layout.map.osm.omap.setView(coordinatesArray, layout.map.osm.zoom);
                                break;
                        }
                    },

                    focusEvent: function (event) {

                        var id = event.id;
                        var offset = event.repeat_time_offset ? event.repeat_time_offset : 0;
                        var $eventsContainer = $instance.$map.find('.stec-layout-events');
                        var $event = $eventsContainer.find('.stec-layout-event[data-id="' + id + '"][data-repeat-time-offset="' + offset + '"]');

                        if (!helper.isMobile()) {
                            this.scroller.scrollTo(-1 * $event.position().top);
                            this.clearFocus();
                        }

                        $eventsContainer.addClass('container-focused');
                        $event.addClass('event-focused');

                        if (this.isSmallScreen()) {
                            // Small screen displays only the focused event on focus, so this remove the scrollbar
                            this.scroller.removeScroller();
                        }

                    },

                    noEventsDisplay: function () {
                        this.scroller.removeScroller();

                        $instance.$map.find('.stec-layout-map-noevents').show();
                    },

                    listEvents: function (events) {

                        if (!events) {
                            this.noEventsDisplay();
                            return;
                        }

                        $instance.$map.find('.stec-layout-map-noevents').hide();

                        eventsHandler.eventHolder.openEvents(events, function () {
                            layout.map.sidebar.scroller.doScroll();
                        }, true);
                    },

                    bindControls: function () {

                        helper.onResizeEnd(function () {
                            if ($instance.$map.is(":visible")) {
                                layout.map.sidebar.scroller.doScroll();
                                layout.map.sidebar.scroller.scrollTo(0);
                            }
                        }, 50);

                        $instance.on('onEventToggleOpen', function (e, data) {

                            if (glob.options.general_settings.open_event_in !== 'self') {
                                return;
                            }

                            if (false === data.$container.parent().hasClass('event-focused')) {
                                layout.map.sidebar.clearFocus();
                            }

                            $instance.$map.addClass('sidebar-fullwidth');

                            layout.map.sidebar.scroller.removeScroller();

                            $instance.$map.find('.stec-layout-map-sidebar .stec-scrollable').css('max-height', 'none');

                        });

                        $instance.on('onEventToggleClose', function () {

                            if (glob.options.general_settings.open_event_in !== 'self') {
                                return;
                            }

                            $instance.$map.removeClass('sidebar-fullwidth');

                            $instance.$map.find('.stec-layout-map-sidebar .stec-scrollable').css('max-height', layout.map.sidebar.maxHeight);

                            layout.map.sidebar.scroller.doScroll();
                        });

                        $instance.$map.find('.stec-layout-events').on('mouseenter', '.stec-layout-event', function () {

                            var id, event;

                            if ($instance.$map.find('.stec-layout-events').hasClass('container-focused')) {
                                return;
                            }

                            id = $(this).data('id');
                            event = calData.getEventById(id);

                            if (event.location && event.location.coordinates) {
                                layout.map.sidebar.focusLocation(event.location.coordinates.split(','));
                            }
                        });
                    }

                }

            },

            bindControls: function () {

                setInterval(function () {

                    $instance.find('.stec-layout-event-preview-left-icon-images').filter(':visible').each(function () {

                        var currentImage, nextImage;

                        if ($(this).children().length <= 1) {
                            return;
                        }

                        currentImage = $(this).children().filter('.active');
                        nextImage = currentImage.next();

                        if (nextImage.length === 0) {
                            nextImage = $(this).children().first();
                        }

                        nextImage.addClass('transition');

                        setTimeout(function () {
                            currentImage.removeClass('active');
                            nextImage.removeClass('transition').addClass('active');

                        }, 1000);

                    });

                }, 6000);

                setInterval(function () {

                    $instance.$boxgrid.find('.stec-layout-boxgrid-images').filter(':visible').each(function () {

                        var currentImage, nextImage;

                        if ($(this).children().length <= 1) {
                            return;
                        }

                        currentImage = $(this).children().filter('.active');
                        nextImage = currentImage.next();

                        if (nextImage.length === 0) {
                            nextImage = $(this).children().first();
                        }

                        nextImage.addClass('transition');

                        setTimeout(function () {
                            currentImage.removeClass('active');
                            nextImage.removeClass('transition').addClass('active');

                        }, 1000);

                    });

                }, 6000);
            }

        };

        this.refreshState = function (type) {
            switch (type) {
                case 'layout' :
                    layout.set();
                    break;

                case 'top' :
                    top.set();
                    break;

                default:
                    layout.set();
            }
        };

        this.prepareEvents = function (callback) {

            var promiseAddToEventsPool;

            if (window.stecJsonEvents && window.stecJsonEvents[glob.options.id]) {
                // Fixes calData filters execution on initial load
                promiseAddToEventsPool = new window.Promise(function (resolve, reject) {
                    calData.addToEventsPool(window.stecJsonEvents[glob.options.id]);
                    setTimeout(function () {
                        resolve();
                    });
                });

                promiseAddToEventsPool.then(function () {
                    if (typeof callback === 'function') {
                        callback();
                    }
                });

            } else {

                preloader.add();

                calData.queryEvents(callback, function step() {

                    preloader.count(calData.eventsPool.length);

                });

            }

        };

        this.init = function (options) {

            var now = new Date(), monthNumber, theWeek, theMonth, theToday, match;

            DB = new window.StecDB();
            helper = new window.StecHelper();
            calData = new window.StecCalData();

            $.extend(glob.options, options);

            instance = "#" + glob.options.id;
            $instance = $(instance);

            helper
                    .set('glob', glob)
                    .set('calData', calData)
                    .set('$instance', $instance)
                    .set('instance', instance)
                    .set('layout', layout)
                    .set('eventsHandler', eventsHandler);

            calData
                    .set('helper', helper)
                    .set('glob', glob)
                    .set('DB', DB);

            if ('0' !== glob.options.general_settings.reminder) {
                reminder = new window.StecReminder('reminder-popup-id-' + glob.options.id);

                reminder
                        .set('$instance', $instance)
                        .set('instance', instance)
                        .set('helper', helper)
                        .set('calData', calData)
                        .set('displayPrivacyAgreement', glob.options.general_settings.display_privacy_agreement)
                        .set('privacyAgreemntContent', glob.options.general_settings.privacy_agreement_content)
                        .set('glob', glob);

                reminder.controller();
            }


            rsvp = new window.StecRsvpToEvent('rsvp-popup-id-' + glob.options.id);

            rsvp
                    .set('$instance', $instance)
                    .set('helper', helper)
                    .set('calData', calData)
                    .set('layout', layout)
                    .set('displayPrivacyAgreement', glob.options.general_settings.display_privacy_agreement)
                    .set('privacyAgreemntContent', glob.options.general_settings.privacy_agreement_content)
                    .set('glob', glob);

            rsvp.controller();

            glob.rsvp = rsvp;

            if (window.StecBooking) {
                booking = new window.StecBooking('booking-popup-id-' + glob.options.id);

                booking
                        .set('$instance', $instance)
                        .set('helper', helper)
                        .set('glob', glob);

                booking.controller();
            }

            if (window.StecShop) {
                shop = new window.StecShop();

                shop
                        .set('$instance', $instance)
                        .set('helper', helper)
                        .set('glob', glob);

                shop.controller();
            }

            if ($instance.length <= 0) {
                console.log('Stachethemes Event Calendar - License is not activated');
                return;
            }

            if (helper.isMobile()) {
                $instance.addClass('stec-mobile');
            }

            $instance.$top = $instance.children(".stec-top");                                 // inc.top.php wrap
            $instance.$agenda = $instance.children(".stec-layout").find(".stec-layout-agenda"); // layout.agenda.inc.php wrap
            $instance.$month = $instance.children(".stec-layout").find(".stec-layout-month"); // layout.month.inc.php wrap
            $instance.$week = $instance.children(".stec-layout").find(".stec-layout-week"); // layout.month.inc.php wrap
            $instance.$day = $instance.children(".stec-layout").find(".stec-layout-day"); // layout.day.inc.php wrap
            $instance.$grid = $instance.children(".stec-layout").find(".stec-layout-grid"); // layout.grid.inc.php wrap
            $instance.$boxgrid = $instance.children(".stec-layout").find(".stec-layout-boxgrid"); // layout.boxgrid.inc.php wrap
            $instance.$map = $instance.children(".stec-layout").find(".stec-layout-map"); // layout.map.inc.php wrap
            $instance.$events = $instance.children(".stec-layout").find(".stec-layout-events"); // layout.event.inc.php wrap

            $instance.$top.path = instance + " .stec-top ";
            $instance.$agenda.path = instance + " .stec-layout .stec-layout-agenda ";
            $instance.$month.path = instance + " .stec-layout .stec-layout-month ";
            $instance.$week.path = instance + " .stec-layout .stec-layout-week ";
            $instance.$day.path = instance + " .stec-layout .stec-layout-day ";
            $instance.$events.path = instance + " .stec-layout-events ";

            glob.template.eventCreateForm = $('#stec-event-create-form-template').html();
            glob.template.event = $('#stec-event-template').html();
            glob.template.gridevent = $('#stec-grid-event-template').html();
            glob.template.boxgridevent = $('#stec-boxgrid-event-template').html();
            glob.template.eventInner = $('#stec-event-inner-template').html();
            glob.template.tooltip = $('#stec-tooltip-template').html();
            glob.template.preloader = $('#stec-preloader-template').html();
            glob.template.share = $('#stec-share-template').html();
            glob.template.emailEvent = $('#stec-email-event-template').html();

            glob.options.view = glob.options.general_settings.view;

            // Go to user specified date on init
            if (typeof glob.options.start_date !== 'undefined') {
                switch (glob.options.start_date) {

                    case 'prev_week' :
                        now = moment().subtract(1, 'week').startOf('week').toDate();
                        break;
                    case 'prev_month' :
                        now = moment().subtract(1, 'month').startOf('month').toDate();
                        break;

                    case 'this_week' :
                        now = moment().startOf('week').toDate();
                        break;
                    case 'next_week' :
                        now = moment().add(1, 'week').startOf('week').toDate();
                        break;
                    case 'next_month' :
                        now = moment().add(1, 'month').startOf('month').toDate();
                        break;
                    case 'this_month' :
                        now = moment().startOf('month').toDate();
                        break;

                    default:

                        match = glob.options.start_date.match(/(\+|\-)(\d)M/);

                        if (null !== match) {
                            switch (match[1]) {
                                case '+' :
                                    now = moment().add(parseInt(match[2], 10), 'month').startOf('month').toDate();
                                    break;

                                case '-' :
                                    now = moment().subtract(parseInt(match[2], 10), 'month').startOf('month').toDate();
                                    break;
                            }
                        } else {
                            now = new Date(glob.options.start_date);
                        }
                }
            }

            if (glob.options.min_date) {
                switch (glob.options.min_date) {
                    case 'this_week' :
                        theWeek = helper.getWeekInfo(moment().toDate());
                        monthNumber = theWeek.start.month + 1;
                        monthNumber = parseInt(('0' + monthNumber).slice(-2), 10);
                        glob.options.min_date = theWeek.start.year + '-' + monthNumber + '-' + theWeek.start.day;
                        break;
                    case 'prev_week' :
                        theWeek = helper.getWeekInfo(moment().subtract(1, 'week').toDate());
                        monthNumber = theWeek.start.month + 1;
                        monthNumber = parseInt(('0' + monthNumber).slice(-2), 10);
                        glob.options.min_date = theWeek.start.year + '-' + monthNumber + '-' + theWeek.start.day;
                        break;
                    case 'next_week' :
                        theWeek = helper.getWeekInfo(moment().add(1, 'week').toDate());
                        monthNumber = theWeek.start.month + 1;
                        monthNumber = parseInt(('0' + monthNumber).slice(-2), 10);
                        glob.options.min_date = theWeek.start.year + '-' + monthNumber + '-' + theWeek.start.day;
                        break;
                    case 'prev_month' :
                        theMonth = moment().subtract(1, 'month').startOf('month');
                        glob.options.min_date = theMonth.format('YYYY-MM-DD');
                        break;
                    case 'next_month' :
                        theMonth = moment().add(1, 'month').startOf('month');
                        glob.options.min_date = theMonth.format('YYYY-MM-DD');
                        break;
                    case 'this_month' :
                        theMonth = moment().startOf('month');
                        glob.options.min_date = theMonth.format('YYYY-MM-DD');
                        break;
                    case 'today' :
                        theToday = moment().startOf('day');
                        glob.options.min_date = theToday.format('YYYY-MM-DD');
                        break;

                    default:

                        match = glob.options.min_date.match(/(\+|\-)(\d)M/);

                        if (null !== match) {
                            switch (match[1]) {
                                case '+' :
                                    glob.options.min_date = moment().add(parseInt(match[2], 10), 'month').startOf('month').format('YYYY-MM-DD');
                                    break;

                                case '-' :
                                    glob.options.min_date = moment().subtract(parseInt(match[2], 10), 'month').startOf('month').format('YYYY-MM-DD');
                                    break;
                            }
                        }
                }
            }

            if (glob.options.max_date) {
                switch (glob.options.max_date) {
                    case 'this_week' :
                        theWeek = helper.getWeekInfo(moment().toDate());
                        monthNumber = theWeek.start.month + 1;
                        monthNumber = parseInt(('0' + monthNumber).slice(-2), 10);
                        glob.options.max_date = theWeek.end.year + '-' + monthNumber + '-' + theWeek.end.day;
                        break;
                    case 'prev_week' :
                        theWeek = helper.getWeekInfo(moment().subtract(1, 'week').toDate());
                        monthNumber = theWeek.end.month + 1;
                        monthNumber = parseInt(('0' + monthNumber).slice(-2), 10);
                        glob.options.max_date = theWeek.end.year + '-' + monthNumber + '-' + theWeek.end.day;
                        break;
                    case 'next_week' :
                        theWeek = helper.getWeekInfo(moment().add(1, 'week').toDate());
                        monthNumber = theWeek.end.month + 1;
                        monthNumber = parseInt(('0' + monthNumber).slice(-2), 10);
                        glob.options.max_date = theWeek.end.year + '-' + monthNumber + '-' + theWeek.end.day;
                        break;
                    case 'prev_month' :
                        theMonth = moment().subtract(1, 'month').endOf('month');
                        glob.options.max_date = theMonth.format('YYYY-MM-DD');
                        break;
                    case 'next_month' :
                        theMonth = moment().add(1, 'month').endOf('month');
                        glob.options.max_date = theMonth.format('YYYY-MM-DD');
                        break;
                    case 'this_month' :
                        theMonth = moment().endOf('month');
                        glob.options.max_date = theMonth.format('YYYY-MM-DD');
                        break;
                    case 'today' :
                        theToday = moment().endOf('day');
                        glob.options.max_date = theToday.format('YYYY-MM-DD');
                        break;

                    default:
                        match = glob.options.max_date.match(/(\+|\-)(\d)M/);

                        if (null !== match) {
                            switch (match[1]) {
                                case '+' :
                                    glob.options.max_date = moment().add(parseInt(match[2], 10), 'month').endOf('month').format('YYYY-MM-DD');
                                    break;

                                case '-' :
                                    glob.options.max_date = moment().subtract(parseInt(match[2], 10), 'month').endOf('month').format('YYYY-MM-DD');
                                    break;
                            }
                        }
                }
            }

            glob.options.day = now.getDate();
            glob.options.month = now.getMonth();
            glob.options.year = now.getFullYear();

            if (typeof window.StecAnimate !== 'undefined') {
                helper.set('animate', new window.StecAnimate($instance));
            }

            top.init();
            layout.init();
            eventsHandler.init();

            this.prepareEvents(function () {
                preloader.destroy();
                $instance.show();
                main.refreshState('layout');
            });

            helper.extendBind("stachethemes_ec_extend");

        };

        moment.locale('en');

    }

    $(function () {

        // Set boostrap datetimepicker to no conflict mode
        if (typeof $.fn.datepicker.noConflict === 'function') {
            $.fn.bootstrapDP = $.fn.datepicker.noConflict();
        }

        $(document).on('click', '.stec-fixed-message a', function (e) {
            e.preventDefault();

            $(this).parents('.stec-fixed-message').remove();
        });

        if (typeof window.stecInstance !== "undefined") {
            $(window.stecInstance).each(function () {

                var stec;
                var thisInstance = this;

                if (this.init_on_trigger) {
                    $(document).on(this.init_on_trigger, function () {
                        $(document).trigger('stecInitOnTrigger', thisInstance);
                        stec = new StachethemesEventCalendar();
                        stec.init(thisInstance);
                    });
                } else {
                    stec = new StachethemesEventCalendar();
                    stec.init(thisInstance);
                }

            });
        }
    });

    window.StachethemesEventCalendar = StachethemesEventCalendar;

})(window.jQuery);