<?php

namespace Stachethemes\Stec;

class Buddypress {

    public static function get_group_calendars($group_id) {

        $visibility      = sprintf('stec_bp_group_%d', $group_id);
        $group_calendars = array();

        $calendars = get_posts(array(
            'post_type' => 'stec_calendar',
            'fields'    => 'ids'
        ));

        if ($calendars) {
            foreach ($calendars as $calid) {

                $post_meta_visibility = get_post_meta($calid, 'visibility', true);
                
                if (!$post_meta_visibility) {
                    continue;
                }
                
                $calendar_visibility  = explode(',', $post_meta_visibility);

                if (true === in_array($visibility, $calendar_visibility)) {
                    $group_calendars[] = $calid;
                }
            }
        }


        return $group_calendars;
    }

    public static function extend_user_roles() {

        add_filter('stec_get_user_roles_list', function($list) {

            $userid = get_current_user_id();

            if (bp_is_active('groups')) {
                $buddypress_groups = groups_get_groups(array(
                    'show_hidden' => true
                ));

                if ($buddypress_groups['groups']) {
                    foreach ($buddypress_groups['groups'] as $group) {

                        if ($group->status === 'public' || groups_is_user_member($userid, $group->id)) {
                            $list['stec_bp_group_' . $group->id] = sprintf(esc_html__('%s (Buddypress Group)', 'stec'), $group->name);
                        }
                    }
                }
            }
            return $list;
        }, 10);


        add_filter('stec_get_user_roles', function($roles, $userid) {

            if (bp_is_active('groups')) {
                $bpgroups = bp_get_user_groups($userid);
                if ($bpgroups) {
                    foreach ($bpgroups as $bp_group) {
                        $roles[] = 'stec_bp_group_' . $bp_group->group_id;
                    }
                }
            }

            return $roles;
        }, 10, 2);
    }

    public static function get_calendar_bp_groups(Calendar_Post $calendar) {

        $bp_groups  = array();
        $visibility = $calendar->get_visibility();

        foreach ($visibility as $vis) {
            if (strpos($vis, 'stec_bp_group_') === 0) {
                $group_id = filter_var(str_replace('stec_bp_group_', '', $vis), FILTER_VALIDATE_INT);

                if ($group_id) {
                    $bp_groups[] = $group_id;
                }
            }
        }

        return $bp_groups;
    }

    public static function get_event_bp_groups(Event_Post $event) {

        $bp_groups  = array();
        $visibility = array();
        $visibility = $event->get_visibility();

        if (in_array('stec_cal_default', $visibility)) {
            $calendar   = new Calendar_Post($event->get_calid());
            $visibility = $calendar->get_visibility();
        }

        foreach ($visibility as $vis) {
            if (strpos($vis, 'stec_bp_group_') === 0) {
                $group_id = filter_var(str_replace('stec_bp_group_', '', $vis), FILTER_VALIDATE_INT);

                if ($group_id) {
                    $bp_groups[] = $group_id;
                }
            }
        }

        return $bp_groups;
    }

    public static function record_activities() {

        if (!function_exists('bp_activity_add')) {
            return;
        }

        add_action('stec_after_insert_event', function ($result, $event, $is_update) {

            if (is_wp_error($result)) {
                return;
            }

            $bp_groups = self::get_event_bp_groups($event);

            if ($bp_groups) {

                $user         = get_userdata(get_current_user_id());
                $display_name = $user->get('display_name');

                foreach ($bp_groups as $group_id) {

                    if ($is_update) {
                        $activity_id = bp_activity_add(array(
                            'action'       => sprintf(esc_html__('  %s edited an event', 'stec'), $display_name),
                            'component'    => 'groups',
                            'content'      => sprintf('<a href="%s">%s</a>', $event->get_permalink(), $event->get_title()),
                            'type'         => 'edit_stec_event_post',
                            'item_id'      => $group_id,
                            'primary_link' => $event->get_permalink()
                        ));
                    } else {
                        $activity_id = bp_activity_add(array(
                            'action'       => sprintf(esc_html__('  %s submitted an event', 'stec'), $display_name),
                            'component'    => 'groups',
                            'content'      => sprintf('<a href="%s">%s</a>', $event->get_permalink(), $event->get_title()),
                            'type'         => 'new_stec_event_post',
                            'item_id'      => $group_id,
                            'primary_link' => $event->get_permalink()
                        ));
                    }

                    if (is_wp_error($activity_id)) {
                        Admin_Helper::debug_log($activity_id->get_error_message());
                    }
                }
            } else {
                if ($is_update) {
                    $activity_id = bp_activity_add(array(
                        'action'       => esc_html__('Edited an event', 'stec'),
                        'component'    => 'stec',
                        'content'      => sprintf('<a href="%s">%s</a>', $event->get_permalink(), $event->get_title()),
                        'type'         => 'edit_stec_event_post',
                        'item_id'      => $event->get_id(),
                        'primary_link' => $event->get_permalink()
                    ));
                } else {
                    $activity_id = bp_activity_add(array(
                        'action'       => esc_html__('Submitted an event', 'stec'),
                        'component'    => 'stec',
                        'content'      => sprintf('<a href="%s">%s</a>', $event->get_permalink(), $event->get_title()),
                        'type'         => 'new_stec_event_post',
                        'item_id'      => $event->get_id(),
                        'primary_link' => $event->get_permalink()
                    ));
                }

                if (is_wp_error($activity_id)) {
                    Admin_Helper::debug_log($activity_id->get_error_message());
                }
            }
        }, 10, 3);

        add_action('stec_after_delete_event', function($result, $event) {

            if (is_wp_error($result)) {
                return;
            }

            $bp_groups = self::get_event_bp_groups($event);

            if ($bp_groups) {

                $user         = get_userdata(get_current_user_id());
                $display_name = $user->get('display_name');

                foreach ($bp_groups as $group_id) {

                    $activity_id = bp_activity_add(array(
                        'action'       => sprintf(esc_html__('  %s deleted an event', 'stec'), $display_name),
                        'component'    => 'groups',
                        'content'      => $event->get_title(),
                        'type'         => 'delete_stec_event_post',
                        'item_id'      => $group_id,
                        'primary_link' => $event->get_permalink()
                    ));

                    if (is_wp_error($activity_id)) {
                        Admin_Helper::debug_log($activity_id->get_error_message());
                    }
                }
            } else {

                $activity_id = bp_activity_add(array(
                    'action'    => esc_html__('Deleted an event', 'stec'),
                    'component' => 'stec',
                    'content'   => $event->get_title(),
                    'type'      => 'delete_stec_event_post',
                    'item_id'   => $event->get_id()
                ));

                if (is_wp_error($activity_id)) {
                    Admin_Helper::debug_log($activity_id->get_error_message());
                }
            }
        }, 10, 2);

        add_action('stec_after_insert_calendar', function($result, $calendar, $is_update) {

            if (is_wp_error($result)) {
                return;
            }

            $bp_groups = self::get_calendar_bp_groups($calendar);

            if ($bp_groups) {

                $user         = get_userdata(get_current_user_id());
                $display_name = $user->get('display_name');

                foreach ($bp_groups as $group_id) {

                    if ($is_update) {
                        $activity_id = bp_activity_add(array(
                            'action'    => sprintf(esc_html__('  %s edited a calendar', 'stec'), $display_name),
                            'component' => 'groups',
                            'content'   => $calendar->get_title(),
                            'type'      => 'edit_stec_calendar_post',
                            'item_id'   => $group_id,
                        ));
                    } else {
                        $activity_id = bp_activity_add(array(
                            'action'    => sprintf(esc_html__('  %s created a calendar', 'stec'), $display_name),
                            'component' => 'groups',
                            'content'   => $calendar->get_title(),
                            'type'      => 'edit_stec_calendar_post',
                            'item_id'   => $group_id
                        ));
                    }

                    if (is_wp_error($activity_id)) {
                        Admin_Helper::debug_log($activity_id->get_error_message());
                    }
                }
            } else {

                if ($is_update) {
                    $activity_id = bp_activity_add(array(
                        'action'    => esc_html__('Edited a calendar', 'stec'),
                        'component' => 'stec',
                        'content'   => $calendar->get_title(),
                        'type'      => 'edit_stec_calendar_post',
                        'item_id'   => $calendar->get_id()
                    ));
                } else {
                    $activity_id = bp_activity_add(array(
                        'action'    => esc_html__('Created a calendar', 'stec'),
                        'component' => 'stec',
                        'content'   => $calendar->get_title(),
                        'type'      => 'new_stec_calendar_post',
                        'item_id'   => $calendar->get_id()
                    ));
                }

                if (is_wp_error($activity_id)) {
                    Admin_Helper::debug_log($activity_id->get_error_message());
                }
            }
        }, 10, 3);

        add_action('stec_after_delete_calendar', function($result, $calendar) {

            if (is_wp_error($result)) {
                return;
            }

            $bp_groups = self::get_calendar_bp_groups($calendar);

            if ($bp_groups) {

                $user         = get_userdata(get_current_user_id());
                $display_name = $user->get('display_name');

                foreach ($bp_groups as $group_id) {

                    $activity_id = bp_activity_add(array(
                        'action'    => sprintf(esc_html__('  %s deleted a calendar', 'stec'), $display_name),
                        'component' => 'groups',
                        'content'   => $calendar->get_title(),
                        'type'      => 'delete_stec_calendar_post',
                        'item_id'   => $group_id
                    ));

                    if (is_wp_error($activity_id)) {
                        Admin_Helper::debug_log($activity_id->get_error_message());
                    }
                }
            } else {

                $activity_id = bp_activity_add(array(
                    'action'    => esc_html__('Deleted a calendar', 'stec'),
                    'component' => 'stec',
                    'content'   => $calendar->get_title(),
                    'type'      => 'delete_stec_calendar_post',
                    'item_id'   => $calendar->get_id()
                ));

                if (is_wp_error($activity_id)) {
                    Admin_Helper::debug_log($activity_id->get_error_message());
                }
            }
        }, 10, 2);
    }

    public static function activity_filters() {
        add_action('bp_register_activity_actions', function() {
            bp_activity_set_action('stec', 'new_stec_event_post', esc_html__('Created Event', 'stec'));
            bp_activity_set_action('stec', 'edit_stec_event_post', esc_html__('Edited Event', 'stec'));
            bp_activity_set_action('stec', 'delete_stec_event_post', esc_html__('Deleted Event', 'stec'));
            bp_activity_set_action('stec', 'new_stec_calendar_post', esc_html__('Created Calendar', 'stec'));
            bp_activity_set_action('stec', 'edit_stec_calendar_post', esc_html__('Edited Calendar', 'stec'));
            bp_activity_set_action('stec', 'delete_stec_calendar_post', esc_html__('Deleted Calendar', 'stec'));
        });

        add_action('bp_activity_filter_options', '\Stachethemes\Stec\Buddypress::display_activity_actions');
        add_action('bp_member_activity_filter_options', '\Stachethemes\Stec\Buddypress::display_activity_actions');

        // You need to check Groups component is available
        if (bp_is_active('groups')) {
            add_action('bp_group_activity_filter_options', '\Stachethemes\Stec\Buddypress::display_activity_actions');
        }
    }

    public static function display_activity_actions() {
        $bp_plugin_actions = self::list_actions();

        if (empty($bp_plugin_actions)) {
            return;
        }

        foreach ($bp_plugin_actions as $type):
            printf('<option value="%s">%s</option>', esc_attr($type['key']), esc_attr($type['value']));
        endforeach;
    }

    private static function list_actions() {

        $bp_activity_actions = buddypress()->activity->actions;

        $bp_plugin_actions = array();

        if (!empty($bp_activity_actions->stec)) {
            $bp_plugin_actions = array_values((array) $bp_activity_actions->stec);
        }

        return $bp_plugin_actions;
    }
    
    /**
     * @todo Requires fix of the css issues caused by BP stylesheet
     */
    public static function group_calendar() {

        add_action('wp_enqueue_scripts', function() {

            global $bp;

            if ('stec-bp-group-calendar' === $bp->current_action) {
                do_action('stec_load_calendar_styles');
                do_action('stec_load_calendar_scripts');

                add_action('wp_head', function() {
                    Stachethemes_Event_Calendar::get_instance()->load_head();
                }, 9999);
            }
        });

        add_action('bp_setup_nav', function() {

            global $bp;

            $user_access = false;
            $group_link  = '';

            if (bp_is_active('groups') && !empty($bp->groups->current_group)) {

                $group_link      = $bp->root_domain . '/' . bp_get_groups_root_slug() . '/' . $bp->groups->current_group->slug . '/';
                $user_access     = $bp->groups->current_group->user_has_access;
                $group_calendars = self::get_group_calendars($bp->groups->current_group->id);

                if ($group_calendars) {

                    bp_core_new_subnav_item(array(
                        'name'            => esc_html__('Group Calendar', 'stec'),
                        'slug'            => 'stec-bp-group-calendar',
                        'parent_url'      => $group_link,
                        'parent_slug'     => $bp->groups->current_group->slug,
                        'screen_function' => function() use ($group_calendars) {

                            add_action('bp_template_title', function() {
                                        printf('%s <br/>', esc_html__('Group Calendar', 'stec'));
                                    });

                            add_action('bp_template_content', function() use ($group_calendars) {
                                        $shortocode = apply_filters('stec_bp_group_calendar_shortcode', sprintf('[stachethemes_ec cal="%s"]', implode(',', $group_calendars)), $group_calendars);
                                        echo do_shortcode($shortocode);
                                    });

                            bp_core_load_template(apply_filters('bp_core_template_plugin', 'groups/single/plugins'));
                        },
                        'position'        => 50,
                        'user_has_access' => $user_access,
                    ));
                }
            }
        }, 100);
    }

}

if (function_exists('bp_is_active')) {
    Buddypress::extend_user_roles();
    Buddypress::record_activities();
    Buddypress::activity_filters();
    
    /*
    if (false !== apply_filters('stec_bp_group_show_calendar', true)) {
        Buddypress::group_calendar();
    }
     */
}