<?php

namespace Stachethemes\Stec\Elementor\Widgets\Event_Submit_Form;

trait Widget_Panel_Display {

    public function show_in_panel() {

        if (!isset($this->stec_show_in_panel)) {
            $current_document = \Elementor\Plugin::$instance->documents->get_current();
            $post_type        = $current_document->get_post()->post_type;

            $this->stec_show_in_panel = apply_filters('stec_builder_show_in_panel', 'stec_esf' === $post_type, $this->get_name());
        }

        return $this->stec_show_in_panel;
    }

    public function get_setting_or_default($setting, $default = '') {

        return $this->get_settings_for_display($setting) ? $this->get_settings_for_display($setting) : $default;
    }

    public function register_no_options() {
        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'no_options_text',
                array(
                    'label' => esc_html__('No options available', 'stec'),
                    'type'  => \Elementor\Controls_Manager::HEADING,
                )
        );

        $this->end_controls_section();
    }

    // Soft deprecated for some reason keeps true === method_exists eventhough it's not
    public function _register_controls() {
        $this->register_controls();
    }

}

trait Get_Hours_And_Minutes {

    private function get_hours_and_minutes($prefix = 'start') {

        $hours_list   = \Stachethemes\Stec\Admin_Helper::get_hours_array();
        $minutes_list = \Stachethemes\Stec\Admin_Helper::minutes_array();

        ob_start();
        ?>
        <select class="stec-builder-element-content-datetime-style stec-builder-element-content-select-style" name="<?php echo esc_attr__($prefix) . '_time_hours'; ?>">
            <?php
            foreach ($hours_list as $key => $value) :
                ?>
                <option value="<?php esc_attr_e($key); ?>"><?php esc_html_e($value); ?></option>
                <?php
            endforeach;
            ?>
        </select>
        <select class="stec-builder-element-content-datetime-style stec-builder-element-content-select-style" name="<?php echo esc_attr__($prefix) . '_time_minutes'; ?>">
            <?php
            foreach ($minutes_list as $key => $value) :
                ?>
                <option value="<?php esc_attr_e($key); ?>"><?php esc_html_e($value); ?></option>
                <?php
            endforeach;
            ?>
        </select>
        <?php
        return ob_get_clean();
    }

}

class Title extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function init($data) {

        parent::init($data);
    }

    public function get_name() {
        return 'stec_submit_form_title';
    }

    public function get_title() {
        return esc_html__('Title', 'stec');
    }

    public function get_icon() {
        return 'eicon-text-field';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'title', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Event Title', 'stec'),
                ]
        );


        $this->end_controls_section();
    }

    protected function content_template() {
        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-input-style" type="text" name="title" placeholder="%s" required /></div>', '{{{ settings.placeholder }}}');
    }

    protected function render() {

        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-input-style" type="text" name="title" placeholder="%s" required /></div>', $this->get_settings_for_display('placeholder'));
    }

}

class Repeater extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_repeater';
    }

    public function get_title() {
        return esc_html__('Repeater', 'stec');
    }

    public function get_icon() {
        return 'eicon-sync';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'repeater', 'stachethemes');
    }

    protected function register_controls() {
        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Repeater Scheme', 'stec')
                ]
        );

        $this->end_controls_section();
    }

    private function get_repeat_options() {
        $options   = array();
        $options[] = "<option value='0'>" . esc_html__('No repeat', 'stec') . "</option>";
        $options[] = "<option value='1'>" . esc_html__('Daily', 'stec') . "</option>";
        $options[] = "<option value='2'>" . esc_html__('Weekly', 'stec') . "</option>";
        $options[] = "<option value='3'>" . esc_html__('Monthly', 'stec') . "</option>";
        $options[] = "<option value='4'>" . esc_html__('Yearly', 'stec') . "</option>";

        return implode(PHP_EOL, $options);
    }

    private function get_repeat_content() {
        ob_start();
        ?>
        <div class="stec-repeater-content">

            <div class="stec-repeater-content-weekdays">
                <span><?php esc_html_e('By day:', 'stec'); ?></span>
                <div class="stec-repeater-content-repeat-on">
                    <span>
                        <label title="Sunday">
                            <input name="SU" title="Sunday" type="checkbox">
                            <?php esc_html_e('SU', 'stec'); ?></label>
                    </span>
                    <span>
                        <label title="Monday">
                            <input name="MO" title="Monday" type="checkbox">
                            <?php esc_html_e('MO', 'stec'); ?></label>
                    </span>
                    <span>
                        <label title="Tuesday">
                            <input name="TU" title="Tuesday" type="checkbox">
                            <?php esc_html_e('TU', 'stec'); ?>
                        </label>
                    </span>
                    <span>
                        <label title="Wednesday">
                            <input name="WE" title="Wednesday" type="checkbox">
                            <?php esc_html_e('WE', 'stec'); ?>
                        </label>
                    </span>
                    <span>
                        <label title="Thursday">
                            <input name="TH" title="Thursday" type="checkbox">
                            <?php esc_html_e('TH', 'stec'); ?>
                        </label>
                    </span>
                    <span>
                        <label title="Friday">
                            <input name="FR" title="Friday" type="checkbox">
                            <?php esc_html_e('FR', 'stec'); ?>
                        </label>
                    </span>
                    <span>
                        <label title="Saturday">
                            <input name="SA" title="Saturday" type="checkbox">
                            <?php esc_html_e('SA', 'stec'); ?>
                        </label>
                    </span>
                </div>
            </div>

            <div class="stec-repeater-content-repeat-gap">
                <span><?php esc_html_e('Repeat gap:', 'stec'); ?></span>
                <select class="stec-builder-element-content-select-style" name="repeat_gap">
                    <option value=""><?php esc_html_e('No gap', 'stec'); ?></option>
                    <?php
                    $gap = 2;
                    while ($gap <= 30) {
                        printf("<option value='%s'>%s</option>", $gap, ($gap - 1));
                        $gap++;
                    }
                    ?>
                </select>
            </div>

            <div class="stec-repeater-content-repeat-ends-on">
                <span><?php esc_html_e('Repeat ends on:', 'stec'); ?></span>
                <div class="stec-repeater-content-endson-options">
                    <div>
                        <label>
                            <input name="repeat_endson" value="0" checked="checked" type="radio">
                            <?php esc_html_e('Never', 'stec'); ?>
                        </label>
                    </div>
                    <div>
                        <label>
                            <input name="repeat_endson" value="1" type="radio">
                            <span><?php esc_html_e('After', 'stec'); ?></span>
                            <input class="stec-builder-element-content-input-style" type="number" name="repeat_occurences" max="99" min="1" value="" disabled="disabled">
                            <span><?php esc_html_e('occurrences', 'stec'); ?></span>
                        </label>
                    </div>
                    <div>
                        <label>
                            <input name="repeat_endson" value="2" type="radio">
                            <span><?php esc_html_e('On date', 'stec'); ?></span>
                            <input class="stec-builder-element-content-input-style" name="repeat_ends_on_date" size="10" value="" disabled="disabled" autocomplete="off">
                            <input type="hidden" name="repeat_ends_on_date_alt">
                        </label>
                    </div>
                </div>
            </div>

            <div class="stec-repeater-content-exdate-menu">
                <span><?php esc_html_e('Date exception:', 'stec'); ?></span>
                <div class="stec-repeater-content-exdate-options">
                    <div>
                        <input class="stec-builder-element-content-input-style" name="repeat_exdate_datepicker" value="" type="text">
                        <input type="hidden" name="repeat_exdate_datepicker_alt">
                        <a class="stec-repeater-content-add-exdate-button" href="javascript:void(0);"><?php esc_html_e('Add exdate', 'stec'); ?></a>
                    </div>
                    <ul class="stec-repeater-content-exdate-datelist">
                    </ul>
                </div>
            </div>

            <input type="hidden" name="rrule" value="">
            <input type="hidden" name="exdate" value="">

        </div>
        <?php
        return ob_get_clean();
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" name="repeater"><option disabled selected>%s</option>%s</select>%s</div>', '{{{ settings.placeholder }}}', $this->get_repeat_options(), $this->get_repeat_content());
    }

    protected function render() {

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" name="repeater"><option disabled selected>%s</option>%s</select>%s</div>', $this->get_settings_for_display('placeholder'), $this->get_repeat_options(), $this->get_repeat_content());
    }

}

class Description extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_description';
    }

    public function get_title() {
        return esc_html__('Description', 'stec');
    }

    public function get_icon() {
        return 'eicon-text';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'description', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXTAREA,
                    'default' => esc_html__('Description', 'stec')
                ]
        );


        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {
        printf('<div class="stec-builder-element"><textarea class="stec-builder-element-content-textarea-style" name="description" placeholder="%s" %s></textarea></div>', '{{{ settings.placeholder }}}', '{{{ settings.required }}}');
    }

    protected function render() {

        printf('<div class="stec-builder-element"><textarea class="stec-builder-element-content-textarea-style" name="description" placeholder="%s" %s></textarea></div>', $this->get_settings_for_display('placeholder'), $this->get_settings_for_display('required'));
    }

}

class Description_Short extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_description_short';
    }

    public function get_title() {
        return esc_html__('Short Description', 'stec');
    }

    public function get_icon() {
        return 'eicon-text';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'description', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Short Description', 'stec')
                ]
        );

        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-input-style" name="short_desc" placeholder="%s" %s /></div>', '{{{ settings.placeholder }}}', '{{{ settings.required }}}');
    }

    protected function render() {

        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-input-style" name="short_desc" placeholder="%s" %s /></div>', $this->get_settings_for_display('placeholder'), $this->get_settings_for_display('required'));
    }

}

class Location extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_location';
    }

    public function get_title() {
        return esc_html__('Location', 'stec');
    }

    public function get_icon() {
        return 'eicon-map-pin';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'location', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );


        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    private function get_address_form() {

        $locations = \Stachethemes\Stec\Locations::get_location_country_city();

        ob_start();
        ?>
        <div class="stec-builder-submission-new-address">

            <select class="stec-builder-element-content-select-style" type="text" name="location_country" placeholder="<?php esc_attr_e('Country', 'stec'); ?>" >
                <?php
                printf('<option selected disabled>%s</option>', esc_html__('-- Country --', 'stec'));
                foreach ($locations as $country => $cities) {
                    printf('<option value="%s">%s</option>', esc_attr($country), esc_html($country));
                }
                ?>
            </select>

            <select class="stec-builder-element-content-select-style" type="text" name="location_city" placeholder="<?php esc_attr_e('City', 'stec'); ?>" >
                <?php
                printf('<option selected disabled>%s</option>', esc_html__('-- City --', 'stec'));
                foreach ($locations as $country => $cities) {
                    foreach ($cities as $city) {
                        printf('<option data-country="%s" value="%s">%s</option>', $country, esc_attr($city), esc_html($city));
                    }
                }
                ?>
            </select>
            <input class="stec-builder-element-content-input-style" type="text" name="location_address" placeholder="<?php esc_attr_e('Address', 'stec'); ?>" >
            <div class="stec-builder-element-content-coordinates-wrap">
                <input class="stec-builder-element-content-input-style" type="text" name="location_coordinates" placeholder="<?php esc_attr_e('Coordinates', 'stec'); ?>" >
                <button title="<?php esc_html_e('Find coordinates', 'stec'); ?>" type="button" data-input="location_coordinates" data-action="get-coordinates"><i class="fas fa-search-location"></i></button>
            </div>
            <textarea class="stec-builder-element-content-textarea-style" name="location_details" placeholder="<?php esc_attr_e('Details', 'stec'); ?>"></textarea>
        </div>
        <?php
        return ob_get_clean();
    }

    private function get_location_form() {
        ob_start();
        ?>
        <div class="stec-builder-submission-new-location">

            <select class="stec-builder-element-content-select-style" name="location_type">
                <option value="physical"><?php esc_html_e('Physical'); ?></option>
                <option value="virtual"><?php esc_html_e('Virtual'); ?></option>
            </select>

            <input class="stec-builder-element-content-input-style" type="text" name="location_address" placeholder="<?php esc_attr_e('Address', 'stec'); ?>" >

            <div class="stec-builder-submission-new-location-physical-data">
                <input class="stec-builder-element-content-input-style" type="text" name="location_city" placeholder="<?php esc_attr_e('City', 'stec'); ?>" >
                <input class="stec-builder-element-content-input-style" type="text" name="location_country" placeholder="<?php esc_attr_e('Country', 'stec'); ?>" >
                <div class="stec-builder-element-content-coordinates-wrap">
                    <input class="stec-builder-element-content-input-style" type="text" name="location_coordinates" placeholder="<?php esc_attr_e('Coordinates', 'stec'); ?>" >
                    <button title="<?php esc_html_e('Find coordinates', 'stec'); ?>" type="button" data-input="location_coordinates" data-action="get-coordinates"><i class="fas fa-search-location"></i></button>
                </div>
            </div>
            <textarea class="stec-builder-element-content-textarea-style" name="location_details" placeholder="<?php esc_attr_e('Details', 'stec'); ?>"></textarea>
        </div>
        <?php
        return ob_get_clean();
    }

    private function get_location_form_type() {

        $location_type = (int) \Stachethemes\Stec\Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_location_type');

        switch ($location_type) {

            case 1:
                return $this->get_address_form();
            case 2:
            case 3:
                return $this->get_location_form();
        }

        return '';
    }

    private function get_locations() {

        $location_type = (int) \Stachethemes\Stec\Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_location_type');

        $options = array();

        $locations = \Stachethemes\Stec\Locations::get_locations_list();

        if ($location_type > 0) {

            $options[] = "<optgroup label='" . esc_attr__('New location', 'stec') . "'>";

            switch ($location_type) :
                case 1:
                    $options[] = "<option value='custom_address'>" . esc_html__('Add new address', 'stec') . "</option>";
                    break;
                case 2:
                    $options[] = "<option value='custom'>" . esc_html__('Add new location', 'stec') . "</option>";
                    break;
                case 3:
                    $options[] = "<option value='custom'>" . esc_html__('Add new location', 'stec') . "</option>";
                    break;
            endswitch;

            $options[] = "</optgroup>";
        }

        $options[] = "<optgroup label='" . esc_attr__('Locations', 'stec') . "'>";
        $options[] = "<option selected value=''>" . esc_html__('No location', 'stec') . "</option>";
        if ($location_type !== 3) {
            foreach ($locations as $k => $v) :
                $options[] = "<option value='{$k}'>{$v}</option>";
            endforeach;
        }
        $options[] = "</optgroup>";


        return implode(PHP_EOL, $options);
    }

    protected function render() {

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" name="location" %s>%s</select>%s</div>', $this->get_settings_for_display('required'), $this->get_locations(), $this->get_location_form_type());
    }

}

class Rsvp extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_rsvp';
    }

    public function get_title() {
        return esc_html__('RSVP', 'stec');
    }

    public function get_icon() {
        return 'eicon-mail';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'rsvp', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Collect RSVP', 'stec')
                ]
        );


        $this->end_controls_section();
    }

    protected function content_template() {

        echo '<div class="stec-builder-element">';

        echo '<select class="stec-builder-element-content-select-style" name="rsvp[collect]">';
        printf('<option disabled selected>%s</option>', '{{{ settings.placeholder }}}');
        printf('<option value="0">%s</option>', esc_html__('Disable', 'stec'));
        printf('<option value="1">%s</option>', esc_html__('Enable', 'stec'));
        echo '</select>';

        printf('<input class="stec-builder-element-content-input-style" type="number" min="0" name="rsvp[limit]" placeholder="%s" />', esc_attr__('Attendees limit', 'stec'));

        echo '<select class="stec-builder-element-content-select-style" name="rsvp[visibility]">';
        printf('<option disabled selected>%s</option>', esc_html__('Attendance visibility', 'stec'));
        printf('<option value="public">%s</option>', esc_html__('Public', 'stec'));
        printf('<option value="private">%s</option>', esc_html__('Private', 'stec'));
        echo '</select>';

        echo '</div>';
    }

    protected function render() {

        echo '<div class="stec-builder-element">';

        echo '<select class="stec-builder-element-content-select-style" name="rsvp[collect]">';
        printf('<option disabled selected>%s</option>', $this->get_settings_for_display('placeholder'));
        printf('<option value="0">%s</option>', esc_html__('Disable', 'stec'));
        printf('<option value="1">%s</option>', esc_html__('Enable', 'stec'));
        echo '</select>';

        printf('<input class="stec-builder-element-content-input-style" type="number" min="0" name="rsvp[limit]" placeholder="%s" />', esc_attr__('Attendees limit', 'stec'));

        echo '<select class="stec-builder-element-content-select-style" name="rsvp[visibility]">';
        printf('<option disabled selected>%s</option>', esc_html__('Attendance visibility', 'stec'));
        printf('<option value="public">%s</option>', esc_html__('Public', 'stec'));
        printf('<option value="private">%s</option>', esc_html__('Private', 'stec'));
        echo '</select>';

        echo '</div>';
    }

}

class Timezone extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_timezone';
    }

    public function get_title() {
        return esc_html__('Timezone', 'stec');
    }

    public function get_icon() {
        return 'eicon-global-settings';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'timezone', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function get_timezone_content() {

        $timezones = \Stachethemes\Stec\Admin_Helper::timezones_list();

        $list = array_merge(
                array('stec_cal_default' => esc_attr__('Calendar Timezone', 'stec')),
                $timezones
        );

        $html = array();

        foreach ($list as $k => $v) {
            $html[] = sprintf('<option value="%s">%s</option>', $k, $v);
        }

        return implode('', $html);
    }

    protected function render() {
        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" name="timezone" required>%s</select></div>', $this->get_timezone_content());
    }

}

class Color extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_color';
    }

    public function get_title() {
        return esc_html__('Color', 'stec');
    }

    public function get_icon() {
        return 'eicon-eyedropper';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'color', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    protected function render() {
        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-colorpicker-style" type="text" name="event_color" value="#f15e6e" /></div>');
    }

}

class Calendar extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    protected $writable_cals = array();

    public function get_name() {
        return 'stec_submit_form_calendar';
    }

    public function get_title() {
        return esc_html__('Calendar', 'stec');
    }

    public function get_icon() {
        return 'eicon-calendar';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'calendar', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Select Calendar', 'stec')
                ]
        );

        $this->add_control(
                'calendar_default', array(
            'label'   => esc_html__('Default Calendar', 'stec'),
            'type'    => \Elementor\Controls_Manager::SELECT,
            'default' => '',
            'options' => $this->get_writable_calendars_kv()
                )
        );

        $this->end_controls_section();
    }

    private function get_writable_calendars_kv() {

        $options = array();

        if (!$this->writable_cals) {
            $this->writable_cals = \Stachethemes\Stec\Calendars::get_writable_calendar_list();
        }

        foreach ($this->writable_cals as $cal) :
            $options[0]              = '';
            $options[$cal->get_id()] = $cal->get_title();
        endforeach;

        return $options;
    }

    private function get_writable_calendars($default = null) {

        $options = array();

        if (!$this->writable_cals) {
            $this->writable_cals = \Stachethemes\Stec\Calendars::get_writable_calendar_list();
        }

        foreach ($this->writable_cals as $cal) :
            $options[] = sprintf('<option %s data-color="%s" value="%s">%s</option>',
                    ($default && (int) $default === (int) $cal->get_id() ? 'selected="selected"' : ''),
                    $cal->get_color(),
                    $cal->get_id(),
                    $cal->get_title());
        endforeach;

        return implode(PHP_EOL, $options);
    }

    protected function content_template() {
        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" required name="calendar_id"><option disabled selected value="">%s</option>%s</select></div>', '{{{ settings.placeholder }}}', $this->get_writable_calendars());
    }

    protected function render() {
        $default = $this->get_settings_for_display('calendar_default');
        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" required name="calendar_id"><option disabled selected value="">%s</option>%s</select></div>', $this->get_settings_for_display('placeholder'), $this->get_writable_calendars($default));
    }

}

class Counter extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_counter';
    }

    public function get_title() {
        return esc_html__('Counter', 'stec');
    }

    public function get_icon() {
        return 'eicon-countdown';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'counter', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Counter', 'stec')
                ]
        );

        $this->end_controls_section();
    }

    protected function content_template() {

        $options = array(
            sprintf('<option value="0">%s</option>', esc_html__('Disabled', 'stec')),
            sprintf('<option value="1">%s</option>', esc_html__('Enabled', 'stec')),
        );

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" name="counter"><option disabled selected value="">%s</option>%s</select></div>', '{{{ settings.placeholder }}}', implode(PHP_EOL, $options));
    }

    protected function render() {

        $options = array(
            sprintf('<option value="0">%s</option>', esc_html__('Disabled', 'stec')),
            sprintf('<option value="1">%s</option>', esc_html__('Enabled', 'stec')),
        );

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" name="counter"><option disabled selected value="">%s</option>%s</select></div>', $this->get_settings_for_display('placeholder'), implode(PHP_EOL, $options));
    }

}

class Keywords extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_keywords';
    }

    public function get_title() {
        return esc_html__('Keywords', 'stec');
    }

    public function get_icon() {
        return 'eicon-search-results';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'keywords', 'search', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Keywords', 'stec')
                ]
        );

        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-input-style" %s type="text" name="keywords" placeholder="%s" value="" /></div>', '{{{ settings.required }}}', '{{{ settings.placeholder }}}');
    }

    protected function render() {

        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-input-style" %s type="text" name="keywords" placeholder="%s" value="" /></div>', $this->get_settings_for_display('required'), $this->get_settings_for_display('placeholder'));
    }

}

class Comments extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_comments';
    }

    public function get_title() {
        return esc_html__('Comments', 'stec');
    }

    public function get_icon() {
        return 'eicon-comments';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'comments', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Comments', 'stec')
                ]
        );

        $this->end_controls_section();
    }

    protected function content_template() {

        $options = array(
            sprintf('<option value="0">%s</option>', esc_html__('Disabled', 'stec')),
            sprintf('<option value="1">%s</option>', esc_html__('Enabled', 'stec')),
        );

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" name="comments"><option disabled selected value="">%s</option>%s</select></div>', '{{{ settings.placeholder }}}', implode(PHP_EOL, $options));
    }

    protected function render() {

        $options = array(
            sprintf('<option value="0">%s</option>', esc_html__('Disabled', 'stec')),
            sprintf('<option value="1">%s</option>', esc_html__('Enabled', 'stec')),
        );

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" name="comments"><option disabled selected value="">%s</option>%s</select></div>', $this->get_settings_for_display('placeholder'), implode(PHP_EOL, $options));
    }

}

class Starts_On extends \Elementor\Widget_Base {

    use Widget_Panel_Display;
    use Get_Hours_And_Minutes;

    public function get_name() {
        return 'stec_submit_form_starts_on';
    }

    public function get_title() {
        return esc_html__('Starts On', 'stec');
    }

    public function get_icon() {
        return 'eicon-clock-o';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'start', 'time', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Starts On', 'stec')
                ]
        );

        $this->end_controls_section();
    }

    protected function content_template() {
        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-datetime-style stec-builder-element-content-input-style" type="text" name="start_date" placeholder="%s" value="%s" />', '{{{ settings.placeholder }}}', '');
        printf('<input type="hidden" name="start_date_alt" value="%s" />', '');
        echo $this->get_hours_and_minutes();
        echo '</div>';
    }

    protected function render() {

        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-datetime-style stec-builder-element-content-input-style" type="text" name="start_date" placeholder="%s" value="%s" />', $this->get_settings_for_display('placeholder'), '');
        printf('<input type="hidden" name="start_date_alt" value="%s" />', '');
        echo $this->get_hours_and_minutes();
        echo '</div>';
    }

}

class Ends_On extends \Elementor\Widget_Base {

    use Widget_Panel_Display;
    use Get_Hours_And_Minutes;

    public function get_name() {
        return 'stec_submit_form_ends_on';
    }

    public function get_title() {
        return esc_html__('Ends On', 'stec');
    }

    public function get_icon() {
        return 'eicon-clock-o';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'end', 'time', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Ends On', 'stec')
                ]
        );


        $this->end_controls_section();
    }

    protected function content_template() {
        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-datetime-style stec-builder-element-content-input-style" type="text" name="end_date" placeholder="%s" />', ' {{{ settings.placeholder }}} ');
        echo '<input type="hidden" name="end_date_alt" />';
        echo $this->get_hours_and_minutes('end');
        echo '</div>';
    }

    protected function render() {
        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-datetime-style stec-builder-element-content-input-style" type="text" name="end_date" placeholder="%s" />', $this->get_settings_for_display('placeholder'));
        echo '<input type="hidden" name="end_date_alt" />';
        echo $this->get_hours_and_minutes('end');
        echo '</div>';
    }

}

class All_Day extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_all_day';
    }

    public function get_title() {
        return esc_html__('All Day Check', 'stec');
    }

    public function get_icon() {
        return 'eicon-check-circle';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'all', 'day', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'checked',
                array(
                    'label'        => esc_html__('Checked', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'checked',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><label class="stec-builder-element-content-checkbox-style"><input name="all_day" type="checkbox" %s><span class="stec-builder-element-content-p-style">%s</span></label></div>', '{{{ settings.checked }}}', esc_html__('All day', 'stec'));
    }

    protected function render() {

        printf('<div class="stec-builder-element"><label class="stec-builder-element-content-checkbox-style"><input name="all_day" type="checkbox" %s><span class="stec-builder-element-content-p-style">%s</span></label></div>', $this->get_settings_for_display('checked'), esc_html__('All day', 'stec'));
    }

}

class Contact_Email extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_contact';
    }

    public function get_title() {
        return esc_html__('Contact E-mail', 'stec');
    }

    public function get_icon() {
        return 'eicon-email-field';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'contact', 'email', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Contact E-mail', 'stec')
                ]
        );


        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-input-style" type="email" name="contact_email" placeholder="%s" %s /></div>', '{{{ settings.placeholder }}}', '{{{ settings.required }}}');
    }

    protected function render() {

        printf('<div class="stec-builder-element"><input class="stec-builder-element-content-input-style" type="email" name="contact_email" placeholder="%s" %s /></div>', $this->get_settings_for_display('placeholder'), $this->get_settings_for_display('required'));
    }

}

class Event_Status extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_event_status';
    }

    public function get_title() {
        return esc_html__('Event Status', 'stec');
    }

    public function get_icon() {
        return 'eicon-flash';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'status', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Event Status', 'stec')
                ]
        );

        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    private function get_event_statuses($inline_edit = false) {
        ob_start();
        ?>
        <option disabled selected value=''><?php echo true === $inline_edit ? '{{{ settings.placeholder }}} ' : $this->get_settings_for_display('placeholder'); ?></option>
        <option value="EventScheduled"><?php esc_html_e('Scheduled', 'stec'); ?></option>
        <option value="EventMovedOnline"><?php esc_html_e('Moved Online', 'stec'); ?></option>
        <option value="EventPostponed"><?php esc_html_e('Postponed', 'stec'); ?></option>
        <option value="EventRescheduled"><?php esc_html_e('Rescheduled', 'stec'); ?></option>
        <option value="EventCancelled"><?php esc_html_e('Cancelled', 'stec'); ?></option>
        <?php
        return ob_get_clean();
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" %s name="event_status">%s</select></div>', '{{{ settings.required }}}', $this->get_event_statuses(true));
    }

    protected function render() {

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" %s name="event_status">%s</select></div>', $this->get_settings_for_display('required'), $this->get_event_statuses());
    }

}

class Notes extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_event_notes';
    }

    public function get_title() {
        return esc_html__('Notes to the reviewer', 'stec');
    }

    public function get_icon() {
        return 'eicon-pencil';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'notes', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXTAREA,
                    'default' => esc_html__('Notes to the reviewer', 'stec')
                ]
        );


        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><textarea class="stec-builder-element-content-textarea-style" %s name="notes" placeholder="%s"></textarea></div>', '{{{ settings.required }}}', '{{{ settings.placeholder }}}');
    }

    protected function render() {

        printf('<div class="stec-builder-element"><textarea class="stec-builder-element-content-textarea-style" %s name="notes" placeholder="%s"></textarea></div>', $this->get_settings_for_display('required'), $this->get_settings_for_display('placeholder'));
    }

}

class Categories extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_event_categories';
    }

    public function get_title() {
        return esc_html__('Categories', 'stec');
    }

    public function get_icon() {
        return 'eicon-folder-o';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'category', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Select Categories', 'stec')
                ]
        );

        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    private function get_event_categories() {

        $options = array();

        $categories = \Stachethemes\Stec\Categories::get_categories_list();

        foreach ($categories as $k => $v) :
            $options[] = "<option value='{$k}'>{$v}</option>";
        endforeach;

        return implode(PHP_EOL, $options);
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" %s name="event_categories[]" multiple><option disabled value="">%s</option>%s</select></div>', '{{{ settings.required }}}', '{{{ settings.placeholder }}}', $this->get_event_categories());
    }

    protected function render() {

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-select-style" %s name="event_categories[]" multiple><option disabled value="">%s</option>%s</select></div>', $this->get_settings_for_display('required'), $this->get_settings_for_display('placeholder'), $this->get_event_categories());
    }

}

class Featured extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_featured';
    }

    public function get_title() {
        return esc_html__('Featured', 'stec');
    }

    public function get_icon() {
        return 'eicon-heart-o';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'featured', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Featured', 'stec')
                ]
        );


        $this->end_controls_section();
    }

    protected function content_template() {

        $options = array(
            sprintf('<option value="0">%s</option>', esc_html__('No', 'stec')),
            sprintf('<option value="1">%s</option>', esc_html__('Yes', 'stec')),
            sprintf('<option value="2">%s</option>', esc_html__('Yes, with background', 'stec')),
        );

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-input-style" name="featured"><option disabled selected value="">%s</option>%s</select></div>', ' {{{ settings.placeholder }}}', implode(PHP_EOL, $options));
    }

    protected function render() {

        $options = array(
            sprintf('<option value="0">%s</option>', esc_html__('No', 'stec')),
            sprintf('<option value="1">%s</option>', esc_html__('Yes', 'stec')),
            sprintf('<option value="2">%s</option>', esc_html__('Yes, with background', 'stec')),
        );

        printf('<div class="stec-builder-element"><select class="stec-builder-element-content-input-style" name="featured"><option disabled selected value="">%s</option>%s</select></div>', $this->get_settings_for_display('placeholder'), implode(PHP_EOL, $options));
    }

}

class Website_URL extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_website_url';
    }

    public function get_title() {
        return esc_html__('Website URL', 'stec');
    }

    public function get_icon() {
        return 'eicon-url';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'website', 'external', 'link', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('URL Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Website URL', 'stec')
                ]
        );

        $this->add_control(
                'placeholder2',
                [
                    'label'   => esc_html__('Button Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Button text (optional)', 'stec')
                ]
        );

        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {

        echo '<div class="stec-builder-element stec-style-flexbox-no-media">';
        printf('<input class="stec-builder-element-content-input-style" %s type="url" name="link" placeholder="%s" />', '{{{ settings.required }}}', '{{{ settings.placeholder }}}');
        printf('<input class="stec-builder-element-content-input-style" type="text" name="link[text]" placeholder="%s" %s />', '{{{ settings.placeholder2 }}}', 'style="margin-left:10px"');
        echo '</div>';
    }

    protected function render() {

        echo '<div class="stec-builder-element stec-style-flexbox-no-media">';
        printf('<input class="stec-builder-element-content-input-style" %s type="url" name="link[url]" placeholder="%s" />', $this->get_settings_for_display('required'), $this->get_settings_for_display('placeholder'));
        printf('<input class="stec-builder-element-content-input-style" type="text" name="link[text]" placeholder="%s" %s />', $this->get_settings_for_display('placeholder2'), 'style="margin-left:10px"');
        echo '</div>';
    }

}

class Captcha extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_captcha';
    }

    public function get_title() {
        return esc_html__('Captcha', 'stec');
    }

    public function get_icon() {
        return 'eicon-captcha';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'captcha', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    protected function render() {

        if (1 === (int) \Stachethemes\Stec\Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled')) {
            echo '<div class="stec-builder-element"><div class="stec-builder-element-content-captcha-style"><div></div></div></div>';
        } else {
            echo '';
        }
    }

}

class Privacy_Policy extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_privacy_policy';
    }

    public function get_title() {
        return esc_html__('Privacy Policy', 'stec');
    }

    public function get_icon() {
        return 'eicon-lock-user';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'policy', 'privacy', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function get_privacy_policy() {

        $text = \Stachethemes\Stec\Settings::get_admin_setting_value('stec_menu__general_privacy_agreement', 'privacy_agreement_content');

        $html = '<div>'
                . '    <label class="stec-builder-element-content-checkbox-style">'
                . '        <input type="checkbox" name="privacy_policy" required />'
                . '        <span>' . $text . '</span>'
                . '     </label>'
                . '</div>';

        return $html;
    }

    protected function render() {


        if ((int) get_query_var('stec_edit_event') !== 1 && 1 === (int) \Stachethemes\Stec\Settings::get_admin_setting('stec_menu__general_privacy_agreement', 'display_privacy_agreement')) {
            printf('<div class="stec-builder-element"><div class="stec-builder-element-content-privacy-style">%s</div></div>', $this->get_privacy_policy());
        } else {
            echo '';
        }
    }

}

class Booking extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_booking';
    }

    public function get_title() {
        return esc_html__('Booking', 'stec');
    }

    public function get_icon() {
        return 'eicon-slider-push';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'booking', 'tickets', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function get_booking_content() {

        if (false === class_exists('WooCommerce') || false === current_user_can('manage_woocommerce')) {
            return '';
        }

        ob_start();
        ?>

        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-ticket'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('Add Ticket', 'stec'); ?>
        </button>

        <input type="hidden" name="has_tickets" value="1" />
        <?php
        return ob_get_clean();
    }

    protected function render() {
        printf('<div class="stec-builder-element"><div class="stec-builder-element-content-booking-style">%s</div></div>', $this->get_booking_content());
    }

}

class Image extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_image';
    }

    public function get_title() {
        return esc_html__('Image', 'stec');
    }

    public function get_icon() {
        return 'eicon-image';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'image', 'media', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function get_image_content() {
        if (current_user_can('upload_files')) {
            return '<div class="stec-builder-element-content-media-images-style"><button type="button" class="stec-builder-add-media-photo">' . esc_html__('Add Images', 'stec') . '</button></div><input type="hidden" name="stec_has_media" value="1" />';
        } else {

            ob_start();
            ?>
            <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-image'>
                <i class="fas fa-plus"></i>
                <span><?php esc_html_e('Image', 'stec'); ?></span>
            </button>
            <span class="stec-builder-submission-add-image-src stec-builder-element-content-p-style"></span>
            <?php
            $buffer_content = ob_get_clean();

            return '<div class="stec-builder-element-content-media-images-style"><input class="stec-builder-hide-upload" name="stec_public_file_image" type="file" accept="image/png, image/jpeg"/>' . $buffer_content . '</div>';
        }
    }

    protected function render() {
        echo '<div class="stec-builder-element">';
        echo $this->get_image_content();
        echo '</div>';
    }

}

class Attendance extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_attendance';
    }

    public function get_title() {
        return esc_html__('Attendance', 'stec');
    }

    public function get_icon() {
        return 'eicon-person';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'attendance', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function get_attendance_content() {
        ob_start();
        ?>
        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-attendee'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('Add Attendee', 'stec'); ?>
        </button>

        <input type="hidden" name="has_attendance_list" value="1" />

        <?php
        return ob_get_clean();
    }

    protected function render() {

        printf('<div class="stec-builder-element"><div class="stec-builder-element-content-attendance-style">%s</div></div>', $this->get_attendance_content());
    }

}

class Guests extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_guests';
    }

    public function get_title() {
        return esc_html__('Guests', 'stec');
    }

    public function get_icon() {
        return 'eicon-person';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'guests', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function get_guests_content() {
        ob_start();
        ?>

        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-guest'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('Add Guest', 'stec'); ?>
        </button>

        <input type="hidden" name="has_guests_list" value="1" />

        <?php
        return ob_get_clean();
    }

    protected function render() {

        printf('<div class="stec-builder-element"><div class="stec-builder-element-content-guests-style">%s</div></div>', $this->get_guests_content());
    }

}

class Organizers extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_organizers';
    }

    public function get_title() {
        return esc_html__('Organizers', 'stec');
    }

    public function get_icon() {
        return 'eicon-person';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'organizers', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Select Organizer(s)', 'stec')
                ]
        );


        $this->add_control(
                'required',
                array(
                    'label'        => esc_html__('Required', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => 'required',
                    'default'      => '',
                )
        );

        $this->end_controls_section();
    }

    private function get_organizers_content($inline_edit = false) {

        ob_start();


        if ($inline_edit) {
            $required    = '{{{ settings.required }}}';
            $placeholder = '{{{ settings.placeholder }}}';
        } else {
            $required    = $this->get_settings_for_display('required');
            $placeholder = $this->get_settings_for_display('placeholder');
        }
        ?>

        <div class="stec-builder-submission-organizer-list">
            <input type="hidden" name="organizers[]" value="" />

            <select <?php esc_html_e($required) ?> class="stec-builder-element-content-select-style" name="organizers[]" multiple>

                <option disabled value=''><?php esc_html_e($placeholder); ?></option>
                <?php
                $organizers = \Stachethemes\Stec\Organizers::get_organizers_list();

                foreach ($organizers as $id => $name) {
                    printf("<option value='%s'>%s</option>", esc_attr($id), esc_html($name));
                }
                ?>

            </select>

        </div>
     
        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-organizer'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('New Organizer', 'stec'); ?>
        </button>

        <?php
        return ob_get_clean();
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><div class="stec-builder-element-content-organizers-style">%s</div></div>', $this->get_organizers_content(true));
    }

    protected function render() {

        printf('<div class="stec-builder-element"><div class="stec-builder-element-content-organizers-style">%s</div></div>', $this->get_organizers_content());
    }

}

class Submit_Button extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_submit_button';
    }

    public function get_title() {
        return esc_html__('Submit Button', 'stec');
    }

    public function get_icon() {
        return 'eicon-button';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'submit', 'button', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'text',
                [
                    'label'   => esc_html__('Button Text', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Submit Event', 'stec')
                ]
        );


        $this->end_controls_section();
    }

    protected function content_template() {

        printf('<div class="stec-builder-element"><div class="stec-builder-element-content-submit-style"><input type="submit" name="submit" value="%s" /></div></div>', '{{{ settings.text }}}');
    }

    protected function render() {

        printf('<div class="stec-builder-element"><div class="stec-builder-element-content-submit-style"><input type="submit" name="submit" value="%s" /></div></div>', $this->get_setting_or_default('text', esc_html__('Submit Event', 'stec')));
    }

}

class Icon_Type extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_icon';
    }

    public function get_title() {
        return esc_html__('Icon', 'stec');
    }

    public function get_icon() {
        return 'eicon-image-before-after';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'icon', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'placeholder',
                [
                    'label'   => esc_html__('Placeholder', 'stec'),
                    'type'    => \Elementor\Controls_Manager::TEXT,
                    'default' => esc_html__('Select Icon Type', 'stec')
                ]
        );

        $this->end_controls_section();
    }

    private function get_stec_icons() {
        $icons   = \Stachethemes\Stec\Admin_Helper::get_icon_list();
        $options = array();
        foreach ($icons as $k => $icon) {
            $options[] = "<option value='{$k}'>{$icon}</option>";
        }
        return implode(PHP_EOL, $options);
    }

    protected function content_template() {
        $type_options   = array();
        $type_options[] = "<option value='icon'>" . esc_html__('Icon', 'stec') . "</option>";
        $type_options[] = "<option value='date'>" . esc_html__('Date', 'stec') . "</option>";
        if (current_user_can('upload_files')) {
            $type_options[] = "<option value='image'>" . esc_html__('Image', 'stec') . "</option>";
        }

        echo '<div class="stec-builder-element">';

        printf('<select class="stec-builder-element-content-select-style" name="icon_type"><option disabled selected value="">%s</option>%s</select>', '{{{ settings.placeholder }}}', implode('', $type_options));
        printf('<select class="stec-builder-element-content-select-style" name="icon"><option disabled selected value="">%s</option>%s</select>', esc_html__('Select Icon', 'stec'), $this->get_stec_icons());

        if (current_user_can('upload_files')) {
            printf('<div class="stec-builder-element-content-media-icon-images-style"><button type="button" class="stec-builder-add-media-photo">%s</button></div><input type="hidden" name="stec_has_icon_media" value="1" />', esc_html__('Icon Images', 'stec'));
        }

        echo '</div>';
    }

    protected function render() {
        $type_options   = array();
        $type_options[] = "<option value='icon'>" . esc_html__('Icon', 'stec') . "</option>";
        $type_options[] = "<option value='date'>" . esc_html__('Date', 'stec') . "</option>";
        if (current_user_can('upload_files')) {
            $type_options[] = "<option value='image'>" . esc_html__('Image', 'stec') . "</option>";
        }

        echo '<div class="stec-builder-element">';

        printf('<select class="stec-builder-element-content-select-style" name="icon_type"><option disabled selected value="">%s</option>%s</select>', $this->get_settings_for_display('placeholder'), implode('', $type_options));
        printf('<select class="stec-builder-element-content-select-style" name="icon"><option disabled selected value="">%s</option>%s</select>', esc_html__('Select Icon', 'stec'), $this->get_stec_icons());

        if (current_user_can('upload_files')) {
            printf('<div class="stec-builder-element-content-media-icon-images-style"><button type="button" class="stec-builder-add-media-photo">%s</button></div><input type="hidden" name="stec_has_icon_media" value="1" />', esc_html__('Icon Images', 'stec'));
        }

        echo '</div>';
    }

}

class Attachments extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_attachments';
    }

    public function get_title() {
        return esc_html__('Attachments', 'stec');
    }

    public function get_icon() {
        return 'eicon-post-content';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'attachments', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function get_attachments_content() {
        ob_start();

        if (current_user_can('upload_files')) {
            ?>
            <button type="button" class="stec-builder-element-content-button-style stec-builder-add-media-attachments">
                <i class="fas fa-plus"></i>
                <span><?php esc_html_e('Add Attachments', 'stec'); ?></span>
                <input type="hidden" name="has_attachments" value="1">
            </button>
            <?php
        } else {
            ?>
            <button type="button" class="stec-builder-element-content-button-style stec-builder-upload-attachments">
                <i class="fas fa-plus"></i>
                <span><?php esc_html_e('Upload Attachment', 'stec'); ?></span>
            </button>
            <input class="stec-builder-hide-upload" name="stec_public_file_attachment" type="file" />
            <span class="stec-builder-submission-add-attachment-filename stec-builder-element-content-p-style"></span>
            <?php
        }

        return ob_get_clean();
    }

    protected function render() {
        printf('<div class="stec-builder-element"><div class="stec-builder-element-content-attachments-style">%s</div></div>', $this->get_attachments_content());
    }

}

class Schedule extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_submit_form_schedule';
    }

    public function get_title() {
        return esc_html__('Schedule', 'stec');
    }

    public function get_icon() {
        return 'eicon-bullet-list';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'schedule', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function get_schedule_hours_and_minutes() {

        $hours_list   = \Stachethemes\Stec\Admin_Helper::get_hours_array();
        $minutes_list = \Stachethemes\Stec\Admin_Helper::minutes_array();

        ob_start();
        ?>
        <select class="stec-builder-element-content-datetime-style stec-builder-element-content-select-style" name="schedule[0][schedule_time_hours_from]">
            <?php
            foreach ($hours_list as $key => $value) :
                ?>
                <option value="<?php esc_attr_e($key); ?>"><?php esc_html_e($value); ?></option>
                <?php
            endforeach;
            ?>
        </select>
        <select  class="stec-builder-element-content-datetime-style stec-builder-element-content-select-style" name="schedule[0][schedule_time_minutes_from]">
            <?php
            foreach ($minutes_list as $key => $value) :
                ?>
                <option value="<?php esc_attr_e($key); ?>"><?php esc_html_e($value); ?></option>
                <?php
            endforeach;
            ?>
        </select>
        <?php
        return ob_get_clean();
    }

    private function get_stec_icons() {

        $icons   = \Stachethemes\Stec\Admin_Helper::get_icon_list();
        $options = array();
        foreach ($icons as $k => $icon) {
            $options[] = "<option value='{$k}'>{$icon}</option>";
        }
        return implode(PHP_EOL, $options);
    }

    private function get_schedule_content() {
        ob_start();
        ?>
            
        <button type='button' class='stec-builder-element-content-button-style stec-builder-submission-add-schedule'>
            <i class="fas fa-plus"></i>
            <?php esc_html_e('Add Timespan', 'stec'); ?>
        </button>

        <?php
        return ob_get_clean();
    }

    protected function render() {
        printf('<div class="stec-builder-element"><div class="stec-builder-element-content-schedules-style">%s</div></div>', $this->get_schedule_content());
    }

}

\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Title());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Description());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Description_Short());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Color());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Icon_Type());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Image());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Location());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Calendar());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Categories());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Timezone());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Starts_On());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Ends_On());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new All_Day());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Repeater());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Featured());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Schedule());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Organizers());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Guests());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Rsvp());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Attendance());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Attachments());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Booking());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Website_URL());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Keywords());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Counter());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Comments());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Contact_Email());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Event_Status());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Notes());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Captcha());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Privacy_Policy());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Submit_Button());
