<?php

namespace Stachethemes\Stec\Elementor\Widgets\Single_Page;

use Stachethemes\Stec\Admin_Helper as Admin_Helper;

trait Widget_Panel_Display {

    public function show_in_panel() {

        if (!isset($this->stec_show_in_panel)) {
            $current_document = \Elementor\Plugin::$instance->documents->get_current();
            $post_type        = $current_document->get_post()->post_type;

            $this->stec_show_in_panel = apply_filters('stec_builder_show_in_panel', 'stec_sp' === $post_type, $this->get_name());
        }

        return $this->stec_show_in_panel;
    }

    public function get_setting_or_default($setting, $default = '') {

        return $this->get_settings_for_display($setting) ? $this->get_settings_for_display($setting) : $default;
    }

    public function register_no_options() {
        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'no_options_text',
                array(
                    'label' => esc_html__('No options available', 'stec'),
                    'type'  => \Elementor\Controls_Manager::HEADING,
                )
        );

        $this->end_controls_section();
    }

    // Soft deprecated for some reason keeps true === method_exists eventhough it's not
    public function _register_controls() {
        $this->register_controls();
    }

    /**
     * Retrieve event and offset 
     * @global \Stachethemes\Stec\Event_Post $stec_sp_event
     * @global int $stec_sp_repeat_offset
     * @return mixed array [event, offset] or false on failure
     */
    private function get_event_data() {

        global $stec_sp_event;
        global $stec_sp_repeat_offset;

        if (false === $stec_sp_event instanceof \Stachethemes\Stec\Event_Post) {
            return false;
        }

        if (false === filter_var($stec_sp_repeat_offset, FILTER_VALIDATE_INT)) {
            $stec_sp_repeat_offset = 0;
        }

        return array(
            'event'  => $stec_sp_event,
            'offset' => $stec_sp_repeat_offset
        );
    }

}

/**
 * Replace the code with the calendar code
 */
class Preview extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_preview';
    }

    public function get_title() {
        return esc_html__('Preview Block', 'stec');
    }

    public function get_icon() {
        return 'eicon-preview-thin';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'title', 'preview', 'stachethemes');
    }

    protected function register_controls() {

        /**
         * General
         */
        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'icon',
                array(
                    'label'        => esc_html__('Icon', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'tags',
                array(
                    'label'        => esc_html__('Tags', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'date',
                array(
                    'label'        => esc_html__('Date', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'location',
                array(
                    'label'        => esc_html__('Location', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );


        $this->end_controls_section();
    }

    protected function placeholder_content() {
        ?>

        <div class="stec-builder-element">
            <div <?php printf('style="background-color:%s"', 'inherit'); ?> class="stec-layout-single-preview">

                <div class="stec-layout-single-preview-left">

                    <?php if ($this->get_settings_for_display('icon')) : ?>

                        <div class="stec-layout-single-preview-left-icon" style="background:#f9a852">
                            <i class="fa fa-cog"></i>
                        </div>

                    <?php endif; ?>

                    <div class="stec-layout-single-preview-left-text">

                        <?php if ($this->get_settings_for_display('tags')) : ?>

                            <div class="stec-layout-single-preview-left-text-event-status">
                                <span class="postponed"><?php esc_html_e('Postponed', 'stec'); ?></span>
                            </div>
                            <div class="stec-layout-single-preview-left-text-featured">
                                <i class="fa fa-star"></i>
                                <span><?php esc_html_e('Featured', 'stec'); ?></span>
                            </div>

                        <?php endif; ?>

                        <span class="stec-layout-single-preview-left-text-title"><?php esc_html_e('Event Title', 'stec'); ?></span>


                        <?php if ($this->get_settings_for_display('date') || $this->get_settings_for_display('location')) : ?>

                            <span class="stec-layout-single-preview-left-text-secondary">
                                <?php if ($this->get_settings_for_display('date')) : ?>
                                    <span class="stec-layout-single-preview-left-text-date"><i class="fas fa-calendar"></i><?php esc_html_e('Event Date', 'stec'); ?></span>
                                <?php endif; ?>

                                <?php if ($this->get_settings_for_display('location')) : ?>
                                    <span class="stec-layout-single-preview-left-text-location"><i class="fas fa-map-marker-alt"></i><?php esc_html_e('Event Location', 'stec'); ?></span>
                                <?php endif; ?>
                            </span>

                        <?php endif; ?>

                        <div class="stec-layout-single-preview-separator">
                            <a class="stec-layout-single-preview-left-event-edit" href="#"><i class="far fa-edit"></i></a>

                            <a href="javascript:void(0);" class="stec-layout-single-preview-left-reminder-toggle">
                                <i class="fa fa-bell"></i>
                            </a>
                        </div>
                    </div>

                </div>

                <div class="stec-layout-single-preview-right">
                    <a class="stec-layout-single-preview-right-event-edit" href="#"><i class="far fa-edit"></i></a>
                    <div class="stec-layout-single-preview-right-menu">
                        <i class="fa fa-bell"></i>
                    </div>
                </div>

            </div>
        </div>

        <?php
    }

    protected function the_content() {

        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event         = $event_data['event'];
        $repeat_offset = $event_data['offset'];
        $edit_link     = $event->get_edit_link();
        $bg_color      = $this->get_settings_for_display('_background_color', '');
        ?>

        <div class="stec-builder-element">

            <div class="stec-layout-single-preview" <?php
            if ($bg_color) {
                echo sprintf('style="background-color:%s"', $bg_color);
            }
            ?>>

                <div class="stec-layout-single-preview-left">

                    <?php if ($this->get_settings_for_display('icon')) : ?>

                        <div class="stec-layout-single-preview-left-icon <?php
                        if ($event->get_icon_type() === 'icon' && $event->get_icon() === 'fa') {
                            echo 'stec-no-icon';
                        }
                        ?>" style="background:<?php echo $event->get_color('display'); ?>">

                            <?php
                            switch ($event->get_icon_type()) :

                                case 'date' :

                                    $d   = $event->get_start_date('j', $repeat_offset);
                                    $mmm = date_i18n('M', $event->get_start_date('U', $repeat_offset));
                                    ?>
                                    <span class="stec-layout-single-preview-left-icon-date">
                                        <span class="stec-layout-single-preview-left-icon-date-number"><?php esc_html_e($d); ?></span>
                                        <span class="stec-layout-single-preview-left-icon-date-month"><?php esc_html_e($mmm); ?></span>
                                    </span>
                                    <?php
                                    break;

                                case 'image' :

                                    if ($event->get_icon_images()) {
                                        ?>
                                        <div class="stec-layout-single-preview-left-icon-images">
                                            <?php
                                            foreach ($event->get_parsed_icon_images() as $k => $image) {
                                                ?><div class="stec-layout-single-preview-left-icon-image <?php echo (int) $k === 0 ? 'active' : '' ?>" style="background-image: url(<?php echo esc_url($image->src); ?>)"></div><?php
                                            }
                                            ?>
                                        </div>
                                        <?php
                                    }
                                    break;

                                case 'icon' :
                                    ?><i class="<?php esc_attr_e($event->get_icon()); ?>"></i><?php
                                    break;

                            endswitch;
                            ?>

                        </div>

                    <?php endif; ?>

                    <div class="stec-layout-single-preview-left-text">
                        <?php
                        switch ($event->get_event_status()) :
                            case 'EventCancelled' :
                                $event_status       = esc_html__('Cancelled', 'stec');
                                $event_status_class = 'cancelled';
                                break;

                            case 'EventMovedOnline' :
                                $event_status       = esc_html__('Moved Online', 'stec');
                                $event_status_class = 'moved-online';
                                break;

                            case 'EventPostponed' :
                                $event_status       = esc_html__('Postponed', 'stec');
                                $event_status_class = 'postponed';
                                break;

                            case 'EventRescheduled' :
                                $event_status       = esc_html__('Rescheduled', 'stec');
                                $event_status_class = 'rescheduled';
                                break;

                            default:
                                $event_status       = null;
                                $event_status_class = null;

                        endswitch;

                        if ($this->get_settings_for_display('tags')) :

                            if ($event_status) {
                                printf('<div class="stec-layout-single-preview-left-text-event-status"><span class="%s">%s</span></div>', $event_status_class, $event_status);
                            }

                            if ($event->get_approved() === 0) {
                                printf('<div class="stec-layout-single-preview-left-text-event-status"><span class="unapproved">%s</span></div>', esc_html__('Awaiting Approval', 'stec'));
                            }

                            if ($event->get_featured() >= 1) {
                                printf('<div class="stec-layout-single-preview-left-text-featured"><i class="fa fa-star"></i><span>%s</span></div>', esc_html__('Featured', 'stec'));
                            }

                        endif;

                        printf('<span class="stec-layout-single-preview-left-text-title">%s</span>', 'EventCancelled' === $event->get_event_status() ? "<s>{$event->get_title()}</s>" : $event->get_title());
                        ?>

                        <span class="stec-layout-single-preview-left-text-secondary">
                            <?php
                            if ($this->get_settings_for_display('date')) :

                                printf('<span class="stec-layout-single-preview-left-text-date"><i class="fas fa-calendar"></i>%s</span>', Admin_Helper::get_the_timespan($event, $repeat_offset));

                            endif;

                            if ($this->get_settings_for_display('location')) :

                                if ('EventMovedOnline' !== $event->get_event_status() && $event->get_location()) {
                                    $location = $event->get_parsed_location();

                                    if ('virtual' === $location->type) {
                                        $location_text = esc_html__('Online Event', 'stec');
                                    } else {
                                        $location_text = $location->full_address;
                                    }
                                    
                                    printf('<span class="stec-layout-single-preview-left-text-location"><i class="fas fa-map-marker-alt"></i>%s</span>', $location_text);
                                    unset($location);
                                }

                            endif;
                            ?>
                        </span>

                        <div class="stec-layout-single-preview-separator">

                            <?php
                            if ($edit_link) {
                                printf('<a class="stec-layout-single-preview-left-event-edit" href="%s"><i class="far fa-edit"></i></a>', $edit_link);
                            }
                            ?>

                            <a href="javascript:void(0);" class="stec-layout-single-preview-left-reminder-toggle">
                                <i class="fa fa-bell"></i>
                            </a>
                        </div>
                    </div>

                </div>

                <div class="stec-layout-single-preview-right">

                    <?php
                    if ($edit_link) {
                        printf('<a class="stec-layout-single-preview-right-event-edit" href="%s"><i class="far fa-edit"></i></a>', $edit_link);
                    }

                    if (!Admin_Helper::reminder_expired($event, $repeat_offset)) {
                        echo '<div class="stec-layout-single-preview-right-menu"><i class="fa fa-bell"></i></div>';
                    }
                    ?>

                </div>

            </div>
        </div>
        <?php
    }

    protected function render() {

        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Share_And_Export extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_share_and_export';
    }

    public function get_title() {
        return esc_html__('Share and Export', 'stec');
    }

    public function get_icon() {
        return 'eicon-share';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'share', 'export', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event         = $event_data['event'];
        $repeat_offset = $event_data['offset'];

        $the_permalink = $event->get_permalink($repeat_offset);

        echo '<div class="stec-builder-element">';
        echo '  <div class="stec-layout-single-share-and-export">';

        echo '      <div class="stec-layout-single-share">';
        if (\Stachethemes\Stec\Settings::get_admin_setting_value('stec_menu__general', 'social_links') === '1') :
            echo Admin_Helper::get_share_links($the_permalink);
        endif;
        echo '      </div>';

        if (\Stachethemes\Stec\Settings::get_admin_setting_value('stec_menu__general', 'show_export_buttons') === '1') :
            echo '<div class="stec-layout-single-export">';
            echo '<form method="POST">';
            printf('<button class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor">%s</button>', esc_html__('Export to .ICS file', 'stec'));
            printf('<input type="hidden" value="%d" name="event_id">', (int) $event->get_id());
            printf('<input type="hidden" value="%d" name="calendar_id">', (int) $event->get_calid());
            echo '<input type="hidden" value="stec_public_export_to_ics" name="task">';
            echo '</form>';

            printf('<a class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor" href="%s" target="_BLANK" class="stec-style-text">%s</a>', esc_url($event->get_gcal_link($repeat_offset)), esc_html__('Import to Google Calendar', 'stec'));
            echo '</div>';
        endif;
        echo '  </div>';
        echo '</div>';
    }

    private function placeholder_content() {
        echo '<div class="stec-builder-element">';
        echo '  <div class="stec-layout-single-share-and-export">';

        echo '      <div class="stec-layout-single-share">';
        if (\Stachethemes\Stec\Settings::get_admin_setting_value('stec_menu__general', 'social_links') === '1') :
            echo Admin_Helper::get_share_links('#');
        endif;
        echo '      </div>';

        echo '<div class="stec-layout-single-export">';
        echo '<form method="POST">';
        printf('<button class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor">%s</button>', esc_html__('Export to .ICS file', 'stec'));
        echo '</form>';

        printf('<a class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor" href="%s" target="_BLANK" class="stec-style-text">%s</a>', '#', esc_html__('Import to Google Calendar', 'stec'));
        echo '</div>';
        echo '  </div>';
        echo '</div>';
    }

    protected function render() {

        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Organizers extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_organizers';
    }

    public function get_title() {
        return esc_html__('Organizers', 'stec');
    }

    public function get_icon() {
        return 'eicon-person';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'organizers', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {

        echo '<div class="stec-builder-element">';
        echo '  <ul class="stec-layout-single-organizers">';
        echo '      <li class="stec-layout-single-organizers-organizer">';
        echo '          <div class="stec-layout-single-organizers-organizer-image">';
        echo '              <div class="stec-layout-single-organizers-organizer-image-default"></div>';
        echo '          </div>';
        echo '          <div class="stec-layout-single-organizers-organizer-content">';
        printf('            <div class="stec-layout-single-organizers-organizer-title stec-style-title2">%s</div>', esc_html__('John Doe', 'stec'));
        printf('            <div class="stec-layout-single-organizers-organizer-sub-title">%s</div>', sprintf(esc_html__('Organizer of %s', 'stec'), esc_html__('My Event', 'stec')));
        printf('            <div class="stec-layout-single-organizers-organizer-description stec-style-text">%s</div>', esc_html__('Organizer Details', 'stec'));
        echo '              <div class="stec-layout-single-organizers-organizer-social">';
        printf("                <a target='_blank' href='%s'><i class='%s'></i></a>", '#', 'fab fa-facebook');
        echo '              </div>';
        echo '          </div>';
        echo '      </li>';
        echo '  </ul>';
        echo '</div>';
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event      = $event_data['event'];
        $organizers = $event->get_organizers();

        echo '<div class="stec-builder-element">';
        if ($organizers) :
            echo '  <ul class="stec-layout-single-organizers">';
            foreach ($organizers as $organizer_id) :

                $organizer = new \Stachethemes\Stec\Organizer((int) $organizer_id);

                if (!$organizer->get_id()) {
                    continue;
                }

                echo '<li class="stec-layout-single-organizers-organizer">';
                echo '<div class="stec-layout-single-organizers-organizer-image">';
                if ($organizer->get_photo_src()) :
                    printf('<div class="stec-layout-single-organizers-organizer-image-photo" style="background-image:url(%s)"></div>', esc_url($organizer->get_photo_src()));
                else:
                    echo '<div class="stec-layout-single-organizers-organizer-image-default"></div>';
                endif;
                echo '</div>';
                echo '<div class="stec-layout-single-organizers-organizer-content">';
                echo '<div class="stec-layout-single-organizers-organizer-title stec-style-title2">';

                if ($organizer->get_permalink()) {
                    printf("<a href='%s'>%s</a>", esc_url($organizer->get_permalink()), esc_html($organizer->get_name()));
                } else {
                    esc_html_e($organizer->get_name());
                }
                echo '</div>';

                printf('<div class="stec-layout-single-organizers-organizer-sub-title">%s</div>', sprintf(esc_html__('Organizer of %s', 'stec'), $event->get_title()));
                printf('<div class="stec-layout-single-organizers-organizer-description stec-style-text">%s</div>', esc_html__($organizer->get_details()));
                echo '<div class="stec-layout-single-organizers-organizer-social">';
                if ($organizer->get_social()) :
                    foreach ($organizer->get_social() as $social) {
                        if (false !== filter_var($social['link'], FILTER_VALIDATE_EMAIL)) {
                            printf("<a href='mailto:%s'><i class='%s'></i></a>",
                                    $social['link'],
                                    esc_attr($social['ico'])
                            );
                        } elseif ('fas fa-phone' === $social['ico']) {
                            echo "<a href='tel:{$social['link']}'><i class='{$social['ico']}'></i></a>";
                        } else {
                            printf("<a target='_blank' href='%s'><i class='%s'></i></a>",
                                    esc_url($social['link']),
                                    esc_attr($social['ico'])
                            );
                        }
                    }
                endif;
                echo '      </div>';
                echo '  </div>';
                echo '</li>';
            endforeach;
            echo '  </ul>';
        endif;
        echo '</div>';
    }

    protected function render() {

        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Description extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_description';
    }

    public function get_title() {
        return esc_html__('Description', 'stec');
    }

    public function get_icon() {
        return 'eicon-text';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'description', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {

        echo '<div class="stec-builder-element stec-builder-element-placeholder">';
        echo '<span>';
        echo '  <i class="eicon-text"></i>';
        printf('<span>%s</span>', esc_html__('Description', 'stec'));
        echo '</span>';
        echo '</div>';
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event = $event_data['event'];

        echo '<div class="stec-builder-element">';
        if ($event->get_description('raw') !== '') {
            printf('<div class="stec-layout-single-description stec-style-text">%s</div>', $event->get_description());
        }
        echo '</div>';
    }

    protected function render() {

        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Tickets extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_tickets';
    }

    public function get_title() {
        return esc_html__('Tickets', 'stec');
    }

    public function get_icon() {
        return 'eicon-slider-push';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'tickets', 'booking', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {

        echo '<div class="stec-builder-element">';
        printf('<button type="button" class="stec-style-button stec-layout-single-tickets-default" data-action="open-tickets">%s</button>', esc_html__('Tickets', 'stec'));
        echo '</div>';
    }

    private function the_content() {

        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event         = $event_data['event'];
        $repeat_offset = $event_data['offset'];
        $now           = gmdate('U');
        $has_booking   = 'EventCancelled' !== $event->get_event_status() && $event->get_start_date('U', $repeat_offset) > $now;

        if ($has_booking && $event->get_bookable_products()) {
            echo '<div class="stec-builder-element">';
            printf('<button type="button" class="stec-style-button stec-layout-single-tickets-default" data-action="open-tickets">%s</button>', esc_html__('Tickets', 'stec'));
            echo '</div>';
        }
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class RSVP extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_rsvp';
    }

    public function get_title() {
        return esc_html__('RSVP', 'stec');
    }

    public function get_icon() {
        return 'eicon-mail';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'rsvp', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {
        echo '<div class="stec-builder-element">';
        printf('<a class="stec-layout-single-rsvp stec-style-full-width stec-style-button " href="javascript:void(0);">%s</a>', esc_html__('RSVP to Event', 'stec'));
        echo '</div>';
    }

    private function the_content() {

        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event         = $event_data['event'];
        $repeat_offset = $event_data['offset'];
        $rsvp_expired  = 'EventCancelled' === $event->get_event_status() || $event->get_start_date('U', $repeat_offset) < gmdate('U');

        if (false === $rsvp_expired && 1 === (int) $event->get_rsvp() && false === Admin_Helper::user_is_invited($event)) :
            echo '<div class="stec-builder-element">';
            printf('<a class="stec-layout-single-rsvp stec-style-full-width stec-style-button " href="javascript:void(0);">%s</a>', esc_html__('RSVP to Event', 'stec'));
            echo '</div>';
        endif;
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Counter extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_counter';
    }

    public function get_title() {
        return esc_html__('Counter', 'stec');
    }

    public function get_icon() {
        return 'eicon-countdown';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'counter', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    protected function render() {

        $display_counter = true;

        if (!Admin_Helper::is_elementor() && !Admin_Helper::is_preview('stec_sp')) {
            $event_data = $this->get_event_data();

            if (!$event_data || 1 !== (int) $event_data['event']->get_counter() || 'EventCancelled' === $event_data['event']->get_event_status()) {
                $display_counter = false;
            }
        }

        if (false === $display_counter) {
            return;
        }

        echo '<div class="stec-builder-element">';
        ?>
        <div class="stec-layout-single-event-status">
            <span class="stec-layout-single-event-status-text event-startsin"><?php esc_html_e('Event will start in', 'stec'); ?></span>
            <span class="stec-layout-single-event-status-text event-inprogress"><?php esc_html_e('Event will end in', 'stec'); ?></span>
            <span class="stec-layout-single-event-status-text event-expired"><?php esc_html_e('Event expired', 'stec'); ?></span>
        </div>
        <?php
        echo '<ul class="stec-layout-single-counter">';
        echo '<li>';
        echo'<p class="stec-layout-single-counter-num">999</p>';
        printf('<p class="stec-layout-single-counter-label" data-plural-label="%s" data-singular-label="%s">days</p>', esc_html__('Days', 'stec'), esc_html__('Day', 'stec'));
        echo'</li>';
        echo'<li>';
        echo '<p class="stec-layout-single-counter-num">23</p>';
        printf('<p class="stec-layout-single-counter-label" data-plural-label="%s" data-singular-label="%s">hours</p>', esc_html__('Hours', 'stec'), esc_html__('Hour', 'stec'));
        echo'</li>';
        echo '<li>';
        echo '<p class="stec-layout-single-counter-num">59</p>';
        printf('<p class="stec-layout-single-counter-label" data-plural-label="%s" data-singular-label="%s">minutes</p>', esc_html__('Minutes', 'stec'), esc_html__('Minute', 'stec'));
        echo'</li>';
        echo '<li>';
        echo '<p class="stec-layout-single-counter-num">59</p>';
        printf('<p class="stec-layout-single-counter-label" data-plural-label="%s" data-singular-label="%s">seconds</p>', esc_html__('Seconds', 'stec'), esc_html__('Second', 'stec'));
        echo'</li>';
        echo'</ul>';
        echo '</div>';
    }

}

class Website_URL extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_website_url';
    }

    public function get_title() {
        return esc_html__('Website URL', 'stec');
    }

    public function get_icon() {
        return 'eicon-url';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'website', 'external', 'link', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function the_content() {

        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event = $event_data['event'];

        if ($event->get_link('url') !== '') :
            echo '<div class="stec-builder-element">';
            printf('<a href="%s" class="stec-layout-single-external-link stec-style-button" target="_BLANK">%s</a>', esc_url($event->get_link('url')), $event->get_link('text') ? $event->get_link('text') : esc_html__('Visit Website', 'stec'));
            echo '</div>';
        endif;
    }

    private function placeholder_content() {
        echo '<div class="stec-builder-element">';
        printf('<a href="%s" class="stec-layout-single-external-link stec-style-button " target="_BLANK">%s</a>', '#', esc_html__('Visit Website', 'stec'));
        echo '</div>';
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Attachments extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_attachments';
    }

    public function get_title() {
        return esc_html__('Attachments', 'stec');
    }

    public function get_icon() {
        return 'eicon-post-content';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'attachments', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event = $event_data['event'];


        if ($event->get_attachments()) :

            ob_start();
            ?>
            <div class="stec-builder-element">
                <div class="stec-layout-single-attachments">

                    <div class="stec-layout-single-attachments-top">

                        <p class="stec-style-title2"><i class="far fa-file"></i><?php esc_html_e('Attachments', 'stec'); ?></p>

                        <div class="stec-layout-single-attachments-toggle">
                            <i class="fa fa-plus"></i>
                            <i class="fa fa-minus"></i>
                        </div>

                    </div>

                    <ul class="stec-layout-single-attachments-list">

                        <?php foreach ($event->get_parsed_attachments() as $file) : ?>

                            <li class="stec-layout-single-attachment">
                                <div>
                                    <p class="stec-layout-single-attachment-title stec-style-title2"><a href="<?php echo esc_url($file->link) ?>"><?php esc_html_e($file->filename); ?></a></p>
                                    <p class="stec-layout-single-attachment-desc stec-style-text"><?php esc_html_e($file->description); ?></p>
                                </div>

                                <div>
                                    <a href="<?php echo esc_url($file->link) ?>" class="stec-style-title2"><?php esc_html_e('Download', 'stec'); ?></a>
                                    <p class="stec-layout-single-attachment-size stec-style-text"><?php esc_html_e($file->size); ?></p>
                                </div>
                            </li>

                        <?php endforeach; ?>
                    </ul>
                </div>
            </div>
            <?php
            ob_get_flush();

        endif;
    }

    private function placeholder_content() {


        ob_start();
        ?>
        <div class="stec-builder-element">
            <div class="stec-layout-single-attachments">

                <div class="stec-layout-single-attachments-top">

                    <p class="stec-style-title2"><i class="far fa-file"></i><?php esc_html_e('Attachments', 'stec'); ?></p>

                    <div class="stec-layout-single-attachments-toggle">
                        <i class="fa fa-plus"></i>
                        <i class="fa fa-minus"></i>
                    </div>

                </div>

                <ul class="stec-layout-single-attachments-list">

                    <li class="stec-layout-single-attachment">
                        <div>
                            <p class="stec-layout-single-attachment-title stec-style-title2"><a href="#"><?php esc_html_e('Attachment', 'stec'); ?></a></p>
                            <p class="stec-layout-single-attachment-desc stec-style-text"><?php esc_html_e('Attachment Details', 'stec'); ?></p>
                        </div>

                        <div>
                            <a href="#" class="stec-style-title2"><?php esc_html_e('Download', 'stec'); ?></a>
                            <p class="stec-layout-single-attachment-size stec-style-text"><?php esc_html_e('2MB'); ?></p>
                        </div>
                    </li>
                </ul>
            </div>
        </div>
        <?php
        ob_get_flush();
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Categories extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_categories';
    }

    public function get_title() {
        return esc_html__('Categories', 'stec');
    }

    public function get_icon() {
        return 'eicon-folder-o';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'categories', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function the_content() {

        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event = $event_data['event'];

        echo '<div class="stec-builder-element">';

        if ($event->get_categories() && \Stachethemes\Stec\Settings::get_admin_setting_value('stec_menu__general', 'show_event_tags') === '1') :
            echo '<div class="stec-layout-single-categories">';
            foreach ($event->get_parsed_categories() as $category) {
                printf('<span class="stec-layout-single-category" style="background-color: %s">%s</span>', esc_attr__($category['color']), esc_html__($category['title']));
            }
            echo '</div>';
        endif;

        echo '</div>';
    }

    private function placeholder_content() {

        echo '<div class="stec-builder-element">';

        echo '<div class="stec-layout-single-categories">';
        printf('<span class="stec-layout-single-category" style="background-color: %s">%s</span>', '#f08126', esc_html__('Category', 'stec'));
        printf('<span class="stec-layout-single-category" style="background-color: %s">%s</span>', '#388fa1', esc_html__('Category', 'stec'));
        printf('<span class="stec-layout-single-category" style="background-color: %s">%s</span>', '#71ad81', esc_html__('Category', 'stec'));
        echo '</div>';

        echo '</div>';
    }

    protected function render() {

        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Attendance_Buttons extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_attendance_buttons';
    }

    public function get_title() {
        return esc_html__('Attendance Buttons', 'stec');
    }

    public function get_icon() {
        return 'eicon-button';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'attendance', 'buttons', 'stachethemes');
    }

    protected function register_controls() {

        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'gap',
                array(
                    'label'      => esc_html__('Gap', 'stec'),
                    'type'       => \Elementor\Controls_Manager::SLIDER,
                    'size_units' => ['px', '%'],
                    'range'      => array(
                        'px' => array(
                            'min'  => 0,
                            'max'  => 200,
                            'step' => 1,
                        ),
                        '%'  => array(
                            'min' => 0,
                            'max' => 100,
                        ),
                    ),
                    'default'    => array(
                        'unit' => 'px',
                        'size' => 10,
                    ),
                    'selectors'  => array(
                        '{{WRAPPER}} .stec-layout-single-attendance-button-attend'  => 'margin-right: {{SIZE}}{{UNIT}};',
                        '{{WRAPPER}} .stec-layout-single-attendance-button-decline' => 'margin-right: {{SIZE}}{{UNIT}};',
                    ),
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {

        echo '<div class="stec-builder-element">';

        $style = 'margin-right: {{ settings.gap.size }}{{ settings.gap.unit }}'
        ?>

        <ul class="stec-layout-single-attendance-buttons">
            <?php
            printf('<li style="%s" class="stec-style-button stec-layout-single-attendance-button-attend "><span>%s</span></li>', $style, esc_html__('Attend', 'stec'));
            printf('<li style="%s" class="stec-style-button stec-layout-single-attendance-button-decline "><span>%s</span></li>', $style, esc_html__('Decline', 'stec'));
            printf('<li class="stec-style-button stec-layout-single-attendance-button-unlist "><span>%s</span></li>', esc_html__('Unlist', 'stec'));
            ?>
        </ul>

        <?php
        echo '</div>';
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event         = $event_data['event'];
        $repeat_offset = $event_data['offset'];

        if ('EventCancelled' !== $event->get_event_status() && Admin_Helper::user_is_invited($event)) :

            echo '<div class="stec-builder-element">';

            $user_status = (int) Admin_Helper::get_user_attendance_status($event, false, $repeat_offset);
            $free_slots  = \Stachethemes\Stec\Rsvp::has_free_slots($event, $event->get_start_date('Y-m-d', $repeat_offset));
            if (1 === $user_status || $free_slots !== false) :

                $gap   = $this->get_settings_for_display('gap');
                $style = sprintf('margin-right:%s%s', $gap['size'], $gap['unit']);
                ?>
                <ul class="stec-layout-single-attendance-buttons">

                    <?php
                    printf('<li style="%s" class="stec-style-button stec-layout-single-attendance-button-attend  %s"><span>%s</span></li>', $style, 1 === $user_status ? 'active' : '', esc_html__('Attend', 'stec'));
                    printf('<li style="%s" class="stec-style-button stec-layout-single-attendance-button-decline  %s"><span>%s</span></li>', $style, 2 === $user_status ? 'active' : '', esc_html__('Decline', 'stec'));
                    printf('<li class="stec-style-button stec-layout-single-attendance-button-unlist "><span>%s</span></li>', esc_html__('Unlist', 'stec'));
                    ?>

                </ul>
            <?php endif; ?>
            <?php
            echo '</div>';
        endif;
    }

    protected function render() {

        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->content_template();
        } else {
            $this->the_content();
        }
    }

}

class Location extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_location';
    }

    public function get_title() {
        return esc_html__('Location', 'stec');
    }

    public function get_icon() {
        return 'eicon-map-pin';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'map', 'location', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {

        echo '<div class="stec-builder-element">';
        ?>
        <div class="stec-layout-single-location">

            <div class="stec-layout-single-location-flex">

                <div class="stec-layout-single-location-left">
                    <p class="stec-style-title2"><?php esc_html_e('Location', 'stec'); ?></p>
                    <span class="stec-layout-single-location-address stec-style-text"><?php esc_html_e('Location Address', 'stec'); ?></span>
                </div>

                <div class="stec-layout-single-location-right">
                    <p class="stec-layout-single-location-directions-title stec-style-title2"><?php esc_html_e('Directions', 'stec'); ?></p>
                </div>

            </div>

            <div class="stec-layout-single-location-flex">

                <div class="stec-layout-single-location-left">
                    <div class="stec-layout-single-location-map stec-builder-element-placeholder-map-container">
                        <span>
                            <i class="eicon-google-maps"></i>
                            <span><?php esc_html_e('Map Container', 'stec'); ?></span>
                        </span>
                    </div>
                </div>

                <div class="stec-layout-single-location-right">
                    <input class="stec-layout-event-input-fontandcolor" type="text" name="start" placeholder="<?php esc_html_e('Start Location', 'stec'); ?>" />
                    <input class="stec-layout-event-input-fontandcolor" type="text" name="end" placeholder="<?php esc_html_e('Destination', 'stec'); ?>" />
                    <div class="stec-style-flexbox-no-media stec-layout-single-location-directions-buttons">
                        <a href="javascript:void(0);" class="stec-style-button stec-layout-single-location-get-direction-btn "><?php esc_html_e('Get Directions', 'stec'); ?></a>
                        <a target="_blank" href="javascript:void(0);" class="stec-style-button stec-layout-single-location-open-map "><?php esc_html_e('Open Map', 'stec'); ?></a>
                    </div>
                </div>

            </div>

            <p class="stec-layout-single-location-details-title stec-style-title2"><?php esc_html_e('Location Details', 'stec'); ?></p>
            <p class='stec-layout-single-location-details stec-style-text'><?php esc_html_e('Location Description Text', 'stec'); ?></p>

        </div>
        <?php
        echo '</div>';
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event = $event_data['event'];

        echo '<div class="stec-builder-element">';

        if ($event->get_location()) :

            $location = $event->get_parsed_location();
            ?>

            <?php
            switch ($location->type) :

                case 'virtual' :
                    ?>
                    <div class="stec-layout-single-location">
                        <div class="stec-layout-single-location-virtual">
                            <div class="stec-layout-single-location-virtual-tags">
                                <span><i class="fas fa-wifi"></i><?php esc_html_e('Online Event', 'stec'); ?></span>
                            </div>

                            <div class="stec-layout-single-location-virtual-flex">

                                <div>
                                    <div class="stec-layout-single-location-left-qr-code">
                                        <img src="<?php echo esc_url($location->qrcode); ?>" />
                                    </div>
                                </div>

                                <div>
                                    <a class="stec-style-button" target="_blank" href="<?php echo esc_url($location->address); ?>" ><?php esc_html_e('Join Event', 'stec'); ?></a>
                                    <div class="stec-layout-single-location-virtual-share">
                                        <?php
                                        echo Admin_Helper::get_virtual_location_share_links($location->address);
                                        ?>
                                    </div>
                                </div>

                            </div>

                            <?php if ($location->details) : ?>

                                <div class="stec-layout-single-location-optional-details">
                                    <p class="stec-style-title2"><?php esc_html_e('Details', 'stec'); ?></p>
                                    <p class='stec-layout-single-location-details stec-style-text'>
                                        <?php
                                        echo Admin_Helper::esc($location->details, 'noscript');
                                        ?>
                                </div>

                            <?php endif; ?>
                        </div>
                    </div>
                    <?php
                    break;

                case 'physical' :
                default:
                    ?>

                    <div class="stec-layout-single-location">
                        <div class="stec-layout-single-location-flex">

                            <div class="stec-layout-single-location-left">
                                <p class="stec-style-title2"><?php esc_html_e('Location', 'stec'); ?></p>
                                <span class="stec-layout-single-location-address stec-style-text" data-location-coordinates="<?php
                                esc_attr_e($location->coordinates);
                                ?>"><?php
                                          if ($location->title) {
                                              esc_attr_e(implode(', ', array($location->title, $location->full_address)));
                                          } else {
                                              esc_attr_e($location->full_address);
                                          }
                                          ?></span>
                            </div>

                            <div class="stec-layout-single-location-right">
                                <p class="stec-layout-single-location-directions-title stec-style-title2"><?php esc_html_e('Directions', 'stec'); ?></p>
                            </div>

                        </div>

                        <div class="stec-layout-single-location-flex">

                            <div class="stec-layout-single-location-left">
                                <div class="stec-layout-single-location-map"></div>
                            </div>

                            <div class="stec-layout-single-location-right">
                                <input class="stec-layout-event-input-fontandcolor" type="text" name="start" placeholder="<?php esc_html_e('Start Location', 'stec'); ?>" />
                                <input class="stec-layout-event-input-fontandcolor" type="text" name="end" placeholder="<?php esc_html_e('Destination', 'stec'); ?>" />
                                <div class="stec-style-flexbox-no-media stec-layout-single-location-directions-buttons">
                                    <a href="javascript:void(0);" class="stec-style-button stec-layout-single-location-get-direction-btn "><?php esc_html_e('Get Directions', 'stec'); ?></a>
                                    <a target="_blank" href="javascript:void(0);" class="stec-style-button stec-layout-single-location-open-map "><?php esc_html_e('Open Map', 'stec'); ?></a>
                                </div>
                                <p class="stec-layout-single-location-direction-error stec-style-text"><?php esc_html_e('Could not find route!', 'stec'); ?></p>
                            </div>

                        </div>

                        <?php if ($location->details !== '') : ?>

                            <p class="stec-layout-single-location-details-title stec-style-title2"><?php esc_html_e('Location Details', 'stec'); ?></p>

                            <p class='stec-layout-single-location-details stec-style-text'>
                                <?php echo Admin_Helper::esc($location->details, 'noscript'); ?>
                            </p>

                        <?php endif; ?>

                    </div>
                <?php

            endswitch;
        endif;

        echo '</div>';
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Media_Slider extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_media_slider';
    }

    public function get_title() {
        return esc_html__('Media Slider', 'stec');
    }

    public function get_icon() {
        return 'eicon-slideshow';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'media', 'slider', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {

        echo '<div class="stec-builder-element stec-builder-element-placeholder">';
        echo '<span>';
        echo '  <i class="eicon-slideshow"></i>';
        printf('<span>%s</span>', esc_html__('Media Slider', 'stec'));
        echo '</span>';
        echo '</div>';
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event = $event_data['event'];

        if ($event->get_images()) :
            ?>

            <div class="stec-builder-element">

                <div class="stec-layout-single-media">

                    <div class="stec-layout-single-media-content">

                        <?php
                        foreach ($event->get_parsed_images() as $image) :
                            ?>
                            <div style="background-image:url(<?php echo esc_url($image->src); ?>);"></div>
                            <?php
                        endforeach;
                        ?>

                    </div>

                    <div class="stec-layout-single-media-content-subs">
                        <div>
                            <p></p>
                            <span></span>
                        </div>
                    </div>

                    <div class="stec-layout-single-media-controls">
                        <div class="stec-layout-single-media-controls-prev stec-layout-single-btn-fontandcolor"><i class="fa fa-angle-left"></i></div>
                        <div class="stec-layout-single-media-controls-list-wrap">
                            <ul class="stec-layout-single-media-controls-list">

                                <?php
                                if (($event->get_images())) :
                                    foreach ($event->get_parsed_images() as $image) :
                                        $image = (object) $image;
                                        ?>
                                        <li style="background-image: url(<?php echo esc_url($image->thumb); ?>);"></li>
                                        <?php
                                    endforeach;
                                endif;
                                ?>

                            </ul>
                        </div>
                        <div class="stec-layout-single-media-controls-next stec-layout-single-btn-fontandcolor"><i class="fa fa-angle-right"></i></div>
                    </div>

                </div>

            </div>
            <?php
        endif;
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Schedule extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_schedule';
    }

    public function get_title() {
        return esc_html__('Schedule', 'stec');
    }

    public function get_icon() {
        return 'eicon-bullet-list';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'schedule', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {
        ?>

        <div class="stec-builder-element">

            <div class="stec-layout-single-schedule-tab">

                <div class="stec-layout-single-schedule-tab-preview">
                    <div class="stec-layout-single-schedule-tab-left">
                        <span><?php esc_html_e('Schedule Time', 'stec'); ?></span>
                    </div>
                    <div class="stec-layout-single-schedule-tab-right">

                        <div class="stec-layout-single-schedule-tab-right-title">
                            <i style="color:#ebc034" class="eicon-clock"></i>
                            <span class=""><?php esc_html_e('Schedule Title', 'stec'); ?></span>
                        </div>

                        <div class="stec-layout-single-schedule-tab-toggle">
                            <i class="fa fa-plus"></i>
                            <i class="fa fa-minus"></i>
                        </div>
                    </div>
                </div>
            </div>

            <div class="stec-layout-single-schedule-tab">

                <div class="stec-layout-single-schedule-tab-preview">
                    <div class="stec-layout-single-schedule-tab-left">
                        <span><?php esc_html_e('Schedule Time', 'stec'); ?></span>
                    </div>
                    <div class="stec-layout-single-schedule-tab-right">

                        <div class="stec-layout-single-schedule-tab-right-title">
                            <i style="color:#32a84a" class="eicon-clock"></i>
                            <span class=""><?php esc_html_e('Schedule Title', 'stec'); ?></span>
                        </div>

                        <div class="stec-layout-single-schedule-tab-toggle">
                            <i class="fa fa-plus"></i>
                            <i class="fa fa-minus"></i>
                        </div>
                    </div>
                </div>

            </div>

            <div class="stec-layout-single-schedule-tab">

                <div class="stec-layout-single-schedule-tab-preview">
                    <div class="stec-layout-single-schedule-tab-left">
                        <span><?php esc_html_e('Schedule Time', 'stec'); ?></span>
                    </div>
                    <div class="stec-layout-single-schedule-tab-right">

                        <div class="stec-layout-single-schedule-tab-right-title">
                            <i style="color:#6644c9" class="eicon-clock"></i>
                            <span class=""><?php esc_html_e('Schedule Title', 'stec'); ?></span>
                        </div>

                        <div class="stec-layout-single-schedule-tab-toggle">
                            <i class="fa fa-plus"></i>
                            <i class="fa fa-minus"></i>
                        </div>
                    </div>
                </div>

            </div>
        </div>

        <?php
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event         = $event_data['event'];
        $repeat_offset = $event_data['offset'];
        $schedules     = $event->get_schedule();

        if ($schedules) :

            echo '<div class="stec-builder-element">';

            foreach ($schedules as $schedule) :

                if (false === ($schedule instanceof \Stachethemes\Stec\Event_Meta_Schedule)) {
                    continue;
                }
                ?>

                <div class="stec-layout-single-schedule-tab<?php
                if ($schedule->get_details() === '') {
                    echo ' stec-layout-single-schedule-tab-no-desc';
                }

                if ($schedule->get_icon() === 'fa') {
                    echo ' stec-layout-single-schedule-tab-no-icon';
                }
                ?>">

                    <div class="stec-layout-single-schedule-tab-preview">
                        <div class="stec-layout-single-schedule-tab-left">
                            <span class="" data-schedule-timestamp="<?php echo strtotime($schedule->get_start_date()) + $repeat_offset; ?>"><?php echo Admin_Helper::get_the_schedule_timespan($schedule->get_start_date(), $event, $repeat_offset); ?></span>
                        </div>
                        <div class="stec-layout-single-schedule-tab-right">

                            <div class="stec-layout-single-schedule-tab-right-title">
                                <i style="color: <?php esc_attr_e($schedule->get_icon_color()); ?>" 
                                   class="<?php esc_attr_e($schedule->get_icon()); ?>"></i><span class=""><?php esc_html_e($schedule->get_title()); ?></span>
                            </div>

                            <div class="stec-layout-single-schedule-tab-toggle">
                                <i class="fa fa-plus"></i>
                                <i class="fa fa-minus"></i>
                            </div>
                        </div>
                    </div>

                    <?php if ($schedule->get_details() != '') : ?>
                        <div class="stec-layout-single-schedule-tab-desc">
                            <div class="stec-style-text"><?php echo nl2br($schedule->get_details()); ?></div>
                        </div>
                    <?php endif; ?>

                </div>

                <?php
            endforeach;

            echo '</div>';

        endif;
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Forecast extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_forecast';
    }

    public function get_title() {
        return esc_html__('Forecast', 'stec');
    }

    public function get_icon() {
        return 'eicon-cloud-check';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'forecast', 'weather', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {

        echo '<div class="stec-builder-element stec-builder-element-placeholder">';
        echo '<span>';
        echo '  <i class="eicon-cloud-check"></i>';
        printf('<span>%s</span>', esc_html__('Weather Forecast', 'stec'));
        echo '</span>';
        echo '</div>';
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }
        ?>

        <div class="stec-builder-element">
            <div class="stec-layout-single-forecast">

                <p class="errorna stec-layout-event-title-fontandcolor"><?php esc_html_e('Weather data is currently not available for this location', 'stec'); ?></p>

                <div class="stec-layout-single-forecast-content">
                    <div class="stec-layout-single-forecast-top">
                        <p class="stec-layout-single-forecast-top-title"><?php esc_html_e('Weather Report', 'stec'); ?></p>
                        <p class="stec-layout-single-forecast-top-date"><?php esc_html_e('Today', 'stec'); ?> stec_replace_today_date</p>
                    </div>

                    <div class="stec-layout-single-forecast-today">

                        <div class="stec-layout-single-forecast-today-left">
                            <div class="stec-layout-single-forecast-today-left-icon">
                                stec_replace_today_icon_div
                            </div>
                            <div>
                                <p class="stec-layout-single-forecast-today-left-current-text">stec_replace_current_summary_text</p>
                                <p class="stec-layout-single-forecast-today-left-current-temp">stec_replace_current_temp &deg;stec_replace_current_temp_units</p>
                            </div>
                        </div>

                        <div class="stec-layout-single-forecast-today-right">
                            <p class=""><?php esc_html_e('Wind', 'stec'); ?> <span>stec_replace_current_wind stec_replace_current_wind_units stec_replace_current_wind_direction</span></p>
                            <p class=""><?php esc_html_e('Humidity', 'stec'); ?> <span>stec_replace_current_humidity %</span></p>
                            <p class=""><?php esc_html_e('Feels like', 'stec'); ?> <span>stec_replace_current_feels_like &deg;stec_replace_current_temp_units</span></p>
                        </div>
                    </div>

                    <div class="stec-layout-single-forecast-details">

                        <div class="stec-layout-single-forecast-details-left">
                            <p class=""><?php esc_html_e('Forecast', 'stec'); ?></p>
                            <div class="stec-layout-single-forecast-details-left-forecast">
                                <div class="stec-layout-single-forecast-details-left-forecast-top">
                                    <p><?php esc_html_e('Date', 'stec'); ?></p>
                                    <p><?php esc_html_e('Weather', 'stec'); ?></p>
                                    <p><?php esc_html_e('Temp', 'stec'); ?></p>
                                </div>
                                stec_replace_5days
                            </div>

                        </div>

                        <div class="stec-layout-single-forecast-details-right">
                            <p class=""><?php esc_html_e('Next 24 Hours', 'stec'); ?></p>
                            <div class="stec-layout-single-forecast-details-chart">
                                <canvas></canvas>
                            </div>
                        </div>
                    </div>

                    <p class="stec-layout-single-forecast-credits"><?php esc_html_e('Powered by', 'stec'); ?> openweathermap.org</p>
                </div>
            </div>
        </div>
        <?php
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Attendance extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_attendance';
    }

    public function get_title() {
        return esc_html__('Attendance', 'stec');
    }

    public function get_icon() {
        return 'eicon-user-circle-o';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'attendance', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {

        $avatar = get_avatar_url(0);
        ?>
        <div class="stec-builder-element">
            <div class="stec-layout-single-attendance">

                <div class="stec-layout-single-attendance-invited">
                    <p class="stec-style-title2">
                        <?php esc_html_e('You are invited to this event!', 'stec'); ?>                               
                        <br>
                    </p>
                    <ul class="stec-layout-single-attendance-invited-buttons">
                        <li class="stec-style-button stec-layout-single-attendance-invited-buttons-attend ">
                            <span><?php esc_html_e('Attend', 'stec'); ?></span>
                        </li>

                        <li class="stec-style-button stec-layout-single-attendance-invited-buttons-decline  ">
                            <span><?php esc_html_e('Decline', 'stec'); ?></span>
                        </li>

                        <li class="stec-style-button stec-layout-single-attendance-invited-buttons-unlist ">
                            <span><?php esc_html_e('Unlist', 'stec'); ?></span>
                        </li>
                    </ul>
                </div>


                <ul class="stec-layout-single-attendance-attendees">

                    <li class="stec-layout-single-attendance-attendee">

                        <div class="stec-layout-single-attendance-attendee-avatar">
                            <img src="<?php echo esc_url($avatar); ?>">
                            <ul>
                                <li class=""><i class="fa fa-check"></i></li>                                
                            </ul>
                        </div>
                        <p class="stec-style-title2"><?php esc_html_e('Attendee Name', 'stec'); ?></p>
                        <span class="stec-layout-single-attendance-attendee-persons"><span>1</span><i class="fas fa-user"></i></span>
                    </li>

                    <li class="stec-layout-single-attendance-attendee">

                        <div class="stec-layout-single-attendance-attendee-avatar">
                            <img src="<?php echo esc_url($avatar); ?>">
                            <ul>
                                <li class=""><i class="fa fa-times"></i></li>                                
                            </ul>
                        </div>
                        <p class="stec-style-title2"><?php esc_html_e('Attendee Name', 'stec'); ?></p>
                        <span class="stec-layout-single-attendance-attendee-persons"><span>1</span><i class="fas fa-user"></i></span>
                    </li>

                    <li class="stec-layout-single-attendance-attendee">

                        <div class="stec-layout-single-attendance-attendee-avatar">
                            <img src="<?php echo esc_url($avatar); ?>">
                            <ul>
                                <li class=""><i class="fa fa-question"></i></li>                                
                            </ul>
                        </div>
                        <p class="stec-style-title2"><?php esc_html_e('Attendee Name', 'stec'); ?></p>
                        <span class="stec-layout-single-attendance-attendee-persons"><span>1</span><i class="fas fa-user"></i></span>
                    </li>

                </ul>

            </div>
        </div>
        <?php
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event         = $event_data['event'];
        $repeat_offset = $event_data['offset'];

        if ('EventCancelled' !== $event->get_event_status() && Admin_Helper::user_is_invited($event)) :
            ?>
            <div class="stec-builder-element">
                <div class="stec-layout-single-attendance">
                    <?php
                    $user_status = (int) Admin_Helper::get_user_attendance_status($event, false, $repeat_offset);
                    $free_slots  = \Stachethemes\Stec\Rsvp::has_free_slots($event, $event->get_start_date('Y-m-d', $repeat_offset));

                    if (1 === $user_status || $free_slots !== false) :
                        ?>

                        <div class="stec-layout-single-attendance-invited">
                            <p class="stec-style-title2">
                                <?php esc_html_e('You are invited to this event!', 'stec'); ?>
                                <br>
                                <span style="<?php echo 9999 === $free_slots ? 'display: none;' : ''; ?>"><?php printf('Places left: %s', '<span class="stec-layout-single-attendance-slots-count">' . (int) $free_slots . '</span>') ?></span>
                            </p>
                            <ul class="stec-layout-single-attendance-invited-buttons">
                                <li class="stec-style-button stec-layout-single-attendance-invited-buttons-attend  <?php
                                if ($user_status === 1) {
                                    echo 'active';
                                }
                                ?>">
                                    <p><?php esc_html_e('Attend', 'stec'); ?></p>
                                </li>

                                <li class="stec-style-button stec-layout-single-attendance-invited-buttons-decline  <?php
                                if ($user_status === 2) {
                                    echo 'active';
                                }
                                ?>">
                                    <p><?php esc_html_e('Decline', 'stec'); ?></p>
                                </li>

                                <li class="stec-style-button stec-layout-single-attendance-invited-buttons-unlist "><p><?php esc_html_e('Unlist', 'stec'); ?></p></li>
                            </ul>
                        </div>

                    <?php endif; ?>

                    <ul class="stec-layout-single-attendance-attendees">

                        <?php
                        $attendees = $event->get_attendance();

                        foreach ($attendees as $attendee) :

                            if (false === ($attendee instanceof \Stachethemes\Stec\Event_Meta_Attendee)) {
                                continue;
                            }
                            ?>

                            <li class="stec-layout-single-attendance-attendee">

                                <div data-userid="<?php echo (int) $attendee->get_userid(); ?>" class="stec-layout-single-attendance-attendee-avatar">
                                    <img src="<?php echo esc_url($attendee->get_avatar()); ?>" alt="<?php esc_attr_e($attendee->get_name()); ?>" />
                                    <ul>
                                        <?php
                                        $user_or_email = $attendee->get_userid() ?: $attendee->get_email();

                                        switch (Admin_Helper::get_user_attendance_status($event, $user_or_email, $repeat_offset)) :

                                            case 1:
                                                echo '<li class=""><i class="fa fa-check"></i></li>';
                                                break;

                                            case 2:
                                                echo '<li class=""><i class="fa fa-times"></i></li>';
                                                break;

                                            case 0:
                                            default:
                                                echo '<li class=""><i class="fa fa-question"></i></li>';

                                        endswitch;
                                        ?>
                                    </ul>
                                </div>

                                <p class="stec-style-title2"><?php esc_html_e($attendee->get_name()); ?></p>
                                <span class="stec-layout-single-attendance-attendee-persons"><span><?php esc_html_e($attendee->get_people()); ?></span><i class="fas fa-user"></i></span>
                            </li>

                        <?php endforeach; ?>
                    </ul>

                </div>
            </div>
            <?php
        endif;
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Shop extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_shop';
    }

    public function get_title() {
        return esc_html__('Shop', 'stec');
    }

    public function get_icon() {
        return 'eicon-products';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'shop', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {
        echo '<div class="stec-builder-element stec-builder-element-placeholder">';
        echo '<span>';
        echo '  <i class="eicon-products"></i>';
        printf('<span>%s</span>', esc_html__('Shop', 'stec'));
        echo '</span>';
        echo '</div>';
    }

    private function the_content() {
        ?>

        <div class="stec-builder-element">
            <div class="stec-layout-single-shop"></div>
        </div>

        <?php
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Tabs extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    private $stec_icons             = array(
        'description' => 'fa fa-info',
        'attachments' => 'fas fa-paperclip',
        'schedule'    => 'fa fa-th-list',
        'forecast'    => 'fas fa-sun',
        'attendance'  => 'fa fa-user',
        'organizers'  => 'fas fa-user-edit',
        'location'    => 'fa fa-map-marker',
        'shop'        => 'fa fa-shopping-cart',
        'guests'      => 'fas fa-star',
        'comments'    => 'fas fa-comments'
    );
    private $stec_accepted_elements = array(
        'description',
        'attachments',
        'schedule',
        'forecast',
        'attendance',
        'organizers',
        'location',
        'shop',
        'guests',
        'comments'
    );

    public function get_name() {
        return 'stec_single_tabs';
    }

    public function get_title() {
        return esc_html__('Tabs', 'stec');
    }

    public function get_icon() {
        return 'eicon-tabs';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'tabs', 'stachethemes');
    }

    protected function register_controls() {

        /**
         * General
         */
        $this->start_controls_section(
                'settings',
                array(
                    'label' => esc_html__('Content', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $repeater = new \Elementor\Repeater();

        $repeater->add_control(
                'tab_title', array(
            'label'       => esc_html__('Tab Title', 'stec'),
            'type'        => \Elementor\Controls_Manager::TEXT,
            'default'     => '',
            'label_block' => true,
                )
        );

        $repeater->add_control(
                'tab_element', array(
            'label'   => esc_html__('Section', 'stec'),
            'type'    => \Elementor\Controls_Manager::SELECT,
            'default' => '',
            'options' => array(
                'description' => esc_html__('Description', 'stec'),
                'schedule'    => esc_html__('Schedule', 'stec'),
                'forecast'    => esc_html__('Forecast', 'stec'),
                'organizers'  => esc_html__('Organizers', 'stec'),
                'attendance'  => esc_html__('Attendance', 'stec'),
                'shop'        => esc_html__('Shop', 'stec'),
                'guests'      => esc_html__('Guests', 'stec'),
                'location'    => esc_html__('Location', 'stec'),
                'comments'    => esc_html__('Comments', 'stec'),
                'attachments' => esc_html__('Attachments', 'stec'),
            )
                )
        );

        $this->add_control(
                'sections',
                array(
                    'label'       => esc_html__('Tab Sections', 'stec'),
                    'type'        => \Elementor\Controls_Manager::REPEATER,
                    'fields'      => $repeater->get_controls(),
                    'title_field' => '{{{ tab_title }}}'
                )
        );

        $this->end_controls_section();
    }

    private function the_content() {

        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event         = $event_data['event'];
        $repeat_offset = $event_data['offset'];

        $sections = $this->get_settings_for_display('sections');
        $display  = array();
        $has_tabs = false;

        foreach ($sections as $k => $section) :

            if (!in_array($section['tab_element'], $this->stec_accepted_elements)) {
                continue;
            }

            $tab_is_empty = false;

            switch ($section['tab_element']) :


                case 'description' :

                    if (!$event->get_description('raw')) {
                        $tab_is_empty = true;
                    }

                    break;

                case 'schedule' :

                    if (!$event->get_schedule()) {
                        $tab_is_empty = true;
                    }

                    break;

                case 'location' :
                case 'forecast' :

                    if ('EventMovedOnline' === $event->get_event_status() || !$event->get_location()) {
                        $tab_is_empty = true;
                    }

                    break;

                case 'shop' :

                    if (!$event->get_products()) {
                        $tab_is_empty = true;
                    }

                    break;

                case 'guests' :

                    if (!$event->get_guests()) {
                        $tab_is_empty = true;
                    }

                    break;

                case 'attendance' :

                    if ('EventCancelled' === $event->get_event_status() || !$event->get_attendance()) {
                        $tab_is_empty = true;
                    }

                    break;

                case 'comments' :

                    if (!$event->get_comments()) {
                        $tab_is_empty = true;
                    }

                    break;

                case 'attachments' :

                    if (!$event->get_attachments()) {
                        $tab_is_empty = true;
                    }

                    break;

                case 'organizers' :

                    if (!$event->get_organizers()) {
                        $tab_is_empty = true;
                    }

                    break;

            endswitch;

            if (true === $tab_is_empty) {
                continue;
            }

            $has_tabs    = true;
            $widget_type = sprintf('stec_single_%s', $section['tab_element']);
            $element     = \Elementor\Plugin::instance()->widgets_manager->get_widget_types($widget_type);

            ob_start();
            $element->render();
            $tab_content = ob_get_clean();

            $display[] = array(
                'element' => $section['tab_element'],
                'title'   => $section['tab_title'],
                'content' => $tab_content
            );
        endforeach;

        unset($tab_content);
        unset($widget_type);
        unset($element);

        ob_start();
        do_action('stec_elementor_widget_single_tab', $event, $repeat_offset);
        $stec_elementor_widget_single_tab = ob_get_clean();

        ob_start();
        do_action('stec_elementor_widget_single_tab_content', $event, $repeat_offset);
        $stec_elementor_widget_single_tab_content = ob_get_clean();

        if (false === $has_tabs && $stec_elementor_widget_single_tab) {
            $has_tabs = true;
        }

        if ($has_tabs) :
            ?>

            <div class="stec-builder-element">

                <div class="stec-layout-single-tabs">

                    <ul class="stec-layout-single-tabs-list">

                        <?php
                        foreach ($display as $k => $tab) {
                            printf('<li class="stec-style-title2 %s" data-tab="stec-layout-single-%s"><i class="%s"></i><span>%s</span></li>', $k === 0 ? 'active' : '', $tab['element'], $this->stec_icons[$tab['element']], $tab['title']);
                        }
                        unset($tab);

                        echo Admin_Helper::esc($stec_elementor_widget_single_tab, 'raw');
                        ?>

                    </ul>

                    <div class="stec-layout-event-single-tabs-content">

                        <?php
                        foreach ($display as $k => $tab) {
                            printf('<div class="%s" data-tab="stec-layout-single-%s">%s</div>', $k === 0 ? 'active' : '', $tab['element'], $tab['content']);
                        }
                        unset($tab);

                        echo Admin_Helper::esc($stec_elementor_widget_single_tab_content, 'raw');
                        ?>

                    </div>

                </div>
            </div>

            <?php
        endif;
    }

    private function placeholder_content() {

        $sections = $this->get_settings_for_display('sections');
        $display  = array();
        $has_tabs = false;

        foreach ($sections as $k => $section) :

            if (!in_array($section['tab_element'], $this->stec_accepted_elements)) {
                continue;
            }

            $has_tabs    = true;
            $widget_type = sprintf('stec_single_%s', $section['tab_element']);
            $element     = \Elementor\Plugin::instance()->widgets_manager->get_widget_types($widget_type);

            ob_start();
            $element->render();
            $tab_content = ob_get_clean();

            $display[] = array(
                'element' => $section['tab_element'],
                'title'   => $section['tab_title'],
                'content' => $tab_content
            );
        endforeach;

        unset($tab_content);
        unset($widget_type);
        unset($element);


        if ($has_tabs) :
            ?>

            <div class="stec-builder-element">

                <div class="stec-layout-single-tabs">

                    <ul class="stec-layout-single-tabs-list">

                        <?php
                        foreach ($display as $k => $tab) {
                            printf('<li class="stec-style-title2 %s" data-tab="stec-layout-single-%s"><i class="%s"></i><span>%s</span></li>', $k === 0 ? 'active' : '', $tab['element'], $this->stec_icons[$tab['element']], $tab['title']);
                        }
                        unset($tab);
                        ?>

                    </ul>

                    <div class="stec-layout-event-single-tabs-content">

                        <?php
                        foreach ($display as $k => $tab) {
                            printf('<div class="%s" data-tab="stec-layout-single-%s">%s</div>', $k === 0 ? 'active' : '', $tab['element'], $tab['content']);
                        }
                        unset($tab);
                        ?>

                    </div>

                </div>
            </div>

            <?php
        endif;
    }

    protected function render() {

        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Guests extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_guests';
    }

    public function get_title() {
        return esc_html__('Guests', 'stec');
    }

    public function get_icon() {
        return 'eicon-user-circle-o';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'guests', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {
        ?>
        <div class="stec-builder-element">
            <div class="stec-layout-single-guests">

                <div class="stec-layout-single-guests-guest">

                    <div class="stec-layout-single-guests-guest-left">
                        <div class="stec-layout-single-guests-guest-left-avatar">
                            <div class="stec-layout-single-guests-guest-left-avatar-default"></div>
                            <ul>
                                <li class="stec-layout-single-guests-guest-left-avatar-icon-position-0">
                                    <a href="#">
                                        <i class="fab fa-facebook"></i>
                                    </a>
                                </li>
                            </ul>
                        </div>
                    </div>

                    <div class="stec-layout-single-guests-guest-right">
                        <p class="stec-layout-single-guests-guest-right-title stec-style-title2"><?php esc_html_e('Guest Name', 'stec'); ?></p>
                        <div class="stec-layout-single-guests-guest-right-desc  stec-style-text">
                            <p><?php esc_html_e('Guest About', 'stec'); ?></p>
                        </div>
                    </div>

                </div>

                <div class="stec-layout-single-guests-guest">

                    <div class="stec-layout-single-guests-guest-left">
                        <div class="stec-layout-single-guests-guest-left-avatar">
                            <div class="stec-layout-single-guests-guest-left-avatar-default"></div>
                            <ul>
                                <li class="stec-layout-single-guests-guest-left-avatar-icon-position-0">
                                    <a href="#">
                                        <i class="fab fa-facebook"></i>
                                    </a>
                                </li>
                            </ul>
                        </div>
                    </div>

                    <div class="stec-layout-single-guests-guest-right">
                        <p class="stec-layout-single-guests-guest-right-title stec-style-title2"><?php esc_html_e('Guest Name', 'stec'); ?></p>
                        <div class="stec-layout-single-guests-guest-right-desc  stec-style-text">
                            <p><?php esc_html_e('Guest About', 'stec'); ?></p>
                        </div>
                    </div>

                </div>

                <div class="stec-layout-single-guests-guest">

                    <div class="stec-layout-single-guests-guest-left">
                        <div class="stec-layout-single-guests-guest-left-avatar">
                            <div class="stec-layout-single-guests-guest-left-avatar-default"></div>
                            <ul>
                                <li class="stec-layout-single-guests-guest-left-avatar-icon-position-0">
                                    <a href="#">
                                        <i class="fab fa-facebook"></i>
                                    </a>
                                </li>
                            </ul>
                        </div>
                    </div>

                    <div class="stec-layout-single-guests-guest-right">
                        <p class="stec-layout-single-guests-guest-right-title stec-style-title2"><?php esc_html_e('Guest Name', 'stec'); ?></p>
                        <div class="stec-layout-single-guests-guest-right-desc  stec-style-text">
                            <p><?php esc_html_e('Guest About', 'stec'); ?></p>
                        </div>
                    </div>

                </div>

            </div>
        </div>

        <?php
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event = $event_data['event'];

        if ($event->get_guests()) :

            $guests = $event->get_guests();
            ?>

            <div class="stec-builder-element">
                <div class="stec-layout-single-guests">

                    <?php
                    foreach ($guests as $guest) :

                        if (false === ($guest instanceof \Stachethemes\Stec\Event_Meta_Guest)) {
                            continue;
                        }
                        ?>

                        <div class="stec-layout-single-guests-guest">

                            <div class="stec-layout-single-guests-guest-left">
                                <div class="stec-layout-single-guests-guest-left-avatar">
                                    <?php if ($guest->get_photo()) : ?>
                                        <img alt="<?php esc_attr_e($guest->get_name()); ?>" src="<?php echo esc_url($guest->get_photo_full()); ?>">
                                    <?php else : ?>
                                        <div class="stec-layout-single-guests-guest-left-avatar-default"></div>
                                    <?php
                                    endif;
                                    ?>
                                    <ul>
                                        <?php foreach ($guest->get_links() as $k => $link) : $link = (object) $link; ?>
                                            <li class="stec-layout-single-guests-guest-left-avatar-icon-position-<?php echo (int) $k; ?>">

                                                <?php
                                                if (false !== filter_var($link->link, FILTER_VALIDATE_EMAIL)) {
                                                    $link->link = 'mailto:' . $link->link;
                                                } elseif ('fas fa-phone' === $link->ico) {
                                                    $link->link = 'tel:' . $link->link;
                                                }
                                                ?>

                                                <a href="<?php echo esc_url($link->link); ?>">
                                                    <i class="<?php esc_attr_e($link->ico); ?>"></i>
                                                </a>
                                            </li>
                                        <?php endforeach; ?>
                                    </ul>
                                </div>
                            </div>

                            <div class="stec-layout-single-guests-guest-right">
                                <p class="stec-layout-single-guests-guest-right-title stec-style-title2"><?php esc_html_e($guest->get_name()); ?></p>
                                <div class="stec-layout-single-guests-guest-right-desc  stec-style-text">
                                    <p><?php echo Admin_Helper::esc($guest->get_about(), 'noscript'); ?></p>
                                </div>
                            </div>

                        </div>

                    <?php endforeach; ?>

                </div>
            </div>

            <?php
        endif;
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Comments extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_comments';
    }

    public function get_title() {
        return esc_html__('Comments', 'stec');
    }

    public function get_icon() {
        return 'eicon-comments';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'comments', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {
        echo '<div class="stec-builder-element stec-builder-element-placeholder">';
        echo '<span>';
        echo '  <i class="eicon-comments"></i>';
        printf('<span>%s</span>', esc_html__('Comments', 'stec'));
        echo '</span>';
        echo '</div>';
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        echo '<div class="stec-builder-element">';
        echo '  <div class="stec-layout-single-comments"></div>';
        echo '</div>';
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

class Related_Events extends \Elementor\Widget_Base {

    use Widget_Panel_Display;

    public function get_name() {
        return 'stec_single_related_events';
    }

    public function get_title() {
        return esc_html__('Related Events', 'stec');
    }

    public function get_icon() {
        return 'eicon-integration';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'comments', 'stachethemes');
    }

    protected function register_controls() {

        $this->register_no_options();
    }

    private function placeholder_content() {
        echo '<div class="stec-builder-element stec-builder-element-placeholder">';
        echo '<span>';
        echo '  <i class="eicon-integration"></i>';
        printf('<span>%s</span>', esc_html__('Related Events', 'stec'));
        echo '</span>';
        echo '</div>';
    }

    private function the_content() {
        $event_data = $this->get_event_data();

        if (!$event_data) {
            return '';
        }

        $event         = $event_data['event'];
        $repeat_offset = $event_data['offset'];

        include(\Stachethemes\Stec\Stachethemes_Event_Calendar::get_instance()->get_path('VIEW') . "/front/pages/single.related.php");
    }

    protected function render() {
        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->placeholder_content();
        } else {
            $this->the_content();
        }
    }

}

\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Preview());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Share_And_Export());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Organizers());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Description());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Tickets());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Shop());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new RSVP());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Counter());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Website_URL());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Attachments());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Categories());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Attendance_Buttons());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Attendance());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Guests());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Location());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Media_Slider());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Schedule());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Forecast());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Comments());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Tabs());
\Elementor\Plugin::instance()->widgets_manager->register_widget_type(new Related_Events());
