<?php

namespace Stachethemes\Stec\Elementor\Widgets;

use Stachethemes\Stec\Categories as Categories;
use Stachethemes\Stec\Calendars as Calendars;
use Stachethemes\Stec\Organizers as Organizers;

class Calendar_Base extends \Elementor\Widget_Base {

    public function get_name() {
        return 'stachethemes_ec';
    }

    public function get_title() {
        return esc_html__('Stachethemes Event Calendar', 'stec');
    }

    public function get_icon() {
        return 'far fa-calendar-alt';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'calendar', 'stachethemes');
    }

    protected function _register_controls() {

        /**
         * General
         */
        $this->start_controls_section(
                'general',
                array(
                    'label' => esc_html__('General', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'views',
                array(
                    'label'    => esc_html__('Views', 'stec'),
                    'type'     => \Elementor\Controls_Manager::SELECT2,
                    'default'  => array(),
                    'multiple' => true,
                    'options'  => array(
                        'agenda'  => esc_html__('Agenda', 'stec'),
                        'week'    => esc_html__('Week', 'stec'),
                        'month'   => esc_html__('Month', 'stec'),
                        'day'     => esc_html__('Day', 'stec'),
                        'grid'    => esc_html__('Grid', 'stec'),
                        'boxgrid' => esc_html__('Box Grid', 'stec'),
                        'map'     => esc_html__('Map', 'stec'),
                    ),
                )
        );

        $this->add_control(
                'view',
                array(
                    'label'   => esc_html__('Initial View', 'stec'),
                    'type'    => \Elementor\Controls_Manager::SELECT,
                    'default' => '',
                    'options' => array(
                        'agenda'  => esc_html__('Agenda', 'stec'),
                        'week'    => esc_html__('Week', 'stec'),
                        'month'   => esc_html__('Month', 'stec'),
                        'day'     => esc_html__('Day', 'stec'),
                        'grid'    => esc_html__('Grid', 'stec'),
                        'boxgrid' => esc_html__('Box Grid', 'stec'),
                        'map'     => esc_html__('Map', 'stec'),
                    ),
                )
        );

        $this->add_control(
                'open_event_in',
                array(
                    'label'   => esc_html__('Open events in', 'stec'),
                    'type'    => \Elementor\Controls_Manager::SELECT,
                    'default' => 'self',
                    'options' => array(
                        'self'     => esc_html__('Calendar', 'stec'),
                        'single'   => esc_html__('Single Page', 'stec'),
                        'external' => esc_html__('External Link', 'stec'),
                        'modal'    => esc_html__('Popup', 'stec'),
                    ),
                )
        );

        $this->add_control(
                'show_event_tags',
                array(
                    'label'        => esc_html__('Display event categories', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'use_cat_color',
                array(
                    'label'        => esc_html__('Use category color', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => '1',
                    'default'      => 0,
                )
        );

        $this->end_controls_section();


        /**
         * Filters
         */
        $this->start_controls_section(
                'filters',
                array(
                    'label' => esc_html__('Filters', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $calendars = Calendars::get_calendars_list();
        $this->add_control(
                'cal',
                array(
                    'label'    => esc_html__('Calendars', 'stec'),
                    'type'     => \Elementor\Controls_Manager::SELECT2,
                    'default'  => array(),
                    'multiple' => true,
                    'options'  => $calendars
                )
        );

        $categories = Categories::get_categories_list();
        $this->add_control(
                'cat',
                array(
                    'label'    => esc_html__('Categories', 'stec'),
                    'type'     => \Elementor\Controls_Manager::SELECT2,
                    'default'  => array(),
                    'multiple' => true,
                    'options'  => $categories
                )
        );

        $organizers = Organizers::get_organizers_list();
        $this->add_control(
                'organizer_id',
                array(
                    'label'    => esc_html__('Organizers', 'stec'),
                    'type'     => \Elementor\Controls_Manager::SELECT2,
                    'default'  => array(),
                    'multiple' => true,
                    'options'  => $organizers
                )
        );

        $this->add_control(
                'start_date',
                array(
                    'label'          => esc_html__('Initial Start Date', 'stec'),
                    'type'           => \Elementor\Controls_Manager::DATE_TIME,
                    'picker_options' => array(
                        'enableTime' => false,
                        'dateFormat' => 'Y-m-d'
                    )
                )
        );

        $this->add_control(
                'min_date',
                array(
                    'label'          => esc_html__('Range start', 'stec'),
                    'type'           => \Elementor\Controls_Manager::DATE_TIME,
                    'picker_options' => array(
                        'enableTime' => false,
                        'dateFormat' => 'Y-m-d'
                    )
                )
        );

        $this->add_control(
                'max_date',
                array(
                    'label'          => esc_html__('Range end', 'stec'),
                    'type'           => \Elementor\Controls_Manager::DATE_TIME,
                    'picker_options' => array(
                        'enableTime' => false,
                        'dateFormat' => 'Y-m-d'
                    )
                )
        );

        $this->end_controls_section();

        /**
         * Top Menu
         */
        $this->start_controls_section(
                'top_menu',
                array(
                    'label' => esc_html__('Top Menu', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'show_top',
                array(
                    'label'        => esc_html__('Top Menu', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'show_search',
                array(
                    'label'        => esc_html__('Search Menu', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'show_views',
                array(
                    'label'        => esc_html__('Views Menu', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'top_nav',
                array(
                    'label'        => esc_html__('Navigation Menu', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'show_calfilter',
                array(
                    'label'   => esc_html__('Filters Menu', 'stec'),
                    'type'    => \Elementor\Controls_Manager::CHOOSE,
                    'options' => array(
                        '0' => [
                            'title' => esc_html__('Hide', 'stec'),
                            'icon'  => 'eicon-close'
                        ],
                        '1' => [
                            'title' => esc_html__('Toggle', 'stec'),
                            'icon'  => 'eicon-menu-bar'
                        ],
                        '2' => [
                            'title' => esc_html__('Static', 'stec'),
                            'icon'  => 'eicon-preview-thin'
                        ],
                    ),
                    'default' => '1',
                    'toggle'  => true
                )
        );

        $this->end_controls_section();

        /**
         * Agenda
         */
        $this->start_controls_section(
                'agenda',
                array(
                    'label' => esc_html__('Agenda', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'agenda_cal_display',
                array(
                    'label'        => esc_html__('Calendar Slider Display', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'agenda_list_display',
                array(
                    'label'        => esc_html__('Events List Display', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->add_control(
                'reverse_agenda_list',
                array(
                    'label'   => esc_html__('List Order', 'stec'),
                    'type'    => \Elementor\Controls_Manager::SELECT,
                    'default' => 0,
                    'options' => array(
                        0 => esc_html__('Ascending', 'stec'),
                        1 => esc_html__('Descending', 'stec'),
                    )
                )
        );

        $this->add_control(
                'agenda_get_n',
                array(
                    'label'   => esc_html__('Events Per Click', 'stec'),
                    'type'    => \Elementor\Controls_Manager::NUMBER,
                    'min'     => 1,
                    'max'     => 100,
                    'default' => 3
                )
        );

        $this->add_control(
                'agenda_lfm',
                array(
                    'label'        => esc_html__('Look for more button', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );


        $this->end_controls_section();


        /**
         * Month & Week
         */
        $this->start_controls_section(
                'month_and_week',
                array(
                    'label' => esc_html__('Month & Week', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'show_event_title_all_cells',
                array(
                    'label'        => esc_html__('Show event title on all cells', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => '1',
                    'default'      => 0,
                )
        );

        $this->add_control(
                'tooltip_display',
                array(
                    'label'        => esc_html__('Tooltip', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->end_controls_section();


        /**
         * Grid
         */
        $this->start_controls_section(
                'grid',
                array(
                    'label' => esc_html__('Grid', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'grid_columns',
                array(
                    'label'   => esc_html__('Columns', 'stec'),
                    'type'    => \Elementor\Controls_Manager::NUMBER,
                    'min'     => 1,
                    'max'     => 10,
                    'default' => 4,
                )
        );

        $this->add_control(
                'grid_gutter',
                array(
                    'label'   => esc_html__('Gutter', 'stec'),
                    'type'    => \Elementor\Controls_Manager::NUMBER,
                    'min'     => 0,
                    'max'     => 100,
                    'default' => 10,
                )
        );

        $this->add_control(
                'grid_per_click',
                array(
                    'label'   => esc_html__('Events per click', 'stec'),
                    'type'    => \Elementor\Controls_Manager::NUMBER,
                    'min'     => 1,
                    'max'     => 100,
                    'default' => 4,
                )
        );

        $this->add_control(
                'grid_lfm',
                array(
                    'label'        => esc_html__('Look for more button', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->end_controls_section();


        /**
         * Box Grid
         */
        $this->start_controls_section(
                'box_grid',
                array(
                    'label' => esc_html__('Box Grid', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'boxgrid_columns',
                array(
                    'label'   => esc_html__('Columns', 'stec'),
                    'type'    => \Elementor\Controls_Manager::NUMBER,
                    'min'     => 1,
                    'max'     => 10,
                    'default' => 4,
                )
        );

        $this->add_control(
                'boxgrid_per_click',
                array(
                    'label'   => esc_html__('Events per click', 'stec'),
                    'type'    => \Elementor\Controls_Manager::NUMBER,
                    'min'     => 1,
                    'max'     => 100,
                    'default' => 4,
                )
        );

        $this->add_control(
                'boxgrid_lfm',
                array(
                    'label'        => esc_html__('Look for more button', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Show', 'stec'),
                    'label_off'    => esc_html__('Hide', 'stec'),
                    'return_value' => '1',
                    'default'      => '1',
                )
        );

        $this->end_controls_section();


        /**
         * Map
         */
        $this->start_controls_section(
                'map',
                array(
                    'label' => esc_html__('Map', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'map_zoom',
                array(
                    'label'   => esc_html__('Zoom', 'stec'),
                    'type'    => \Elementor\Controls_Manager::NUMBER,
                    'min'     => 1,
                    'max'     => 20,
                    'default' => 10,
                )
        );

        $this->add_control(
                'map_focus',
                array(
                    'label'       => esc_html__('Focus Location', 'stec'),
                    'type'        => \Elementor\Controls_Manager::TEXT,
                    'description' => esc_html__('Center map on location (e.g. London, UK)', 'stec'),
                    'default'     => '',
                )
        );

        $this->end_controls_section();

        /**
         * Submission form
         */
        $this->start_controls_section(
                'submission_form',
                array(
                    'label' => esc_html__('Event Submission', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'show_create_event_form',
                array(
                    'label'        => esc_html__('Display event submission form', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => '1',
                    'default'      => '1'
                )
        );


        $this->add_control(
                'create_event_form_allow_expired',
                array(
                    'label'        => esc_html__('Allow submitting expired events', 'stec'),
                    'type'         => \Elementor\Controls_Manager::SWITCHER,
                    'label_on'     => esc_html__('Yes', 'stec'),
                    'label_off'    => esc_html__('No', 'stec'),
                    'return_value' => '1',
                    'default'      => 0,
                )
        );

        $this->end_controls_section();

        /**
         * Other
         */
        $this->start_controls_section(
                'other',
                array(
                    'label' => esc_html__('Other', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'other_parameters',
                array(
                    'label'        => esc_html__('Additional parameters', 'stec'),
                    'type'         => \Elementor\Controls_Manager::TEXTAREA,
                    'return_value' => '',
                    'default'      => ''
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {
        ?>
        <div class="stachethemes-shortcode-placeholder">
            <div class="stachethemes-shortcode-placeholder-logo"></div>
            <p class="stachethemes-shortcode-placeholder-text"><?php esc_html_e('Stachethemes Event Calendar', 'stec'); ?></p></div>
        <?php
    }

    protected function render() {

        $settings = $this->get_settings_for_display();

        $attributes       = array();
        $attributes_array = array(
            'views'                           => $settings['views'],
            'view'                            => $settings['view'],
            'cal'                             => $settings['cal'],
            'cat'                             => $settings['cat'],
            'organizer_id'                    => $settings['organizer_id'],
            'start_date'                      => $settings['start_date'],
            'min_date'                        => $settings['min_date'],
            'max_date'                        => $settings['max_date'],
            'show_top'                        => $settings['show_top'] ?: 0,
            'show_views'                      => $settings['show_views'] ?: 0,
            'top_nav'                         => $settings['top_nav'] ?: 0,
            'show_search'                     => $settings['show_search'] ?: 0,
            'show_calfilter'                  => $settings['show_calfilter'] ?: 0,
            'agenda_cal_display'              => $settings['agenda_cal_display'] ?: 0,
            'agenda_list_display'             => $settings['agenda_list_display'] ?: 0,
            'agenda_get_n'                    => $settings['agenda_get_n'],
            'reverse_agenda_list'             => $settings['reverse_agenda_list'],
            'open_event_in'                   => $settings['open_event_in'],
            'show_event_title_all_cells'      => $settings['show_event_title_all_cells'] ?: 0,
            'tooltip_display'                 => $settings['tooltip_display'] ?: 0,
            'grid_columns'                    => $settings['grid_columns'],
            'grid_gutter'                     => $settings['grid_gutter'],
            'grid_per_click'                  => $settings['grid_per_click'],
            'boxgrid_columns'                 => $settings['boxgrid_columns'],
            'boxgrid_per_click'               => $settings['boxgrid_per_click'],
            'map_zoom'                        => $settings['map_zoom'],
            'map_focus'                       => $settings['map_focus'],
            'show_create_event_form'          => $settings['show_create_event_form'] ?: 0,
            'create_event_form_allow_expired' => $settings['create_event_form_allow_expired'] ?: 0,
            'use_cat_color'                   => $settings['use_cat_color'] ?: 0,
            'show_event_tags'                 => $settings['show_event_tags'] ?: 0,
            'agenda_lfm'                      => $settings['agenda_lfm'] ?: 0,
            'grid_lfm'                        => $settings['grid_lfm'] ?: 0,
            'boxgrid_lfm'                     => $settings['boxgrid_lfm'] ?: 0
        );

        foreach ($attributes_array as $k => $attrib) {
            if (!$attrib && 0 !== $attrib) {
                continue;
            }

            $attributes[] = sprintf('%s="%s"', $k, is_array($attrib) ? implode(',', $attrib) : $attrib);
        }

        $inline_attributes = implode(' ', $attributes);

        echo do_shortcode(shortcode_unautop(sprintf('[stachethemes_ec %s %s]', $inline_attributes, $settings['other_parameters'])));
    }

    public function render_plain_content() {
        echo '[stachethemes_ec]';
    }

    public function show_in_panel() {

        $exclude          = array('stec_esf', 'stec_sp');
        $current_document = \Elementor\Plugin::$instance->documents->get_current();
        $post_type        = $current_document->get_post()->post_type;

        foreach ($exclude as $exclude_pt) {
            if ($exclude_pt === $post_type) {
                return false;
            }
        }

        return true;
    }

}
