<?php

namespace Stachethemes\Stec\Elementor\Widgets;

use Stachethemes\Stec\Admin_Helper as Admin_Helper;

class Submit_Form extends \Elementor\Widget_Base {

    public function get_name() {
        return 'stec_submit_form';
    }

    public function get_title() {
        return esc_html__('Event Submission Form', 'stec');
    }

    public function get_icon() {
        return 'far fa-calendar-alt';
    }

    public function get_categories() {
        return array('stec');
    }

    public function get_keywords() {
        return array('stec', 'submit', 'event', 'form', 'calendar', 'stachethemes');
    }

    protected function _register_controls() {

        /**
         * Submission form
         */
        $this->start_controls_section(
                'submission_form',
                array(
                    'label' => esc_html__('Settings', 'stec'),
                    'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
                )
        );

        $this->add_control(
                'template_id',
                array(
                    'label'   => esc_html__('Template', 'stec'),
                    'type'    => \Elementor\Controls_Manager::SELECT,
                    'default' => \Stachethemes\Stec\Settings::get_admin_setting_value('stec_menu__general_event_submit', 'submission_form_id'),
                    'options' => array(0 => esc_html__('Default', 'stec')) + \Stachethemes\Stec\Builder::get_projects_list_array('stec_esf'),
                )
        );

        $writables_cals     = \Stachethemes\Stec\Calendars::get_writable_calendar_list();
        $writable_cals_list = array();

        foreach ($writables_cals as $writables_cal) {
            $writable_cals_list[$writables_cal->get_id()] = $writables_cal->get_title();
        }

        $this->add_control(
                'cal',
                array(
                    'label'    => esc_html__('Calendars', 'stec'),
                    'multiple' => true,
                    'type'     => \Elementor\Controls_Manager::SELECT2,
                    'default'  => '',
                    'options'  => $writable_cals_list,
                )
        );


        $this->add_control(
                'cat',
                array(
                    'label'    => esc_html__('Categories', 'stec'),
                    'multiple' => true,
                    'type'     => \Elementor\Controls_Manager::SELECT2,
                    'default'  => '',
                    'options'  => \Stachethemes\Stec\Categories::get_categories_list(),
                )
        );

        $this->end_controls_section();
    }

    protected function content_template() {
        ?>
        <div class="stachethemes-shortcode-placeholder">
            <div class="stachethemes-shortcode-placeholder-logo"></div>
            <p class="stachethemes-shortcode-placeholder-text"><?php esc_html_e('Event Submission Form', 'stec'); ?></p>
        </div>
        <?php
    }

    protected function render() {

        if (Admin_Helper::is_elementor() || Admin_Helper::is_preview('stec_sp')) {
            $this->content_template();
        } else {
            echo do_shortcode(
                    shortcode_unautop(
                            sprintf('[stec_submit_form template_id="%s" cal="%s" cat="%s"]',
                                    $this->get_settings_for_display('template_id'),
                                    $this->get_settings_for_display('cal') ? implode(',', $this->get_settings_for_display('cal')) : '',
                                    $this->get_settings_for_display('cat') ? implode(',', $this->get_settings_for_display('cat')) : ''
                            )
                    )
            );
        }
    }

    public function render_plain_content() {
        echo '[stec_submit_form]';
    }

    public function show_in_panel() {

        $exclude          = array('stec_esf', 'stec_sp');
        $current_document = \Elementor\Plugin::$instance->documents->get_current();
        $post_type        = $current_document->get_post()->post_type;

        foreach ($exclude as $exclude_pt) {
            if ($exclude_pt === $post_type) {
                return false;
            }
        }

        return true;
    }

}
