<?php

namespace Stachethemes\Stec;

if (!$this instanceof Stachethemes_Event_Calendar) {
    wp_die('-1');
}

header('Content-Type: application/json');

if (!is_admin() || false === check_ajax_referer('stecApiVerifyNonce', 'security', false)) {
    echo wp_json_encode(array(
        'error'     => 1,
        'error_msg' => esc_html__('Invalid nonce', 'stec')
    ));
    exit;
}

$task = Admin_Helper::post('task', false, FILTER_SANITIZE_STRING);

switch ($task) :

    case 'add_fa_icon' :

        $icon          = Admin_Helper::post('icon', false, FILTER_SANITIZE_STRING);
        $filtered_icon = sanitize_text_field($icon);

        if ($filtered_icon) {
            $icons = get_option('stec_user_fa_icons', array());
            if (!in_array($filtered_icon, $icons)) {
                $icons[$filtered_icon] = $filtered_icon;
                update_option('stec_user_fa_icons', $icons, false);
            }
        }

        echo wp_json_encode(array(
            'error' => 0
        ));

        break;

    case 'activate_license' :

        $purchase_code = Admin_Helper::post('purchase_code', false, FILTER_SANITIZE_STRING);
        $site_url      = get_site_url();
        $parsed        = parse_url($site_url);
        $domain        = $parsed['host'];


        try {

            if (!$purchase_code) {
                throw new Stec_Exception(esc_html__('No purchase code provided', 'stec'));
            }

            $remote = wp_remote_get(sprintf('https://api.stachethemes.com/stec/activate/%s/%s', $purchase_code, $domain));
            
            if (is_wp_error($remote)) {
                throw new Stec_Exception($remote->get_error_message());
            }
            
            if (200 !== wp_remote_retrieve_response_code($remote)) {
                throw new Stec_Exception(wp_remote_retrieve_response_message($remote));
            }

            $body     = wp_remote_retrieve_body($remote);
            $response = json_decode($body);

            if ($response->errorMessage) {
                throw new Stec_Exception($response->errorMessage);
            }
        } catch (Stec_Exception $ex) {
            echo wp_json_encode(array(
                'error' => 1,
                'msg'   => $ex->getMessage()
            ));

            exit;
        }

        update_option('stec_activated', array(
            'purchase_code' => $purchase_code,
            'server_name'   => $domain
                ), false);

        Settings::update_admin_settings('stec_menu__license');

        Admin_Helper::set_message(esc_html__('License Activated', 'stec'));

        echo wp_json_encode(array(
            'error' => 0
        ));

        break;

    case 'deactivate_license' :

        $purchase_code = Admin_Helper::post('purchase_code', false, FILTER_SANITIZE_STRING);
        $site_url      = get_site_url();
        $parsed        = parse_url($site_url);
        $domain        = $parsed['host'];

        try {

            if (!$purchase_code) {
                throw new Stec_Exception(esc_html__('No purchase code provided', 'stec'));
            }

            $remote = wp_remote_get(sprintf('https://api.stachethemes.com/stec/deactivate/%s/%s', $purchase_code, $domain));

            if (200 !== wp_remote_retrieve_response_code($remote)) {
                throw new Stec_Exception(wp_remote_retrieve_response_message($remote));
            }

            $body     = wp_remote_retrieve_body($remote);
            $response = json_decode($body);

            if ($response->errorMessage) {
                throw new Stec_Exception($response->errorMessage);
            }
        } catch (Stec_Exception $ex) {
            echo wp_json_encode(array(
                'error' => 1,
                'msg'   => $ex->getMessage()
            ));

            exit;
        }

        update_option('stec_activated', false, false);

        Settings::update_admin_settings('stec_menu__license');

        Admin_Helper::set_message(esc_html__('License Deactivated', 'stec'));

        echo wp_json_encode(array(
            'error' => 0
        ));

        break;

    case 'stec_builder_delete' :

        $project_id = Admin_Helper::post('project_id', false, FILTER_DEFAULT);

        $result = Builder::delete_project($project_id);

        $r = new response_object();
        $r->set_data(array(
            'success'  => 1,
            'projects' => Builder::get_projects_list()
        ));

        $r->get();

        break;

    case 'stec_builder_bulk_delete' :

        $project_ids = Admin_Helper::post('project_ids', array(), FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY);

        $result = Builder::bulk_delete_projects($project_ids);

        $r = new response_object();
        $r->set_data(array(
            'success'  => 1,
            'projects' => Builder::get_projects_list()
        ));

        $r->get();

        break;

    case 'get_users_by_role' :

        $roles = Admin_Helper::post('role', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

        $users_by_role = get_users(array(
            'role__in' => $roles,
            'fields'   => array(
                'ID',
                'display_name',
                'user_email'
            )
        ));


        $r = new response_object();
        $r->set_data(array(
            'success' => 1,
            'users'   => $users_by_role
        ));

        $r->get();

        break;

    case 'doi_revoke_consents' :

        DTools::delete_doi_emails();

        $r = new response_object();
        $r->set_data(array(
            'success' => 1
        ));

        $r->get();

        break;

    case 'insert_new_organizer' :

        $r = new response_object();

        try {

            if (false === Admin_Helper::isset_post('name')) {
                throw new Stec_Exception(esc_html__('Invalid organizer name', 'stec'));
            }

            $organizer = new Organizer();
            $organizer->set_name(Admin_Helper::post('name', '', FILTER_SANITIZE_STRING));
            $organizer->set_email(Admin_Helper::post('email', '', FILTER_VALIDATE_EMAIL));
            $organizer->set_social(Admin_Helper::post('social', array(), FILTER_DEFAULT, FILTER_REQUIRE_ARRAY));
            $organizer->set_details(Admin_Helper::post('details', '', FILTER_DEFAULT));
            $organizer->set_notes(Admin_Helper::post('notes', '', FILTER_DEFAULT));
            $organizer->set_photo(Admin_Helper::post('photo', '', FILTER_VALIDATE_INT));
            $organizer->set_visibility(Admin_Helper::post('visibility', 'public', FILTER_SANITIZE_STRING));

            $id = $organizer->insert_organizer();

            $r->set_data(array(
                'success'   => 1,
                'organizer' => $organizer->get_data()
            ));
        } catch (Stec_Exception $ex) {
            $r->set_error_msg($ex->getMessage());
            $r->set_error(1);
        }

        $r->get();

        break;

    case 'migrate_261_locations' :

        try {
            $r         = new response_object();
            $event_ids = get_posts(array(
                'posts_per_page'   => 5,
                'offset'           => 0,
                'post_type'        => 'stec_event',
                'fields'           => 'ids',
                'meta_key'         => 'location',
                'suppress_filters' => true
            ));

            if ($event_ids) {

                set_time_limit(STEC_EXEC_TIME_LIMIT);

                foreach ($event_ids as $event_id) {

                    $location = get_post_meta($event_id, 'location', true);

                    if ($location) {

                        $found_location = Locations::get_location_by_name($location);
                        if ($found_location) {
                            $new_location_id = $found_location->get_id();
                        } else {
                            $new_location = new Location();
                            $new_location->set_address($location);
                            $details      = get_post_meta($event_id, 'location_details', true);
                            $coordinates  = get_post_meta($event_id, 'location_forecast', true);

                            if ($details) {
                                $new_location->set_details($details);
                            }

                            if ($coordinates) {
                                $new_location->set_coordinates($coordinates);
                            }

                            $new_location_id = $new_location->insert_location();
                        }

                        $result = wp_set_object_terms($event_id, $new_location_id, 'stec_event_location');

                        if (is_wp_error($result)) {
                            throw new Stec_Exception($result->get_error_message());
                        }
                    }

                    delete_post_meta($event_id, 'location');
                    delete_post_meta($event_id, 'location_details');
                    delete_post_meta($event_id, 'location_forecast');
                    delete_post_meta($event_id, 'location_use_coord');

                    Admin_Helper::debug_log($event_id);
                }
            }

            $has_event_left = get_posts(array(
                'posts_per_page'   => 1,
                'offset'           => 0,
                'post_type'        => 'stec_event',
                'fields'           => 'ids',
                'meta_key'         => 'location',
                'suppress_filters' => true
            ));

            $r->set_data(array(
                'success' => 1,
                'rerun'   => count($has_event_left)
            ));
        } catch (Stec_Exception $ex) {
            $r->set_error_msg($ex->getMessage());
            $r->set_error(1);
        }

        $r->get();
        break;

    case 'migrate_261_organizers' :

        try {
            $r = new response_object();

            $event_ids = get_posts(array(
                'posts_per_page'   => 5,
                'offset'           => 0,
                'post_type'        => 'stec_event',
                'fields'           => 'ids',
                'meta_key'         => 'organizers',
                'suppress_filters' => true
            ));

            if ($event_ids) {

                set_time_limit(STEC_EXEC_TIME_LIMIT);

                foreach ($event_ids as $event_id) {

                    $organizers     = get_post_meta($event_id, 'organizers', true);
                    $organizers_ids = array();

                    if ($organizers) {

                        foreach ($organizers as $organizer) {
                            $found_organizer = Organizers::get_organizer_by_name($organizer->name);
                            if ($found_organizer) {
                                $new_organizer_id = $found_organizer->get_id();
                            } else {
                                $new_organizer = new Organizer();
                                $new_organizer->set_name($organizer->name);
                                if ($organizer->photo) {
                                    $new_organizer->set_photo($organizer->photo);
                                }
                                $new_organizer->set_details($organizer->about);

                                if ($organizer->links) {
                                    $new_organizer->set_social($organizer->links);
                                }

                                $new_organizer_id = $new_organizer->insert_organizer();
                            }

                            if ($new_organizer_id) {
                                $organizers_ids[] = $new_organizer_id;
                            }

                            unset($new_organizer_id);
                        }
                    }
                }

                $result = wp_set_object_terms($event_id, array_map('intval', $organizers_ids), 'stec_event_organizer');

                if (is_wp_error($result)) {
                    throw new Stec_Exception($result->get_error_message());
                }

                delete_post_meta($event_id, 'organizers');
            }

            $has_event_left = get_posts(array(
                'posts_per_page'   => 1,
                'offset'           => 0,
                'post_type'        => 'stec_event',
                'fields'           => 'ids',
                'meta_key'         => 'organizers',
                'suppress_filters' => true
            ));

            $r->set_data(array(
                'success' => 1,
                'rerun'   => count($has_event_left)
            ));
        } catch (Stec_Exception $ex) {
            $r->set_error_msg($ex->getMessage());
            $r->set_error(1);
        }

        $r->get();
        break;

    case 'clear_session_message' :

        Admin_Helper::session_set('stec-admin-message', false);

        $r = new response_object();

        $r->set_data(array(
            'success' => 1,
        ));

        $r->get();

        break;

    case 'refresh_permalinks' :

        $r = new response_object();

        try {

            if (!current_user_can('manage_options')) {
                throw new Stec_Exception(esc_html__('Permission denied', 'stec'));
            }

            $permalinks = Admin_Helper::post('permalinks', null, FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);

            if (!is_array($permalinks)) {
                throw new Stec_Exception(esc_html__('Invalid permalinks', 'stec'));
            }

            $defs = Admin_Helper::get_permalinks();

            foreach ($defs as $k => &$def) {
                if (isset($permalinks[$k]) && trim($permalinks[$k] !== '')) {
                    $def = trim($permalinks[$k]);
                }
            }
            unset($def);

            update_option('stec_permalinks', $defs, false);

            add_action('shutdown', function() {
                do_action('stec_add_rewrite_rules');
                flush_rewrite_rules();
            });

            $r->set_data(array(
                'success' => 1,
                'msg'     => esc_html__('Permalinks have been updated', 'stec')
            ));
        } catch (Stec_Exception $ex) {
            $r->set_error(1);
            $r->set_error_msg($ex->getMessage());
        }

        $r->get();

        break;

    case 'mark_tickets_order_scanned' :

        $r = new response_object();

        try {
            $order_id     = Admin_Helper::post('order_id', null, FILTER_VALIDATE_INT);
            $mark_scanned = Admin_Helper::post('mark_scanned', 0, FILTER_VALIDATE_INT);

            if (!$order_id) {
                throw new Stec_Exception(esc_html__('Item or order not found', 'stec'));
            }

            $order = new \WC_Order($order_id);

            if (!Booking::order_has_tickets($order)) {
                throw new Stec_Exception(esc_html__('Order has no tickets', 'stec'));
            }

            $items = $order->get_items();

            foreach ($items as $item) {

                $product_id = $item->get_product_id();

                if ('1' !== get_post_meta($product_id, 'stec_bookable', true)) {
                    continue;
                }

                $item->update_meta_data('stec_scanned', $mark_scanned, true);
                $item->save_meta_data();
            }

            if ($mark_scanned) {
                $message = esc_html__('Tickets are marked as scanned', 'stec');
            } else {
                $message = esc_html__('Tickets unmarked', 'stec');
            }

            $r->set_data(array(
                'success'      => 1,
                'mark_scanned' => $mark_scanned,
                'msg'          => $message
            ));
        } catch (\Exception $ex) {

            $r->set_error(1);
            $r->set_error_msg($ex->getMessage());
        }

        $r->get();

        break;

    case 'mark_ticket_scanned' :

        $r = new response_object();

        try {
            $item_id      = Admin_Helper::post('item_id', null, FILTER_VALIDATE_INT);
            $order_id     = Admin_Helper::post('order_id', null, FILTER_VALIDATE_INT);
            $mark_scanned = Admin_Helper::post('mark_scanned', 0, FILTER_VALIDATE_INT);

            if (!$item_id || !$order_id) {
                throw new Stec_Exception(esc_html__('Item or order not found', 'stec'));
            }

            $item = new \WC_Order_Item_Product($item_id);

            $product_id = $item->get_product_id();

            if ('1' !== get_post_meta($product_id, 'stec_bookable', true)) {
                throw new Stec_Exception(esc_html__('Product is not ticket', 'stec'));
            }

            $item->update_meta_data('stec_scanned', $mark_scanned, true);

            $item->save_meta_data();

            if ($mark_scanned) {
                $message = esc_html__('Ticket marked as scanned', 'stec');
            } else {
                $message = esc_html__('Ticket unmarked', 'stec');
            }

            $r->set_data(array(
                'success'      => 1,
                'mark_scanned' => $mark_scanned,
                'msg'          => $message
            ));
        } catch (\Exception $ex) {

            $r->set_error(1);
            $r->set_error_msg($ex->getMessage());
        }

        $r->get();

        break;

    case 'get_search_products' :

        $r        = new response_object();
        $products = array();
        $exclude  = Booking::get_bookable_products_ids();

        try {

            $search     = Admin_Helper::post('search', '', FILTER_SANITIZE_STRING);
            $data_store = \WC_Data_Store::load('product');
            $productIds = $data_store->search_products(wc_clean(wp_unslash($search)), '', false, false, 10, null, $exclude);
            $products   = array();

            foreach ($productIds as $id) {
                if ($id <= 0) {
                    continue;
                }

                $product = WC()->product_factory->get_product($id);

                $products[] = array(
                    'id'    => $id,
                    'title' => $product->get_title(),
                    'image' => $product->get_image('woocommerce_thumbnail'),
                    'price' => wc_price($product->get_price())
                );
            }

            $r->set_data(array(
                'success'  => 1,
                'products' => $products
            ));
        } catch (\Exception $ex) {

            $r->set_error(1);
            $r->set_error_msg($ex->getMessage());
        }

        $r->get();

        break;

endswitch;

exit;
