<?php

namespace Stachethemes\Stec;

if (!$this instanceof Stachethemes_Event_Calendar) {
    wp_die('-1');
}

/**
 * Apply admin settings filters for event single page
 */
add_action('stec_single_apply_admin_filters', function () {
    if (1 === (int) Settings::get_admin_setting_value('stec_menu__general_single_page', 'disable_title')) {
        add_filter('the_title', function() {
            return '';
        }, 9999);
    }

    if (1 === (int) Settings::get_admin_setting_value('stec_menu__general_single_page', 'disable_sidebar')) {
        add_filter('is_active_sidebar', function() {
            return false;
        }, 9999);
    }

    if (1 === (int) Settings::get_admin_setting_value('stec_menu__general_single_page', 'disable_thumbnail')) {
        add_filter('has_post_thumbnail', function() {
            return false;
        }, 9999);
    }

    if (1 === (int) Settings::get_admin_setting_value('stec_menu__general_single_page', 'disable_default_comments')) {
        add_filter('comments_open', function() {
            return false;
        }, 9999);
    }
});

/**
 * Single page css
 */
add_action('stec_single_load_styles', function() {

    if ('1' === Settings::get_admin_setting_value('stec_menu__general_other', 'load_fontawesome')) {
        Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-font-awesome", "css/fonts/css/all.min.css");
    }

    Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-single", "css/front/style.single-page-elements.min.css");
    if ('osm' === Settings::get_admin_setting_value('stec_menu__general', 'map_type')) {
        Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-leaflet", "//unpkg.com/leaflet@1.6.0/dist/leaflet.css");
    }

    Stachethemes_Event_Calendar::get_instance()->add_defer_css_handle(apply_filters('stec_single_defer_css', array(
        'stec-font-awesome',
        'stec-leaflet',
    )));
});

/**
 * Single page js
 */
add_action('stec_single_load_scripts', function() {

    $deps = array('jquery', 'jquery-ui-draggable', 'jquery-effects-core', 'jquery-ui-datepicker');

    foreach ($deps as $dep) {
        wp_enqueue_script($dep);
    }

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jmobile_js') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-jquery-mobile-js", "js/libs/jquery.mobile.min.js", "jquery");
    }

    switch (Settings::get_admin_setting_value('stec_menu__general', 'map_type')) :
        case 'osm':
            Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-leaflet-js", "//unpkg.com/leaflet@1.6.0/dist/leaflet.js");
            break;

        case 'gmap':
            if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_gmaps') === '1') {
                Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-google-maps", "//maps.googleapis.com/maps/api/js?key=" . Settings::get_admin_setting_value('stec_menu__general', 'gmaps_api_key'));
            }
        default:

    endswitch;

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_chart_js') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-chart", "js/libs/chart-2.8.0.min.js");
    }
    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_moment_js') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-moment-tz", "js/libs/moment-timezone.min.js", "moment");
    }
    if (Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-google-captcha-api", "https://www.google.com/recaptcha/api.js?render=explicit async defer");
    }

    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-helper-js", "js/front/helper.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-single-js", "js/front/stec-single.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-db-js", "js/db.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-reminder-js", "js/front/reminder.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-rsvp-js", "js/front/rsvp.min.js");

    if (class_exists('WooCommerce')) {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-booking-js", "js/front/booking.min.js");
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-shop-js", "js/front/shop.min.js");

        // Localize the wc checkout path
        Stachethemes_Event_Calendar::get_instance()->localize('stec-shop-js', 'stec_wc_checkout_params', array(
            'i18n_checkout' => esc_html__('Checkout', 'stec'),
            'checkout_url'  => wc_get_checkout_url(),
        ));
    }

    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-scroll-js", "js/libs/scroll.min.js");

    // Share and embeding
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-sharer-js", "js/front/share.min.js");

    // Add DB nonce
    Stachethemes_Event_Calendar::get_instance()->localize('stec-db-js', 'stecApiNonce', array(
        'nonce' => wp_create_nonce('stecApiVerifyNonce')
    ));

    Stachethemes_Event_Calendar::get_instance()->load_js_locales();

    add_action('wp_head', function() {
        Stachethemes_Event_Calendar::get_instance()->load_head();
    }, 9999);

    add_action('wp_footer', function() {
        echo Html_Templates::get_share_modal_template();
        echo Html_Templates::get_single_page_templates();
    });
});

add_filter('single_template', function($single) {

    global $post;

    // If is not part of plugin single pages return normal
    if (strpos($post->post_type, 'stec_') === false) {
        return $single;
    }

    /**
     * EDIT EVENT TEMPLATE
     */
    if ($post->post_type === 'stec_event' && null !== (get_query_var('stec_edit_event', null))) {

        do_action('stec_single_apply_admin_filters');

        $event = new Event_Post(get_the_ID());

        add_action('wp_enqueue_scripts', function() {
            do_action('stec_submit_form_load_styles');
            do_action('stec_submit_form_load_scripts');
        });

        add_action('wp_head', function() use ($event) {
            do_action('stec_single_edit_wp_head', $event);
        });

        $atts                 = array();
        $atts['id']           = uniqid('stec-submission-form-');
        $atts['project_type'] = 'event_submission_form';

        if (!isset($atts['project_id']) || !$atts['project_id']) {
            $atts['project_id'] = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'submission_form_id');

            if (class_exists('SitePress')) {
                $atts['project_id'] = apply_filters('wpml_object_id', $atts['project_id'], 'post', true);
            }
        }

        $atts['include_media_resize']                  = true;
        $atts['captcha']['enabled']                    = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled');
        $atts['captcha']['site_key']                   = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'site_key');
        $atts['create_event_form_allow_expired']       = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_expired');
        $atts['create_event_form_allow_location_type'] = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_location_type');

        add_filter('stec_event_get_front_data', function($data) use ($event) {
            $data['contact_email'] = $event->get_custom_meta('contact_email');
            $data['review_note']   = $event->get_custom_meta('review_note');
            return $data;
        });

        $atts['edit_event']             = $event->get_front_data();
        $atts['edit_event']['timezone'] = $event->get_timezone('admin');
        $atts['date_format']            = Settings::get_admin_setting_value('stec_menu__general', 'date_format');

        $json = wp_json_encode($atts);

        add_filter('the_content', function() use($event, $atts) {

            $template_id = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'submission_form_id');
            $classes     = array('stec-event-submission-form');

            try {
                $template_content = Builder::get_project_content($template_id);
            } catch (Stec_Exception $ex) {
                if (404 === $ex->getCode()) {
                    $classes[]        = 'stec-event-submission-form-default';
                    $default_path     = Stachethemes_Event_Calendar::get_instance()->get_path('VIEW') . 'front/forms/default-submit-form.php';
                    ob_start();
                    include ($default_path);
                    $template_content = ob_get_clean();
                } else {
                    $template_content = '';
                    Admin_Helper::debug_log($ex->getMessage());
                }
            }

            return sprintf('<div id="%s" class="%s"><form>%s</form></div>', $atts['id'], implode(' ', $classes), apply_filters('stec_submit_form_content', $template_content, $atts, $event));
        });

        add_action('wp_footer', function() use ($event, $json) {
            ?>
            <script type="text/javascript">
                if (typeof window.stecSubmitFormInstance === "undefined") {
                    window.stecSubmitFormInstance = [];
                }
                window.stecSubmitFormInstance.push(<?php echo Admin_Helper::esc($json, 'json'); ?>);
            </script>
            <?php
            do_action('stec_single_edit_wp_footer', $event);
        });

        return apply_filters('stec_single_page_template', $single, 'edit');
    }

    // Check if single stec file exists in the theme and return it
    if (strpos($single, 'stec_') !== false) {
        if (file_exists($single)) {
            return $single;
        }
    }

    switch ($post->post_type) {

        /**
         * SINGLE PAGE TEMPLATE
         */
        case 'stec_event' :

            do_action('stec_single_apply_admin_filters');

            $event         = new Event_Post(get_the_ID());
            $repeat_offset = (int) get_query_var('stec_repeat_offset', 0);

            if (!Events::user_can_view_event($event) || false === $event->is_valid_offset($repeat_offset)) {
                global $wp_query;
                $wp_query->set_404();
                status_header(404);
                get_template_part(404);
                exit();
            }

            add_action('wp_head', function() use ($event, $repeat_offset) {
                do_action('stec_single_wp_head', $event);

                echo Admin_Helper::get_event_og($event, $repeat_offset);
            });

            add_action('wp_enqueue_scripts', function() {
                do_action('stec_single_load_styles');
                do_action('stec_single_load_scripts');
            });

            add_filter('the_content', function($default) use ($event, $repeat_offset) {

                if (is_single() && in_the_loop() && is_main_query()) :

                    global $stec_sp_event;
                    global $stec_sp_repeat_offset;

                    $stec_sp_event         = $event;
                    $stec_sp_repeat_offset = $repeat_offset;
                    $template_id           = Settings::get_admin_setting_value('stec_menu__general_single_page', 'single_page_template');

                    ob_start();

                    try {
                        printf('<div class="stec-layout-single">%s</div>', Builder::get_project_content($template_id));
                    } catch (Stec_Exception $ex) {
                        if (404 === $ex->getCode()) {
                            $default_path = Stachethemes_Event_Calendar::get_instance()->get_path('VIEW') . 'front/pages/single.php';
                            include ($default_path);
                        } else {
                            Admin_Helper::debug_log($ex->getMessage());
                        }
                    }

                    $template_content = ob_get_clean();

                    return apply_filters('stec_single_page_content', $template_content, $event, $repeat_offset);

                endif;

                return $default;
            });

            add_action('wp_footer', function() use ($event, $repeat_offset) {
                $general_settings            = Settings::get_admin_settings_kv('stec_menu__general');
                $general_settings['captcha'] = Settings::get_admin_settings_kv('stec_menu__general_google_captcha');
                if (is_user_logged_in()) {
                    $general_settings['userid']    = get_current_user_id();
                    $userdata                      = get_userdata($general_settings['userid']);
                    $general_settings['useremail'] = $userdata->user_email;
                    $general_settings['username']  = $userdata->display_name;
                }
                $general_settings['site_url']                  = get_site_url();
                $general_settings['repeat_offset']             = $repeat_offset;
                $general_settings['display_privacy_agreement'] = Settings::get_admin_setting_value('stec_menu__general_privacy_agreement', 'display_privacy_agreement');
                $general_settings['privacy_agreement_content'] = Settings::get_admin_setting_value('stec_menu__general_privacy_agreement', 'privacy_agreement_content');

                if (class_exists('WooCommerce')) {
                    $general_settings['woocommerce'] = array(
                        'price_format'        => get_woocommerce_price_format(),
                        'symbol'              => get_woocommerce_currency_symbol(),
                        'decimals'            => wc_get_price_decimals(),
                        'decimals_separator'  => wc_get_price_decimal_separator(),
                        'thousands_separator' => wc_get_price_thousand_separator(),
                    );
                }
                ?>
                <script type="text/javascript">
                    var stecSingleOptions = <?php echo wp_json_encode($general_settings); ?>;
                    var stecSingleEvent = <?php echo wp_json_encode($event->get_front_data()); ?>;
                </script>
                <?php
                Admin_Helper::get_event_schema($event, $repeat_offset);

                do_action('stec_single_wp_footer', $event);
            });

            return apply_filters('stec_single_page_template', $single, 'view');

        default: return $single;
    }
});


/**
 * Applies the post template (defined from "Post Attributes")
 */
add_filter('stec_single_page_template', function($single, $mode) {

    if ($mode === 'edit') {
        $template_id = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'submission_form_id');
    } else {
        $template_id = Settings::get_admin_setting_value('stec_menu__general_single_page', 'single_page_template');
    }

    if ($template_id) {
        $single_page_slug = get_page_template_slug($template_id);

        if ($single_page_slug) {
            $single_template_path = locate_template($single_page_slug);

            if ($single_template_path) {
                $single = $single_template_path;
            }
        }
    }

    return $single;
}, 9999, 2);
