<?php

namespace Stachethemes\Stec;

class Event_Meta_Product implements Event_Meta_Object {

    private $id;

    public function get_data() {
        return array(
            'id' => $this->get_id()
        );
    }

    public function get_id() {
        return filter_var($this->id, FILTER_VALIDATE_INT);
    }

    public function set_id($id) {
        $this->id = (int) $id;
    }

    private function get_variation_options(\WC_Product_Variable $wc_product) {

        $vars    = $wc_product->get_attributes();
        $options = array();

        foreach ($vars as $var) {
            $var instanceof \WC_Product_Attribute;

            if (true === $var->get_visible() && true === $var->get_variation()) {
                $options[] = array(
                    'name'    => $var->get_name(),
                    'slug'    => sanitize_title($var->get_name()),
                    'options' => $var->get_options()
                );
            }
        }


        return $options;
    }

    private function get_product_variations(\WC_Product_Variable $wc_product) {

        $variables = array();
        $vars      = $wc_product->get_available_variations('objects');

        foreach ($vars as $var) {

            $var instanceof \WC_Product_Variation;
            $variables[$var->get_id()] = array();
            $v                         = & $variables[$var->get_id()];

            $v['id']             = $var->get_id();
            $v['image']          = array();
            $v['stock_quantity'] = $var->get_stock_quantity();
            $v['is_in_stock']    = $var->is_in_stock();
            $v['is_purchasable'] = $var->is_purchasable();
            $v['attributes']     = $var->get_attributes();
            $v['price']          = $var->get_price();
            $v['html_price']     = $var->get_price_html();

            $image_id = $var->get_image_id();

            if ($image_id) {
                $thumb      = wp_get_attachment_image_src($image_id, 'woocommerce_thumbnail');
                $full       = wp_get_attachment_image_src($image_id, 'large');
                $v['image'] = array(
                    'id'    => $image_id,
                    'thumb' => $thumb[0],
                    'full'  => $full[0]
                );
            }
            $variables[] = $v;
        }
        unset($v);

        return $variables;
    }

    public function get_parsed_data() {

        $wc_product = WC()->product_factory->get_product($this->get_id());
        $data       = array();
        
        if (!$wc_product) {
            return $data;
        }

        switch ($wc_product->get_type()) {
            case 'variable' :

                $wc_product instanceof \WC_Product_Variable;

                $data['id']                = $wc_product->get_id();
                $data['sku']               = $wc_product->get_sku();
                $data['title']             = $wc_product->get_title();
                $data['short_desc']        = $wc_product->get_short_description();
                $data['stock_quantity']    = $wc_product->get_stock_quantity();
                $data['is_featured']       = $wc_product->is_featured();
                $data['is_on_sale']        = $wc_product->is_on_sale();
                $data['is_in_stock']       = $wc_product->is_in_stock();
                $data['is_purchasable']    = $wc_product->is_purchasable();
                $data['type']              = $wc_product->get_type();
                $data['price']             = $wc_product->get_price();
                $data['html_price']        = $wc_product->get_price_html();
                $data['variation_options'] = $this->get_variation_options($wc_product);
                $data['variations']        = $this->get_product_variations($wc_product);
                $data['permalink']         = $wc_product->get_permalink();
                $data['image']             = array();

                $image_id = $wc_product->get_image_id();

                if ($image_id) {
                    $thumb         = wp_get_attachment_image_src($image_id, 'woocommerce_thumbnail');
                    $full          = wp_get_attachment_image_src($image_id, 'large');
                    $data['image'] = array(
                        'id'    => $image_id,
                        'thumb' => $thumb[0],
                        'full'  => $full[0]
                    );
                } else {
                    $data['image'] = array(
                        'id'    => 0,
                        'thumb' => wc_placeholder_img_src('woocommerce_thumbnail'),
                        'full'  => wc_placeholder_img_src('large')
                    );
                }

                $data['gallery'] = array();

                $gallery = $wc_product->get_gallery_image_ids();

                if ($gallery) {
                    foreach ($gallery as $img_id) {

                        $thumb = wp_get_attachment_image_src($img_id, 'woocommerce_thumbnail');
                        $full  = wp_get_attachment_image_src($img_id, 'large');

                        $data['gallery'][] = array(
                            'id'    => $img_id,
                            'thumb' => $thumb[0],
                            'full'  => $full[0]
                        );
                    }
                }

                return $data;

            default:

                $data['id']             = $wc_product->get_id();
                $data['sku']            = $wc_product->get_sku();
                $data['title']          = $wc_product->get_title();
                $data['short_desc']     = $wc_product->get_short_description();
                $data['stock_quantity'] = $wc_product->get_stock_quantity();
                $data['is_featured']    = $wc_product->is_featured();
                $data['is_on_sale']     = $wc_product->is_on_sale();
                $data['is_in_stock']    = $wc_product->is_in_stock();
                $data['is_purchasable'] = $wc_product->is_purchasable();
                $data['type']           = $wc_product->get_type();
                $data['price']          = $wc_product->get_price();
                $data['html_price']     = $wc_product->get_price_html();
                $data['permalink']      = $wc_product->get_permalink();
                $data['image']          = array();

                $image_id = $wc_product->get_image_id();

                if ($image_id) {
                    $thumb = wp_get_attachment_image_src($image_id, 'woocommerce_thumbnail');
                    $full  = wp_get_attachment_image_src($image_id, 'large');

                    $data['image'] = array(
                        'id'    => $image_id,
                        'thumb' => $thumb[0],
                        'full'  => $full[0]
                    );
                } else {

                    $data['image'] = array(
                        'id'    => 0,
                        'thumb' => wc_placeholder_img_src('woocommerce_thumbnail'),
                        'full'  => wc_placeholder_img_src('large')
                    );
                }

                return $data;
        }
    }

}
