<?php

namespace Stachethemes\Stec;

add_shortcode('stachethemes_ec', function($shortcode_atts = array()) {

    ob_start();

    if (Admin_Helper::is_placeholder()) {
        include (Stachethemes_Event_Calendar::get_instance()->get_path('VIEW') . 'front/placeholder/stachethemes_ec.php');
        return ob_get_clean();
    }

    if (true === Admin_Helper::check_migration_required()) {
        printf(esc_html__('<p>Stachethemes Event Calendar requires database update. To update go to %s</p>', 'stec'), '<a href="' . esc_url(admin_url('admin.php?page=stec_menu__migrate')) . '">Dashboard -> Stachethemes Event Calendar -> Migrate</a>');
        return;
    }

    $atts     = apply_filters('stachethemes_ec_atts', $shortcode_atts);
    $calendar = new Calendar_Instance($atts);

    // Before html hook
    do_action('stec_before_html');

    if (Stachethemes_Event_Calendar::get_instance()->lcns() !== false) {
        include (Stachethemes_Event_Calendar::get_instance()->get_path('VIEW') . 'front/default.php');
    } else {
        Stachethemes_Event_Calendar::get_instance()->display_activation_required();
    }

    // After html hook
    do_action('stec_after_html');

    // If ajax events is disabled add events to js var
    if (Settings::get_admin_setting_value('stec_menu__general_other', 'ajax_events') === '0') {
        add_action('wp_footer', function() use ($calendar, $atts) {

            $events             = array();
            $cat                = isset($atts['cat']) ? $atts['cat'] : null;
            $organizer_id       = isset($atts['organizer_id']) ? $atts['organizer_id'] : null;
            $cal                = isset($atts['cal']) ? $atts['cal'] : null;
            $min_date           = isset($atts['min_date']) ? $atts['min_date'] : null;
            $max_date           = isset($atts['max_date']) ? $atts['max_date'] : null;
            $events_per_request = Settings::get_admin_setting_value('stec_menu__general_other', 'events_per_request');

            if ($cal) {
                $cal = explode(',', $cal);
            }

            $meta_query = array();
            $tax_query  = array();

            if ($min_date || $max_date) {

                $meta_query[] = array(
                    'relation' => 'OR',
                    array(
                        'key'     => 'start_date',
                        'value'   => array(($min_date ? $min_date : '1970-01-01'), ($max_date ? $max_date : '2038-01-19')),
                        'compare' => 'BETWEEN',
                        'type'    => 'DATE'
                    ),
                    array(
                        'key'     => 'rrule',
                        'value'   => '',
                        'compare' => '!=',
                    )
                );
            }

            if ($cat) {
                $cat         = explode(',', $cat);
                $tax_query[] = array(
                    'taxonomy' => 'stec_event_category',
                    'field'    => 'term_id',
                    'terms'    => $cat,
                );
            }

            if ($organizer_id) {
                $organizer_id_array = explode(',', $organizer_id);
                $tax_query[]        = array(
                    'taxonomy' => 'stec_event_organizer',
                    'field'    => 'term_id',
                    'terms'    => $organizer_id_array,
                );
            }

            $params = array(
                'calid'      => $cal,
                'meta_query' => $meta_query,
                'tax_query'  => $tax_query,
                'page'       => 0,
                'per_page'   => $events_per_request
            );

            while ($events_p = Events::get_front_events($params, true)) {
                $params['page']++;
                $events = array_merge($events, $events_p);
            }

            if ($events) {
                foreach ($events as $key => $event) {
                    $events[$key] = $event->get_front_data();
                }
            }
            ?>
            <script type="text/javascript">
                if (typeof stecJsonEvents === 'undefined') {
                    var stecJsonEvents = [];
                }
                stecJsonEvents['<?php esc_attr_e($calendar); ?>'] = <?php echo wp_json_encode($events); ?>;
            </script>
            <?php
        }, 9999);
    }


    return ob_get_clean();
});

add_action('wp_enqueue_scripts', function() {

    if (Admin_Helper::is_placeholder()) {
        Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-shortcode-placeholder", "css/front/placeholder.min.css");
        return;
    }

    if (Stachethemes_Event_Calendar::get_instance()->scripts_are_forced() === true || Admin_Helper::post_has_shortcode('stachethemes_ec')) {

        do_action('stec_load_calendar_styles');
        do_action('stec_load_calendar_scripts');

        add_action('wp_head', function() {
            Stachethemes_Event_Calendar::get_instance()->load_head();
        }, 9999);
    }
});

add_action('stec_load_calendar_styles', function() {

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jquery_ui_css') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-ui", "https://ajax.googleapis.com/ajax/libs/jqueryui/1.11.4/themes/smoothness/jquery-ui.css");
    }

    Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-css", "css/front/style.min.css");
    Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-colorpicker", "css/colorpicker/css/colorpicker.min.css");
    Stachethemes_Event_Calendar::get_instance()->add_asset_css('stec-submit-event-elements', 'css/front/style.submit-form-elements.min.css');

    if ('1' === Settings::get_admin_setting_value('stec_menu__general_other', 'load_fontawesome')) {
        Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-font-awesome", "css/fonts/css/all.min.css");
    }

    if ('osm' === Settings::get_admin_setting_value('stec_menu__general', 'map_type') && '1' === Settings::get_admin_setting_value('stec_menu__general_other', 'load_leaflet')) {
        Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-leaflet", "//unpkg.com/leaflet@1.6.0/dist/leaflet.css");
    }

    // Defer styles
    Stachethemes_Event_Calendar::get_instance()->add_defer_css_handle(apply_filters('stachethemes_ec_defer_css', array(
        'stec-css',
        'stec-ui',
        'stec-colorpicker',
        'stec-submit-event-elements',
        'stec-font-awesome',
        'stec-leaflet'
    )));
});

add_action('stec_load_calendar_scripts', function() {

    $deps = array('jquery', 'jquery-ui-draggable', 'jquery-effects-core', 'jquery-touch-punch', 'jquery-ui-datepicker');

    foreach ($deps as $dep) {
        wp_enqueue_script($dep);
    }

    if (Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-google-captcha-api", "//www.google.com/recaptcha/api.js?render=explicit async defer");
    }

    switch (Settings::get_admin_setting_value('stec_menu__general', 'map_type')) :
        case 'osm':
            if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_leaflet') === '1') {
                Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-leaflet-js", "//unpkg.com/leaflet@1.6.0/dist/leaflet.js");
            }

            break;

        case 'gmap':
            if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_gmaps') === '1') {
                Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-google-maps", "//maps.googleapis.com/maps/api/js?key=" . Settings::get_admin_setting_value('stec_menu__general', 'gmaps_api_key'));
            }
        default:

    endswitch;

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jmobile_js') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-jquery-mobile-js", "js/libs/jquery.mobile.min.js", "jquery");
    }

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_chart_js') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-chart", "js/libs/chart-2.8.0.min.js", "jquery");
    }

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_moment_js') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-moment-tz-js", "js/libs/moment-timezone.min.js", "moment");
    }

    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-rrule-js", "js/libs/rrule.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-db-js", "js/db.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-helper-js", "js/front/helper.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-reminder-js", "js/front/reminder.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-rsvp-js", "js/front/rsvp.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-scroll-js", "js/libs/scroll.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-js", "js/front/stec.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-adds-js", "js/front/stec-adds.min.js");

    if (class_exists('WooCommerce')) {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-booking-js", "js/front/booking.min.js");
        Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-shop-js", "js/front/shop.min.js");

        // Localize the wc checkout path
        Stachethemes_Event_Calendar::get_instance()->localize('stec-shop-js', 'stec_wc_checkout_params', array(
            'i18n_checkout' => esc_html__('Checkout', 'stec'),
            'checkout_url'  => wc_get_checkout_url(),
        ));
    }

    // Front Event Creation
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-colorpicker-js", "js/libs/colorpicker/colorpicker.min.js");
    Stachethemes_Event_Calendar::get_instance()->add_asset_js('stec-submit-form-js', 'js/front/submit-form.min.js');

    // Share and embeding
    Stachethemes_Event_Calendar::get_instance()->add_asset_js("stec-sharer-js", "js/front/share.min.js");

    // Add front-end wp tinymce
    if (Settings::get_admin_setting_value('stec_menu__general_other', 'wp_tinymce') === '1') {
        wp_enqueue_editor();
    }

    // Add front-end wp media
    if (Settings::get_admin_setting_value('stec_menu__general_other', 'wp_media') === '1') {
        wp_enqueue_media();
    }

    // Defer scripts
    Stachethemes_Event_Calendar::get_instance()->add_defer_js_handle(apply_filters('stachethemes_ec_defer_js', array(
        'stec-google-captcha-api',
        'stec-booking-js',
        'stec-shop-js',
        'stec-rrule-js',
        'stec-js',
        'stec-helper-js',
        'stec-db-js',
        'stec-reminder-js',
        'stec-sharer-js',
        'stec-rsvp-js',
        'stec-scroll-js',
        'stec-adds-js',
        'stec-moment-tz-js',
        'stec-chart',
        'stec-jquery-mobile-js',
        'stec-leaflet-js',
        'stec-google-maps',
        'stec-submit-form-js',
        'stec-colorpicker-js',
    )));

    // Add DB nonce
    Stachethemes_Event_Calendar::get_instance()->localize('stec-db-js', 'stecApiNonce', array(
        'nonce' => wp_create_nonce('stecApiVerifyNonce')
    ));

    // Localize the plugin path (for workers)
    Stachethemes_Event_Calendar::get_instance()->localize('stec-adds-js', 'stecPluginPath', array(
        'url' => plugin_dir_url(STACHETHEMES_EC_FILE__)
    ));

    Stachethemes_Event_Calendar::get_instance()->load_js_locales();

    add_action('wp_footer', function() {
        echo Html_Templates::get_calendar_templates();
        echo Html_Templates::get_share_modal_template();
        echo Html_Templates::get_submit_form_templates();
        
    });
});
