<?php

namespace Stachethemes\Stec;

add_shortcode('stec_submit_form', function($atts = array()) {

    if (!$atts) {
        $atts = array();
    }

    if (!isset($atts['id'])) {
        $atts['id'] = uniqid('stec-event-submission-form-');
    }

    $atts['include_media_resize']                  = true;
    $atts['captcha']['enabled']                    = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled');
    $atts['captcha']['site_key']                   = Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'site_key');
    $atts['create_event_form_allow_expired']       = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_expired');
    $atts['create_event_form_allow_location_type'] = Settings::get_admin_setting_value('stec_menu__general_event_submit', 'create_event_form_allow_location_type');
    $atts['date_format']                           = Settings::get_admin_setting_value('stec_menu__general', 'date_format');
    if (is_user_logged_in()) {
        $userdata          = get_userdata(get_current_user_id());
        $atts['useremail'] = $userdata->user_email;
    }

    $classes     = array('stec-event-submission-form');
    $template_id = isset($atts['template_id']) ? (int) $atts['template_id'] : Settings::get_admin_setting_value('stec_menu__general_event_submit', 'submission_form_id');
    if (class_exists('SitePress')) {
        $template_id = apply_filters('wpml_object_id', $template_id, 'post', true);
    }
    $options = wp_json_encode($atts);

    add_action('wp_footer', function() use ($options) {
        ?>
        <script type="text/javascript">
            if (typeof stecSubmitFormInstance === "undefined") {
                var stecSubmitFormInstance = [];
            }
            stecSubmitFormInstance.push(<?php echo Admin_Helper::esc($options, 'json'); ?>);
        </script>
        <?php
    });

    /**
     * @todo Don't use try catch for logic
     */
    try {
        $template_content = Builder::get_project_content($template_id);
    } catch (Stec_Exception $ex) {
        if (404 === $ex->getCode()) {
            $classes[]    = 'stec-event-submission-form-default';
            $default_path = Stachethemes_Event_Calendar::get_instance()->get_path('VIEW') . 'front/forms/default-submit-form.php';
            ob_start();

            if (locate_template('stec-submit-form.php')) {
                locate_template('stec-submit-form.php', true, false, array(
                    'calendar' => $atts
                ));
            } else {
                include ($default_path);
            }

            $template_content = ob_get_clean();
        } else {
            $template_content = '';
            Admin_Helper::debug_log($ex->getMessage());
        }
    }


    return sprintf('<div id="%s" class="%s"><form>%s</form></div>', $atts['id'], implode(' ', $classes), apply_filters('stec_submit_form_content', $template_content, $atts, false));
});

add_action('wp_enqueue_scripts', function() {
    if (Admin_Helper::post_has_shortcode('stec_submit_form')) {

        do_action('stec_submit_form_load_styles');
        do_action('stec_submit_form_load_scripts');
    }
});

add_action('stec_submit_form_load_styles', function() {

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jquery_ui_css') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_css('stec-ui', '//ajax.googleapis.com/ajax/libs/jqueryui/1.11.4/themes/smoothness/jquery-ui.css');
    }

    Stachethemes_Event_Calendar::get_instance()->add_asset_css('stec-submit-event-elements', 'css/front/style.submit-form-elements.min.css');
    Stachethemes_Event_Calendar::get_instance()->add_asset_css('stec-colorpicker', 'css/colorpicker/css/colorpicker.min.css');

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jquery_ui_css') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_css('stec-ui', '//ajax.googleapis.com/ajax/libs/jqueryui/1.11.4/themes/smoothness/jquery-ui.css');
    }

    if ('1' === Settings::get_admin_setting_value('stec_menu__general_other', 'load_fontawesome')) {
        Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-font-awesome", "css/fonts/css/all.min.css");
    }
});

add_action('stec_submit_form_load_scripts', function() {

    $deps = array('jquery', 'jquery-ui-datepicker');

    foreach ($deps as $dep) {
        wp_enqueue_script($dep);
    }

    // Add front-end wp tinymce
    if (Settings::get_admin_setting_value('stec_menu__general_other', 'wp_tinymce') === '1') {
        wp_enqueue_editor();
    }

    // Add front-end wp media
    if (Settings::get_admin_setting_value('stec_menu__general_other', 'wp_media') === '1') {
        wp_enqueue_media();
    }

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_moment_js') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js('stec-moment-js', 'js/libs/moment-timezone.min.js', 'moment');
    }

    if (Settings::get_admin_setting_value('stec_menu__general_other', 'load_jmobile_js') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js('stec-jquery-mobile-js', 'js/libs/jquery.mobile.min.js');
    }

    Stachethemes_Event_Calendar::get_instance()->add_asset_js('stec-colorpicker-js', 'js/libs/colorpicker/colorpicker.min.js');
    Stachethemes_Event_Calendar::get_instance()->add_asset_js('stec-helper-js', 'js/front/helper.min.js');
    Stachethemes_Event_Calendar::get_instance()->add_asset_js('stec-rrule-js', 'js/libs/rrule.min.js');
    Stachethemes_Event_Calendar::get_instance()->add_asset_js('stec-db-js', 'js/db.min.js');
    Stachethemes_Event_Calendar::get_instance()->add_asset_js('stec-submit-form-js', 'js/front/submit-form.min.js');

    if (Settings::get_admin_setting_value('stec_menu__general_google_captcha', 'enabled') === '1') {
        Stachethemes_Event_Calendar::get_instance()->add_asset_js('stec-google-captcha-api', 'https://www.google.com/recaptcha/api.js?render=explicit async defer');
    }

    Stachethemes_Event_Calendar::get_instance()->localize('stec-db-js', 'stecApiNonce', array(
        'nonce' => wp_create_nonce('stecApiVerifyNonce')
    ));

    Stachethemes_Event_Calendar::get_instance()->load_js_locales();

    add_action('wp_head', function() {
        Stachethemes_Event_Calendar::get_instance()->load_head();
    }, 9999);

    add_action('wp_footer', function() {
        echo Html_Templates::get_submit_form_templates();
    });
});
