<?php

namespace Stachethemes\Stec;

if (false === $order instanceof \WC_Order) {
    wp_die('-1');
}

get_header();
?>

<div class="stec-order-details">
    <div class="stec-order-details-wrap">

        <?php do_action('stec_order_details_before_account_details', $order); ?>

        <p class="stec-order-details-header"><?php esc_html_e('Account Details', 'stec') ?></p>

        <div class="stec-order-details-separator"></div>

        <div class="stec-order-details-billing">
            <?php
            $order instanceof \WC_Order;
            printf('<p class="stec-order-details-billing-name">%s %s</p>', $order->get_billing_first_name(), $order->get_billing_last_name());
            printf('<a class="stec-order-details-billing-email" href="mailto:%s">%s</a>', $order->get_billing_email(), $order->get_billing_email());
            printf('<p class="stec-order-details-billing-date">%s <strong>%s</strong></p>',
                    esc_html__('Order submitted on:', 'stec'),
                    $order->get_date_created()->format(sprintf('%s %s', get_option('date_format'), get_option('time_format'))));
            ?>
        </div>

        <div class="stec-order-details-separator"></div>

        <div class="stec-order-details-flex">
            <p class="stec-order-details-header"><?php esc_html_e('Order Details', 'stec') ?></p>
            <?php
            $status = $order->get_status();

            printf('<p class="stec-order-details-status %s">%s</p>', esc_attr($status), strtoupper($status));
            ?>
        </div>
        <div class = "stec-order-details-separator"></div>

        <?php
        $items      = apply_filters('stec_order_details_items', $order->get_items(), $order);
        $all_marked = true;

        echo '<ul class="stec-order-details-items">';
        foreach ($items as $item) {

            echo '<li class="stec-order-details-item" data-id="' . $item->get_id() . '">';

            $item_data = $item->get_data();

            $product_id = filter_var($item_data['product_id'], FILTER_VALIDATE_INT);

            if (!$product_id) {
                continue;
            }

            $product = wc_get_product($product_id);

            $event_title     = $item->get_meta('event_title', true);
            $event_start     = $item->get_meta('event_start', true);
            $event_end       = $item->get_meta('event_end', true);
            $event_permalink = $item->get_meta('event_permalink', true);

            printf('<p class="stec-order-details-item-title">%s x %s<span>%s</span></p>', $item->get_name(), $item->get_quantity(), wc_price($order->get_item_total($item)));

            if ($event_title) {
                printf('<p class="stec-order-details-item-event-title"><a href="%s">%s</a></p>', $event_permalink, $event_title);
            }

            if ($event_start) {
                printf('<p class="stec-order-details-item-event-date">%s - %s</p>', $event_start, $event_end);
            }

            do_action('stec_order_details_after_item_details', $item, $order);

            if ('completed' === $order->get_status() && current_user_can('manage_woocommerce') && '1' === get_post_meta($item->get_product_id(), 'stec_bookable', true)) {

                $is_marked = $item->get_meta('stec_scanned', true);

                if (!$is_marked) {
                    $all_marked = false;
                }

                printf('<button class="stec-order-details-item-mark stec-style-button-secondary %s" data-mark-scanned=1 data-action="mark-item" data-id="%s">%s</button>', $is_marked ? '' : 'active', $item_data['id'] . '-' . $item_data['order_id'], esc_html__('Mark as scanned', 'stec'));
                printf('<button class="stec-order-details-item-mark stec-style-button-secondary %s" data-mark-scanned=0 data-action="mark-item" data-id="%s">%s</button>', $is_marked ? 'active' : '', $item_data['id'] . '-' . $item_data['order_id'], esc_html__('Scanned', 'stec'));
            }

            echo '</li>';
        }

        echo '</ul>';

        do_action('stec_order_details_after_items', $order);
        ?>
    </div>

    <?php
    printf('<p class="stec-order-details-order-total">' . esc_html__('Total:', 'stec') . '<span>%s</span></p>', wc_price($order->get_total()));

    do_action('stec_order_details_after_order_total', $order);

    if (current_user_can('manage_woocommerce')) {
        echo '<div class="stec-style-full-width">';
        
        $admin_url = admin_url(sprintf('post.php?post=%d&action=edit', $order->get_id()));
        printf('<a class="stec-order-details-admin-url" href="%s">%s</a>', esc_url($admin_url), esc_html__('View Order in Dashboard', 'stec'));

        if ('completed' === $order->get_status() && Booking::order_has_tickets($order)) {
            printf('<button class="stec-order-details-mark-all stec-style-button %s" data-id="%d" data-mark-scanned="1" data-action="mark-order">%s</button>', $all_marked ? '' : 'active', $order->get_id(), esc_html__('Mark all as scanned', 'stec'));
            printf('<button class="stec-order-details-mark-all stec-style-button %s" data-id="%d" data-mark-scanned="0" data-action="mark-order">%s</button>', $all_marked ? 'active' : '', $order->get_id(), esc_html__('UnMark all', 'stec'));
        }

        echo '</div>';
    }
    ?>
</div>

<?php
get_footer();
