<?php

namespace Stachethemes\Stec;

do_action('stec_organizer_page_before_content');

if ($organizer && $organizer instanceof Organizer) :
    ?>

    <div class='stec-page-organizer'>

        <div class='stec-page-organizer-person' itemscope itemtype="http://schema.org/Person">

            <?php
            if ($organizer->get_photo()) {
                ?>
                <div class='stec-page-organizer-person-photo'>
                    <div style="background-image:url('<?php echo esc_url($organizer->get_photo_src('medium')); ?>');"></div>
                </div>
                <?php
            }
            ?>

            <h1 class='stec-page-organizer-person-name stec-layout-event-title-fontandcolor'><?php esc_html_e($organizer->get_name()); ?></h1>
            <div class='stec-page-organizer-person-about stec-style-text'><?php esc_html_e($organizer->get_details()); ?></div>
            <div class='stec-page-organizer-person-social'><?php
                $socials = $organizer->get_social();
                if ($socials) {
                    foreach ($socials as $social) {
                        if (!trim($social['link'])) {
                            continue;
                        }
                        
                        if (false !== filter_var($social['link'], FILTER_VALIDATE_EMAIL)) {
                            echo "<a href='mailto:{$social['link']}'><i class='{$social['ico']}'></i></a>";
                        } 
                        
                        elseif ('fas fa-phone' === $social['ico']) {
                            echo "<a href='tel:{$social['link']}'><i class='{$social['ico']}'></i></a>";
                        } 
                        
                        else {
                            echo "<a target='_blank' href='{$social['link']}'><i class='{$social['ico']}'></i></a>";
                        }
                    }
                }
                ?></div>

            <?php
            printf('<meta itemprop="name" content="%s" />', $organizer->get_name());
            printf('<meta itemprop="email" content="%s" />', $organizer->get_email());
            printf('<meta itemprop="disambiguatingDescription" content="%s" />', $organizer->get_details());
            printf('<meta itemprop="potentialAction" content="%s" />', esc_html__('Event Organizer', 'stec'));
            printf('<meta itemprop="url" content="%s" />', $organizer->get_permalink());
            ?>

        </div>

        <div class='stec-page-organizer-events'>
            <?php
            $shortcode = apply_filters('stec_organizer_page_shortcode',
                    sprintf('[stachethemes_ec views="agenda" view="agenda" agenda_cal_display=0 show_top=0 show_create_event_form="0" organizer_id="%s"]', $organizer->get_id()), $organizer);

            echo do_shortcode($shortcode);
            ?>
        </div>
    </div>

    <?php
elseif (isset($organizers) && is_array($organizers) && $organizers) :
    ?>
    <ul class="stec-page-organizer-organizers">
        <?php
        foreach ($organizers as $organizer) :
            if ($organizer instanceof Organizer) :
                ?>
                <li class="stec-page-organizer-organizers-organizer" itemscope itemtype="http://schema.org/Person">
                    <div class="stec-page-organizer-organizers-organizer-image">
                        <?php if ($organizer->get_photo()) : ?>
                            <div class="stec-page-organizer-organizers-organizer-image-photo" style="background-image:url(<?php echo esc_url($organizer->get_photo_src('medium')); ?>)"></div>
                        <?php else: ?> 
                            <div class="stec-page-organizer-organizers-organizer-image-default"></div>
                        <?php endif; ?>
                    </div>
                    <div class="stec-page-organizer-organizers-organizer-content">
                        <div class="stec-page-organizer-organizers-organizer-title stec-style-title2"><?php
                            if ($organizer->get_permalink()) {
                                echo "<a href='" . esc_url($organizer->get_permalink()) . "'>" . esc_html__($organizer->get_name()) . "</a>";
                            } else {
                                esc_html_e($organizer->get_name());
                            }
                            ?></div>
                        <div class="stec-page-organizer-organizers-organizer-description stec-style-text"><?php esc_html_e($organizer->get_details()); ?></div>
                        <div class="stec-page-organizer-organizers-organizer-social">
                            <?php
                            if ($organizer->get_social()) :
                                foreach ($organizer->get_social() as $social) {
                                    if (false !== filter_var($social['link'], FILTER_VALIDATE_EMAIL)) {
                                        printf("<a href='mailto:%s'><i class='%s'></i></a>", filter_var($social['link'], FILTER_VALIDATE_EMAIL), esc_attr($social['ico']));
                                    } 
                                    elseif ('fas fa-phone' === $social['ico']) {
                                        echo "<a href='tel:{$social['link']}'><i class='{$social['ico']}'></i></a>";
                                    } else {
                                        printf("<a target='_blank' href='%s'><i class='%s'></i></a>", esc_url($social['link']), esc_attr($social['ico']));
                                    }
                                }
                            endif;
                            ?>
                        </div>
                    </div>

                    <?php
                    printf('<meta itemprop="name" content="%s" />', $organizer->get_name());
                    printf('<meta itemprop="email" content="%s" />', $organizer->get_email());
                    printf('<meta itemprop="disambiguatingDescription" content="%s" />', $organizer->get_details());
                    printf('<meta itemprop="potentialAction" content="%s" />', esc_html__('Event Organizer', 'stec'));
                    printf('<meta itemprop="url" content="%s" />', $organizer->get_permalink());
                    ?>
                </li>
                <?php
            endif;
        endforeach;
        ?>
    </ul>
    <?php
else :
    do_action('stec_organizer_page_is_empty');
endif;

do_action('stec_organizer_page_after_content');
