<?php

namespace Stachethemes\Stec;

$edit_link       = $event->get_edit_link();
$the_permalink   = $event->get_permalink($repeat_offset);
$is_cancelled    = 'EventCancelled' === $event->get_event_status();
$rsvp_expired    = $is_cancelled || $event->get_start_date('U', $repeat_offset) < gmdate('U');
$booking_expired = $rsvp_expired;
$has_tickets     = class_exists('WooCommerce') && $event->get_bookable_products() && false === $booking_expired;
$can_rsvp        = false === $rsvp_expired && 1 === (int) $event->get_rsvp() && false === Admin_Helper::user_is_invited($event);

add_action('stec_single_show_related', function() use ($event, $repeat_offset) {
    include(Stachethemes_Event_Calendar::get_instance()->get_path('VIEW') . "/front/pages/single.related.php");
});

do_action('stec_single_before_content');
?>

<div class="stec-layout-single stec-layout-single-default <?php echo $is_cancelled ? 'cancelled' : '' ?>">

    <div class="stec-layout-single-preview">

        <div class="stec-layout-single-preview-left">

            <div class="stec-layout-single-preview-left-icon <?php
            if ($event->get_icon_type() === 'icon' && $event->get_icon() === 'fa') {
                echo 'stec-no-icon';
            }
            ?>" style="background:<?php echo $event->get_color('display'); ?>">

                <?php
                switch ($event->get_icon_type()) :

                    case 'date' :

                        $d   = $event->get_start_date('j', $repeat_offset);
                        $mmm = date_i18n('M', $event->get_start_date('U', $repeat_offset));
                        ?>
                        <span class="stec-layout-single-preview-left-icon-date"><span class="stec-layout-single-preview-left-icon-date-number"><?php esc_html_e($d); ?></span><span class="stec-layout-single-preview-left-icon-date-month"><?php esc_html_e($mmm); ?></span></span>
                        <?php
                        break;

                    case 'image' :

                        if ($event->get_icon_images()) {
                            ?>
                            <div class="stec-layout-single-preview-left-icon-images">
                                <?php
                                foreach ($event->get_parsed_icon_images() as $k => $image) {
                                    ?><div class="stec-layout-single-preview-left-icon-image <?php echo (int) $k === 0 ? 'active' : '' ?>" style="background-image: url(<?php echo esc_url($image->src); ?>)"></div><?php
                                }
                                ?>
                            </div>
                            <?php
                        }
                        break;

                    case 'icon' :
                        ?><i class="<?php esc_attr_e($event->get_icon()); ?>"></i><?php
                        break;

                endswitch;
                ?>

            </div>


            <div class="stec-layout-single-preview-left-text">

                <?php
                switch ($event->get_event_status()) :
                    case 'EventCancelled' :
                        $event_status       = esc_html__('Cancelled', 'stec');
                        $event_status_class = 'cancelled';
                        break;

                    case 'EventMovedOnline' :
                        $event_status       = esc_html__('Moved Online', 'stec');
                        $event_status_class = 'moved-online';
                        break;

                    case 'EventPostponed' :
                        $event_status       = esc_html__('Postponed', 'stec');
                        $event_status_class = 'postponed';
                        break;

                    case 'EventRescheduled' :
                        $event_status       = esc_html__('Rescheduled', 'stec');
                        $event_status_class = 'rescheduled';
                        break;

                    default:
                        $event_status       = null;
                        $event_status_class = null;

                endswitch;

                if ($event_status) {
                    printf('<div class="stec-layout-single-preview-left-text-event-status"><span class="%s">%s</span></div>', $event_status_class, $event_status);
                }

                if ($event->get_approved() === 0) {
                    printf('<div class="stec-layout-single-preview-left-text-event-status"><span class="unapproved">%s</span></div>', esc_html__('Awaiting Approval', 'stec'));
                }

                if ($event->get_featured() >= 1) {
                    printf('<div class="stec-layout-single-preview-left-text-featured"><i class="fa fa-star"></i><span>%s</span></div>', esc_html__('Featured', 'stec'));
                }
                ?>

                <?php
                printf('<span class="stec-layout-single-preview-left-text-title">%s</span>', $is_cancelled ? "<s>{$event->get_title()}</s>" : $event->get_title());
                ?>

                <span class="stec-layout-single-preview-left-text-secondary">
                    <?php
                    printf('<span class="stec-layout-single-preview-left-text-date"><i class="fas fa-calendar"></i>%s</span>', Admin_Helper::get_the_timespan($event, $repeat_offset));


                    if ($event->get_location()) {
                        $location = $event->get_parsed_location();

                        if ('virtual' === $location->type) {
                            printf('<span class="stec-layout-single-preview-left-text-location"><i class="fas fa-map-marker-alt"></i>%s</span>', esc_html__('Online Event', 'stec'));
                        } else {
                            printf('<span class="stec-layout-single-preview-left-text-location"><i class="fas fa-map-marker-alt"></i>%s</span>', $location->full_address);
                        }
                        unset($location);
                    }
                    ?>
                </span>

                <div class="stec-layout-single-preview-separator">

                    <?php
                    if ($edit_link) {
                        printf('<a class="stec-layout-single-preview-left-event-edit" href="%s"><i class="far fa-edit"></i></a>', $edit_link);
                    }
                    ?>

                    <a href="javascript:void(0);" class="stec-layout-single-preview-left-reminder-toggle">
                        <i class="fa fa-bell"></i>
                    </a>
                </div>
            </div>

        </div>

        <div class="stec-layout-single-preview-right">

            <?php
            if ($edit_link) {
                printf('<a class="stec-layout-single-preview-right-event-edit" href="%s"><i class="far fa-edit"></i></a>', $edit_link);
            }

            if (!Admin_Helper::reminder_expired($event, $repeat_offset)) {
                echo '<div class="stec-layout-single-preview-right-menu"><i class="fa fa-bell"></i></div>';
            }
            ?>

        </div>

    </div>

    <?php if ($event->get_images()) : ?>

        <div class="stec-layout-single-section stec-layout-single-media">

            <div class="stec-layout-single-media-content">

                <?php
                foreach ($event->get_parsed_images() as $image) :
                    ?>
                    <div style="background-image:url(<?php echo esc_url($image->src); ?>);"></div>
                    <?php
                endforeach;
                ?>

            </div>

            <div class="stec-layout-single-media-content-subs">
                <div>
                    <p></p>
                    <span></span>
                </div>
            </div>

            <div class="stec-layout-single-media-controls">
                <div class="stec-layout-single-media-controls-prev stec-layout-single-btn-fontandcolor"><i class="fa fa-angle-left"></i></div>
                <div class="stec-layout-single-media-controls-list-wrap">
                    <ul class="stec-layout-single-media-controls-list">

                        <?php
                        if (($event->get_images())) :
                            foreach ($event->get_parsed_images() as $image) :
                                $image = (object) $image;
                                ?>
                                <li style="background-image: url(<?php echo esc_url($image->thumb); ?>);"></li>
                                <?php
                            endforeach;
                        endif;
                        ?>

                    </ul>
                </div>
                <div class="stec-layout-single-media-controls-next stec-layout-single-btn-fontandcolor"><i class="fa fa-angle-right"></i></div>
            </div>

        </div>

    <?php endif; ?>

    <?php
    if ($event->get_categories() && Settings::get_admin_setting_value('stec_menu__general', 'show_event_tags') === '1') :
        echo '<div class="stec-layout-single-categories">';
        foreach ($event->get_parsed_categories() as $category) {
            printf('<span class="stec-layout-single-category" style="background-color: %s">%s</span>', esc_attr__($category['color']), esc_html__($category['title']));
        }
        echo '</div>';
    endif;
    ?>

    <?php
    if ($has_tickets) :
        ?>
        <button type="button" class="stec-style-button stec-layout-single-tickets-default" data-action="open-tickets"><?php esc_html_e('Tickets', 'stec'); ?></button>
        <?php
    endif;
    ?>

    <?php if ($event->get_description('raw') !== '') : ?>

        <div class="stec-layout-single-section stec-layout-single-description stec-style-text">
            <?php echo $event->get_description(); ?>
        </div>

    <?php endif; ?>

    <?php
    if ($event->get_organizers()) :
        ?>            

        <ul class="stec-layout-single-section stec-layout-single-organizers">

            <?php
            $organizers = $event->get_organizers();

            foreach ($organizers as $organizer_id) :

                $organizer = new Organizer((int) $organizer_id);

                if (!$organizer->get_id()) {
                    continue;
                }
                ?>
                <li class="stec-layout-single-organizers-organizer">
                    <div class="stec-layout-single-organizers-organizer-image">
                        <?php if ($organizer->get_photo_src()) : ?>
                            <div class="stec-layout-single-organizers-organizer-image-photo" style="background-image:url(<?php echo esc_url($organizer->get_photo_src()); ?>)"></div>
                        <?php else: ?> 
                            <div class="stec-layout-single-organizers-organizer-image-default"></div>
                        <?php endif; ?>
                    </div>
                    <div class="stec-layout-single-organizers-organizer-content">
                        <div class="stec-layout-single-organizers-organizer-title stec-style-title2"><?php
                            if ($organizer->get_permalink()) {
                                printf("<a href='%s'>%s</a>", esc_url($organizer->get_permalink()), esc_html($organizer->get_name()));
                            } else {
                                esc_html_e($organizer->get_name());
                            }
                            ?></div>
                        <div class="stec-layout-single-organizers-organizer-sub-title"><?php printf(esc_html__('Organizer of %s', 'stec'), $event->get_title()); ?></div>
                        <div class="stec-layout-single-organizers-organizer-description stec-style-text"><?php esc_html_e($organizer->get_details()); ?></div>
                        <div class="stec-layout-single-organizers-organizer-social">
                            <?php
                            if ($organizer->get_social()) :
                                foreach ($organizer->get_social() as $social) {
                                    if (false !== filter_var($social['link'], FILTER_VALIDATE_EMAIL)) {
                                        printf("<a href='mailto:%s'><i class='%s'></i></a>",
                                                $social['link'],
                                                esc_attr($social['ico'])
                                        );
                                    } elseif ('fas fa-phone' === $social['ico']) {
                                        echo "<a href='tel:{$social['link']}'><i class='{$social['ico']}'></i></a>";
                                    } else {
                                        printf("<a target='_blank' href='%s'><i class='%s'></i></a>",
                                                esc_url($social['link']),
                                                esc_attr($social['ico'])
                                        );
                                    }
                                }
                            endif;
                            ?>
                        </div>
                    </div>
                </li>
            <?php endforeach; ?>
        </ul>

    <?php endif; ?>

    <?php if ($can_rsvp || $event->get_link('url') !== '') : ?>
        <div class="stec-layout-single-section stec-style-full-width stec-style-float-left">

            <div class="stec-style-flexbox-no-media stec-style-full-width stec-style-float-left">

                <?php
                if ($event->get_link('url') !== '') :
                    printf('<a href="%s" class="stec-layout-single-external-link stec-style-full-width stec-style-button stec-layout-event-btn-fontandcolor" target="_BLANK">%s</a>', esc_url($event->get_link('url')), $event->get_link('text') ? $event->get_link('text') : esc_html__('Visit Website', 'stec'));
                endif;
                ?>

                <?php if (false === $rsvp_expired && 1 === (int) $event->get_rsvp() && false === Admin_Helper::user_is_invited($event)) : ?>
                    <a class="stec-layout-single-rsvp stec-style-full-width stec-style-button stec-layout-event-btn-fontandcolor" href="javascript:void(0);"><?php esc_html_e('RSVP to Event', 'stec'); ?></a>
                <?php endif; ?>
            </div>
        </div>
    <?php endif; ?>

    <?php if (!$is_cancelled && (int) $event->get_counter() === 1) : ?>

        <div class="stec-layout-single-section">

            <div class="stec-layout-single-event-status">
                <span class="stec-layout-single-event-status-text event-startsin"><?php esc_html_e('Event start in', 'stec'); ?></span>
                <span class="stec-layout-single-event-status-text event-inprogress"><?php esc_html_e('Event end in', 'stec'); ?></span>
                <span class="stec-layout-single-event-status-text event-expired"><?php esc_html_e('Event expired', 'stec'); ?></span>
            </div>

            <ul class="stec-layout-single-counter">
                <li>
                    <p class="stec-layout-single-counter-num">0</p>
                    <p class="stec-layout-single-counter-label" data-plural-label="<?php esc_html_e('Days', 'stec'); ?>" data-singular-label="<?php esc_html_e('Day', 'stec'); ?>">days</p>
                </li>
                <li>
                    <p class="stec-layout-single-counter-num">0</p>
                    <p class="stec-layout-single-counter-label" data-plural-label="<?php esc_html_e('Hours', 'stec'); ?>" data-singular-label="<?php esc_html_e('Hour', 'stec'); ?>">hours</p>
                </li>
                <li>
                    <p class="stec-layout-single-counter-num">0</p>
                    <p class="stec-layout-single-counter-label" data-plural-label="<?php esc_html_e('Minutes', 'stec'); ?>" data-singular-label="<?php esc_html_e('Minute', 'stec'); ?>">minutes</p>
                </li>
                <li>
                    <p class="stec-layout-single-counter-num">0</p>
                    <p class="stec-layout-single-counter-label" data-plural-label="<?php esc_html_e('Seconds', 'stec'); ?>" data-singular-label="<?php esc_html_e('Second', 'stec'); ?>">seconds</p>
                </li>
            </ul>

        </div>

    <?php endif; ?>

    <?php
    if (Admin_Helper::user_is_invited($event)) :

        $user_status = (int) Admin_Helper::get_user_attendance_status($event, false, $repeat_offset);
        $free_slots  = Rsvp::has_free_slots($event, $event->get_start_date('Y-m-d', $repeat_offset));

        if (1 === $user_status || $free_slots !== false) :
            $style = sprintf('margin-right:%s%s', '10', 'px');
            ?>
            <div class="stec-layout-single-section">
                <ul class="stec-layout-single-attendance-buttons">

                    <?php
                    printf('<li style="%s" class="stec-style-button stec-layout-single-attendance-button-attend  %s"><span>%s</span></li>', $style, 1 === $user_status ? 'active' : '', esc_html__('Attend', 'stec'));
                    printf('<li style="%s" class="stec-style-button stec-layout-single-attendance-button-decline  %s"><span>%s</span></li>', $style, 2 === $user_status ? 'active' : '', esc_html__('Decline', 'stec'));
                    printf('<li class="stec-style-button stec-layout-single-attendance-button-unlist "><span>%s</span></li>', esc_html__('Unlist', 'stec'));
                    ?>

                </ul>
            </div>
        <?php endif; ?>
    <?php endif; ?>

    <div class="stec-single-sections">

        <?php if ($event->get_attachments()) : ?>

            <div class="stec-layout-single-section stec-layout-single-attachments">

                <div class="stec-layout-single-attachments-top">

                    <p class="stec-style-title2"><i class="far fa-file"></i><?php esc_html_e('Attachments', 'stec'); ?></p>

                    <div class="stec-layout-single-attachments-toggle">
                        <i class="fa fa-plus"></i>
                        <i class="fa fa-minus"></i>
                    </div>

                </div>

                <ul class="stec-layout-single-attachments-list">

                    <?php foreach ($event->get_parsed_attachments() as $file) : ?>

                        <li class="stec-layout-single-attachment">
                            <div>
                                <p class="stec-layout-single-attachment-title stec-style-title2"><a href="<?php echo esc_url($file->link) ?>"><?php esc_html_e($file->filename); ?></a></p>
                                <p class="stec-layout-single-attachment-desc stec-style-text"><?php esc_html_e($file->description); ?></p>
                            </div>

                            <div>
                                <a href="<?php echo esc_url($file->link) ?>" class="stec-style-title2"><?php esc_html_e('Download', 'stec'); ?></a>
                                <p class="stec-layout-single-attachment-size stec-style-text"><?php esc_html_e($file->size); ?></p>
                            </div>
                        </li>

                    <?php endforeach; ?>
                </ul>
            </div>


        <?php endif; ?>

        <?php
        if (Settings::get_admin_setting_value('stec_menu__general', 'social_links') === '1' ||
                Settings::get_admin_setting_value('stec_menu__general', 'show_export_buttons') === '1') :
            ?>  

            <div class="stec-layout-single-section stec-layout-single-share-and-export">

                <div class="stec-layout-single-share">
                    <?php
                    if (Settings::get_admin_setting_value('stec_menu__general', 'social_links') === '1') :
                        echo Admin_Helper::get_share_links($the_permalink);
                    endif;
                    ?>
                </div>

                <?php if (Settings::get_admin_setting_value('stec_menu__general', 'show_export_buttons') === '1') : ?>
                    <div class="stec-layout-single-export">
                        <form method="POST"> 
                            <button class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor"><?php esc_html_e('Export to .ICS file', 'stec'); ?></button>
                            <input type="hidden" value="<?php echo (int) $event->get_id(); ?>" name="event_id">
                            <input type="hidden" value="<?php echo (int) $event->get_calid(); ?>" name="calendar_id">
                            <input type="hidden" value="stec_public_export_to_ics" name="task">
                        </form>

                        <a class="stec-style-button-secondary stec-layout-event-btn-sec-fontandcolor" href="<?php echo esc_url($event->get_gcal_link($repeat_offset)); ?>" target="_BLANK" class="stec-style-text"><?php esc_html_e('Import to Google Calendar', 'stec'); ?></a>
                    </div>
                <?php endif; ?>
            </div>
        <?php endif; ?>

        <?php
        if ($event->get_location()) :

            $location = $event->get_parsed_location();
            switch ($location->type) :

                case 'virtual' :
                    ?>

                    <div class="stec-layout-single-section stec-layout-single-location">
                        <div class="stec-layout-single-location-virtual">
                            <div class="stec-layout-single-location-virtual-tags">
                                <span><i class="fas fa-wifi"></i><?php esc_html_e('Online Event', 'stec'); ?></span>
                            </div>

                            <div class="stec-layout-single-location-virtual-flex">

                                <div>
                                    <div class="stec-layout-single-location-left-qr-code">
                                        <img src="<?php echo esc_url($location->qrcode); ?>" />
                                    </div>
                                </div>

                                <div>
                                    <a class="stec-style-button" target="_blank" href="<?php echo esc_url($location->address); ?>"><?php esc_html_e('Join Event', 'stec'); ?></a>
                                    <div class="stec-layout-single-location-virtual-share">
                                        <?php
                                        echo Admin_Helper::get_virtual_location_share_links($location->address);
                                        ?>
                                    </div>
                                </div>

                            </div>

                            <?php if ($location->details) : ?>

                                <div class="stec-layout-single-location-optional-details">
                                    <p class="stec-style-title2"><?php esc_html_e('Details', 'stec'); ?></p>
                                    <div class='stec-layout-single-location-details stec-style-text'>
                                        <?php
                                        echo Admin_Helper::esc($location->details, 'noscript');
                                        ?>
                                    </div>
                                </div>

                            <?php endif; ?>
                        </div>
                    </div>
                    <?php
                    break;

                case 'physical' :
                default:
                    ?>
                    <div class="stec-layout-single-section stec-layout-single-location">
                        <div class="stec-layout-single-location-flex">

                            <div class="stec-layout-single-location-left">
                                <p class="stec-style-title2"><?php esc_html_e('Location', 'stec'); ?></p>
                                <span class="stec-layout-single-location-address stec-style-text" data-location-coordinates="<?php
                                esc_attr_e($location->coordinates);
                                ?>"><?php
                                          if ($location->title) {
                                              esc_attr_e(implode(', ', array($location->title, $location->full_address)));
                                          } else {
                                              esc_attr_e($location->full_address);
                                          }
                                          ?></span>
                            </div>

                            <div class="stec-layout-single-location-right">
                                <p class="stec-layout-single-location-directions-title stec-style-title2"><?php esc_html_e('Directions', 'stec'); ?></p>
                            </div>

                        </div>

                        <div class="stec-layout-single-location-flex">

                            <div class="stec-layout-single-location-left">
                                <div class="stec-layout-single-location-map"></div>
                            </div>

                            <div class="stec-layout-single-location-right">
                                <input class="stec-layout-event-input-fontandcolor" type="text" name="start" placeholder="<?php esc_html_e('Start Location', 'stec'); ?>" />
                                <input class="stec-layout-event-input-fontandcolor" type="text" name="end" placeholder="<?php esc_html_e('Destination', 'stec'); ?>" />
                                <div class="stec-style-flexbox-no-media stec-layout-single-location-directions-buttons">
                                    <a href="javascript:void(0);" class="stec-style-button stec-layout-single-location-get-direction-btn "><?php esc_html_e('Get Directions', 'stec'); ?></a>
                                    <a target="_blank" href="javascript:void(0);" class="stec-style-button stec-layout-single-location-open-map "><?php esc_html_e('Open Map', 'stec'); ?></a>
                                </div>
                                <p class="stec-layout-single-location-direction-error stec-style-text"><?php esc_html_e('Could not find route!', 'stec'); ?></p>
                            </div>

                        </div>

                        <?php if ($location->details !== '') : ?>

                            <p class="stec-layout-single-location-details-title stec-style-title2"><?php esc_html_e('Location Details', 'stec'); ?></p>

                            <p class='stec-layout-single-location-details stec-style-text'>
                                <?php echo Admin_Helper::esc($location->details, 'noscript'); ?>
                            </p>

                        <?php endif; ?>

                    </div>
                <?php

            endswitch;
            ?>


        <?php endif; ?>


        <div class="stec-layout-single-section stec-layout-single-tabs">

            <ul class="stec-layout-single-tabs-list">

                <?php do_action('stec_add_event_single_tab'); ?>

            </ul>

            <div class="stec-layout-event-single-tabs-content">

                <?php do_action('stec_add_event_single_tab_content'); ?>

            </div>

        </div>


    </div>

    <?php do_action('stec_single_show_related'); ?>

</div>

<?php
do_action('stec_single_after_content');
