<?php

namespace Stachethemes\Stec;

trait Stec_Widget_Event {

    private $stec_instance     = array();
    private $events            = array();

    public function stec_is_online_location(Event_Post $event) {

        if (!$event->get_location()) {
            return false;
        }

        $location = $event->get_parsed_location();

        return 'virtual' === $location->type;
    }

    public function stec_prepare_events() {
        $params = array();
        
        if (!isset($this->stec_instance['status'])) {
            $this->stec_instance['status'] = 'upcoming';
        }

        if (true === $this->stec_instance['exclude_cancelled']) {
            $params['meta_query'] = array(
                array(
                    'key'     => 'event_status',
                    'value'   => array('EventCancelled', 'EventPostponed'),
                    'compare' => 'NOT IN'
                )
            );
        }

        if (true === $this->stec_instance['featured_only']) {
            $params['meta_query'] = array(
                array(
                    'key'     => 'featured',
                    'value'   => array('1', '2'),
                    'compare' => 'IN'
                )
            );
        }

        if ($this->stec_instance['calendar_id']) {
            $params['calid'] = $this->stec_instance['calendar_id'];
        }

        if ($this->stec_instance['category_id']) {
            $params['tax_query']   = array();
            $params['tax_query'][] = array(
                'taxonomy'         => 'stec_event_category',
                'field'            => 'id',
                'terms'            => $this->stec_instance['category_id'],
                'include_children' => false
            );
        }

        $cache_string_data_array = array(
            array($this->stec_instance['status'], $this->stec_instance['calendar_id'], $this->stec_instance['category_id'], $this->stec_instance['max_events_number']),
            get_current_user_id()
        );

        $cached_events = Cache::get_cache($cache_string_data_array, apply_filters('stec_widget_force_cache', false));

        if (!$cached_events) {
            
            switch($this->stec_instance['status']) :
                
                case 'expired' :
                    
                    $this->events = Events::get_expired_events($this->stec_instance['max_events_number'], $params);
                    
                    break;
                
                case 'upcoming' :
                default:
                    $this->events = Events::get_upcoming_events($this->stec_instance['max_events_number'], $params);
                    
            endswitch;
            
            Cache::set_cache($cache_string_data_array, $this->events, 300, apply_filters('stec_widget_force_cache', false));
        } else {
            $this->events = $cached_events;
        }
    }

    public function stec_event_start_string(Event_Post $event) {

        $format = Admin_Helper::get_datetime_format((bool) !$event->get_all_day());

        $date_time_string = $event->get_start_date($format);

        return apply_filters('stec_widget_date_string', $date_time_string, $event->get_all_day());
    }

    public function stec_event_location_string(Event_Post $event) {
        if ($event->get_location()) {
            $location = $event->get_parsed_location();

            if ('virtual' === $location->type) {
                return esc_html__('Online Event', 'stec');
            }

            return $location->title ? $location->title : $location->full_address;
        }

        return '';
    }

    public function stec_event_has_icon(Event_Post $event) {
        return $event->get_icon() !== '' && $event->get_icon() !== 'fa' ? true : false;
    }

    public function stec_event_thumb_html(Event_Post $event) {

        switch ($event->get_icon_type()) :

            case 'image' :
                $images = $event->get_parsed_icon_images();
                $image  = array_shift($images);
                printf('<span class="%1$s-event-thumb" style="background-color:%2$s"><span class="%1$s-event-thumb-image" style="background-image:url(%3$s)"></span></span>', $this->stec_class_prefix, esc_attr($event->get_color('display')), $image->thumb);
                break;

            case 'date' :
                $d = $event->get_start_date('j');
                $m = date_i18n('M', $event->get_start_date('U'));
                printf('<span class="%1$s-event-thumb" style="background-color:%2$s"><span class="%1$s-event-thumb-j">%3$s</span><span class="%1$s-event-thumb-m">%4$s</span></span>', $this->stec_class_prefix, esc_attr($event->get_color('display')), $d, $m);
                break;

            case 'icon' :
            default:
                printf('<span class="%s-event-thumb %s" style="background-color:%s"><i class="%s"></i></span>', $this->stec_class_prefix, $this->stec_event_has_icon($event) ? '' : 'event-no-icon', esc_attr($event->get_color('display')), $event->get_icon());

        endswitch;
    }

    public function stec_event_html(Event_Post $event) {

        $extraClasses = array();

        if ('EventCancelled' === $event->get_event_status()) {
            $extraClasses[] = 'cancelled';
        }

        printf('<div class="%s-event %s">', $this->stec_class_prefix, implode(' ', $extraClasses));
        printf('<div class="%s-wrap">', $this->stec_class_prefix);

        printf('<div class="%s-flex">', $this->stec_class_prefix);

        if (true === $this->stec_instance['display_thumbs']) {
            $this->stec_event_thumb_html($event);
        }

        echo '<div>';

        if (true === $this->stec_instance['display_tags']) :

            // If event has displayed tag use star icon otherwise use text
            if ($event->get_featured() > 0) {
                if ($this->stec_is_online_location($event) || in_array($event->get_event_status(), array('EventMovedOnline', 'EventCancelled', 'EventPostponed', 'EventRescheduled'))) {
                    $is_featured_text = '<i class="fa fa-star"></i>';
                } else {
                    $is_featured_text = sprintf('<span>%s</span>', esc_html__('Featured', 'stec'));
                }
                printf('<span class="%1$s-event-featured">%2$s</span>', $this->stec_class_prefix, $is_featured_text);
            }

            switch ($event->get_event_status()) :

                case 'EventCancelled' :
                    printf('<span class="%1$s-event-status %1$s-event-cancelled">%2$s</span>', $this->stec_class_prefix, esc_html__('Cancelled', 'stec'));
                    break;

                case 'EventMovedOnline' :
                    printf('<span class="%1$s-event-status %1$s-event-moved-online">%2$s</span>', $this->stec_class_prefix, esc_html__('Moved Online', 'stec'));
                    break;

                case 'EventPostponed' :
                    printf('<span class="%1$s-event-status %1$s-event-postponed">%2$s</span>', $this->stec_class_prefix, esc_html__('Postponed', 'stec'));
                    break;

                case 'EventRescheduled' :
                    printf('<span class="%1$s-event-status %1$s-event-rescheduled">%2$s</span>', $this->stec_class_prefix, esc_html__('Rescheduled', 'stec'));
                    break;

            endswitch;

            if ('EventMovedOnline' !== $event->get_event_status() && $this->stec_is_online_location($event)) {
                printf('<span class="%1$s-event-status %1$s-event-online">%2$s</span>', $this->stec_class_prefix, esc_html__('Online Event', 'stec'));
            }

        endif;

        echo apply_filters('stec-widget-event-title', sprintf('<span class="%s-event-title">%s</span> ', $this->stec_class_prefix, esc_html__($event->get_title())), $event, $this->stec_instance, $this->stec_class_prefix);
        echo '</div>';


        echo '</div>';

        printf('    <div class="%s-event-sub">', $this->stec_class_prefix);
        printf('            <span class="%1$s-event-date %1$s-event-sub-item"><i class="fas fa-calendar"></i>%2$s</span>', $this->stec_class_prefix, $this->stec_event_start_string($event));


        if (true === $this->stec_instance['display_location'] && $event->get_location()) {
            printf('<span class="%1$s-event-location %1$s-event-sub-item"><i class="fas fa-map-marker"></i><span>%2$s</span></span>', $this->stec_class_prefix, $this->stec_event_location_string($event));
        }

        if (true === $this->stec_instance['display_categories'] && $event->get_categories()) {
            $categories = $event->get_parsed_categories();
            $cats_list  = array();

            foreach ($categories as $category) {
                $cats_list[] = sprintf('<span>%s</span>', $category['title']);
            }

            printf('<span class="%1$s-event-categories %1$s-event-sub-item"><i class="fas fa-folder"></i><span>%2$s</span></span>', $this->stec_class_prefix, implode(', ', $cats_list));
        }

        if (true === $this->stec_instance['display_tickets'] && $event->get_bookable_products()) {
            printf('<span class="%1$s-event-tickets %1$s-event-sub-item"><i class="fas fa-ticket-alt"></i>%2$s</span>', $this->stec_class_prefix, esc_html__('Tickets', 'stec'));
        }

        echo apply_filters('stec-widget-after-content', '', $event, $this->stec_instance, $this->stec_class_prefix);

        echo '      </div>';
        echo '  </div>';
        
        
        printf('<a class="%s-event-permalink" href="%s" target="'. $this->stec_instance['a_target'] .'"></a>', 
                $this->stec_class_prefix, 
                $this->stec_instance['click_link'] === 'external' ? $event->get_link('url') : $event->get_permalink());
        echo '</div>';
    }

}
