<?php

namespace Stachethemes\Stec;

class Stec_Widget_Events_List extends \WP_Widget {

    private $stec_class_prefix = 'stec-widget-events-list';

    use Stec_Widget_Event;

    function __construct() {
        parent::__construct(
                'stec_widget_events_list',
                esc_html__('STEC Events List', 'stec'),
                array('description' => esc_html__('Displays list of events', 'stec'),)
        );

        Stachethemes_Event_Calendar::get_instance()->add_asset_css($this->stec_class_prefix, 'css/front/style.widget-events.min.css');

        if ('1' === Settings::get_admin_setting_value('stec_menu__general_other', 'load_fontawesome')) {
            Stachethemes_Event_Calendar::get_instance()->add_asset_css("stec-font-awesome", "css/fonts/css/all.min.css", true);
        }
    }

    public function widget($args, $instance) {
        $title = apply_filters('widget_title', $instance['title']);

        echo $args['before_widget'];
        if (!empty($title)) {
            echo $args['before_title'] . $title . $args['after_title'];
        }

        $this->stec_instance = $instance;
        $this->stec_prepare_events();
        $this->stec_render_widget_content();

        echo $args['after_widget'];
    }

    // Admin
    public function form($instance) {
        if (isset($instance['title'])) {
            $title = $instance['title'];
        } else {
            $title = esc_html__('Events', 'stec');
        }

        $calendars          = Calendars::get_calendars_list();
        $categories         = Categories::get_categories_list();
        $calendar_id        = isset($instance['calendar_id']) && is_array($instance['calendar_id']) ? $instance['calendar_id'] : array();
        $category_id        = isset($instance['category_id']) && is_array($instance['category_id']) ? $instance['category_id'] : array();
        $status             = isset($instance['status']) ? $instance['status'] : 'upcoming';
        $a_target           = isset($instance['a_target']) ? $instance['a_target'] : '_self';
        $click_link         = isset($instance['click_link']) ? $instance['click_link'] : 'permalink';
        $max_events_number  = max(1, isset($instance['max_events_number']) ? (int) $instance['max_events_number'] : 5);
        $display_tags       = isset($instance['display_tags']) ? (bool) $instance['display_tags'] : true;
        $display_thumbs     = isset($instance['display_thumbs']) ? (bool) $instance['display_thumbs'] : true;
        $display_categories = isset($instance['display_categories']) ? (bool) $instance['display_categories'] : true;
        $display_location   = isset($instance['display_location']) ? (bool) $instance['display_location'] : true;
        $display_tickets    = isset($instance['display_tickets']) ? (bool) $instance['display_tickets'] : true;
        $exclude_cancelled  = isset($instance['exclude_cancelled']) ? (bool) $instance['exclude_cancelled'] : true;
        $featured_only      = isset($instance['featured_only']) ? (bool) $instance['featured_only'] : false;
        ?>

        <p>
            <label for="<?php echo $this->get_field_id('title'); ?>"><?php esc_html_e('Widget Title', 'stec'); ?></label> 
            <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php esc_attr_e($title); ?>" />
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('calendar_id[]'); ?>"><?php esc_html_e('Calendar'); ?></label>
            <select class="widefat" id="<?php echo $this->get_field_id('calendar_id[]'); ?>" name="<?php echo $this->get_field_name('calendar_id[]'); ?>" multiple="multiple">
                <?php
                foreach ($calendars as $id => $title) {
                    $is_selected = in_array($id, $calendar_id) ? 'selected' : '';
                    printf('<option %s value="%d">%s</option>', $is_selected, esc_attr($id), esc_html__($title));
                }
                ?>
            </select>
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('category_id[]'); ?>"><?php esc_html_e('Category'); ?></label>
            <select class="widefat" id="<?php echo $this->get_field_id('category_id[]'); ?>" name="<?php echo $this->get_field_name('category_id[]'); ?>" multiple="multiple">
                <?php
                foreach ($categories as $id => $title) {
                    $is_selected = in_array($id, $category_id) ? 'selected' : '';
                    printf('<option %s value="%d">%s</option>', $is_selected, esc_attr($id), esc_html__($title));
                }
                ?>
            </select>
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('status'); ?>"><?php esc_html_e('Status', 'stec'); ?></label>
            <select class="widefat" id="<?php echo $this->get_field_id('status'); ?>" name="<?php echo $this->get_field_name('status'); ?>">
                <?php
                printf('<option %s value="%s">%s</option>', $status === 'upcoming' ? 'selected' : '', esc_attr('upcoming'), esc_html__('Upcoming', 'stec'));
                printf('<option %s value="%s">%s</option>', $status === 'expired' ? 'selected' : '', esc_attr('expired'), esc_html__('Expired', 'stec'));
                ?>
            </select>
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('a_target'); ?>"><?php esc_html_e('Open In', 'stec'); ?></label>
            <select class="widefat" id="<?php echo $this->get_field_id('a_target'); ?>" name="<?php echo $this->get_field_name('a_target'); ?>">
                <?php
                printf('<option %s value="%s">%s</option>', $a_target === '_self' ? 'selected' : '', esc_attr('_self'), esc_html__('Same Window', 'stec'));
                printf('<option %s value="%s">%s</option>', $a_target === '_blank' ? 'selected' : '', esc_attr('_blank'), esc_html__('New Window', 'stec'));
                ?>
            </select>
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('click_link'); ?>"><?php esc_html_e('Click Link', 'stec'); ?></label>
            <select class="widefat" id="<?php echo $this->get_field_id('click_link'); ?>" name="<?php echo $this->get_field_name('click_link'); ?>">
                <?php
                printf('<option %s value="%s">%s</option>', $click_link === 'permalink' ? 'selected' : '', esc_attr('permalink'), esc_html__('Permalink', 'stec'));
                printf('<option %s value="%s">%s</option>', $click_link === 'external' ? 'selected' : '', esc_attr('external'), esc_html__('External Link', 'stec'));
                ?>
            </select>
        </p>

        <p>
            <label for="<?php echo $this->get_field_id('max_events_number'); ?>"><?php esc_html_e('Numbers of events to show', 'stec') ?></label>
            <input class="tiny-text" id="<?php echo $this->get_field_id('max_events_number'); ?>" name="<?php echo $this->get_field_name('max_events_number'); ?>" type="number" step="1" max="20" min="1" value="<?php esc_attr_e($max_events_number); ?>" size="3">
        </p>

        <p>
            <input class="checkbox" type="checkbox" <?php echo $display_tags ? 'checked="checked"' : ''; ?> id="<?php echo $this->get_field_id('display_tags'); ?>" name="<?php echo $this->get_field_name('display_tags'); ?>">
            <label for="<?php echo $this->get_field_id('display_tags'); ?>"><?php esc_html_e('Display tags', 'stec') ?> </label>
            <br>
            <input class="checkbox" type="checkbox" <?php echo $display_thumbs ? 'checked="checked"' : ''; ?> id="<?php echo $this->get_field_id('display_thumbs'); ?>" name="<?php echo $this->get_field_name('display_thumbs'); ?>">
            <label for="<?php echo $this->get_field_id('display_thumbs'); ?>"><?php esc_html_e('Display thumbs', 'stec') ?> </label>
            <br>
            <input class="checkbox" type="checkbox" <?php echo $display_location ? 'checked="checked"' : ''; ?> id="<?php echo $this->get_field_id('display_location'); ?>" name="<?php echo $this->get_field_name('display_location'); ?>">
            <label for="<?php echo $this->get_field_id('display_location'); ?>"><?php esc_html_e('Display locations', 'stec') ?> </label>
            <br>
            <input class="checkbox" type="checkbox" <?php echo $display_categories ? 'checked="checked"' : ''; ?> id="<?php echo $this->get_field_id('display_categories'); ?>" name="<?php echo $this->get_field_name('display_categories'); ?>">
            <label for="<?php echo $this->get_field_id('display_categories'); ?>"><?php esc_html_e('Display categories', 'stec') ?> </label>
            <br>
            <input class="checkbox" type="checkbox" <?php echo $display_tickets ? 'checked="checked"' : ''; ?> id="<?php echo $this->get_field_id('display_tickets'); ?>" name="<?php echo $this->get_field_name('display_tickets'); ?>">
            <label for="<?php echo $this->get_field_id('display_tickets'); ?>"><?php esc_html_e('Display tickets', 'stec') ?> </label>
            <br>
            <input class="checkbox" type="checkbox" <?php echo $exclude_cancelled ? 'checked="checked"' : ''; ?> id="<?php echo $this->get_field_id('exclude_cancelled'); ?>" name="<?php echo $this->get_field_name('exclude_cancelled'); ?>">
            <label for="<?php echo $this->get_field_id('exclude_cancelled'); ?>"><?php esc_html_e('Exclude cancelled events', 'stec') ?> </label>
            <br>
            <input class="checkbox" type="checkbox" <?php echo $featured_only ? 'checked="checked"' : ''; ?> id="<?php echo $this->get_field_id('featured_only'); ?>" name="<?php echo $this->get_field_name('featured_only'); ?>">
            <label for="<?php echo $this->get_field_id('featured_only'); ?>"><?php esc_html_e('Featured events only', 'stec') ?> </label>

        </p>

        <?php
    }

    public function update($new_instance, $old_instance) {

        $instance                       = array();
        $instance['title']              = (!empty($new_instance['title']) ) ? strip_tags($new_instance['title']) : '';
        $instance['calendar_id']        = filter_var($new_instance['calendar_id'], FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY) ?: array();
        $instance['category_id']        = filter_var($new_instance['category_id'], FILTER_VALIDATE_INT, FILTER_REQUIRE_ARRAY) ?: array();
        $instance['status']             = filter_var($new_instance['status'], FILTER_SANITIZE_STRING) ?: 'upcoming';
        $instance['a_target']           = filter_var($new_instance['a_target'], FILTER_SANITIZE_STRING) ?: '_self';
        $instance['click_link']         = filter_var($new_instance['click_link'], FILTER_SANITIZE_STRING) ?: 'permalink';
        $instance['max_events_number']  = filter_var($new_instance['max_events_number'], FILTER_VALIDATE_INT) ?: 5;
        $instance['display_tags']       = isset($new_instance['display_tags']) ? (bool) $new_instance['display_tags'] : false;
        $instance['display_thumbs']     = isset($new_instance['display_thumbs']) ? (bool) $new_instance['display_thumbs'] : false;
        $instance['display_location']   = isset($new_instance['display_location']) ? (bool) $new_instance['display_location'] : false;
        $instance['display_categories'] = isset($new_instance['display_categories']) ? (bool) $new_instance['display_categories'] : false;
        $instance['display_tickets']    = isset($new_instance['display_tickets']) ? (bool) $new_instance['display_tickets'] : false;
        $instance['exclude_cancelled']  = isset($new_instance['exclude_cancelled']) ? (bool) $new_instance['exclude_cancelled'] : false;
        $instance['featured_only']      = isset($new_instance['featured_only']) ? (bool) $new_instance['featured_only'] : false;

        return $instance;
    }

    public function stec_render_widget_content() {

        printf('<div class="%s" data-loaded="false">', $this->stec_class_prefix);

        if (Stachethemes_Event_Calendar::get_instance()->lcns() !== false) {

            if ($this->events) {
                foreach ($this->events as $event) {
                    $this->stec_event_html($event);
                }
            }
        } else {
            Stachethemes_Event_Calendar::get_instance()->display_activation_required();
        }

        echo '</div>';
    }

}

add_action('widgets_init', function() {
    register_widget('\Stachethemes\Stec\Stec_Widget_Events_List');
});
